/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2012 by Chuck Chen <ycchen@realtek.com>
 *
 */

#ifndef _ASM_MACH_SYSTEM_H_
#define _ASM_MACH_SYSTEM_H_

#ifdef LD_SCRIPT_MACRO
#ifdef CONFIG_QC_VERIFY
#define BOOTCODE_RAM_LEN 0x600000
#else
#define BOOTCODE_RAM_LEN 0xE0000
#endif
#else

#include <asm/arch/io.h>
#include <rbus/stb_reg.h>
#include <rbus/sys_reg_reg.h>

// this extra info is append to each kernel image in secure booting
// this needs to sync with the structuer image_file_creator uses
struct image_extra_info {
	unsigned int marker;
	unsigned int size;
	unsigned int version;
	unsigned char reserved[20];
	unsigned char signature[256];
};

// Mips kseq base address
#define MIPS_KSEG0BASE		0x80000000
#define MIPS_KSEG1BASE		0xa0000000
#define MIPS_KSEG_MSK		0xe0000000

#define VERSION_A			0
#define VERSION_B			1
#define VERSION_C			2

// support 4 uarts, uart0 in ISO_MISC_OFF, uart1/2/3 in MISC
// this is configured by include/configs/xxx.h

// support 8 timers, timer0 for VCPU, timer1 for ACPU, timer2 for SCPU
#define SYSTEM_TIMER 		CONFIG_SYS_TIMER
//#define TIMER_CLOCK			50000000	//FPGA
#define TIMER_CLOCK			27000000
#define MAX_HWTIMER_NUM		8

// linux will do the remapping of rbus so that you have to read/write rbus via rtd_xxx series
// or just tells a correct address to the system
//#define SYSTEM_GIC_DIST_BASE		GET_MAPPED_RBUS_ADDR(0xb801f000)
//#define SYSTEM_GIC_CPU_BASE			GET_MAPPED_RBUS_ADDR(0xb801e100)
//#define SYSTEM_PL310_BASE			GET_MAPPED_RBUS_ADDR(0xb801d000)

// The definition of ddr address mapping in u-boot.
// Attention: if you need to modify the following settings,
//            please update the excel file in romcode/reference simultaneously.
#define ARM_REDIRECT_CODE_ADDR                  (0x00000000)
#define MIPS_AV_RESET_HANDLER_CODE_ADDR         (0x00010000)
#define MIPS_EXC_REDIRECT_OPT_CODE_ADDR         (0x00000054)
#define MIPS_AUDIO_EXC_VECTOR_A2_ADDR           (0x00000054)
#define MIPS_AUDIO_GDB_VECTOR_A2_ADDR           (0x00000074)
#define MIPS_AUDIO_FLAG_PTR_A2_ADDR             (0x00000078)
#define MIPS_AUDIO_FW_ENTRY_PTR_A2_ADDR         (0x0000007c)
#define MIPS_EXC_REDIRECT_CODE_ADDR             (0x00000080)
#define MIPS_AUDIO_GDB_VECTOR_A1_ADDR           (0x00000088)
#define MIPS_VIDEO_GDB_VECTOR_V1_ADDR           (0x0000008c)
#define MIPS_VIDEO_GDB_VECTOR_V2_ADDR           (0x00000090)
#define MIPS_AUDIO_EXC_VECTOR_A1_ADDR           (0x00000094)
#define MIPS_VIDEO_EXC_VECTOR_V1_ADDR           (0x000000a4)
#define MIPS_VIDEO_EXC_VECTOR_V2_ADDR           (0x000000b4)
#define MIPS_UART_REG_BASE_ADDR                 (0x000000c4)
#define MIPS_AUDIO_FLAG_STR_ADDR                (0x000000c8)
#define MIPS_BOOT_AV_INFO_PTR_ADDR              (0x000000cc)
#define MIPS_AUDIO_FLAG_PTR_A1_ADDR             (0x000000d0)
#define MIPS_VIDEO_FLAG_PTR_V1_ADDR             (0x000000d4)
#define MIPS_VIDEO_FLAG_PTR_V2_ADDR             (0x000000d8)
#define MIPS_VIDEO_FW_ENTRY_PTR_V2_ADDR         (0x000000dc)
#define MIPS_AUDIO_FW_ENTRY_PTR_A1_ADDR         (0x000000e0)
#define MIPS_VIDEO_FW_ENTRY_PTR_V1_ADDR         (0x000000e4)
#define MIPS_SHARED_PRINTK_BUFFER_ADDR          (0x000000e8)    // 0x000000e8~0x000000ff
#define MIPS_SHARED_PRINTK_BUFFER_SIZE          (0x18)
#define MIPS_EXC_DISPATCH_CODE_ADDR             (0x00000100)
#define MIPS_K_ENTRY_CODE_ADDR                  (0x00000160)
#define MIPS_V_ENTRY_CODE_ADDR                  (0x00000200)
#if 0 // remove kcpu
#define MIPS_KCPU_FLAG_PTR_ADDR                 (0x00000800)
#endif
#define FIRST_BOOT_BOOTING_TIME_VALUE           (0x0000098C)
#define MIPS_VIDEO_FLAG_STR_ADDR                (0x00000814)
#define MIPS_VIDEO2_FLAG_STR_ADDR               (0x00000818)
#define MIPS_AUDIO_OUTPUT_INFO_ADDR             (0x00001030)
#define MIPS_AUDIO_OUTPUT_INFO_SIZE             (0x50)
#define MIPS_BOOT_PLATFORM_INFO_ADDR            (0x00001010)
#define UBOOT_PLATFORM_INFO_ADDR                (0x00001020)
#define KERNEL_SEMAPHORE_BUFFER_ADDR            (0x00001200)    // 0x00001200~0x1244  0x000FFE80~0x000FFEC4
#define KERNEL_SEMAPHORE_BUFFER_SIZE            (0x48)
#define MIPS_BOOT_AV_INFO_ADDR                  (0x00001400)
#define UBOOT_MEM_INFO_ADDR                     (0x00001480)
#define MIPS_ISR_VIDEO_IMG_ADDR                 (0x00001500)
#define MIPS_ROS_BOOT_VECTOR_IMG_ADDR           (0x00001900)
#define MIPS_A_ENTRY_CODE_ADDR                  (0x00002000)
#define MIPS_A_ENTRY2_CODE_ADDR                 (0x00002400)
#define DDR_CALIBRATION_ADDR                    (0x00003FC0)    // 0x00003fc0~0x00003fff (32B:calibration, 32B:reserved)

#define ECPU_KERNEL_1_ENTRY_ADDR                (0x00004000)    // 0x00004000~0x00005fff
#define ECPU_KERNEL_2_ENTRY_ADDR                (0x00006000)    // 0x00006000~0x00007fff

#define ARM_REDIRECT_CODE2_ADDR                 (0x0001f800)    // 0x0001f800~0x0001ffff
#define KCPU_CMDLINE_ADDR                       (0x000fff00)    // 0x000fff00~0x00100000
#define KCPU_CMDLINE_SIZE                       (0x00000100)    // 0x000fff00~0x00100000

#define SHARE_CMDLINE_ADDR                      (0x000ffe00)    // 000f_fe00~00ff_ff00 /* command line to kcpu */
#define SHARE_CMDLINE_SIZE                      (0x00000080)    // 128
#define BOOTCODE_TEXT_ADDR                      (CONFIG_SYS_TEXT_BASE)
#define LINUX_KERNEL_ADDR                       (0x00100000)
#define LINUX_KERNEL_ENTRY                      (0x00108000)
#define LINUX_KERNEL_SIZE                       (0x02000000 - LINUX_KERNEL_ENTRY)
#define LINUX64_KERNEL_ENTRY                    (0x00180000)
#define LINUX64_KERNEL_SIZE                     (0x02000000 - LINUX64_KERNEL_ENTRY)
#define ROM_HASH_RESULT_ADDR                    (0x001FFF80)    // 128 byte for rom code hash result
#define ENCRYPTED_BOOTCODE_ADDR                 (0x00200000)
#define DVRBOOT_EXE_BIN_ENTRY_ADDR              (0x01500000)

/* Memory for power on music, video, and image. Include stream buffer and decode buffer */
//#define ENCRYPTED_LINUX_KERNEL_ADDR             (0x0B000000)    // 0B00_0000~0C00_0000 (16M)
//#define COMPRESSED_FW_IMAGE_ADDR                (0x0C000000)    // 0C00_0000~0E00_0000 (32M)
#define FIRMWARE_DESCRIPTION_TABLE_ADDR         (0x06800000)    // 0680_0000~0681_0000 (64K)
#define FACTORY_RO_DATA_ADDR                    (0x10990000)    // 1099_0000~10d9_0000 (4M)

// POM/POV/Resource Table
#define KERNEL_RESERVE_FOR_POM_POV_ADDR         (0x03000000)    // (must allign MB)
#define POWER_ON_MUSIC_BUFFER_ADDR              (KERNEL_RESERVE_FOR_POM_POV_ADDR)
#define POWER_ON_MUSIC_BUFFER_SIZE              (0x00500000)    // 5M
#define POWER_ON_VIDEO_BUFFER_ADDR              (POWER_ON_MUSIC_BUFFER_ADDR+POWER_ON_MUSIC_BUFFER_SIZE)
#define POWER_ON_VIDEO_BUFFER_SIZE              (0x02000000)    // 32M
#define POWER_ON_MUSIC_STREAM_ADDR              (POWER_ON_VIDEO_BUFFER_ADDR+POWER_ON_VIDEO_BUFFER_SIZE)
#define POWER_ON_MUSIC_STREAM_SIZE              (0x001ff000)    // (2M-4K)
#define POWER_ON_RESOURCE_ADDR                  (POWER_ON_MUSIC_STREAM_ADDR+POWER_ON_MUSIC_STREAM_SIZE)
#define POWER_ON_RESOURCE_LOGO_OFF_ADDR         (0x15f80000)    // special for the scenario without POV
#define POWER_ON_RESOURCE_SIZE                  (0x00001000)    // 4k
#define POWER_ON_VIDEO_STREAM_ADDR              (POWER_ON_RESOURCE_ADDR + POWER_ON_RESOURCE_SIZE)
#define POWER_ON_VIDEO_STREAM_SIZE              (0x01900000)    // 25M
#define POWER_ON_VIDEO_LAST_IMAGE_ADDR          (POWER_ON_VIDEO_STREAM_ADDR + POWER_ON_VIDEO_STREAM_SIZE)    // (must allign MB)
#define POWER_ON_VIDEO_LAST_IMAGE_SIZE          (0x00800000)    // 8M (must allign MB) (4M:720p, 8M:1080p)
//
#define POWER_ON_IR_TABLE_ADDR                  (0x1ffff000)    // 0x1ffff000 ~ 0x20000000
#define POWER_ON_IR_TABLE_SIZE                  (0x1000)        // 4k
#define SECURE_STORE_KEY_RESERVE_ADDR           (0x11590000)    // 1159_0000~1179_0000 (2M)
#define TEMP_BUFFER_FOR_PQ_FILE_DECOMPRESS      (0x11790000)    // 1179_0000~11B9_0000 (4M)
#define UBOOT_SECURE_MCP_MEMORY_ADDR            (0x12000000)    // 1200_0000~1400_0000
#define UBOOT_SECURE_MCP_MEMORY_SIZE            (0x02000000)    // 32M
#define MIPS_KCPU_FW_ENTRY_ADDR                 (0x96008000)

#ifdef CONFIG_JUMP_TO_64BIT_KERNEL
#define TEE_FW_ENTRY_ADDR                       (0x16008800)
#else
#define TEE_FW_ENTRY_ADDR                       (0x16008000)
#endif
#define ATF_CODE_SIZE                           (0x28000)
#define MIPS_AUDIO_FW_1_ENTRY_ADDR              (0x1B200000)    // 1B20_0000~1BBF_F000 (~10MB)
#define MIPS_AUDIO_FW_2_ENTRY_ADDR              (0x1A300000)    // 1A30_0000~1A50_0000 (2MB)
#define MIPS_VIDEO_FW_1_ENTRY_ADDR              (0x1A900000)    // 1A90_0000~1AE0_0000 (5MB)
#define MIPS_VIDEO_FW_2_ENTRY_ADDR              (0x1AE00000)    // 1AE0_0000~1B00_0000 (2MB)
#define RPC_SHARE_MEMORY_ADDR                   (0x1B1E0000)    // 1B1E_0000~1B20_0000 (128KB)
#define POWER_ON_VIDEO_VBM_BUFFER_ADDR          (0x20000000)    // 2000_0000~2200_0000
#define POWER_ON_VIDEO_VBM_BUFFER_SIZE          (0x02C00000)    // 44M for VBM

#define TEMP_SIZE_READ_FROM_USB_IMAGE           (0x1D000000)
#define SMP_CORE1_IRQ_SP_ADDR                   (0x23002000)    // 2300_0000~2300_2000 (8K)
#define SMP_CORE2_IRQ_SP_ADDR                   (0x23004000)    // 2300_2000~2300_4000 (8K)
#define SMP_CORE3_IRQ_SP_ADDR                   (0x23006000)    // 2300_4000~2300_6000 (8K)
#define SMP_CORE4_IRQ_SP_ADDR                   (0x23008000)    // 2300_6000~2300_8000 (8K)
#define SMP_COREX_IRQ_SP_SIZE                   (0x2000)
#define UBOOT_NONCACHE_MEMORY_ADDR              (0x23100000)    // 0x2310_0000~0x2320_0000 (1M)
#define UBOOT_HEAP_RELOCATION_ADDR	        (0x23200000)    // 0x2320_0000~0x2390_0000 (7M)

#define FACTORY_DATA_ADDR                       (0x23900000)    // 0x2390_0000~0x2490_0000 (16M)
#define TEMP_BUFFER_FOR_FLASH_DATA_ADDR         (0x24900000)    // 0x24900000~

#define SNAPSHOT_IMAGE_LOAD_ADDR                (0x24D00000)    // 24D0_0000~0x2fe_0000 (177M)
#define CONFIG_FASTBOOT_BUF_ADDR                (0x34000000)
#define CONFIG_FASTBOOT_BUF_SIZE                (0x04000000)
#define CONFIG_AVB_BUFFER_ADDR                  (0x38000000)
#define CONFIG_AVB_BUFFER_SIZE                  (0x04000000)

#define CONFIG_BOOT_IMG_BUF_ADDR                (CONFIG_FASTBOOT_BUF_ADDR)
#define CONFIG_BOOT_IMG_BUF_SIZE                (0x08000000)
#define RAMFS_IMG_ADDR                           0x10000000
#define ENCRYPTED_LINUX_KERNEL_ADDR             (0x3C000000)    // 3C00_0000~3E00_0000 (32M)
#define COMPRESSED_FW_IMAGE_ADDR                (0x30000000)    // 3000_0000~3200_0000 (32M) 
#define CONFIG_DTBO_BUFFER_ADDR                 (0x3E000000)
#define CONFIG_DTBO_BUFFER_SIZE                 (0x00800000)
#define CONFIG_DTB_MEGRE_BUFFER_ADDR        	(0x3E800000)
#define CONFIG_DTB_MEGRE_BUFFER_SIZE            (0x00100000)
#define CONFIG_LINUX_BK_BUFFER_ADDR      	(0x3E900000) //tmp buffer for tftp download vmlinux
#define CONFIG_LINUX_BK_BUFFER_SIZE           LINUX_KERNEL_SIZE

#define VIP_UBOOT_RESERVED_ADDR                 (CONFIG_VIP_RESERVED_ADDR)    // 30600000~3080_0000 (2M)
#define VIP_UBOOT_RESERVED_SIZE                 (CONFIG_VIP_RESERVED_SIZE)
#define OD_UBOOT_RESERVED_ADDR                  (CONFIG_OD_RESERVED_ADDR)     // 3080_0000~3100_0000 (8M)
#define OD_UBOOT_RESERVED_SIZE                  (CONFIG_OD_RESERVED_SIZE)

#define FW_DESC_TABLE_SIZE                      (0x10000)

typedef struct {
	uint heap_relocation_addr;
	uint noncache_mem_addr;
	uint factory_data_addr;
	uint factory_ro_data_addr;
	uint fw_desc_table_base_addr;
	uint encrypted_fw_image_addr;
	uint secure_mcp_mem_buffer_addr;
	uint temp_buffer_for_flash_data_addr;
} rtk_mem_info_t;

extern rtk_mem_info_t *rtk_mem_info;

enum {
	UBOOT_STAMP_BOARD_INIT_F_END = _BIT0,
	UBOOT_STAMP_BOARD_INIT_R_START = _BIT1,
	UBOOT_STAMP_BEFORE_DCACHE_ENABLE = _BIT2,
	UBOOT_STAMP_BEFORE_MMC_INIT = _BIT3,
	UBOOT_STAMP_BEFORE_BOOT_SECONDARY = _BIT4,
	UBOOT_STAMP_BEFORE_ENV_RELOCATE = _BIT5,
	UBOOT_STAMP_BEFORE_THREAD_START = _BIT6,
	UBOOT_STAMP_BEFORE_WAKE_UP_SMP = _BIT7,
	UBOOT_STAMP_BEFORE_LG_BOOT_SPLASH = _BIT8,
	UBOOT_STAMP_BEFORE_MAIN_LOOP = _BIT9,
	UBOOT_STAMP_BEFORE_RUN_BOOT_CMD = _BIT10,
	UBOOT_STAMP_11 = _BIT11,

	UBOOT_STAMP_BEFORE_MEM_CLEANUP = _BIT12,
	UBOOT_STAMP_13 = _BIT13,
	UBOOT_STAMP_14 = _BIT14,
	UBOOT_STAMP_15 = _BIT15,

	UBOOT_STAMP_BEFORE_DO_HIB = _BIT16,
	UBOOT_STAMP_BEFORE_GO_A_GO_V = _BIT17,
	UBOOT_STAMP_BEFORE_RESUME_KERNEL = _BIT18,
	UBOOT_STAMP_19 = _BIT19,

	UBOOT_STAMP_PANEL_INIT_START = _BIT20,
	UBOOT_STAMP_SPLASH_LOAD_LOGO_START = _BIT21,
	UBOOT_STAMP_BOOT_SPLASH_START = _BIT22,
	UBOOT_STAMP_DECOMPRESS_TZFW_START = _BIT23,

	UBOOT_STAMP_24 = _BIT24,
	UBOOT_STAMP_25 = _BIT25,
	UBOOT_STAMP_BEFORE_FAST_BOOT = _BIT26,
	UBOOT_STAMP_GO_ALL_FW_START = _BIT27,
	UBOOT_STAMP_BEFORE_ENABLE_A_V_CLK = _BIT28,
	UBOOT_STAMP_BEFORE_RUN_BOOTM = _BIT29,
	UBOOT_STAMP_PREP_LINUX_START = _BIT30,
	UBOOT_STAMP_JUMP_LINUX_START = _BIT31,
	UBOOT_STAMP_MAGIC = 0xcafe2379
};

// SPI flash base address
#define BOARD_FLASH_ADDR_BASE	(0xb8200000)

// Dummy Register
#define KCPU_JUMP_ADDR			(0xb801a608)	/* register which represent KCPU jump address (specified in Aux code or DTE Bootcode) */
#define VCPU_JUMP_ADDR_reg		(0xb801a61c)	/* register which represent VCPU jump address */
#ifdef CONFIG_VCPU2_SUPPORT
#define VCPU2_JUMP_ADDR_reg		(0xb801a618)	/* register which represent VCPU2 jump address */
#endif
#define ACPU_LOAD_ADDR			(0xb801a614)	/* register which represent ACPU load address (specified in HW setting) */
#ifdef IS_FPGA
#define ACPU_JUMP_ADDR_reg		(0xb801a618)	/* register which represent ACPU jump address (FPGA does not include ISO registers) */
#else
#define ACPU_JUMP_ADDR_reg		(0xb806012c)//(WDOG_DATA12)	/* register which represent ACPU jump address */
#endif

#define REG_CRT_SUSPEND_reg		(0xb8060100)//(WDOG_DATA1)	/* register which represent suspend to ram or disk */
#define REG_STR_JUMP_ADDR_reg	(0xb8060104)//(WDOG_DATA2)    /* register which represent STR jump address */
#define REG_KERNEL_RSV_MEM_reg	(0xb8060108)//(WDOG_DATA3)	/* register which represent kernel reserve memory or not */
#define REG_DDR_CAL_FLAG_reg	(0xb806010c)//(WDOG_DATA4)	/* register which represent DDR calibration status */
#define REG_PACKAGE_TYPE_reg	(0xb8060120)//(WDOG_DATA9)	/* register which represent boot package type */
#define REG_AC_DC_ON_reg		(0xb8060128)//(WDOG_DATA11)	/* register which represent AC or DC on */

/*
 * Description of REG_DDR_CAL_FLAG_reg:
 *	   BIT0: scpu will set this bit as 1, when scpu notice vcpu1 to enter calibration.
 *	   BIT1: vcpu1 will set this bit as 1, after vcpu1 has been finished calibration.
 *	   BIT2~31: reserved
 */
#define MIPS_V_DDR_CAL_CODE_reg		(0xb8064000)	/* register which represent DDR calibration code in DI-sram */
#define MIPS_V_DDR_CAL_C_CODE_reg	(0xb8064100)	/* register which represent DDR calibration C code in DI-sram */
#define DI_SRAM_MEM_SHARE_SIZE		(0x4000)		/* 16K */

// DC_SYS Register
#define DC_SYS_MISC_reg			(0xb8007004)
#define DC2_SYS_MISC_reg		(0xb8003004)

// ISO Register
#define ISO_MISC_OFF_INT_EN_reg 		0xB8062100

// MISC Register
#if 0 // FIXME in Mac3 (unknown register)
#define SCPU_INT_EN_reg 				0xB801B088
#endif
#endif
#endif	// _ASM_MACH_SYSTEM_H_

