/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2012 by Chuck Chen <ycchen@realtek.com>
 *
 */

#include <common.h>
#include <command.h>
#include <asm/arch/io.h>

//============================================================================

#define MAX_TEST_RUN 100
#define MAX_TEST_PT 10000

#define COHERENCE_TEST  1
#define STOP_ERR_CNT    1

#define PRINT_INFO      1
#define PRINT_SPIN_INFO 0
#define DEBUG           0

typedef unsigned int uint;

//static unsigned int SEM_SYNC_POINT2 = 0xb400000-0x4000; //0x104000;   // TODO, a writeable memroy address, size word
unsigned int LOG_ADDR   = 0xb400000-0x4000 + 0x4; //0x104004;               // TODO, a writeable memory address, size 4 words
unsigned int SRAM_BASE  = 0x10000000;               // TODO, a writeable memory space size TEST_BLOCK
unsigned int DRAM_BASE  = 0x11000000;               // TODO, a writeable memory space size TEST_BLOCK
unsigned int TEST_BLOCK = 0x00080000;               // TODO, test block can be reduced. Larger block size takes longer initial time

//============================================================================

enum CacheType  { CT_CACHED = 0, CT_UNCACHED, CT_TOTAL };
enum DataType   { DT_BYTE = 0, DT_HWORD, DT_WORD, DT_TOTAL };
enum AccessType { AT_WRITE = 0, AT_READ, AT_TOTAL };
char AccessTypeStr [16][16] = { "Write", "Read" };
char DataTypeStr [16][16]   = { "Byte", "HalfWord", "Word" };

enum MemInitType { IT_ZERO = 0, IT_ONE, IT_RANDOM, IT_BACKDOOR, IT_NON_BACKDOOR };

#define ALIGN32_FLOOR(addr) ((addr) & 0xFFFFFFFC)
#define ALIGN16_FLOOR(addr) ((addr) & 0xFFFFFFFe)

#define WRITE_MEM32(addr, val)   (*(volatile unsigned int *) (addr)) = (val)
#define WRITE_MEM16(addr, val)   (*(volatile unsigned short *) (addr)) = (val)
#define WRITE_MEM8(addr, val)    (*(volatile unsigned char *) (addr)) = (val)
#define READ_MEM32(addr)         (*(volatile unsigned int *) (addr))
#define READ_MEM16(addr)         (*(volatile unsigned short *) (addr))
#define READ_MEM8(addr)          (*(volatile unsigned char *) (addr))

#define SHOW_ERR_READ 1
#define RANDOM_BG 1
#define BG_DATA 0xdeadbeef
// for mmcr_cp3
#define WR   1
#define RD   0
// #define CTRL 0
// #define GEN  1
// #define CORE0 0x1
// #define CORE1 0x2

//============================================================================

int pass_scu_test(void);
int fail_scu_test(int status, unsigned int fail_cnt, unsigned int rd_err_cnt,
                  unsigned int wr_err_cnt);


unsigned int num_of_core = 4;               // TODO, make sure num_of_core is 4?
unsigned int period_congruent_addr = 4;
unsigned int big_endian;

int summary_check(void);
unsigned int scu_rand2(unsigned int *x)
{
   // Multiply-with-carry method invented by George Marsaglia
   static unsigned int z = 0x87654321;
   z = 36969 * (z & 65535) + (z >> 16);
   *x = 18000 * (*x & 65535) + (*x >> 16);
   return ((z << 16) + *x);
}

unsigned int pgm_is_big (void) {
    union {int x; unsigned char y[4];}z;
    z.x = 0xaabbccdd;
    return (z.y[0]==0xaa);
}


//TODO, replace with arm's function
//proc_no stands for which core it is.
int GET_PROC_NO(void)
{
	unsigned int val=0;

   asm("mrc   p15, 0, %0, c0, c0, 5 @ get PID" : "=r" (val) : : "cc");
	if (val & 0x100)
		if (val & 0x1)
			return 3;
		else
			return 2;
	else
		if (val & 0x1)
			return 1;
		else
			return 0;
}

void wait_loop (unsigned int loops)
{
  unsigned int i;
  for (i=0; i<=loops; i++) {}
}

int scu_compare_data(unsigned int data0, unsigned int data1, unsigned int mask, unsigned int type)
{
    unsigned int result;

    // Randomly writeback-invalidate few cache lines of the argument stack
    result = data0 ^ data1;

    if(type == 0)
        result = result & (mask >> 24);
    else if(type == 1)
        result = result & (mask >> 16);
    else
        result = result & mask;

    if(result == 0)
      return 1;
    else
      return 0;
}

int scu_compare_data2(  volatile unsigned int data0,   volatile unsigned int data1,
                    volatile unsigned int data1_0, volatile unsigned int data1_1,
                    volatile unsigned int data2_0, volatile unsigned int data2_1,
                    volatile unsigned int data3_0, volatile unsigned int data3_1,
                    volatile unsigned int data4_0, volatile unsigned int data4_1,
                    volatile unsigned int data5_0, volatile unsigned int data5_1,
                    volatile unsigned int data6_0, volatile unsigned int data6_1,
                    volatile unsigned int data7_0, volatile unsigned int data7_1,
                    unsigned int mask, unsigned int type)
{
    unsigned int result;

    // Randomly writeback-invalidate few cache lines of the argument stack
    // so that there may have multiple outstanding loads happen for the
    // following statement.
    result = (data0 ^ data1) | (data1_0 ^ data1_1) | (data2_0 ^ data2_1) | (data3_0 ^ data3_1) | (data4_0 ^ data4_1) | (data5_0 ^ data5_1) | (data6_0 ^ data6_1) | (data7_0 ^ data7_1);
    if(type == 0)
        result = result & (mask >> 24);
    else if(type == 1)
        result = result & (mask >> 16);
    else
        result = result & mask;

    if(result == 0)
        return 1;
    else
      return 0;
}

int scu_compare_data3(
                            volatile unsigned int foo0_1, volatile unsigned int foo1_1,
                            volatile unsigned int foo0_2, volatile unsigned int foo1_2,
                            volatile unsigned int foo0_3, volatile unsigned int foo1_3,
                    volatile unsigned int data0,
                                    volatile unsigned int foo0_1_1, volatile unsigned int foo1_1_1,
                                    volatile unsigned int foo0_2_1, volatile unsigned int foo1_2_1,
                                    volatile unsigned int foo0_3_1, volatile unsigned int foo1_3_1,
                                    volatile unsigned int foo0_4_1, volatile unsigned int foo1_4_1,
                    volatile unsigned int data1,
                            volatile unsigned int foo0, volatile unsigned int foo1,
                            volatile unsigned int foo2, volatile unsigned int foo3,
                            volatile unsigned int foo4, volatile unsigned int foo5,
                    volatile unsigned int data1_0, volatile unsigned int data1_1,
                            volatile unsigned int foo6, volatile unsigned int foo7,
                            volatile unsigned int foo8, volatile unsigned int foo9,
                            volatile unsigned int foo8_1, volatile unsigned int foo9_1,
                            volatile unsigned int foo8_2, volatile unsigned int foo9_2,
                    volatile unsigned int data2_0,
                                    volatile unsigned int foo0_1_2, volatile unsigned int foo1_1_2,
                                    volatile unsigned int foo0_2_2, volatile unsigned int foo1_2_2,
                                    volatile unsigned int foo0_3_2, volatile unsigned int foo1_3_2,
                                    volatile unsigned int foo0_4_2, volatile unsigned int foo1_4_2,
                    volatile unsigned int data2_1,
                            volatile unsigned int foo10, volatile unsigned int foo11,
                            volatile unsigned int foo10_1, volatile unsigned int foo11_1,
                            volatile unsigned int foo10_2, volatile unsigned int foo11_2,
                    volatile unsigned int data3_0, volatile unsigned int data3_1,
                            volatile unsigned int foo12, volatile unsigned int foo13,
                            volatile unsigned int foo14_1, volatile unsigned int foo15_1,
                            volatile unsigned int foo14_2, volatile unsigned int foo15_2,
                            volatile unsigned int foo14_3, volatile unsigned int foo15_3,
                            volatile unsigned int foo14_4, volatile unsigned int foo15_4,
                    volatile unsigned int data4_0,
                                    volatile unsigned int foo0_1_3, volatile unsigned int foo1_1_3,
                                    volatile unsigned int foo0_2_3, volatile unsigned int foo1_2_3,
                                    volatile unsigned int foo0_3_3, volatile unsigned int foo1_3_3,
                                    volatile unsigned int foo0_4_3, volatile unsigned int foo1_4_3,
                    volatile unsigned int data4_1,
                            volatile unsigned int foo16, volatile unsigned int foo17,
                            volatile unsigned int foo18, volatile unsigned int foo19,
                            volatile unsigned int foo20, volatile unsigned int foo21,
                    volatile unsigned int data5_0, volatile unsigned int data5_1,
                            volatile unsigned int foo22, volatile unsigned int foo23,
                            volatile unsigned int foo24, volatile unsigned int foo25,
                            volatile unsigned int foo26, volatile unsigned int foo27,
                            volatile unsigned int foo28, volatile unsigned int foo29,
                    volatile unsigned int data6_0,
                                    volatile unsigned int foo0_1_4, volatile unsigned int foo1_1_4,
                                    volatile unsigned int foo0_2_4, volatile unsigned int foo1_2_4,
                                    volatile unsigned int foo0_3_4, volatile unsigned int foo1_3_4,
                                    volatile unsigned int foo0_4_4, volatile unsigned int foo1_4_4,
                    volatile unsigned int data6_1,
                            volatile unsigned int foo30, volatile unsigned int foo31,
                            volatile unsigned int foo32, volatile unsigned int foo33,
                            volatile unsigned int foo34, volatile unsigned int foo35,
                            volatile unsigned int foo36, volatile unsigned int foo37,
                            volatile unsigned int foo38, volatile unsigned int foo39,
                    volatile unsigned int data7_0, volatile unsigned int data7_1,
                    unsigned int mask, unsigned int type)
{

    unsigned int result;

    // Randomly writeback-invalidate few cache lines of the argument stack
    // so that there may have multiple outstanding loads happen for the
    // following statement.
    result = (data0 ^ data1)
              | (data1_0 ^ data1_1)
              | (data2_0 ^ data2_1)
              | (data3_0 ^ data3_1)
              | (data4_0 ^ data4_1)
              | (data5_0 ^ data5_1)
              | (data6_0 ^ data6_1)
              | (data7_0 ^ data7_1)

              | (foo0_1 ^ foo1_1)
              | (foo0_2 ^ foo1_2)
              | (foo0_3 ^ foo1_3)

              | (foo0_1_1 ^ foo1_1_1)
              | (foo0_2_1 ^ foo1_2_1)
              | (foo0_3_1 ^ foo1_3_1)
              | (foo0_4_1 ^ foo1_4_1)

              | (foo0 ^ foo1)
              | (foo2 ^ foo3)
              | (foo4 ^ foo5)

              | (foo6   ^ foo7  )
              | (foo8   ^ foo9  )
              | (foo8_1 ^ foo9_1)
              | (foo8_2 ^ foo9_2)

              | (foo0_1_2 ^ foo1_1_2)
              | (foo0_2_2 ^ foo1_2_2)
              | (foo0_3_2 ^ foo1_3_2)
              | (foo0_4_2 ^ foo1_4_2)
              ;
    if(type == 0)
        result = result & (mask >> 24);
    else if(type == 1)
        result = result & (mask >> 16);
    else
        result = result & mask;

    if(result == 0)
      return 1;
    else
      return 0;
}

void mem_init (uint start_addr1, uint start_addr2, uint size, uint type, uint backdoor) {
    uint i, j;
    uint rand_bg = 0;

    if (type == IT_ZERO) rand_bg = 0; else if (type == IT_ONE) rand_bg = 0xffffffff;
    for (i = start_addr1, j = start_addr2; i <= size + start_addr1; i+=4, j+=4)
    {
//        if ((type == IT_RANDOM)) rand_bg = scu_rand2(&init_seed);
        WRITE_MEM32(i, rand_bg);
        WRITE_MEM32(j, rand_bg);
    }
} // void mem_init (uint start_addr1, uint start_addr2, uint size, uint type)

struct ACC {
  unsigned int cache_type;
  unsigned int data_type;
  unsigned int access_type;
  unsigned int sram_addr;
  unsigned int gold_data;
  unsigned int dram_addr;
  unsigned int rdata;
  unsigned int wdata;
};

int scu_test(void)
{
#if 0
	 unsigned int proc_no;
    unsigned int i, j, k=0, l=0;
    unsigned int data_type;
    unsigned int cache_type;
    unsigned int access_type=0;
    unsigned int offset_addr=0;
    unsigned int sram_addr;
    unsigned int dram_addr;
    int          test_run;
    unsigned int rdata, wdata;
    unsigned int golddata;
    unsigned int fail_count = 0;
    unsigned int overall_fail = 0;
    unsigned int mask=0;
    struct ACC last_acc;

    unsigned int rd_err_count = 0;
    unsigned int pos_rd_err_count = 0;
    unsigned int pos_wr_err_count = 0;

    unsigned int comp_error;

    unsigned int rn; // temp variable for saving a random number

    big_endian = pgm_is_big();

    //TODO
    proc_no = GET_PROC_NO();

    //User define the test type
    cache_type = CT_CACHED;
    data_type = DT_WORD;

    last_acc.data_type = DT_WORD;
    last_acc.access_type = AT_READ;
    last_acc.sram_addr = 0xdeadbeef;
    last_acc.gold_data = 0xdeadbeef;
    last_acc.dram_addr = 0xdeadbeef;
    last_acc.rdata = 0xdeadbeef;
    last_acc.wdata = 0xdeadbeef;

    unsigned int init_seed;
    init_seed = 0x12345678 + proc_no;
    if (GET_PROC_NO() == 3) printf ("Initial seed: 0x%08x\n", init_seed);
//----------------------------------------------------------------------//
// TEST4
//----------------------------------------------------------------------//
    if (COHERENCE_TEST == 1)
    {
        period_congruent_addr = (scu_rand2(&init_seed) % 128) + 4;
        if (PRINT_INFO && (GET_PROC_NO() == 0)) printf("\nTEST4: Coherent access test\n");

        // Init dram and sram
        if (proc_no == 0)
        {
            // Wait other cores to complete before pollute the memory
            mem_init (DRAM_BASE, SRAM_BASE, TEST_BLOCK, IT_ONE, IT_NON_BACKDOOR); // Backdoor works with Taroko TBU only
            WRITE_MEM32(SEM_SYNC_POINT2, 0xaabbccdd); // Ready to go TEST4 single for CPU 1/2/3
        } else {
            // Make other cores to wait core0 init memory
            i = 0x100;
            while (READ_MEM32 (SEM_SYNC_POINT2) != 0xaabbccdd) {
                if (i< 0x2000) i = i << 1;
                if (proc_no == 2) printf ("CPU%d wait_loop (%d loops) for CPU0 init memory (TEST4)...\n", proc_no, i);
                wait_loop (i);
            }
        }

        for (test_run = 1; test_run <= MAX_TEST_RUN; test_run++)
        {
            if (PRINT_INFO && (GET_PROC_NO() == 0)) printf("  Run:%d/%d sram:%08x dram:%08x range:%x points:%d llsc_pairs:%d\n",
                            test_run, MAX_TEST_RUN, SRAM_BASE, DRAM_BASE, TEST_BLOCK, MAX_TEST_PT, 0);

            for(i = 0; i < MAX_TEST_PT; i++)
            {
                access_type = scu_rand2(&init_seed) % AT_TOTAL; // R+W
                rn = scu_rand2(&init_seed);
                // Make it congruent address
                offset_addr = ((0x20 * (rn % 16)) + (0x1000 * ((rn >> 8) % period_congruent_addr))) % TEST_BLOCK;
                // TODO, make cores virtually share the same cache line, line size have to be larger then 4 words.
                offset_addr += proc_no*4; // different cores use differnt words of a same cache line
                data_type   = (rn >> 16) % DT_TOTAL; //Byte/Half/Word/TW

                for (j = offset_addr; j < (offset_addr+((rn>>26)*6)); j+=32 /*cache line size*/)
                {
                    // printf ("cpu %d point %08d addr %08x rn %8d\n", proc_no, i, j, (rn>>26)*3);
                    dram_addr = offset_addr + DRAM_BASE;
                    sram_addr = offset_addr + SRAM_BASE;
                    if (access_type == AT_WRITE)
                    {
                        wdata = (rn >> 1) & mask;
                        // wdata = proc_no | (proc_no << 2);
                        if (data_type == DT_BYTE) //byte
                        {
                            if (DEBUG && (GET_PROC_NO()==0)) printf ("write %s sram %8x, dram %8x, wdata %8x, offset %8x\n", DataTypeStr[data_type], sram_addr, dram_addr, wdata, offset_addr);
                            WRITE_MEM8(dram_addr, wdata);
                            WRITE_MEM8(sram_addr, wdata);
                        }
                        else if (data_type == DT_HWORD) //hword
                        {
                            if (DEBUG && (GET_PROC_NO()==0)) printf ("write %s sram %8x, dram %8x, wdata %8x, offset %8x\n", DataTypeStr[data_type], sram_addr, dram_addr, wdata, offset_addr);
                            WRITE_MEM16(dram_addr, wdata);
                            WRITE_MEM16(sram_addr, wdata);
                        }
                        else if (data_type == DT_WORD) //word
                        {
                            if (DEBUG && (GET_PROC_NO()==0)) printf ("write %s sram %8x, dram %8x, wdata %8x, offset %8x\n", DataTypeStr[data_type], sram_addr, dram_addr, wdata, offset_addr);
                            WRITE_MEM32(dram_addr, wdata);
                            WRITE_MEM32(sram_addr, wdata);
                        }
                        else //twin-word
                        {
                          continue;
                        }
                    }
                    else  //read
                    {
                        if (data_type == DT_BYTE) //byte
                        {
                            rdata     = READ_MEM8(dram_addr);
                            golddata  = READ_MEM8(sram_addr);
                            if (DEBUG && (GET_PROC_NO()==0)) printf ("read %s sram %8x, dram %8x, rdata %8x, golddata %8x, offset %8x\n", DataTypeStr[data_type], sram_addr, dram_addr, rdata, golddata, offset_addr);
                        }
                        else if (data_type == DT_HWORD) //hword
                        {
                            rdata     = READ_MEM16(dram_addr);
                            golddata  = READ_MEM16(sram_addr);
                            if (DEBUG && (GET_PROC_NO()==0)) printf ("read %s sram %8x, dram %8x, rdata %8x, golddata %8x, offset %8x\n", DataTypeStr[data_type], sram_addr, dram_addr, rdata, golddata, offset_addr);
                        }
                        else if (data_type == DT_WORD) //word
                        {
                            rdata     = READ_MEM32(dram_addr);
                            golddata  = READ_MEM32(sram_addr);
                            if (DEBUG && (GET_PROC_NO()==0)) printf ("read %s sram %8x, dram %8x, rdata %8x, golddata %8x, offset %8x\n", DataTypeStr[data_type], sram_addr, dram_addr, rdata, golddata, offset_addr);
                        }
                        else //twin-word
                        {
                          continue;
                        }

                        if (scu_rand2(&init_seed) % 2)
                          comp_error = scu_compare_data3(
                                            rn,rn, // dummy stuff to make it load different cache lines
                                            i,i, j,j,
                                                  rdata, k,k,l,l,i,i,j,j, golddata,
                                            0,0, 0,0, offset_addr,offset_addr, // dummy stuff to make it load different cache lines
                                            rdata, golddata,
                                            0,0, 0,0, 0,0, data_type,data_type, // dummy stuff to make it load different cache lines
                                                  rdata, 0,0,0,0,0,0,0,0, golddata,
                                            0,0, 0,0, 0,0,
                                            rdata, golddata,
                                            0,0, 0,0, 0,0, 0,0, 0,0,
                                                  rdata, 0,0,0,0,0,0,0,0, golddata,
                                            0,0, 0,0, 0,0,
                                            rdata, golddata,
                                            0,0, 0,0, 0,0, 0,0,
                                                  rdata, 0,0,0,0,0,0,0,0, golddata,
                                            0,0, 0,0, 0,0, 0,0, 0,0,
                                            rdata, golddata,
                                            mask, data_type)? 0:1;
#if (1)
                        else if (scu_rand2(&init_seed) % 2)
                          comp_error = scu_compare_data2(rdata, golddata,
                                          rdata, golddata,
                                          rdata, golddata,
                                          rdata, golddata,
                                          rdata, golddata,
                                          rdata, golddata,
                                          rdata, golddata,
                                          rdata, golddata,
                                          mask, data_type)? 0:1;
#endif
                        else
                          comp_error = scu_compare_data(rdata, golddata, mask, data_type)? 0:1;

                        if (comp_error)
                        {
                            if (SHOW_ERR_READ == 1)
                            {
                              if (GET_PROC_NO() == 0) printf("[Fail] Core:%d, Run:%d, PT:%d, data:%s, Addr:0x%08x Expect:0x%08x Actual:0x%08x ErrBit:%08x\n",
                                proc_no, test_run, i, DataTypeStr[data_type], dram_addr, golddata, rdata, golddata^rdata);
                              if (GET_PROC_NO() == 0) printf("\n");
                            } // SHOW_ERR_READ
                            fail_count = fail_count + 1;

                            if (STOP_ERR_CNT != 0)
                            {
                              if (fail_count >= STOP_ERR_CNT)
                              {
                                if (GET_PROC_NO() == 0) printf("[ABORT] TEST4 abort, exceed max err count runs:%d, faults:%d.\n",
                                       test_run, fail_count);
                                return fail_scu_test(0, fail_count, rd_err_count+pos_rd_err_count, pos_wr_err_count);
                              }
                            }
                        } // if (comp_error)
                    } // if (access_type == AT_WRITE)
                } // for (j = offset_addr; j < (offset_addr+(rn>>24)); j+=32 /*cache line size*/)
            } // for(i = 0; i < MAX_TEST_PT; i++)

            if(DEBUG == 1 && (GET_PROC_NO() == 0))
            {
              if(test_run % 100 == 0)
                printf("Complete %3d runs from %x, fail count = %d\n", test_run, DRAM_BASE, fail_count);
              else
                printf("Complete %3d runs from %x, fail count = %d\n", test_run, DRAM_BASE, fail_count);
            }
            else
            {
              if(test_run % 100 == 0)
					if (GET_PROC_NO() == 0)
                  printf("Complete %5d runs from %x, fail count = %d\n", test_run, DRAM_BASE, fail_count);
            }
            last_acc.data_type = data_type;
            last_acc.access_type = access_type;
            last_acc.sram_addr = sram_addr;
            last_acc.gold_data = golddata;
            last_acc.dram_addr = dram_addr;
            last_acc.wdata = wdata;
            last_acc.rdata = rdata;
        } // for (test_run = 1; test_run <= MAX_TEST_RUN; test_run++)

        if (PRINT_INFO && (GET_PROC_NO()==0))
				printf("   [DONE] TEST4 done %d runs, %d faults!\n", test_run-1, fail_count);

        overall_fail += fail_count;
    }

    if(overall_fail == 0)
    {
      pass_scu_test();
    }
    else
    {
      fail_scu_test(0, overall_fail, rd_err_count+pos_rd_err_count, pos_wr_err_count);
    }

	 if (GET_PROC_NO() == 0)
		summary_check();
	 else
		return 0;
#endif
         return 0;
}

int summary_check(void)
{
	volatile unsigned int *status_ptr;
	int i;

	for (i=0;i<num_of_core;i++) {
		printf("Core %d...", i);
		status_ptr = (unsigned int *)(LOG_ADDR + i*16);
		if ((*status_ptr & 0xffff) == 0xabcd)
			printf("PASS\n");
		else if (*status_ptr == 0x0000dead) {
			printf("FAIL\n");
			printf("Failed count %d\n", *(status_ptr+1));
			printf("Read error count %d\n", *(status_ptr+2));
			printf("Write error count %d\n", *(status_ptr+3));
		}
		else if (*status_ptr == 0x0001dead)
			printf("FAIL\n");
		else
			printf("Unknown\n");
	}
        return 0;
	
}
int fail_scu_test(int status, unsigned int fail_cnt, unsigned int rd_err_cnt,
                  unsigned int wr_err_cnt)
{
  volatile unsigned int *status_ptr;
 	int id = GET_PROC_NO();
	status_ptr = (unsigned int *)(LOG_ADDR + id*16);

  if (status == 0)
  {
    *status_ptr       = 0x0000dead;
    *(status_ptr+1)   = fail_cnt;
    *(status_ptr+2)   = rd_err_cnt;
    *(status_ptr+3)   = wr_err_cnt;
  }
  else if (status == 1)
  {
    *status_ptr       = 0x0001dead;
  }
  return 0;
}

int pass_scu_test(void)
{
  volatile unsigned int *status_ptr;
	int id = GET_PROC_NO();

	while ((rtd_inl(0xb8062314) & _BIT5) == 0)
            ;
		rtd_outl(0xb8062300, 0x51);

  status_ptr = (unsigned int *)(LOG_ADDR + id*16);

  *status_ptr       = 0xabcd;
  *(status_ptr+1)   = 0x0;
  *(status_ptr+2)   = 0x0;
  *(status_ptr+3)   = 0x0;
  return 0;
}

int do_scu_test (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
#if (0)
     unsigned int _dram_base;
     unsigned int _dram_size;
     unsigned int _test_block;
     unsigned int _sram_base;
     unsigned int _iteration_limit;
     unsigned int _pt_iteration_limit;
     unsigned int _change_seed;

     if (argc > 1)
      _dram_base = (unsigned int)simple_strtoul(argv[1], NULL, 16);
   else
      _dram_base = (unsigned int)0x02000000UL; // offset@32MB

   if (argc > 2)
      _test_block = (unsigned int)simple_strtoul(argv[3], NULL, 16);
   else
      _test_block = (unsigned int)0x00002000;

   if (argc > 3)
      _sram_base = (unsigned int)simple_strtoul(argv[4], NULL, 16);
   else
      _sram_base = (unsigned int)0x03000000UL;// offset@48MB

   if (argc > 4)
      _iteration_limit = (unsigned int)simple_strtoul(argv[5], NULL, 16);
   else
      _iteration_limit = 100UL;

   if (argc > 5)
      _pt_iteration_limit = (unsigned int)simple_strtoul(argv[6], NULL, 16);
   else
      _pt_iteration_limit = 100UL;
#endif

#ifdef CONFIG_CMD_COPROC
	run_command_list("coproc start", -1, 0);
	mdelay(50);
	run_command_list("coproc scutest e", -1, 0);
#else
	printf("start SCPU core1...\n");
	rtd_outl(0xb801a610, 0x20000);

	printf("start SCPU core2...\n");
	rtd_outl(0xb801a614, 0x20000);

	printf("start SCPU, core3...\n");
	rtd_outl(0xb801a618, 0x20000);
#endif

	scu_test();
	return 0;
}

U_BOOT_CMD(
		scutest,  8,   1,   do_scu_test,
		"SCU read/write test",
		"[dram_base [dram_size [test_block [sram_base [iterations [pt_iterations [cheng_seed]]]]]]]"
);

