/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2012 by Chuck Chen <ycchen@realtek.com>
 *
 * panel select command
*/
#include <common.h>
#include <command.h>
#include <malloc.h>
#include <asm/arch/system.h>
#include <tar.h>
#include <factorylib.h>

struct TUNER_MODEL
{
    unsigned char id;
    char *model_name;
};

static struct TUNER_MODEL tuner_list[] =
{
    {0, "Application Default Setting"},
    {1, "DTV_FE_REALTEK_TDA18275A_ALL"},
    {2, "DTV_FE_REALTEK_SI2151_ALL"},
};
#define TUNER_MODEL_LIST_SIZE   (sizeof(tuner_list)/sizeof(struct TUNER_MODEL))


const char* tuner_config_name[] =
{
    "TP0_FE",
    "TP1_FE",
};

const char* boot_default_tuner_name[] =
{
    "default_boot_TP0",
    "default_boot_TP1",
};

#define MAX_TUNER_COUNT      sizeof(tuner_config_name) / sizeof(char*)


int find_tuner_select(const char* name)
{
    int i;

    if (name==NULL)
        return 0;

    for( i = 0; i < TUNER_MODEL_LIST_SIZE ; i++)
        if (strcmp(name, tuner_list[i].model_name)==0)
            return i;

    return -1;
}

extern char* strtrim(char *s);
extern void dump_tuner_config(int tuner_id);
extern int fgets(char *s, int n);
 
int do_tuner_selection(int tuner_id)
{
    char buf[128] = {0};
    int i;
    int select = -1;

    if (tuner_id >= MAX_TUNER_COUNT )
        return CMD_RET_FAILURE;

    dump_tuner_config(tuner_id);

    printf("\n\n===================================================\n");
    printf("Please select the tuner model you want:\n");

    for( i = 0; i < TUNER_MODEL_LIST_SIZE ; i++)
        printf("[%d] %s\n", i, tuner_list[i].model_name);

    printf("=> ");
    fgets(buf, 127);

    select = simple_strtoul(strtrim(buf), NULL, 10);

    if (select < TUNER_MODEL_LIST_SIZE && select > 0)
    {
        printf("\n===================================================\n");
        printf("you have selected [%d] %s\n", select, tuner_list[select].model_name);
        printf("===================================================\n");
        setenv(tuner_config_name[tuner_id], tuner_list[select].model_name);
        setenv(boot_default_tuner_name[tuner_id], "y");
        saveenv();
    }
    else if((buf[0] == '0') && (select == 0))
    {
        printf("\nClear irda setting! \n");
        setenv(tuner_config_name[tuner_id], NULL);
        setenv(boot_default_tuner_name[tuner_id], NULL);
        saveenv();
    }
    else
    {
        printf("\n [ERR] wrong input! \n");
    }

    return CMD_RET_SUCCESS;
}




void dump_tuner_config(int tuner_id)
{
    const char *tuner_model = NULL;

    if (tuner_id >= MAX_TUNER_COUNT )
        return;

    tuner_model = getenv(tuner_config_name[tuner_id]);

    if (tuner_model == NULL)
        printf("\n%s : NULL (Application Default Setting)\n", tuner_config_name[tuner_id]);
    else
        printf("\n%s : %s\n", tuner_config_name[tuner_id], tuner_model);
}


int do_tuner_help(void)
{
    printf("tuner : dume help message of tuner command\n");
    printf("tuner show : show config of tuner selection\n");
    printf("tuner select : tuner selection\n");
    return CMD_RET_SUCCESS;
}

int do_tuner_show_config(void)
{
    int i;

    for (i=0;i<MAX_TUNER_COUNT; i++)
        dump_tuner_config(i);

    return CMD_RET_SUCCESS;
}

int do_tuner_set_config(void)
{
    char buf[128] = {0};
    int tuner_id = -1;
    printf("===================================================\n");
    printf("Which tuner to be configured (0~%d):\n", MAX_TUNER_COUNT-1);

    printf("=> ");
    fgets(buf, 127);

    tuner_id = simple_strtoul(strtrim(buf), NULL, 10);

    if (tuner_id < 0 || tuner_id >= MAX_TUNER_COUNT)
        return CMD_RET_SUCCESS;

    return do_tuner_selection(tuner_id);
}



int do_tuner_command(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    int ret = CMD_RET_SUCCESS;

    if (argc <= 1) {
        do_tuner_help();
        return CMD_RET_SUCCESS;
    }

    if (strcmp(argv[1], "show")==0)
        ret = do_tuner_show_config();
    else if (strcmp(argv[1], "select")==0) {
        ret = do_tuner_set_config();
        do_tuner_show_config();
    }
    else
    {
        printf("unknown command - %s\n", argv[1]);
        do_tuner_help();
    }

    return ret;
}

U_BOOT_CMD(
    tuner, 5, 0, do_tuner_command,
    "tuner - select tuner model;",
    "This command is used to tuner model in uboot level"
    " "
);

