#include <common.h>
#include <command.h>
//#include <watchdog.h>
#include "watchdog_api.h"
#include <rbus/wdog_reg.h>
//#include <io.h>

int enable_watchdog(unsigned int times)
{
	unsigned int mtime = 0x08000000/5*times;
	rtd_outl(ISO_MISC_TCWOV_reg, mtime);
	rtd_outl(ISO_MISC_TCWNMI_reg, 0x07ff0000);
	rtd_maskl(ISO_MISC_TCWCR_reg, ~0xFF, 0x00);
	return 0;
}

int disable_watchdog(void)
{
	rtd_maskl(ISO_MISC_TCWCR_reg, ~0xFF, 0xA5);
	return 0;
}

#ifdef CONFIG_REALTEK_FREE_CNT
int test_free_cnt(void)
{
	unsigned int cnt1, cnt2;
	printf("case 1: default counter\n");
	printf("FCEN = %x, FCCLR = %x\n", WDOG_FCCTRL_get_fcen(rtd_inl(WDOG_FCCTRL_reg)), WDOG_FCCTRL_get_fcclr(rtd_inl(WDOG_FCCTRL_reg)));
	cnt1 = rtd_inl(WDOG_FCSTS_reg);
	mdelay(78);
	cnt2 = rtd_inl(WDOG_FCSTS_reg);
	printf("FREE_CNT1 = %u, FREE_CNT2 = %u, delay: %u ms\n\n", cnt1, cnt2, ((cnt2-cnt1)/27/1000));

	printf("case 2: stop then continue\n");
	rtd_clearbits(WDOG_FCCTRL, WDOG_FCCTRL_fcen_mask); //stop
	printf("FCEN = %x, FCCLR = %x\n", WDOG_FCCTRL_get_fcen(rtd_inl(WDOG_FCCTRL_reg)), WDOG_FCCTRL_get_fcclr(rtd_inl(WDOG_FCCTRL_reg)));
	printf("FREE_CNT = %u\n", rtd_inl(WDOG_FCSTS_reg));
	mdelay(10);
	printf("FREE_CNT = %u\n", rtd_inl(WDOG_FCSTS_reg));
	rtd_setbits(WDOG_FCCTRL_reg, WDOG_FCCTRL_fcen_mask); //enable
	printf("FCEN = %x, FCCLR = %x\n", WDOG_FCCTRL_get_fcen(rtd_inl(WDOG_FCCTRL_reg)), WDOG_FCCTRL_get_fcclr(rtd_inl(WDOG_FCCTRL_reg)));
	printf("FREE_CNT = %u\n", rtd_inl(WDOG_FCSTS_reg));
	mdelay(20);
	printf("FREE_CNT = %u\n\n", rtd_inl(WDOG_FCSTS_reg));

	printf("case 3: clear free counter\n");
	rtd_setbits(WDOG_FCCTRL, WDOG_FCCTRL_fcclr_mask); //clear
	printf("FCEN = %x, FCCLR = %x\n", WDOG_FCCTRL_get_fcen(rtd_inl(WDOG_FCCTRL_reg)), WDOG_FCCTRL_get_fcclr(rtd_inl(WDOG_FCCTRL_reg)));
	printf("FREE_CNT = %u\n", rtd_inl(WDOG_FCSTS_reg));
	mdelay(30);
	printf("FREE_CNT = %u\n\n", rtd_inl(WDOG_FCSTS_reg));

	printf("case 4: disable and clear free counter, then enable\n");
	rtd_clearbits(WDOG_FCCTRL, WDOG_FCCTRL_fcen_mask); //disable
	rtd_setbits(WDOG_FCCTRL, WDOG_FCCTRL_fcclr_mask); //clear
	printf("FCEN = %x, FCCLR = %x\n", WDOG_FCCTRL_get_fcen(rtd_inl(WDOG_FCCTRL_reg)), WDOG_FCCTRL_get_fcclr(rtd_inl(WDOG_FCCTRL_reg)));
	printf("FREE_CNT = %u\n", rtd_inl(WDOG_FCSTS_reg));
	mdelay(10);
	printf("FREE_CNT = %u\n", rtd_inl(WDOG_FCSTS_reg));
	rtd_setbits(WDOG_FCCTRL_reg, WDOG_FCCTRL_fcen_mask); //enable
	printf("FCEN = %x, FCCLR = %x\n", WDOG_FCCTRL_get_fcen(rtd_inl(WDOG_FCCTRL_reg)), WDOG_FCCTRL_get_fcclr(rtd_inl(WDOG_FCCTRL_reg)));
	printf("FREE_CNT = %u\n", rtd_inl(WDOG_FCSTS_reg));

	return 0;
}
#endif
int do_switch_watchdog (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    unsigned int default_second = 5;
    if (argc < 2 || argc > 3 ) // print usage
    {
        printf("wdt current status is %s\n\n",getenv("wdt"));
        return CMD_RET_USAGE;
    }

    if(!strcmp("on", argv[1]))
    {
        if(argc == 3)
            default_second = simple_strtoul(argv[2], NULL, 16);

        enable_watchdog(default_second);
        printf("Enabled watchdog ,timeout:%d second(s)...\n",default_second);
        setenv("wdt", argv[1]);
        saveenv();
    }
    else if(!strcmp("off", argv[1]))
    {
        printf("Disable watchdog.\n");
        setenv("wdt", argv[1]);
        saveenv();
    }
#ifdef CONFIG_REALTEK_FREE_CNT
    else if(!strcmp("test_fc", argv[1]))
    {
        printf("test free_cnt.\n");
        test_free_cnt();
    }
#endif
	else{
        printf("input wdt format error,please re-input\n");
        printf("wdt on [seconds]: enable watchdog always.\n");
        printf("wdt off: disable watchdog always.\n");
    }
    return 0;
}

U_BOOT_CMD(
	wdt,	3,	1,	do_switch_watchdog,
	"watch dog enable/disable",
	"[on/off [seconds]]\n"
	"wdt on [seconds]: enable watchdog always.\n"
	"wdt off: disable watchdog always.\n"
#ifdef CONFIG_REALTEK_FREE_CNT
	"wdt test_fc: test free run counter.\n"
#endif
);

