/*
 * (C) Copyright 2013
 * EJ Hsu <ejhsu@realtek.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#include <linux/linkage.h>
#include <configs/rtd287o_demo_J64.h>
#include "macro.h"

            /* per-CPU inerrupts config:
                         * ID0-ID7(SGI)   for Non-secure interrupts
                         * ID8-ID15(SGI)  for Secure interrupts.
                         * All PPI config as Non-secure interrupts.
             */
#define GICD_IGROUPR_RTK   0xDfff0fff
#define LOOP_COUNT         0x1000

#define GIC_BASE                                       0x18080000
#define GICD_OFFSET                                    0x1000
#define GICC_OFFSET                                    0x2000
#define GICC_BASE                                      (GIC_BASE + GICC_OFFSET)
#define GICD_BASE                                      (GIC_BASE + GICD_OFFSET)
//#define DEBUG_REDIRECT2
/*************************************************************************
 *
 * Startup Code (reset vector)
 *
 *************************************************************************/


#define L2_TAG_RAM_SETUP         	(0x0 << 9)  /* 0~1 */
#define L2_TAG_RAM_LATENCY         	(0x0 << 6)  /* 0~7 */

#define L2_DATA_RAM_SETUP         	(0x1 << 5)  /* 0~1 */
#define L2_DATA_RAM_LATENCY             (0x2 << 0)  /* 0~7 */

#define MAGIC_TAG1                            0x508
#define MAGIC_TAG2                            (MAGIC_TAG1+0x8)
#define MAGIC_TAG3                            (MAGIC_TAG1+0x10)
#define MAGIC_BRANCH_ADDR                     (MAGIC_TAG1+0x18)

#define MAGIC_COPROC_OP_SMP_DISABLE  0x24000000
#define COUNTER_FREQUENCY			27000000
#define BOOTCODE_ADDR				0x20000
#define KERNEL_ADDR                             0x180000
.globl	start
start:
	b	reset

ENTRY(armv8_switch_to_el1)
	switch_el x5, 1f, 0f, 0f
0:	ret
1:	armv8_switch_to_el1_m x5, x6, x7
ENDPROC(armv8_switch_to_el1)


ENTRY(jump_to_el1_kernel)
	/* TODO: check if SVC */
	/* TODO: check magic number */
	/* TODO: prepare the parameters */

	/* clear the upper 32 bits of x0, x1, x2 */
	lsl	x0, x0, 32
	lsr	x0, x0, 32
	lsl	x1, x1, 32
	lsr	x1, x1, 32
	lsl	x2, x2, 32
	lsr	x2, x2, 32

	mrs	x5, scr_el3
	mov	x5, #0xc30			/* Set SCR_EL3.ST | SCR_EL3.RW */
	msr	scr_el3, x5

	branch_if_cpuid_0 x5, x6, 2f
	ldr	x3, =MAGIC_TAG1
	cmp	x5, #1
	beq	clear_tag_value
	ldr	x3, =MAGIC_TAG2
	cmp	x5, #2
	beq	clear_tag_value
	ldr	x3, =MAGIC_TAG3
clear_tag_value:
	mov	x4, #0
	str	x4, [x3]
5:
	ldr	x4, [x3]
	cmp	x4, #0
	beq	5b
	br	x4

	mov	x6, #0x3c5			/* EL1_SP1 | D | A | I | F */
	mov	x7, x4
//	bl	armv8_switch_to_el1
2:
	mov	x5, #0x3			/* Set CPUs */
	msr	actlr_el3, x5

	/* switch to EL1 with A64 execution state */
	ldr	x6, =MAGIC_BRANCH_ADDR
	ldr	x7, [x6]
//	ldr	x7, =0x1b100000 //hard code to boot
	ldr	x6, =KERNEL_ADDR
	cmp	x7,x6
	beq	linux_boot
	b	atf_boot
linux_boot:
	ldr	x7, =KERNEL_ADDR
	mov	x6, #0x3c5			/* EL1_SP1 | D | A | I | F */
	bl	armv8_switch_to_el1
atf_boot:
//	mov	x6, #0x3cc			/* EL1_SP1 | D | A | I | F */
el_return:
//	msr	spsr_el3, x6
//	msr	elr_el3, x7
	br	x7
//	eret
1:
	b	1b				/* Error */
ENDPROC(jump_to_el1_kernel)

reset:
	ldr	x5, =0x18062300
	ldr	w6, ='R'
	str	w6, [x5]

	/* Initialize Generic Timers */
	msr	cntvoff_el2, xzr

	/*
	 * Could be EL3/EL2/EL1, Initial State:
	 * Little Endian, MMU Disabled, i/dCache Disabled
	 */
	adr	x0, vectors
	switch_el x1, 3f, 2f, 1f
3:	msr	vbar_el3, x0
	mrs	x0, scr_el3
	mov	x0, #0x830			/* Set SCR_EL3.ST */
	msr	scr_el3, x0
	mov	x0, #0x73			/* Set CPUs | L2s */
	msr	actlr_el3, x0
	msr	cptr_el3, xzr			/* Enable FP/SIMD */
	ldr	x0, =COUNTER_FREQUENCY
	msr	cntfrq_el0, x0			/* Initialize CNTFRQ */
	b	0f
2:	msr	vbar_el2, x0
	mov	x0, #0x33ff
	msr	cptr_el2, x0			/* Enable FP/SIMD */
	b	0f
1:	msr	vbar_el1, x0
	mov	x0, #3 << 20
	msr	cpacr_el1, x0			/* Enable FP/SIMD */
0:
//	branch_if_a57_core x0, secondary
	branch_if_cpuid_0 x0, x1, 4f
	ldr	x3, =MAGIC_TAG1
	cmp	x0, #1
	beq	wait_sig
	ldr	x3, =MAGIC_TAG2
	cmp	x0, #2
	beq	wait_sig
	ldr	x3, =MAGIC_TAG3
	cmp	x0, #3
	beq	wait_sig

4:
	ldr	x6, =0x18062300
	ldr	x7, ='U'
	str	w7, [x6]
	/* switch to EL1 with A32 execution state */
	mov	x6, #0x1d3			/* A | I | F | SVC mode */
	ldr	x7, =BOOTCODE_ADDR
	bl	armv8_switch_to_el1

secondary:
	ldr	x3, =MAGIC_TAG2
	branch_if_cpuid_0 x0, x1, wait_sig
	ldr	x3, =MAGIC_TAG3

wait_sig:
	mov	x0, #0
	str	x0, [x3]
1:
	ldr	x2, [x3]
	cmp	x2, #0
	beq	1b
	ldr	x1, =MAGIC_COPROC_OP_SMP_DISABLE
	cmp	x2, x1
	beq	jump_to_el1_kernel
	mov	x7, x2

	mrs	x5, scr_el3
	mov	x5, #0x830			/* Set SCR_EL3.ST | SCR_EL3.RW */
	msr	scr_el3, x5
	mov	x5, #0x3			/* Set CPUs */
	msr	actlr_el3, x5

	/* switch to EL1 with A64 execution state */
	mov	x6, #0x1d3			/* EL1_SP1 | D | A | I | F */
	bl	armv8_switch_to_el1
