
.text
.global start2

#include "bootcode_addr_tmp.inc"
#define CONFIG_USE_IRQ
#define PPI_DISABLE                                     0xffff0000
#define SGI_ENABLE                                      0x0000ffff
#define INT_PRI                                             0xa0a0a0a0
#define GIC_BASE                                          0x18080000
#define GICD_OFFSET                                    0x1000
#define GICC_OFFSET                                    0x2000

#define GICC_BASE                                        (GIC_BASE + GICC_OFFSET)
#define GIC_CPU_CTRL                                  0x00
#define GIC_CPU_PRIMASK                            0x04
#define GIC_CPU_BINPOINT                          0x08
#define GIC_CPU_INTACK	                            0x0c
#define GIC_CPU_EOI                                    0x10
#define GIC_CPU_RUNNINGPRI                      0x14
#define GIC_CPU_HIGHPRI	                            0x18

#define GICD_BASE                                        (GIC_BASE + GICD_OFFSET)
#define GIC_DIST_CTRL                                 0x000
#define GIC_DIST_CTR                                   0x004
#define GIC_DIST_IGROUP	                            0x080
#define GIC_DIST_ENABLE_SET                     0x100
#define GIC_DIST_ENABLE_CLEAR	              0x180
#define GIC_DIST_PENDING_SET	              0x200
#define GIC_DIST_PENDING_CLEAR              0x280
#define GIC_DIST_ACTIVE_SET                     0x300
#define GIC_DIST_ACTIVE_CLEAR                 0x380
#define GIC_DIST_PRI                                   0x400
#define GIC_DIST_TARGET                            0x800
#define GIC_DIST_CONFIG	                            0xc00
#define GIC_DIST_SOFTINT                           0xf00

#define LEN_USR_STACK 0
#define LEN_ABT_STACK 0
#define LEN_UND_STACK 0
#define LEN_SYS_STACK 0
#define LEN_IRQ_STACK 0
#define LEN_FIQ_STACK 0
#define LEN_SVC_STACK 64

#define OFFSET_USR_STACK	(0)
#define OFFSET_SVC_STACK	(OFFSET_USR_STACK + LEN_USR_STACK)
#define OFFSET_FIQ_STACK	(OFFSET_SVC_STACK + LEN_SVC_STACK)
#define OFFSET_IRQ_STACK	(OFFSET_FIQ_STACK + LEN_FIQ_STACK)
#define OFFSET_ABT_STACK	(OFFSET_IRQ_STACK + LEN_IRQ_STACK)
#define OFFSET_UND_STACK	(OFFSET_ABT_STACK + LEN_ABT_STACK)
#define OFFSET_SYS_STACK	(OFFSET_UND_STACK + LEN_UND_STACK)

#define STACK_UPPER_BOUND	(STACK_BASE-LEN_SVC_STACK-LEN_SVC_STACK)
#define STACK_SIZE			(LEN_SVC_STACK+LEN_SVC_STACK)

#define MAGIC_TAG0			  0x500	/* Primary */
#define MAGIC_TAG1			  0x504	/* Secondary */
#define MAGIC_TAG2			  0x508
#define MAGIC_TAG3			  0x50c

#define MAGIC_TAG_KBOOT_FLAG 0x510
#define MAGIC_TAG_VALUE_NON_KBOOT 0x1234abcd

#define CONFIG_ROLLBACK_CODE
#ifdef CONFIG_ROLLBACK_CODE // rollback backup text data if QC code is too large
#define BOOTCODE_OVER_AUX_BACKUP_DATA_ADDR	0x010f0000
#define BOOTCODE_OVER_AUX_MAGIC_ADDR		0x010e0000
#define BOOTCODE_OVER_AUX_BACKUP_SIZE_ADDR	0x010e0004
#define BOOTCODE_OVER_AUX_MAGIC_NUMBER		0xa1a2a3a4
#define BOOTCODE_OVER_AUX_MAGIC_CLEAR		0x00000000
#endif

#define ENABLE_UART_DEBUG_IN_REDIRECT

CPU0_MAGIC_TAG:
	.word	0xdeadbeef
CPU1_MAGIC_TAG:
	.word	0xdeadbeef

start2:
@ -- clear the content of magic tag
#ifdef ENABLE_UART_DEBUG_IN_REDIRECT
	LDR  r1 , =0x00000052	/* 'R' */
	LDR  r0 , =0x18062300
	STR  r1 , [r0]
#endif
	LDR  r1 , =0x00000000	
	LDR  r0 , =MAGIC_TAG0
	STR  r1 , [r0]
	LDR  r0 , =MAGIC_TAG1
	STR  r1 , [r0]
	LDR  r0 , =MAGIC_TAG2
	STR  r1 , [r0]
	LDR  r0 , =MAGIC_TAG3
	STR  r1 , [r0]

#if 0 // simulate 2nd CPU
     B     run_secondary
#endif

	MRC   p15, 0, r0, c0, c0, 5
	AND   r0, r0, #0x3
	CMP   r0, #1
	BEQ   run_secondary

	CMP   r0, #2
	BEQ   run_third

	CMP   r0, #3
	BEQ   run_fourth

#ifdef CONFIG_ROLLBACK_CODE
@ -- check magic
	LDR   r0, =BOOTCODE_OVER_AUX_MAGIC_ADDR
	LDR   r1, [r0]
	LDR   r2, =BOOTCODE_OVER_AUX_MAGIC_NUMBER
	CMP   r1, r2
	BNE   skip_rollback_code

@ -- clear magic
	LDR   r3, =BOOTCODE_OVER_AUX_MAGIC_CLEAR
	STR   r3, [r0]

@ -- start rollback
	LDR   r0, =BOOTCODE_OVER_AUX_BACKUP_SIZE_ADDR
	LDR   r1, [r0]
	LDR   r0, =BOOTCODE_OVER_AUX_BACKUP_DATA_ADDR
//	LDR   r1, =0x200000
	LDR   r2, =JUMP_ADDRESS
	ADD   r3, r2, r1

rollback_loop:
	CMP   r3, r2
	BEQ   rollback_loop_end
	LDR   r4, [r0]
	STR   r4, [r2]
	ADD   r0, r0, #4
	ADD   r2, r2, #4
	B     rollback_loop

rollback_loop_end:
skip_rollback_code:
#endif

@ -- set default stack of super visor mdoe
@ -- also update line 124
     LDR  r0, =STACK_BASE
     MOV  SP, r0

@ -- NOP
     MOV  r0 , #0
     MOV  r0 , #0
     MOV  r0 , #0

@ -- Clear RAM
@ -- r0  start address
@ -- r1  zero
@ -- r2  end address
     LDR  r0 , =STACK_UPPER_BOUND
     MOV  r1 , #0
     ADD  r2 , r0 , #(STACK_SIZE)
INIT_SRAM:
     CMP  r0 , r2
     STRLO r1 , [r0]
     ADDLO r0 , r0 , #4
     BLO  INIT_SRAM

@ -- initial cpu register ( heap, stack... )
@ -- set stack base address
     LDR     r0, =STACK_BASE

     MSR     CPSR_c, #0xD3 ; #MODE_SVC:OR:DIS_IRQ:OR:DIS_FIRQ
     SUB     r3, r0, #OFFSET_SVC_STACK
     MOV     SP, r3

     MSR     CPSR_c, #0xD1 ; #MODE_FIQ:OR:DIS_IRQ:OR:DIS_FIRQ
     SUB     r3, r0, #OFFSET_FIQ_STACK
     MOV     SP, r3

     MSR     CPSR_c, #0xD2 ; #MODE_IRQ:OR:DIS_IRQ:OR:DIS_FIRQ
     SUB     r3, r0, #OFFSET_IRQ_STACK
     MOV     SP, r3

@ -- MSR     CPSR_c, #0xD7 ; #MODE_ABT:OR:DIS_IRQ:OR:DIS_FIRQ
@ -- SUB     r3, r0, #OFFSET_ABT_STACK
@ -- MOV     SP, r3

@ -- MSR     CPSR_c, #0xDB ; #MODE_UND:OR:DIS_IRQ:OR:DIS_FIRQ
@ -- SUB     r3, r0, #OFFSET_UND_STACK
@ -- MOV     SP, r3

     MSR     CPSR_c, #0xDF ; #MODE_SYS:OR:DIS_IRQ:OR:DIS_FIRQ
     SUB     r3, r0, #OFFSET_SYS_STACK
     MOV     SP, r3

@ -- please not set CPSR_c to user mode
@ -- MSR     CPSR_c, #0xD0 ; #MODE_USR:OR:DIS_IRQ:OR:DIS_FIRQ
@ -- SUB     r3, r0, #OFFSET_USR_STACK
@ -- MOV     SP, r3

@ -- back to supervisor mode
     MSR     CPSR_c, #0xD3 ; #MODE_SVC:OR:DIS_IRQ:OR:DIS_FIRQ

@ -- Jump to bootcode
check_cpu0_tag:
     LDR     r0 , =0x0001F800
     LDR     r1, [r0]
     LDR     r2 , =0xdeadbeef
     CMP     r1, r2
     BNE     check_cpu0_tag

     LDR  r0 , =MAGIC_TAG_KBOOT_FLAG
     LDR  r1, [r0]
     LDR  r0, =MAGIC_TAG_VALUE_NON_KBOOT
     CMP  r1, r0
     BEQ  loop1

#ifdef ENABLE_UART_DEBUG_IN_REDIRECT
     LDR  r1 , =0x0000004B	/* 'K' */
     LDR  r0 , =0x18062300
     STR  r1 , [r0]
#endif

loop0:
     LDR   r0 , =MAGIC_TAG0
     LDR   r1, [r0]
     CMP   r1, #0
     BEQ   loop0
     @load adddress to run

#ifdef ENABLE_UART_DEBUG_IN_REDIRECT
     LDR  r2 , =0x0000004F	/* 'O' */
     LDR  r0 , =0x18062300
     STR  r2 , [r0]
#endif

@ -- Jump to OPTEE after kcpu setup OPTEE image
     MOV   PC, r1
loop1:

#ifdef ENABLE_UART_DEBUG_IN_REDIRECT
     LDR  r1 , =0x0000004A	/* 'J' */
     LDR  r0 , =0x18062300
     STR  r1 , [r0]
#endif

@ -- Jump to bootcode
     LDR     r0, =JUMP_ADDRESS
     MOV     PC, r0

run_secondary:
     MSR   CPSR_c, #0xD3 ; #MODE_SVC:OR:DIS_IRQ:OR:DIS_FIRQ
#ifdef CONFIG_USE_IRQ
    bl gic_cpu_init
#endif
loop2:
#ifdef CONFIG_USE_IRQ
     WFI
#else
     WFE
#endif
     LDR   r0 , =MAGIC_TAG1
     LDR   r1, [r0]
     CMP   r1, #0
     BEQ   loop2
     @load adddress to run
     MOV   PC, r1

run_third:
     MSR   CPSR_c, #0xD3 ; #MODE_SVC:OR:DIS_IRQ:OR:DIS_FIRQ
#ifdef CONFIG_USE_IRQ
     bl gic_cpu_init
#endif
loop3:
#ifdef CONFIG_USE_IRQ
     WFI
#else
     WFE
#endif
     LDR   r0 , =MAGIC_TAG2
     LDR   r1, [r0]
     CMP   r1, #0
     BEQ   loop3
     @load adddress to run
     MOV   PC, r1

run_fourth:
     MSR   CPSR_c, #0xD3 ; #MODE_SVC:OR:DIS_IRQ:OR:DIS_FIRQ
#ifdef CONFIG_USE_IRQ
     bl gic_cpu_init
#endif
loop4:
#ifdef CONFIG_USE_IRQ
     WFI
#else
     WFE
#endif
     LDR   r0 , =MAGIC_TAG3
     LDR   r1, [r0]
     CMP   r1, #0
     BEQ   loop4
     @load adddress to run
     MOV   PC, r1
#ifdef CONFIG_USE_IRQ
gic_cpu_init:
    ldr r0, =GICC_BASE
    ldr r1, =GIC_DIST_CTRL
    mov r2, #0
    str r2, [r0, r1]
    ldr r7, =GICD_BASE
    ldr r3, =GIC_DIST_ENABLE_CLEAR
    ldr r2, =PPI_DISABLE
    str r2, [r7, r3]
    ldr r1, =GIC_DIST_ENABLE_SET
    ldr r2, =SGI_ENABLE
    str r2, [r7, r1]
    mov r4, #0
    mov r5, #32
    ldr r6, =GIC_DIST_PRI
    ldr r2, =INT_PRI
for_loop:
    str r2, [r7, r6]
    add r4, r4, #4
    add r6, r6, #4
    cmp r4, r5
    bne for_loop
    ldr r1, =GIC_CPU_PRIMASK
    ldr r2, =0xf0
    str r2, [r0, r1]
    ldr r3, =GIC_CPU_CTRL
    mov r2, #1
    str r2, [r0, r3]
    mov  pc, lr
#endif

