#ifndef __RTK_PWM_LIB_H__
#define __RTK_PWM_LIB_H__

#include <rbus/rbusCRTReg.h>
#include <rbus/rbusStandbyReg.h>
#include <rbus/iso_misc_pwm_reg.h>
#include <rbus/misc_pwm_reg.h>


#define ISO_PWM_MAX            1
#define MISC_PWM_MAX           1


#define N_MAX                  (4096)
#define M_MAX                  (3)
#define CONFIG_CPU_XTAL_FREQ   (27000000)
#define CONFIG_CPU_XTAL_NS     (37)    /*1/27000000*/
#define CONFIG_VSYNC_FREQ      (60)
#define ABS(x, y)              ((x) > (y) ? ((x)-(y)) : ((y)-(x)))
#define CONFIG_VSYNC_FREQ      (60)

typedef struct RTK_PWM_INFO {
	int m_index;
	int m_run;
	int m_duty;
	int m_freq;
	int m_adpt_en;
	int m_freq48n;
	int m_freq50n;
	int m_freq60n;
	int m_vsync;
	int m_pos;
} RTK_PWM_INFO_T;


struct rtk_pwm_reg_map_t {
	unsigned long CTRL;
	unsigned long TIMING_CTRL;
	unsigned long DUTY_SET;
};
typedef struct rtk_pwm_reg_map_t rtk_pwm_reg_map;

#define  ISO_MIS_PWM_DB_CTRL_VADDR          ISO_MISC_PWM_MIS_PWM_DB_CTRL_reg
#define  ISO_MIS_PWM0_CTRL_VADDR            ISO_MISC_PWM_MIS_PWM0_CTRL_reg
#define  ISO_MIS_PWM0_TIMING_CTRL_VADDR     ISO_MISC_PWM_MIS_PWM0_Timing_CTRL_reg
#define  ISO_MIS_PWM0_DUTY_SET_VADDR        ISO_MISC_PWM_MIS_PWM0_DUTY_SET_reg
#define  ISO_MIS_PWM1_CTRL_VADDR            ISO_MISC_PWM_MIS_PWM1_CTRL_reg
#define  ISO_MIS_PWM1_TIMING_CTRL_VADDR     ISO_MISC_PWM_MIS_PWM1_Timing_CTRL_reg
#define  ISO_MIS_PWM1_DUTY_SET_VADDR        ISO_MISC_PWM_MIS_PWM1_DUTY_SET_reg


#define  MIS_PWM_DB_CTRL_VADDR              MISC_PWM_PWM_DB0_CTRL_reg
#define  MIS_PWM0_CTRL_VADDR                MISC_PWM_PWM0_CTRL_reg
#define  MIS_PWM0_TIMING_CTRL_VADDR         MISC_PWM_PWM0_Timing_CTRL_reg
#define  MIS_PWM0_DUTY_SET_VADDR            MISC_PWM_PWM0_DUTY_SET_reg
#define  MIS_PWM1_CTRL_VADDR                MISC_PWM_PWM1_CTRL_reg
#define  MIS_PWM1_TIMING_CTRL_VADDR         MISC_PWM_PWM1_Timing_CTRL_reg
#define  MIS_PWM1_DUTY_SET_VADDR            MISC_PWM_PWM1_DUTY_SET_reg
#define  MIS_PWM2_CTRL_VADDR                MISC_PWM_PWM2_CTRL_reg
#define  MIS_PWM2_TIMING_CTRL_VADDR         MISC_PWM_PWM2_Timing_CTRL_reg
#define  MIS_PWM2_DUTY_SET_VADDR            MISC_PWM_PWM2_DUTY_SET_reg
#define  MIS_PWM3_CTRL_VADDR                MISC_PWM_PWM3_CTRL_reg
#define  MIS_PWM3_TIMING_CTRL_VADDR         MISC_PWM_PWM3_Timing_CTRL_reg
#define  MIS_PWM3_DUTY_SET_VADDR            MISC_PWM_PWM3_DUTY_SET_reg
#define  MIS_PWM4_CTRL_VADDR                MISC_PWM_PWM4_CTRL_reg
#define  MIS_PWM4_TIMING_CTRL_VADDR         MISC_PWM_PWM4_Timing_CTRL_reg
#define  MIS_PWM4_DUTY_SET_VADDR            MISC_PWM_PWM4_DUTY_SET_reg
#define  MIS_PWM5_CTRL_VADDR                MISC_PWM_PWM5_CTRL_reg
#define  MIS_PWM5_TIMING_CTRL_VADDR         MISC_PWM_PWM5_Timing_CTRL_reg
#define  MIS_PWM5_DUTY_SET_VADDR            MISC_PWM_PWM5_DUTY_SET_reg
#define  MIS_PWM6_CTRL_VADDR                MISC_PWM_PWM6_CTRL_reg
#define  MIS_PWM6_TIMING_CTRL_VADDR         MISC_PWM_PWM6_Timing_CTRL_reg
#define  MIS_PWM6_DUTY_SET_VADDR            MISC_PWM_PWM6_DUTY_SET_reg
#define  MIS_PWM7_CTRL_VADDR                MISC_PWM_PWM7_CTRL_reg
#define  MIS_PWM7_TIMING_CTRL_VADDR         MISC_PWM_PWM7_Timing_CTRL_reg
#define  MIS_PWM7_DUTY_SET_VADDR            MISC_PWM_PWM7_DUTY_SET_reg

struct rtk_pwm_phy_t {
	const rtk_pwm_reg_map *p_reg_map;
};

typedef struct rtk_pwm_phy_t rtk_pwm_phy;


struct rtk_pwm_pin_t {
	const char* pcb_name;
	rtk_pwm_phy pwm_phy;
	int index;
};
typedef struct rtk_pwm_pin_t rtk_pwm_pin;

static const rtk_pwm_reg_map iso_pwm0_reg = {
	.CTRL = ISO_MIS_PWM0_CTRL_VADDR,
	.TIMING_CTRL = ISO_MIS_PWM0_TIMING_CTRL_VADDR,
	.DUTY_SET = ISO_MIS_PWM0_DUTY_SET_VADDR,
};

static const rtk_pwm_reg_map iso_pwm1_reg = {
	.CTRL = ISO_MIS_PWM1_CTRL_VADDR,
	.TIMING_CTRL = ISO_MIS_PWM1_TIMING_CTRL_VADDR,
	.DUTY_SET = ISO_MIS_PWM1_DUTY_SET_VADDR,
};

static const rtk_pwm_reg_map mis_pwm0_reg = {
	.CTRL = MIS_PWM0_CTRL_VADDR,
	.TIMING_CTRL = MIS_PWM0_TIMING_CTRL_VADDR,
	.DUTY_SET = MIS_PWM0_DUTY_SET_VADDR,
};

static const rtk_pwm_reg_map mis_pwm1_reg = {
	.CTRL = MIS_PWM1_CTRL_VADDR,
	.TIMING_CTRL = MIS_PWM1_TIMING_CTRL_VADDR,
	.DUTY_SET = MIS_PWM1_DUTY_SET_VADDR,
};

static const rtk_pwm_reg_map mis_pwm2_reg = {
	.CTRL = MIS_PWM2_CTRL_VADDR,
	.TIMING_CTRL = MIS_PWM2_TIMING_CTRL_VADDR,
	.DUTY_SET = MIS_PWM2_DUTY_SET_VADDR,
};

static const rtk_pwm_reg_map mis_pwm3_reg = {
	.CTRL = MIS_PWM3_CTRL_VADDR,
	.TIMING_CTRL = MIS_PWM3_TIMING_CTRL_VADDR,
	.DUTY_SET = MIS_PWM3_DUTY_SET_VADDR,
};

static const rtk_pwm_reg_map mis_pwm4_reg = {
	.CTRL = MIS_PWM4_CTRL_VADDR,
	.TIMING_CTRL = MIS_PWM4_TIMING_CTRL_VADDR,
	.DUTY_SET = MIS_PWM4_DUTY_SET_VADDR,
};

static const rtk_pwm_reg_map mis_pwm5_reg = {
	.CTRL = MIS_PWM5_CTRL_VADDR,
	.TIMING_CTRL = MIS_PWM5_TIMING_CTRL_VADDR,
	.DUTY_SET = MIS_PWM5_DUTY_SET_VADDR,
};

static const rtk_pwm_reg_map mis_pwm6_reg = {
	.CTRL = MIS_PWM6_CTRL_VADDR,
	.TIMING_CTRL = MIS_PWM6_TIMING_CTRL_VADDR,
	.DUTY_SET = MIS_PWM6_DUTY_SET_VADDR,
};

static const rtk_pwm_reg_map mis_pwm7_reg = {
	.CTRL = MIS_PWM7_CTRL_VADDR,
	.TIMING_CTRL = MIS_PWM7_TIMING_CTRL_VADDR,
	.DUTY_SET = MIS_PWM7_DUTY_SET_VADDR,
};

#if 0
static rtk_pwm_phy pwm_iso_phy[] = {
	{&iso_pwm0_reg},
	{&iso_pwm1_reg},
};

static rtk_pwm_phy pwm_mis_phy[] = {
	{&mis_pwm0_reg},
	{&mis_pwm1_reg},
	{&mis_pwm2_reg},
	{&mis_pwm3_reg},
	{&mis_pwm4_reg},
	{&mis_pwm5_reg},
	{&mis_pwm6_reg},
	{&mis_pwm7_reg},
};
#endif

#define pwm_info_print(fmt,args...)		printf(fmt,## args)
#define pwm_debug_print(fmt,args...)		//printf(fmt,## args)

#if 0
static rtk_pwm_pin g_pwm_phy_avlb[] = {
	{"PIN_ISO_PWM_DIM1", {&iso_pwm0_reg}, -1},
	{"PIN_ISO_PWM_DIM2", {&iso_pwm1_reg}, -1},
	{"PIN_PWM_DIM1", {&mis_pwm0_reg}, -1},
	{"PIN_PWM_DIM2", {&mis_pwm1_reg}, -1},
	{"PIN_PWM_DIM3", {&mis_pwm2_reg}, -1},
	{"PIN_PWM_DIM4", {&mis_pwm3_reg}, -1},
	{"PIN_PWM_DIM5", {&mis_pwm4_reg}, -1},
	{"PIN_PWM_DIM6", {&mis_pwm5_reg}, -1},
	{"PIN_PWM_DIM7", {&mis_pwm6_reg}, -1},
	{"PIN_PWM_DIM8", {&mis_pwm7_reg}, -1},
};

#define RTK_PWM_CANDIDATE_CNT  (sizeof(g_pwm_phy_avlb)/sizeof(g_pwm_phy_avlb[0]))
#endif

/** these function code in driver/pwm/pwm.c **/
extern void rtk_hw_get_divider(int pwm_freq_hz, int duty_max,
				unsigned short *M_best, unsigned short *N_best);
extern int get_pwm_frequency(char type, int index);
extern int set_pwm_frequency(char type, int index, int freq);

extern int get_pwm_duty(char type, int index);
extern int set_pwm_duty(char type, int index, int duty);
extern int set_pwm_duty_full(int duty);

extern int get_pwm_enable(char type, int index);
extern int set_pwm_enable(char type, int index, int enable);

extern int get_pwm_sync(char type, int index);
extern int set_pwm_sync(char type, int index, int rtk_vsync);
/** these function code in driver/pwm/pwm.c **/


int rtk_pwm_init(void);
int rtk_pwm_set(RTK_PWM_INFO_T* param);
void rtk_pwm_test(void);

#endif
