#include <qc_verify_common.h>
#include <common.h>
#include <malloc.h>
#include <asm/io.h>
#include <errno.h>
#include <linux/mii.h>
#include <asm/arch/system.h>

#if CONFIG_FT
#undef CONFIG_DMA_TRANSFER
#undef BIST_TEST
#else
#define CONFIG_DMA_TRANSFER
#define BIST_TEST
#endif


//#define QC_test_1000M
#define QC_test_100M
#define QC_test_10M

//#define QC_ETN_DEBUG

#define ETH_TAG_NAME                 "[ETN]"

//#ifdef QC_ETN_DEBUG
//#define QC_ETN_DBG		     0
#ifdef QC_ETN_DBG
#define ETH_DBG(fmt, args...)       printf(ETH_TAG_NAME "[Debg] " fmt, ## args)
#else
#define ETH_DBG(fmt, args...)
#endif

#define REG_BASE    0xb8016000
#define RX_BUF_SIZE 	1536
#define TX_BUF_SIZE	1514
#define LAN_QC_VERIFY_LEN	(sizeof(txbuffer))//(0x40)
#define ETH_ZLEN	60
#define HZ 		1000
#define currticks()	get_timer(0)

#define R8168_TX_RING_BYTES (NUM_TX_DESC * sizeof(struct TxDesc))
#define R8168_RX_RING_BYTES (NUM_RX_DESC * sizeof(struct RxDesc))

#define NUM_TX_DESC	16	/* Number of Tx descriptor registers */
#define NUM_RX_DESC	16	/* Number of Rx descriptor registers */

#define TX_TIMEOUT  (6*HZ)
#define RX_DMA_BURST    7   /* Maximum PCI burst, '6' is 1024 */

#define RX_DMA_BURST_128 3
#define TX_DMA_BURST_unlimited  7
#define TX_DMA_BURST_1024   6
#define TX_DMA_BURST_512    5
#define TX_DMA_BURST_256    4
#define TX_DMA_BURST_128    3
#define TX_DMA_BURST_64     2
#define TX_DMA_BURST_32     1
#define TX_DMA_BURST_16     0

#define Reserved1_data  0x3F
#define RxEarly_off_V1 (0x07 << 11)
#define RxEarly_off_V2 (1 << 11)

#define InterFrameGap   0x03    /* 3 means InterFrameGap = the shortest one */

#define ALIGN_8                 (0x7)
#define ALIGN_16                (0xf)
#define ALIGN_32                (0x1f)
#define ALIGN_64                (0x3f)
#define ALIGN_256               (0xff)
#define _RXDESC_PER_CACHELINE (ARCH_ALIGN/sizeof(struct RxDesc)) 
#define ARCH_ALIGN 64

#define RTL_W8(reg, val8)	rtd_outb(ioaddr + (reg), (val8))
#define RTL_W16(reg, val16)	rtd_outw(ioaddr + (reg), (val16))
#define RTL_W32(reg, val32)	rtd_outl(ioaddr + (reg), (val32))
#define RTL_R8(reg)		rtd_inb(ioaddr + (reg))
#define RTL_R16(reg)		rtd_inw(ioaddr + (reg))
#define RTL_R32(reg)		rtd_inl(ioaddr + (reg))

typedef struct _GDUMP_TALLY {
    u64 TxOK;
    u64 RxOK;
    u64 TxERR;
    u32 RxERR;
    u16 MissPkt;
    u16 FAE;
    u32 Tx1Col;
    u32 TxMCol;
    u64 RxOKPhy;
    u64 RxOKBrd;
    u32 RxOKMul;
    u16 TxAbt;
    u16 TxUndrn; //only possible on jumbo frames
}

GDUMP_TALLY, *PGDUMP_TALLY;

enum RTL8168_registers {
    MAC0            = 0x00,     /* Ethernet hardware address. */
    MAC4            = 0x04,
    MAR0            = 0x08,     /* Multicast filter. */
    CounterAddrLow      = 0x10,
    CounterAddrHigh     = 0x14,
    CustomLED       = 0x18,
    TxDescStartAddrLow  = 0x20,
    TxDescStartAddrHigh = 0x24,
    TxHDescStartAddrLow = 0x28,
    TxHDescStartAddrHigh    = 0x2c,
    FLASH           = 0x30,
    ERSR            = 0x36,
    ChipCmd         = 0x37,
    TxPoll          = 0x38,
    IntrMask        = 0x3C,
    IntrStatus      = 0x3E,
    TxConfig        = 0x40,
    RxConfig        = 0x44,
    TCTR            = 0x48,
    Cfg9346         = 0x50,
    Config0         = 0x51,
    Config1         = 0x52,
    Config2         = 0x53,
    Config3         = 0x54,
    Config4         = 0x55,
    Config5         = 0x56,
    TDFNR           = 0x57,
    TimeIntr        = 0x58,
    PHYAR           = 0x60,
    CSIDR           = 0x64,
    CSIAR           = 0x68,
    PHYstatus       = 0x6C,
    MACDBG          = 0x6D,
    GPIO            = 0x6E,
    PMCH            = 0x6F,
    ERIDR           = 0x70,
    ERIAR           = 0x74,
    EPHY_RXER_NUM       = 0x7C,
    EPHYAR          = 0x80,
    OCPDR           = 0xB0,
    MACOCP          = 0xB0,
    OCPAR           = 0xB4,
    PHYOCP          = 0xB8,
    DBG_reg         = 0xD1,
    MCUCmd_reg      = 0xD3,
    RxMaxSize       = 0xDA,
    EFUSEAR         = 0xDC,
    CPlusCmd        = 0xE0,
    IntrMitigate        = 0xE2,
    RxDescAddrLow       = 0xE4,
    RxDescAddrHigh      = 0xE8,
    MTPS            = 0xEC,
    FuncEvent       = 0xF0,
    FuncEventMask       = 0xF4,
    FuncPresetState     = 0xF8,
    FuncForceEvent      = 0xFC,
};

enum RTL8168_register_content {
    /* InterruptStatusBits */
    SYSErr      = 0x8000,
    PCSTimeout  = 0x4000,
    SWInt       = 0x0100,
    TxDescUnavail   = 0x0080,
    RxFIFOOver  = 0x0040,
    LinkChg     = 0x0020,
    RxDescUnavail   = 0x0010,
    TxErr       = 0x0008,
    TxOK        = 0x0004,
    RxErr       = 0x0002,
    RxOK        = 0x0001,

    /* RxStatusDesc */
    RxRWT = (1 << 22),
    RxRES = (1 << 21),
    RxRUNT = (1 << 20),
    RxCRC = (1 << 19),

    /* ChipCmdBits */
    StopReq  = 0x80,
    CmdReset = 0x10,
    CmdRxEnb = 0x08,
    CmdTxEnb = 0x04,
    RxBufEmpty = 0x01,

    /* Cfg9346Bits */
    Cfg9346_Lock = 0x00,
    Cfg9346_Unlock = 0xC0,
    Cfg9346_EEDO = (1 << 0),
    Cfg9346_EEDI = (1 << 1),
    Cfg9346_EESK = (1 << 2),
    Cfg9346_EECS = (1 << 3),
    Cfg9346_EEM0 = (1 << 6),
    Cfg9346_EEM1 = (1 << 7),

    /* rx_mode_bits */
    AcceptErr = 0x20,
    AcceptRunt = 0x10,
    AcceptBroadcast = 0x08,
    AcceptMulticast = 0x04,
    AcceptMyPhys = 0x02,
    AcceptAllPhys = 0x01,

    /* Transmit Priority Polling*/
    HPQ = 0x80,
    NPQ = 0x40,
    FSWInt = 0x01,

    /* RxConfigBits */
    Reserved2_shift = 13,
    RxCfgDMAShift = 8,
    RxCfg_128_int_en = (1 << 15),
    RxCfg_fet_multi_en = (1 << 14),
    RxCfg_half_refetch = (1 << 13),
    RxCfg_9356SEL = (1 << 6),

    /* TxConfigBits */
    TxInterFrameGapShift = 24,
    TxDMAShift = 8, /* DMA burst value (0-7) is shift this many bits */
    TxMACLoopBack = (1 << 17),  /* MAC loopback */

    /* Config1 register */
    LEDS1       = (1 << 7),
    LEDS0       = (1 << 6),
    Speed_down  = (1 << 4),
    MEMMAP      = (1 << 3),
    IOMAP       = (1 << 2),
    VPD         = (1 << 1),
    PMEnable    = (1 << 0), /* Power Management Enable */

    /* Config2 register */
    PMSTS_En    = (1 << 5),

    /* Config3 register */
    Isolate_en  = (1 << 12), /* Isolate enable */
    MagicPacket = (1 << 5), /* Wake up when receives a Magic Packet */
    LinkUp      = (1 << 4), /* This bit is reserved in RTL8168B.*/
    /* Wake up when the cable connection is re-established */
    ECRCEN      = (1 << 3), /* This bit is reserved in RTL8168B*/
    Jumbo_En0   = (1 << 2), /* This bit is reserved in RTL8168B*/
    RDY_TO_L23  = (1 << 1), /* This bit is reserved in RTL8168B*/
    Beacon_en   = (1 << 0), /* This bit is reserved in RTL8168B*/

    /* Config4 register */
    Jumbo_En1   = (1 << 1), /* This bit is reserved in RTL8168B*/

    /* Config5 register */
    BWF     = (1 << 6), /* Accept Broadcast wakeup frame */
    MWF     = (1 << 5), /* Accept Multicast wakeup frame */
    UWF     = (1 << 4), /* Accept Unicast wakeup frame */
    LanWake     = (1 << 1), /* LanWake enable/disable */
    PMEStatus   = (1 << 0), /* PME status can be reset by PCI RST# */

    /* CPlusCmd */
    EnableBist  = (1 << 15),
    Macdbgo_oe  = (1 << 14),
    Normal_mode = (1 << 13),
    Force_halfdup   = (1 << 12),
    Force_rxflow_en = (1 << 11),
    Force_txflow_en = (1 << 10),
    Cxpl_dbg_sel    = (1 << 9),//This bit is reserved in RTL8168B
    ASF     = (1 << 8),//This bit is reserved in RTL8168C
    PktCntrDisable  = (1 << 7),
    RxVlan      = (1 << 6),
    RxChkSum    = (1 << 5),
    Macdbgo_sel = 0x001C,
    INTT_0      = 0x0000,
    INTT_1      = 0x0001,
    INTT_2      = 0x0002,
    INTT_3      = 0x0003,

    /* rtl8168_PHYstatus */
    PowerSaveStatus = 0x80,
    TxFlowCtrl = 0x40,
    RxFlowCtrl = 0x20,
    _1000bpsF = 0x10,
    _100bps = 0x08,
    _10bps = 0x04,
    LinkStatus = 0x02,
    FullDup = 0x01,

    /* DBG_reg */
    Fix_Nak_1 = (1 << 4),
    Fix_Nak_2 = (1 << 3),
    DBGPIN_E2 = (1 << 0),

    /* DumpCounterCommand */
    CounterDump = 0x8,

    /* PHY access */
    PHYAR_Flag = 0x80000000,
    PHYAR_Write = 0x80000000,
    PHYAR_Read = 0x00000000,
    PHYAR_Reg_Mask = 0x1f,
    PHYAR_Reg_shift = 16,
    PHYAR_Data_Mask = 0xffff,

    /* EPHY access */
    EPHYAR_Flag = 0x80000000,
    EPHYAR_Write = 0x80000000,
    EPHYAR_Read = 0x00000000,
    EPHYAR_Reg_Mask = 0x1f,
    EPHYAR_Reg_shift = 16,
    EPHYAR_Data_Mask = 0xffff,

    /* CSI access */
    CSIAR_Flag = 0x80000000,
    CSIAR_Write = 0x80000000,
    CSIAR_Read = 0x00000000,
    CSIAR_ByteEn = 0x0f,
    CSIAR_ByteEn_shift = 12,
    CSIAR_Addr_Mask = 0x0fff,

    /* ERI access */
    ERIAR_Flag = 0x80000000,
    ERIAR_Write = 0x80000000,
    ERIAR_Read = 0x00000000,
    ERIAR_Addr_Align = 4, /* ERI access register address must be 4 byte alignment */
    ERIAR_ExGMAC = 0,
    ERIAR_MSIX = 1,
    ERIAR_ASF = 2,
    ERIAR_OOB = 2,
    ERIAR_Type_shift = 16,
    ERIAR_ByteEn = 0x0f,
    ERIAR_ByteEn_shift = 12,

    /* OCP GPHY access */
    OCPDR_Write = 0x80000000,
    OCPDR_Read = 0x00000000,
    OCPDR_Reg_Mask = 0xFF,
    OCPDR_Data_Mask = 0xFFFF,
    OCPDR_GPHY_Reg_shift = 16,
    OCPAR_Flag = 0x80000000,
    OCPAR_GPHY_Write = 0x8000F060,
    OCPAR_GPHY_Read = 0x0000F060,
    OCPR_Write = 0x80000000,
    OCPR_Read = 0x00000000,
    OCPR_Addr_Reg_shift = 16,
    OCPR_Flag = 0x80000000,
    OCP_STD_PHY_BASE_PAGE = 0x0A40,

    /* MCU Command */
    Now_is_oob = (1 << 7),
    Txfifo_empty = (1 << 5),
    Rxfifo_empty = (1 << 4),

    /* E-FUSE access */
    EFUSE_WRITE = 0x80000000,
    EFUSE_WRITE_OK  = 0x00000000,
    EFUSE_READ  = 0x00000000,
    EFUSE_READ_OK   = 0x80000000,
    EFUSE_Reg_Mask  = 0x03FF,
    EFUSE_Reg_Shift = 8,
    EFUSE_Check_Cnt = 300,
    EFUSE_READ_FAIL = 0xFF,
    EFUSE_Data_Mask = 0x000000FF,

    /* GPIO */
    GPIO_en = (1 << 0),

};

enum _DescStatusBit {
    DescOwn     = (1 << 31), /* Descriptor is owned by NIC */
    RingEnd     = (1 << 30), /* End of descriptor ring */
    FirstFrag   = (1 << 29), /* First segment of a packet */
    LastFrag    = (1 << 28), /* Final segment of a packet */

    /* Tx private */
    /*------ offset 0 of tx descriptor ------*/
    LargeSend   = (1 << 27), /* TCP Large Send Offload (TSO) */
    LargeSend_DP = (1 << 16), /* TCP Large Send Offload (TSO) */
    MSSShift    = 16,        /* MSS value position */
    MSSMask     = 0x7FFU,    /* MSS value 11 bits */
    TxIPCS      = (1 << 18), /* Calculate IP checksum */
    TxUDPCS     = (1 << 17), /* Calculate UDP/IP checksum */
    TxTCPCS     = (1 << 16), /* Calculate TCP/IP checksum */
    TxVlanTag   = (1 << 17), /* Add VLAN tag */

    /*@@@@@@ offset 4 of tx descriptor => bits for RTL8168C/CP only     begin @@@@@@*/
    TxUDPCS_C   = (1 << 31), /* Calculate UDP/IP checksum */
    TxTCPCS_C   = (1 << 30), /* Calculate TCP/IP checksum */
    TxIPCS_C    = (1 << 29), /* Calculate IP checksum */
    /*@@@@@@ offset 4 of tx descriptor => bits for RTL8168C/CP only     end @@@@@@*/


    /* Rx private */
    /*------ offset 0 of rx descriptor ------*/
    PID1        = (1 << 18), /* Protocol ID bit 1/2 */
    PID0        = (1 << 17), /* Protocol ID bit 2/2 */

#define RxProtoUDP  (PID1)
#define RxProtoTCP  (PID0)
#define RxProtoIP   (PID1 | PID0)
#define RxProtoMask RxProtoIP

    RxIPF       = (1 << 16), /* IP checksum failed */
    RxUDPF      = (1 << 15), /* UDP/IP checksum failed */
    RxTCPF      = (1 << 14), /* TCP/IP checksum failed */
    RxVlanTag   = (1 << 16), /* VLAN tag available */

    /*@@@@@@ offset 0 of rx descriptor => bits for RTL8168C/CP only     begin @@@@@@*/
    RxUDPT      = (1 << 18),
    RxTCPT      = (1 << 17),
    /*@@@@@@ offset 0 of rx descriptor => bits for RTL8168C/CP only     end @@@@@@*/

    /*@@@@@@ offset 4 of rx descriptor => bits for RTL8168C/CP only     begin @@@@@@*/
    RxV6F       = (1 << 31),
    RxV4F       = (1 << 30),
    /*@@@@@@ offset 4 of rx descriptor => bits for RTL8168C/CP only     end @@@@@@*/
};

struct rtl8168_private {
	u32 mmio_addr;	/* memory map physical address */
	int chipset;
	u32 mcfg;
	u32 cur_page;
	u32 cur_rx;	/* Index into the Rx descriptor buffer of next Rx pkt. */
	u32 cur_tx;	/* Index into the Tx descriptor buffer of next Rx pkt. */
	u32 dirty_rx;
	u32 dirty_tx;
	u8 *DescArrays;	/* Index of Descriptor buffer */
	struct TxDesc *TxDescArray;	/* Index of 256-alignment Tx Descriptor buffer */
	struct RxDesc *RxDescArray;	/* Index of 256-alignment Rx Descriptor buffer */
	u8 *RxBufferRings;	/* Index of Rx Buffer  */
	u8 *RxBufferRing[NUM_RX_DESC];	/* Index of Rx Buffer array */

	u8  efuse;
	u32 rx_buf_sz;
	u32 rtl8168_rx_config;

	u16 cp_cmd;
	u32 tx_count;
	u32 rx_count;
};

struct TxDesc {
	u32 opts1;
	u32 opts2;
	u64 addr;
};

struct RxDesc {
	u32 opts1;
	u32 opts2;
	u64 addr;
};

static u8 dummy[sizeof( GDUMP_TALLY ) + ALIGN_64];
static u8  txbuffer[TX_BUF_SIZE] __attribute__((aligned(32)));
static int rtl8168_recv(struct rtl8168_private *tp);
static int rtl8168_recv_count(struct rtl8168_private *tp);

static void mdio_write(struct rtl8168_private *tp, u32 RegAddr, u32 value)
{
	u32 ioaddr = tp->mmio_addr;
	int i;

	if (RegAddr == 0x1F) {
		tp->cur_page = value;
	}

	RTL_W32(PHYAR, PHYAR_Write |
			(RegAddr & PHYAR_Reg_Mask) << PHYAR_Reg_shift |
			(value & PHYAR_Data_Mask));
	//printf("WPHY addr %x value %x \n",RegAddr,value);
	for (i = 0; i < 10; i++) {
		udelay(100);

		/* Check if the RTL8168 has completed writing to the specified MII register */
		if (!(RTL_R32(PHYAR) & PHYAR_Flag)) {
			udelay(20);
			break;
		}
	}
}

#if 0
static u32 mdio_fast_read(struct rtl8168_private *tp, u32 RegAddr)
{
        u32 ioaddr = tp->mmio_addr;
        int i, value = 0;

        RTL_W32(PHYAR,
                        PHYAR_Read | (RegAddr & PHYAR_Reg_Mask) << PHYAR_Reg_shift);

        for (i = 0; i < 10; i++) {
                udelay(10);

                /* Check if the RTL8168 has completed retrieving data from the specified MII register */
                if (RTL_R32(PHYAR) & PHYAR_Flag) {
                        value = RTL_R32(PHYAR) & PHYAR_Data_Mask;
                        //udelay(20);
                        break;
                }
        }

	if (i == 10)
		printf("mdio fast read timeout....\n");
        //printf("RPHY addr %x value %x \n",RegAddr,value);

        return value;
}
#endif


static u32 mdio_read(struct rtl8168_private *tp, u32 RegAddr)
{
	u32 ioaddr = tp->mmio_addr;
	int i, value = 0;

	RTL_W32(PHYAR,
			PHYAR_Read | (RegAddr & PHYAR_Reg_Mask) << PHYAR_Reg_shift);

	for (i = 0; i < 10; i++) {
		udelay(100);

		/* Check if the RTL8168 has completed retrieving data from the specified MII register */
		if (RTL_R32(PHYAR) & PHYAR_Flag) {
			value = RTL_R32(PHYAR) & PHYAR_Data_Mask;
			udelay(20);
			break;
		}
	}
	//printf("RPHY addr %x value %x \n",RegAddr,value);

	return value;
}

static void rtl8168_init_ring_indexes(struct rtl8168_private *tp)
{
	tp->dirty_tx = 0;
	tp->dirty_rx = 0;
	tp->cur_tx = 0;
	tp->cur_rx = 0;
}

static void rtl8168_tx_desc_init(struct rtl8168_private *tp)
{
	int i = 0;

	memset(tp->TxDescArray, 0x0, NUM_TX_DESC * sizeof(struct TxDesc));

	for (i = 0; i < NUM_TX_DESC; i++) {
		if (i == (NUM_TX_DESC - 1))
			tp->TxDescArray[i].opts1 = cpu_to_le32(RingEnd);
	}
}

static void rtl8168_rx_desc_init(struct rtl8168_private *tp)
{
	memset(tp->RxDescArray, 0x0, NUM_RX_DESC * sizeof(struct RxDesc));
}

static u32 rtl8168_rx_fill(struct rtl8168_private *tp,
		u32 start,
		u32 end)
{
	unsigned long rxb;
	u32 cur;

	rxb = (unsigned long)tp->RxBufferRings;
	rxb = (rxb + ALIGN_8) & ~ALIGN_8;

	for (cur = start; end - cur > 0; cur++) {
		if (cur == (NUM_RX_DESC - 1))
			tp->RxDescArray[cur].opts1 =
				cpu_to_le32((DescOwn | RingEnd) + tp->rx_buf_sz);
		else
			tp->RxDescArray[cur].opts1 =
				cpu_to_le32(DescOwn + tp->rx_buf_sz);

		tp->RxBufferRing[cur] = (u8 *)(rxb + cur * tp->rx_buf_sz);
		tp->RxDescArray[cur].addr =
			cpu_to_le64((unsigned int)(tp->RxBufferRing[cur]));

		flush_cache((unsigned long)tp->RxBufferRing[cur], tp->rx_buf_sz);
	}

	return cur - start;
}

static int rtl8168_init_ring(struct rtl8168_private *tp)
{
#ifdef QC_ETN_DEBUG
	printf ("%s\n", __FUNCTION__);
#endif

	rtl8168_init_ring_indexes(tp);

	rtl8168_tx_desc_init(tp);
	rtl8168_rx_desc_init(tp);

	rtl8168_rx_fill(tp, 0, NUM_RX_DESC);

	//rtl8168_mark_as_last_descriptor(tp->RxDescArray + NUM_RX_DESC - 1);

    return 0;
}

static void rtl8168_tally_counter_addr_fill(struct rtl8168_private *tp)
{
	u32 ioaddr = tp->mmio_addr;
	unsigned long tally;

	tally = ((unsigned long)dummy + ALIGN_64) & ~ALIGN_64;

	if (!tally)
		return;

	RTL_W32(CounterAddrLow, cpu_to_le32(tally));
	RTL_W32(CounterAddrHigh, 0);
    ETH_DBG("tally phy=0x%x\n",cpu_to_le32(tally));
}

static void rtl8168_tally_counter_clear(struct rtl8168_private *tp)
{
	u32 ioaddr = tp->mmio_addr;
	unsigned long tally;

	tally = ((unsigned long)dummy + ALIGN_64) & ~ALIGN_64;

	if (!tally)
		return;

	tally |= _BIT0;
	RTL_W32(CounterAddrHigh, 0);
	RTL_W32(CounterAddrLow, cpu_to_le32(tally));
}

static void rtl8168_desc_addr_fill(struct rtl8168_private *tp)
{
	u32 ioaddr = tp->mmio_addr;

	RTL_W32(TxDescStartAddrLow, (unsigned int)(tp->TxDescArray));
	RTL_W32(TxDescStartAddrHigh, (unsigned long)0);
	RTL_W32(RxDescAddrLow, (unsigned int)(tp->RxDescArray));
	RTL_W32(RxDescAddrHigh, (unsigned long)0);
}

static void rtl8168_hw_init(struct rtl8168_private *tp)
{
	u32 ioaddr = tp->mmio_addr;

	// ? disable aspm and clock request before access ephy
	RTL_W8(Cfg9346, Cfg9346_Unlock);
	RTL_W8(Config5, RTL_R8(Config5) & ~_BIT0);
	RTL_W8(Config2, RTL_R8(Config2) & ~_BIT7);
	RTL_W8(Cfg9346, Cfg9346_Lock);
	RTL_W8(0xF1, RTL_R8(0xF1) & ~_BIT7);  //??

#if 1
	/*disable ocp phy power saving*/
	mdio_write(tp, 0x1F, 0x0C41);
	mdio_write(tp, 0x13, 0x0000);
	mdio_write(tp, 0x13, 0x0500);
	mdio_write(tp, 0x1F, 0x0000);
	// printf("PHY status: %08x, %08x\n", mdio_read(tp, 0x00), RTL_R8(0x6c));
#endif
}

static void rtl8168_enable_rxdvgate(struct rtl8168_private *tp)
{
	u32 ioaddr = tp->mmio_addr;

	RTL_W8(0xF2, RTL_R8(0xF2) | _BIT3); 
	mdelay(2);
}

static void rtl8168_disable_rxdvgate(struct rtl8168_private *tp)
{
	u32 ioaddr = tp->mmio_addr;

	RTL_W8(0xF2, RTL_R8(0xF2) & ~_BIT3);
	mdelay(2);
}

static void rtl8168_wait_txrx_fifo_empty(struct rtl8168_private *tp)
{
	u32 ioaddr = tp->mmio_addr;
	int i;

	for (i = 0; i < 10; i++) {
		udelay(100);
		if (RTL_R32(TxConfig) & _BIT11)
			break;
	}

	for (i = 0; i < 10; i++) {
		udelay(100);
		if ((RTL_R8(MCUCmd_reg) & (Txfifo_empty | Rxfifo_empty)) == (Txfifo_empty | Rxfifo_empty))
			break;

	}
}

static void rtl8168_irq_mask_and_ack(u32 ioaddr)
{
	RTL_W16(IntrMask, 0x0000);
	RTL_W16(IntrStatus, RTL_R16(IntrStatus));
}

static void rtl8168_nic_reset(struct rtl8168_private *tp)
{
	u32 ioaddr = tp->mmio_addr;
//	int i;

#ifdef QC_ETN_DEBUG
	printf("%s\n", __func__);
#endif
	RTL_W32(RxConfig, (RX_DMA_BURST << RxCfgDMAShift));

	rtl8168_enable_rxdvgate(tp);

	rtl8168_wait_txrx_fifo_empty(tp);

	mdelay(2);

//	/* Software reset the chip. */
//	RTL_W8(ChipCmd, CmdReset);
//
//	///* Check that the chip has finished the reset. */
//	for (i = 100; i > 0; i--) {
//		if ((RTL_R8(ChipCmd) & CmdReset) == 0)
//			break;
//		udelay(100);
//	}
	//printf("%s, i = %d\n", __func__, i);
	//printf("reset chip\n");
}

static void rtl8168_hw_reset(struct rtl8168_private *tp)
{
	u32 ioaddr = tp->mmio_addr;

	/* Disable interrupts and clear interrupt status */
	rtl8168_irq_mask_and_ack(ioaddr);

	rtl8168_nic_reset(tp);
}

static void rtl8168_phy_power_up (struct rtl8168_private *tp)
{
	//u32 ioaddr = tp->mmio_addr;

	mdio_write(tp, 0x1F, 0x0000);
	mdio_write(tp, MII_BMCR, BMCR_ANENABLE);  // Enable auto negotiation
}

static void rtl8168_powerup_pll(struct rtl8168_private *tp)
{
	//u32 ioaddr = tp->mmio_addr;

	//RTL_W8(PMCH, RTL_R8(PMCH) | _BIT7 | _BIT6);  // default value = 0xf0

	rtl8168_phy_power_up(tp);
}

#ifdef QC_test_1000M
static void rtl8168_giga_link_patch(struct rtl8168_private *tp) 
{
	u32 ioaddr = tp->mmio_addr;

	mdio_write(tp, 0x1f, 0x0a40);
	//mdelay(100);
	mdio_write(tp, 0x1b, 0x8013);// disable UC auto k r/rc address
	//mdelay(10);
	mdio_write(tp, 0x1c, 0x0f08);// disable UC data
	//mdelay(10);

	mdio_write(tp, 0x1f, 0x0a40);// change Page to a40
	//mdelay(100);
	mdio_write(tp, 0x1b, 0x8011);// set reg27 bit[15:0]=0
	//mdelay(10);
	mdio_write(tp, 0x1c, 0x9137);// set reg28 bit[10:10]=0
	//mdelay(10);
	mdio_write(tp, 0x1f, 0x0bc0);// change Page to bc0
	//mdelay(100);
	mdio_write(tp, 0x14, 0x3465);// set reg20 bit[11]=0
	//mdelay(10);
	
	/* Delay cap */
	mdio_write(tp, 0x1f, 0x0bc0);// change Page to bc0
	//mdelay(100);
	mdio_write(tp, 0x15, 0x8160);// set reg21 bit[1:0]=0
	//mdelay(10);
	printf("Reg21 = %04x\n", mdio_read(tp, 0x15));// set reg21 bit[1:0]=0
	//mdelay(10);

	/* Delay cap */
	mdio_write(tp, 0x1f, 0x0bcd);// change Page to bcd
	//mdelay(100);
	mdio_write(tp, 0x15, 0x2222);// set reg21 bit[15:0]=0
	//mdelay(10);
	mdio_read(tp, 0x15);// set reg21 bit[15:0]=0
	//mdelay(10);

	/* tapbin R value */
	mdio_write(tp, 0x1f, 0x0bce);// change Page to bce
	//mdelay(100);
	mdio_write(tp, 0x10, 0x5555);// set reg16 bit[15:0]=aaaa
	//mdelay(10);
	printf("Reg16 = %04x\n", mdio_read(tp, 0x10));// set reg16 bit[15:0]=aaaa
	//mdelay(10);

	/* LPF Rx Len value */
	mdio_write(tp, 0x1f, 0x0bcd);// change Page to bcd
	//mdelay(100);
	mdio_write(tp, 0x16, 0xffff);// set reg22 bit[15:0]=ffff
	//mdelay(10);
	mdio_read(tp, 0x16);// set reg16 bit[15:0]=aaaa
	//mdelay(10);

	mdelay(10000);
	printf("status = %x, %04x, %04x\n", RTL_R8(PHYstatus), mdio_read(tp, MII_CTRL1000), mdio_read(tp, MII_STAT1000));
}
#endif

static void rtl8168_set_speed_duplex(struct rtl8168_private *tp, u16 mode, u16 duplex)
{
	//u32 ioaddr = tp->mmio_addr;

	mdio_write(tp, 0x1F, 0x0000);
	mdio_write(tp, MII_BMCR, mode | duplex);  
#ifdef QC_ETN_DEBUG
	printk("Reg0 = %04x\n", mdio_read(tp, MII_BMCR));
#endif
}

static int rtl8168_check_phy_status(struct rtl8168_private *tp, u8 mode)
{
	u32 ioaddr = tp->mmio_addr;
	u8 status = 0x0;
	u32 i = 0;

	while(i <= 550) {
		status = RTL_R8(PHYstatus);
		if((status & LinkStatus) && (status & mode)) {
			printk("Link OK\n");
			break;
		}
		mdelay(1);
		i++;
	}
	
	if (i > 550) {
		printk("Link fail\n");
		return 0;
	}
	else
		return 1;
}

static int rtl8168_recv_packet(struct rtl8168_private *tp)
{
        u32 ioaddr = tp->mmio_addr;
        int ret = 0;
        u32 count = 30;
        int stat = 0;

        do {
                if((RTL_R16(IntrStatus) & RxErr) || (RTL_R16(IntrStatus) & RxOK)) {
                        if((stat=rtl8168_recv(tp)) < 0) {
                                        ret = stat; //-1,-2
                        }
                        break;
                }
                //udelay(10);
                mdelay(10);
                count--;
                if (count == 0)
                {
                        printk("timeout\n");
                        ret = -1;
                        break;
                }
        } while(1);

        return ret;
}


static int rtl8168_recv_waiting(struct rtl8168_private *tp)
{
    u32 ioaddr = tp->mmio_addr;
    int waiting = 0;

    for(waiting=0; waiting<30; waiting++)
    {
        if(((RTL_R16(IntrStatus) & RxErr)
                || (RTL_R16(IntrStatus) & RxOK)))
            return 1;
        mdelay(10);
    }
    return 0;
}

static int rtl8168_recv_multi_packet(struct rtl8168_private *tp,int packetNum)
{
	u32 ioaddr  = tp->mmio_addr;
	int ret     = 0;
    int i       = 0;

    if(rtl8168_recv_waiting(tp))
    {
        for(i=0;i<packetNum;i++)
        {
            ret = rtl8168_recv_count(tp);
            if(ret < 0)
                break;
            else if(!ret && !rtl8168_recv_waiting(tp))
            {
                printf("rx ir status timeout cur_rx=%u\n",tp->cur_rx);
                ret = -1;
                break;
            }
        }
    }
    else
    {
        printf("rx ir status timeout cur_rx=%u\n",tp->cur_rx);
        ret = -1;
    }

	return ret;
}

static void rtl8168_rx_desc_offset0_init(struct rtl8168_private *tp, int own)
{
	int i = 0;
	int ownbit = 0;

	if (own)
		ownbit = DescOwn;

	for (i = 0; i < NUM_RX_DESC; i++) {
		if (i == (NUM_RX_DESC - 1))
			tp->RxDescArray[i].opts1 = cpu_to_le32((ownbit | RingEnd) | (unsigned long)tp->rx_buf_sz);
		else
			tp->RxDescArray[i].opts1 = cpu_to_le32(ownbit | (unsigned long)tp->rx_buf_sz);

	}
}

static u32 rtl8168_eri_read(u32 ioaddr, int addr, int len, int type)
{
	int i, val_shift, shift = 0;
	u32 value1 = 0, value2 = 0, mask;

	if (len > 4 || len <= 0)
		return -1;

	while (len > 0) {
		val_shift = addr % ERIAR_Addr_Align;
		addr = addr & ~0x3;

		RTL_W32(ERIAR,
				ERIAR_Read |
				type << ERIAR_Type_shift |
				ERIAR_ByteEn << ERIAR_ByteEn_shift |
				addr);

		for (i = 0; i < 10; i++) {
			udelay(100);

			/* Check if the RTL8168 has completed ERI read */
			if (RTL_R32(ERIAR) & ERIAR_Flag)
				break;
		}

		if (len == 1)       mask = (0xFF << (val_shift * 8)) & 0xFFFFFFFF;
		else if (len == 2)  mask = (0xFFFF << (val_shift * 8)) & 0xFFFFFFFF;
		else if (len == 3)  mask = (0xFFFFFF << (val_shift * 8)) & 0xFFFFFFFF;
		else            mask = (0xFFFFFFFF << (val_shift * 8)) & 0xFFFFFFFF;

		value1 = RTL_R32(ERIDR) & mask;
		value2 |= (value1 >> val_shift * 8) << shift * 8;

		if (len <= 4 - val_shift) {
			len = 0;
		} else {
			len -= (4 - val_shift);
			shift = 4 - val_shift;
			addr += 4;
		}
	}

	return value2;
}

static int rtl8168_eri_write(u32 ioaddr, int addr, int len, u32 value, int type)
{
	int i, val_shift, shift = 0;
	u32 value1 = 0, mask;

	if (len > 4 || len <= 0)
		return -1;

	while (len > 0) {
		val_shift = addr % ERIAR_Addr_Align;
		addr = addr & ~0x3;

		if (len == 1)       mask = (0xFF << (val_shift * 8)) & 0xFFFFFFFF;
		else if (len == 2)  mask = (0xFFFF << (val_shift * 8)) & 0xFFFFFFFF;
		else if (len == 3)  mask = (0xFFFFFF << (val_shift * 8)) & 0xFFFFFFFF;
		else            mask = (0xFFFFFFFF << (val_shift * 8)) & 0xFFFFFFFF;

		value1 = rtl8168_eri_read(ioaddr, addr, 4, type) & ~mask;
		value1 |= ((value << val_shift * 8) >> shift * 8);

		RTL_W32(ERIDR, value1);
		RTL_W32(ERIAR,
				ERIAR_Write |
				type << ERIAR_Type_shift |
				ERIAR_ByteEn << ERIAR_ByteEn_shift |
				addr);

		for (i = 0; i < 10; i++) {
			udelay(100);

			/* Check if the RTL8168 has completed ERI write */
			if (!(RTL_R32(ERIAR) & ERIAR_Flag))
				break;
		}

		if (len <= 4 - val_shift) {
			len = 0;
		} else {
			len -= (4 - val_shift);
			shift = 4 - val_shift;
			addr += 4;
		}
	}

	return 0;
}

static void rtl8168_hw_set_rx_packet_filter(struct rtl8168_private *tp)
{
	u32 ioaddr = tp->mmio_addr;
	int rx_mode;
	u32 tmp = 0;

#ifdef QC_ETN_DEBUG
	printf ("%s\n", __FUNCTION__);
#endif

	/* IFF_ALLMULTI */
	/* Too many to filter perfectly -- accept all multicasts. */
	rx_mode = RTL_R32(RxConfig);
	rx_mode |= AcceptBroadcast | AcceptMulticast | AcceptMyPhys | AcceptAllPhys | AcceptRunt | AcceptErr;

	tp->rtl8168_rx_config = RxEarly_off_V2 | (RX_DMA_BURST << RxCfgDMAShift);
	tmp = tp->rtl8168_rx_config | rx_mode | (RTL_R32(RxConfig) & 0xff7e1880);

	RTL_W32(RxConfig, tmp);
#ifdef QC_ETN_DEBUG
	printk("RxConfig = %08x\n", RTL_R32(RxConfig));
#endif

	RTL_W32(MAR0 + 0, 0xffffffff);
	RTL_W32(MAR0 + 4, 0xffffffff);
}

static void rtl8168_hw_start(struct rtl8168_private *tp)
{
	u32 ioaddr = tp->mmio_addr;
	u32 csi_tmp;

#ifdef QC_ETN_DEBUG
	printf ("%s\n", __FUNCTION__);
#endif

	RTL_W32(RxConfig, (RX_DMA_BURST << RxCfgDMAShift));

	rtl8168_hw_reset(tp);
	//    printf("RxConfig = %08x\n", RTL_R32(RxConfig));

	rtl8168_rx_desc_offset0_init(tp, 1); //? init functin already initailze 

	RTL_W8(Cfg9346, Cfg9346_Unlock);

	RTL_W8(0xF1, RTL_R8(0xF1) & ~_BIT7);
	RTL_W8(Config2, RTL_R8(Config2) & ~_BIT7);
	RTL_W8(Config5, RTL_R8(Config5) & ~_BIT0);

	/* Max tx packet size */
	RTL_W8(MTPS, Reserved1_data);

	rtl8168_tally_counter_addr_fill(tp);

	rtl8168_desc_addr_fill(tp);

	/* Set DMA burst size and Interframe Gap Time */
	RTL_W32(TxConfig, (TX_DMA_BURST_128 << TxDMAShift) |
			(InterFrameGap << TxInterFrameGapShift));

	tp->cp_cmd = (RTL_R16(CPlusCmd) | PktCntrDisable);

	rtl8168_eri_write(ioaddr, 0xC8, 4, 0x00080002, ERIAR_ExGMAC);
	rtl8168_eri_write(ioaddr, 0xCC, 1, 0x38, ERIAR_ExGMAC);
	rtl8168_eri_write(ioaddr, 0xD0, 1, 0x48, ERIAR_ExGMAC);
	rtl8168_eri_write(ioaddr, 0xE8, 4, 0x00100006, ERIAR_ExGMAC);

	RTL_W32(TxConfig, RTL_R32(TxConfig) | _BIT7);

	csi_tmp = rtl8168_eri_read(ioaddr, 0xDC, 1, ERIAR_ExGMAC);
	csi_tmp &= ~_BIT0;
	rtl8168_eri_write(ioaddr, 0xDC, 1, csi_tmp, ERIAR_ExGMAC);
	csi_tmp |= _BIT0;
	rtl8168_eri_write(ioaddr, 0xDC, 1, csi_tmp, ERIAR_ExGMAC);

	RTL_W8(Config3, RTL_R8(Config3) & ~Beacon_en);

	tp->cp_cmd = RTL_R16(CPlusCmd) &
		~(EnableBist | Macdbgo_oe | Force_halfdup |
				Force_rxflow_en | Force_txflow_en |
				Cxpl_dbg_sel | ASF | PktCntrDisable |
				Macdbgo_sel);

	RTL_W8(0x1B, RTL_R8(0x1B) & ~0x07); //? for led gpio

	RTL_W8(TDFNR, 0x1);

	RTL_W8(Config2, RTL_R8(Config2) & ~PMSTS_En);

	RTL_W8(0xD0, RTL_R8(0xD0) | _BIT6);
	RTL_W8(0xF2, RTL_R8(0xF2) | _BIT6);

	RTL_W8(0xD0, RTL_R8(0xD0) | _BIT7);

	rtl8168_eri_write(ioaddr, 0xC0, 2, 0x0000, ERIAR_ExGMAC);
	rtl8168_eri_write(ioaddr, 0xB8, 4, 0x00000000, ERIAR_ExGMAC);

	rtl8168_eri_write(ioaddr, 0x5F0, 2, 0x4f87, ERIAR_ExGMAC);

	csi_tmp = rtl8168_eri_read(ioaddr, 0xD4, 4, ERIAR_ExGMAC);
	csi_tmp  |= ( _BIT7 | _BIT8 | _BIT9 | _BIT10 | _BIT11 | _BIT12 );
	rtl8168_eri_write(ioaddr, 0xD4, 4, csi_tmp, ERIAR_ExGMAC);

	csi_tmp = rtl8168_eri_read(ioaddr, 0x1B0, 4, ERIAR_ExGMAC);
	csi_tmp &= ~_BIT12;
	rtl8168_eri_write(ioaddr, 0x1B0, 4, csi_tmp, ERIAR_ExGMAC);

	csi_tmp = rtl8168_eri_read(ioaddr, 0x2FC, 1, ERIAR_ExGMAC);
	csi_tmp &= ~(_BIT0 | _BIT1 | _BIT2);
	csi_tmp |= _BIT0;
	rtl8168_eri_write(ioaddr, 0x2FC, 1, csi_tmp, ERIAR_ExGMAC);

	//csi_tmp = rtl8168_eri_read(ioaddr, 0x1D0, 1, ERIAR_ExGMAC);
	//csi_tmp |= _BIT1;
	//rtl8168_eri_write(ioaddr, 0x1D0, 1, csi_tmp, ERIAR_ExGMAC);
	//other hw parameters
	rtl8168_eri_write(ioaddr, 0x2F8, 2, 0x1D8F, ERIAR_ExGMAC);

	RTL_W16(CPlusCmd, tp->cp_cmd);

	RTL_W16(RxMaxSize, tp->rx_buf_sz);

	//rtl8168_disable_EEE(tp);

	rtl8168_disable_rxdvgate(tp);

	/* Set Rx packet filter */
	rtl8168_hw_set_rx_packet_filter(tp);
	
	RTL_W16(IntrMask, 0x00ff);
#ifdef QC_ETN_DEBUG
	//printk("IntrMask = %08x\n", RTL_R32(IntrMask));
#endif

	RTL_W8(Cfg9346, Cfg9346_Lock);

	wmb();

	//RTL_W8(ChipCmd, CmdTxEnb | CmdRxEnb);
	//RTL_W8(ChipCmd, CmdRxEnb);

	wmb();

	udelay(10);
}

static int rtl8168_init(struct rtl8168_private *tp)
{
#ifdef QC_ETN_DEBUG
	printf ("%s\n", __FUNCTION__);
#endif

	tp->DescArrays = (u8 *)malloc_noncache(R8168_TX_RING_BYTES + R8168_RX_RING_BYTES + ALIGN_256);
	if (!tp->DescArrays) {
		printf("Can not allocate memory of r8168\n");
		return -ENOMEM;
	}

	tp->RxBufferRings = (u8 *)memalign(64, NUM_RX_DESC * tp->rx_buf_sz + ALIGN_8);
	if (!tp->RxBufferRings) {
		printf("Can not allocate memory of r8168\n");
		free(tp->DescArrays);
		tp->DescArrays = 0;
		return -ENOMEM;
	}

	/* Tx Descriptors needs 256 bytes alignment; */
	tp->TxDescArray = (struct TxDesc *)((unsigned long)(tp->DescArrays +
				ALIGN_256) & ~ALIGN_256);

	/* Rx Descriptors needs 256 bytes alignment; */
	tp->RxDescArray = (struct RxDesc *)((u8 *)tp->TxDescArray +
			R8168_TX_RING_BYTES);

	rtl8168_init_ring(tp);
	rtl8168_tally_counter_clear(tp);
	rtl8168_hw_init(tp);
	rtl8168_hw_reset(tp); 
	rtl8168_powerup_pll(tp);
	rtl8168_hw_start(tp);

	//    mdio_write(tp, 0x1f, 0x0000);
	//    printf("PHY status: %08x, %08x\n", mdio_read(tp, 0x00), RTL_R8(0x6c));

	return 0;
}

static void rtl8168_hw_d3_para(struct rtl8168_private *tp)
{
	u32 ioaddr = tp->mmio_addr;

	RTL_W16(RxMaxSize, 0);

	RTL_W8(0xF1, RTL_R8(0xF1) & ~_BIT7);
	RTL_W8(Cfg9346, Cfg9346_Unlock);
	RTL_W8(Config2, RTL_R8(Config2) & ~_BIT7);
	RTL_W8(Config5, RTL_R8(Config5) & ~_BIT0);
	RTL_W8(Cfg9346, Cfg9346_Lock);

	rtl8168_eri_write(ioaddr, 0x2F8, 2, 0x0064, ERIAR_ExGMAC);

	/*disable ocp phy power saving*/
	mdio_write(tp, 0x1F, 0x0C41);
	mdio_write(tp, 0x13, 0x0000);
	mdio_write(tp, 0x13, 0x0500);
	mdio_write(tp, 0x1F, 0x0000);
	rtl8168_disable_rxdvgate(tp);
}

static void rtl8168_down(struct rtl8168_private *tp)
{
	rtl8168_hw_reset(tp);

	rtl8168_hw_d3_para(tp);

	if (tp->RxBufferRings) {
		free(tp->RxBufferRings);
		tp->RxBufferRings = NULL;
	}

	if (tp->DescArrays) {
		tp->DescArrays = NULL;
	}
}

static void rtl8168_halt(struct rtl8168_private *tp)
{
#ifdef QC_ETN_DEBUG
	printf ("%s\n", __FUNCTION__);
#endif
    
	rtl8168_down(tp);
}

#ifdef QC_ETN_DEBUG
static void print_packet( u8 * buf, int length )
{
	int i;
	int remainder;
	int lines;

	printf("Packet of length %d \n", length );

	lines = length / 16;
	remainder = length % 16;

	for ( i = 0; i < lines ; i ++ ) {
		int cur;

		for ( cur = 0; cur < 8; cur ++ ) {
			u8 a, b;

			a = *(buf ++ );
			b = *(buf ++ );
			printf("%02x%02x ", a, b );
		}
		printf("\n");
	}
	for ( i = 0; i < remainder/2 ; i++ ) {
		u8 a, b;

		a = *(buf ++ );
		b = *(buf ++ );
		printf("%02x%02x ", a, b );
	}
	printf("\n");
}
#endif

static int rtl8168_send(struct rtl8168_private *tp, void *packet, int length)
{
	/* send the packet to destination */
	u32 ioaddr = tp->mmio_addr;
	u32 to, len = length;
	int ret, cur_tx; //dx=0;

	//u8 *testpacket;
	//int a,Temp=0;
	//u8 val=0;

#ifdef QC_ETN_DEBUG
	printf ("%s\n", __FUNCTION__);
	printf("sending %d bytes, cur_tx=%u\n"
	        ,len
	        ,tp->cur_tx);
#endif

	cur_tx = tp->cur_tx;

#ifdef QC_ETN_DEBUG
	u8 *buf;
	buf = (u8 *) packet;

	//printf("pointer %x\n",packet);
	print_packet(buf, length);
#endif

	flush_cache((unsigned long)packet,length);
	tp->TxDescArray[cur_tx].addr = cpu_to_le64((unsigned int)packet);

	if (cur_tx == (NUM_TX_DESC - 1)) {
		tp->TxDescArray[cur_tx].opts1 =
			cpu_to_le32((DescOwn | RingEnd | FirstFrag | LastFrag) |
					((len > ETH_ZLEN) ? len : ETH_ZLEN));
	} else {
		tp->TxDescArray[cur_tx].opts1 =
			cpu_to_le32((DescOwn | FirstFrag | LastFrag) |
					((len > ETH_ZLEN) ? len : ETH_ZLEN));
	}

	wmb();

	RTL_W8(ChipCmd, CmdTxEnb | CmdRxEnb);
	RTL_W8(TxPoll, NPQ);    /* set polling bit */

	tp->cur_tx = (cur_tx+1) % NUM_TX_DESC;
	to = currticks() + TX_TIMEOUT;
	do {
		RTL_W8(TxPoll, NPQ);
	} while ((le32_to_cpu(tp->TxDescArray[cur_tx].opts1) & DescOwn)
			&& (currticks() < to));	/* wait */

	if (currticks() >= to) {
		printf("tx timeout/error\n");
		ret = -1;
	} else {
	    ETH_DBG("tx done irq status=0x%x\n",RTL_R16(IntrStatus));
		ret = length;
	}

	/* Delay to make net console (nc) work properly */
	//udelay(20);
	return ret;
}

static int rtl8168_recv(struct rtl8168_private *tp)
{
    /* return true if there's an ethernet packet ready to read */
    /* nic->packet should contain data on return */
    /* nic->packetlen should contain length of data */
    u32 ioaddr = tp->mmio_addr;
    int cur_rx;
    int length = 0;
    u16 sts = RTL_R16(IntrStatus);
    int count =0;
    int i = 0, ret = 0;
    u32 addr,size;
#ifdef QC_ETN_DEBUG
    printf ("%s\n", __FUNCTION__);
    // printf("IntrStatus %x \n",IntrStatus);
#endif

    RTL_W16(IntrStatus, sts);

    rmb();

    // if ((sts & (RxOK)))
    //     printf("RxOK %x \n",sts);
    //  if ((sts & (RxDescUnavail | RxErr | RxOK)) == 0)
    //      return 0;
    //if (sts & (RxDescUnavail)) {
    //    printf("error: RX desc. unavailable ,status = 0x%x \n", sts);
    //}
    //
    //if (sts & (RxErr)) {
    //    printf("error: RX error ,status = 0x%x \n", sts);
    //}
    //
    //if (sts & (RxFIFOOver)) {
    //    printf("error: RX FIFO full ,status = 0x%x \n", sts);
    //    rtl8168_init_ring(dev);
    //    rtl8168_hw_start(tp);
    //
    //    return 0;
    //}


    cur_rx = tp->cur_rx;

    // while(1) {
    addr=(u32)(&tp->RxDescArray[cur_rx]);
    addr &= ~(ARCH_ALIGN - 1);
    size = roundup(sizeof(struct RxDesc), ARCH_ALIGN);

    //printf ("RX cur_rx = %X opts1 = %X \n", cur_rx, le32_to_cpu(tp->RxDescArray[cur_rx].opts1));

    if ((le32_to_cpu(tp->RxDescArray[cur_rx].opts1) & DescOwn) == 0) {

        count = count +1;

        if (!(le32_to_cpu(tp->RxDescArray[cur_rx].opts1) & RxRES)) {
            length = (int) (le32_to_cpu(tp->RxDescArray[cur_rx].
                        opts1) & 0x00003fff) - 4;


            invalidate_dcache_range((unsigned long)tp->RxBufferRing[cur_rx],(unsigned long)tp->RxBufferRing[cur_rx]+length);
            dmb();
#ifdef QC_ETN_DEBUG
            print_packet(tp->RxBufferRing[cur_rx], length);
#endif
            if (length != LAN_QC_VERIFY_LEN) {
                puts("Error:length not correct\n");
                return -2;
            }

            if (memcmp(tp->RxBufferRing[cur_rx], txbuffer, length) != 0) {
                puts("Error:content not equal\n");
                return -2;
            }
        } else {
            puts("Error Rx");
            ret = -1;
        }
        /*
         * Free the current buffer, restart the engine and move forward
         * to the next buffer. Here we check if the whole cacheline of
         * descriptors was already processed and if so, we mark it free
         * as whole.
         */
        size = _RXDESC_PER_CACHELINE - 1;
        if ((cur_rx & size) == size)
        {
            for (i = cur_rx - size; i <= cur_rx ; i++) {
                if (i == NUM_RX_DESC - 1){
                    tp->RxDescArray[i].opts1 =
                        cpu_to_le32(DescOwn | RingEnd | tp->rx_buf_sz);
                }
                else{
                    tp->RxDescArray[i].opts1 =
                        cpu_to_le32(DescOwn | tp->rx_buf_sz);
                }
            }

        }
        cur_rx = (cur_rx + 1) % NUM_RX_DESC;
        tp->cur_rx = cur_rx;

    } else {
        printf ("NetReceive %d packets\n",count);
        if (count == 0)
            return -1;
        return count;
    }

    return ret;     /* initially as this is called to flush the input */
}

static int rtl8168_recv_count(struct rtl8168_private *tp)
{
	/* return true if there's an ethernet packet ready to read */
	/* nic->packet should contain data on return */
	/* nic->packetlen should contain length of data */
	u32 ioaddr = tp->mmio_addr;
	int cur_rx;
	int length = 0;
	int count =0;
	int i = 0, ret = 0;
	u16 sts = RTL_R16(IntrStatus);
	u32 addr,size;
	ETH_DBG("%s IntrStatus=0x%x\n", __FUNCTION__,IntrStatus);

    RTL_W16(IntrStatus, sts);
	rmb();

#ifdef QC_ETN_DEBUG
    if ((sts & (RxOK)))
         printf("RxOK %x \n",sts);
    if ((sts & (RxDescUnavail | RxErr | RxOK)) == 0)
          return 0;
    if (sts & (RxDescUnavail)) {
        printf("error: RX desc. unavailable ,status = 0x%x \n", sts);
    }

    if (sts & (RxErr)) {
        printf("error: RX error ,status = 0x%x \n", sts);
    }

    if (sts & (RxFIFOOver)) {
        printf("error: RX FIFO full ,status = 0x%x \n", sts);
        rtl8168_init_ring(dev);
        rtl8168_hw_start(tp);

        return 0;
    }
#endif

	cur_rx = tp->cur_rx;

	// while(1) {
	addr=(u32)(&tp->RxDescArray[cur_rx]);
	addr &= ~(ARCH_ALIGN - 1);
	size = roundup(sizeof(struct RxDesc), ARCH_ALIGN);

	//printf ("RX cur_rx = %X opts1 = %X \n", cur_rx, le32_to_cpu(tp->RxDescArray[cur_rx].opts1));

	if ((le32_to_cpu(tp->RxDescArray[cur_rx].opts1) & DescOwn) == 0) {

		if (!(le32_to_cpu(tp->RxDescArray[cur_rx].opts1) & RxRES)) {
		    count = count +1;
		    tp->rx_count++;
		    ETH_DBG("rx count=%d\n",tp->rx_count);
			length = (int) (le32_to_cpu(tp->RxDescArray[cur_rx].
						opts1) & 0x00003fff) - 4;


			invalidate_dcache_range((unsigned long)tp->RxBufferRing[cur_rx],(unsigned long)tp->RxBufferRing[cur_rx]+length);
			dmb();
#ifdef QC_ETN_DEBUG
			printf("recv packet cur_rx=%u len=%u\n",cur_rx,length);
			print_packet(tp->RxBufferRing[cur_rx], length);
#endif
			if (length != LAN_QC_VERIFY_LEN) {
			    printf("Rx Error:length is not correct\n");
				return -2;
			}
				
			if (memcmp(tp->RxBufferRing[cur_rx], txbuffer, length) != 0) {
			    printf("Rx Error:content is not the same\n");
				return -2;		
			}	
		}
		else {
		    printf("Rx Error: RES cur_rx=%u opts1=0x%x opts2=0x%x addr=0x%llx\n"
		                ,cur_rx
		                ,le32_to_cpu(tp->RxDescArray[cur_rx].opts1)
		                ,le32_to_cpu(tp->RxDescArray[cur_rx].opts2)
		                ,(unsigned long)tp->RxBufferRing[cur_rx]);
			return -3;
		}
		/*
		 * Free the current buffer, restart the engine and move forward
		 * to the next buffer. Here we check if the whole cacheline of
		 * descriptors was already processed and if so, we mark it free
		 * as whole.
		 */		 		 
		size = _RXDESC_PER_CACHELINE - 1;
		if ((cur_rx & size) == size)		
		{	 		   
			for (i = cur_rx - size; i <= cur_rx ; i++) {
				if (i == NUM_RX_DESC - 1){
					tp->RxDescArray[i].opts1 =
						cpu_to_le32(DescOwn | RingEnd | tp->rx_buf_sz);	
				}
				else{
					tp->RxDescArray[i].opts1 =
						cpu_to_le32(DescOwn | tp->rx_buf_sz);					
				}
			}

		}		
		cur_rx = (cur_rx + 1) % NUM_RX_DESC;
		tp->cur_rx = cur_rx;
	}
//	else {
//		printf("NetReceive %d packets, IntrStatus =0x%x\n"
//		        ,count
//		        ,RTL_R16(IntrStatus));
//		return count;
//	}

	return count;		/* initially as this is called to flush the input */
}


unsigned int ABS(int value)
{
   if (0 > value)
      return(0 - value);
   else
      return(value);
}

#if 0
static void ETN_MDIO_write(struct rtl8168_private *tp, u32 page, u32 RegAddr, u32 value)
{
	mdio_write(tp, 0x1F, page);
	mdio_write(tp, RegAddr, value);
}


static u32 ETN_MDIO_read(struct rtl8168_private *tp, u32 page , u32 RegAddr)
{
	mdio_write(tp, 0x1F, page);
	return mdio_read(tp,RegAddr);

}


static u32 ETN_MDIO_read_from_current_page(struct rtl8168_private *tp,unsigned int reg_addr)
{
   //unsigned int wdata;
   //unsigned int rdata;

   return mdio_fast_read(tp,reg_addr);

}
#endif

int polling_prog_status(void)
{
	int i;
	unsigned int status;
	
	for(i = 0 ; i < 1000 ; ++i)
	{
		if ( (status = (rtd_inl(0xb8038804) & 0x07)))
		{
			if (status == 1)
			{
				printk("status success...\n");
				break;
			}
			else
			{
				printk("status fail %x\n",status);
				return -1;
			}
		}
		else
			continue;
		
	}

	if (i == 1000)
	{
		printk("status polling timeout\n");
		return -1;
	}

	return 0;
}


void Efuse_Write(unsigned int adc)
{

//	rtd_outl(0xB8000900,0x00000230);       //set debug pin 
//	rtd_outl(0xB8000848,0xe0e0e000);       //set debug pin 
//	rtd_outl(0xB8038834,0x00000007);       //set debug pin 

	rtd_outl(0xB8038820,0xa5a50000);       //set program start initial 

	//------------------------------------------------------------------------
	// OTP field name : GPHY_PAR                        
	// OTP bit : [991:928]
	// Rbus addr : 0xb8038078[31]~ 0xb8038074[0]

	//------------------------------------------------------------------------

	rtd_outl(0xb8038074,adc << 16);       //program bit ,ADC bit[31:16]
	rtd_outl(0xB8038800,0x0000000f);       //trigger program enable and repair enable 
	//-------polling(pad_gpio76)-------    //polling pad_gpio76 from 1 falling to 0 

	if (polling_prog_status())
		return;

//	printk("status=%x\n",rtd_inl(0xB8038804));                   //check program_status, bit0:success, bit1:fail, bit2:lock

	//------------------------------------------------------------------------
	// OTP field name : double_GPHY_PAR                        
	// OTP bit : [16351:16288]
	// Rbus addr : 0xb80387F8[31]~ 0xb80387F4[0]

	//------------------------------------------------------------------------

	rtd_outl(0xb80387f4,adc << 16);       //program bit, ADC bit[31:16]
	rtd_outl(0xB8038800,0x0000000f);       //trigger program enable and repair enable 
	//-------polling(pad_gpio76)-------    //polling pad_gpio76 from 1 falling to 0 

	if (polling_prog_status())
                return;

//	printk("status=%x\n",rtd_inl(0xB8038804));                   //check program_status, bit0:success, bit1:fail, bit2:lock 

	//------------------------------------------------------------------------

	rtd_outl(0xB8038820,0x00000000);       //close program start initial 

}


//------------------------------------------------------------------------
// OTP field name : GPHY_PAR                        
// OTP bit : [991:928]
// Rbus addr : 0xb8038078[31]~ 0xb8038074[0]
//------------------------------------------------------------------------
// OTP field name : double_GPHY_PAR                        
// OTP bit : [16351:16288]
// Rbus addr : 0xb80387F8[31]~ 0xb80387F4[0]
//------------------------------------------------------------------------

int Efuse_Read(unsigned int adc)
{
	//int result = 0;

	rtd_outl(0xB8038814,0x00000008);       // set normal read enable

	if(!(((rtd_inl(0xb8038074) >> 16) == adc) && ((rtd_inl(0xb80387F4) >> 16) == adc)))
	{
		printk("Efuse normal read not equal\n");
                printk("adc=%x %x %x\n",adc,rtd_inl(0xb8038074),rtd_inl(0xb80387f4));
		return -1;
	}


	rtd_outl(0xB8038814,0x0000000C);       // set margin read enable

	
	if(!(((rtd_inl(0xb8038074) >> 16) == adc) && ((rtd_inl(0xb80387F4) >> 16) == adc)))
        {
                printk("Efuse margin read not equal\n");
		printk("adc=%x %x %x\n",adc,rtd_inl(0xb8038074),rtd_inl(0xb80387f4));
		return -1;
        }
	return 0;

}

// 1: already burned , 0 not burned
int Efuse_done_bit_status(void)
{
	if ((rtd_inl(0xb803807c) & 0x00000800) || (rtd_inl(0xb80387fc) & 0x00000800))
        {
                return 1;
        }

        rtd_outl(0xB8038814,0x0000000C);       //set margin read enable

        if ((rtd_inl(0xb803807c) & 0x00000800) || (rtd_inl(0xb80387fc) & 0x00000800))
        {
                return 1;
        }

	return 0;

}

//etn script
//0x1803807C[8]   GPHY_PAR_done
//0x180387FC[8]   double_GPHY_PAR_done

//firmware
//0x1803807C[11]	GPHY_PAR_done_2
//0x180387FC[11]	double_GPHY_PAR_done_2


int Efuse_write_done_bit(void)
{

	rtd_outl(0xB8038814,0x00000008);       //set normal read enable

	if ((rtd_inl(0xb803807c) & 0x00000800) || (rtd_inl(0xb80387fc) & 0x00000800))
	{
		printk("error: normal read done bit not zero\n");
		return -1;
	}

	rtd_outl(0xB8038814,0x0000000C);       //set margin read enable

        if ((rtd_inl(0xb803807c) & 0x00000800) || (rtd_inl(0xb80387fc) & 0x00000800))
        {
                printk("error: margin read done bit not zero\n");
		return -1;
        }
	
	//------------------------------------------------------------------------
//	rtd_outl(0xB8000900,0x00000230);       //set debug pin 
//	rtd_outl(0xB8000848,0xe0e0e000);       //set debug pin 
//	rtd_outl(0xB8038834,0x00000007);       //set debug pin 

	rtd_outl(0xB8038820,0xa5a50000);       //set program start initial


	rtd_outl(0xb803807C,0x00000800);       //program bit 
	rtd_outl(0xB8038800,0x0000000f);       //trigger program enable and repair enable 
	//-------polling(pad_gpio76)-------    //polling pad_gpio76 from 1 falling to 0 
	if (polling_prog_status())
        	return -1;

//	rtd_inl(0xB8038804);                   //check program_status, bit0:success, bit1:fail, bit2:lock check status after burn

	rtd_outl(0xb80387FC,0x00000800);       //program bit 
	rtd_outl(0xB8038800,0x0000000f);       //trigger program enable and repair enable 
	//-------polling(pad_gpio76)-------    //polling pad_gpio76 from 1 falling to 0 
        if (polling_prog_status())
                return -1;

//	rtd_inl(0xB8038804);                   //check program_status, bit0:success, bit1:fail, bit2:lock

	rtd_outl(0xB8038820,0x00000000);       //close program start initial 

	//------------------------------------------------------------------------

	rtd_outl(0xB8038814,0x00000008);       //set normal read enable

        if (!((rtd_inl(0xb803807c) & 0x00000800) && (rtd_inl(0xb80387fc) & 0x00000800)))
        {
                printk("error: normal read done bit not one\n");
		return -1;
        }


	rtd_outl(0xB8038814,0x0000000C);       //set margin read enable

	if (!((rtd_inl(0xb803807c) & 0x00000800) && (rtd_inl(0xb80387fc) & 0x00000800)))
        {
                printk("error: margin read done bit not one\n");
		return -1;
        }

	return 0;
}

#ifdef BIST_TEST

static int mbist_test(void)
{

	int result = 0;
// Wait init_autoload_done

  // ##########################
  // ##### from ip_init.c #####
  // ##########################


  rtd_outl(0xb8016050, 0x000000c0);
  rtd_outl(0xb8016050, 0x00000400);
  rtd_outl(0xb8016050, 0x00000000);
  rtd_outl(0xb8016050, 0x3d000000);
  rtd_outl(0xb8016054, 0x00000026);
  rtd_outl(0xb8016054, 0x00000000);
  rtd_outl(0xb8016054, 0x00020000);
  rtd_outl(0xb801600c, 0xffffffff);
  rtd_outl(0xb8016008, 0xffffffff);
  rtd_outl(0xb8016000, 0x004c5452);
  rtd_outl(0xb8016004, 0x00000800);
  rtd_outl(0xb8016070, 0x54520000);
  rtd_outl(0xb8016074, 0x8000c0f0);
//  rtd_inl(0xb8016074, 0x0000c0f0);
  rtd_outl(0xb8016070, 0x0800004c);
  rtd_outl(0xb8016074, 0x8000f0f4);
//  rtd_inl(0xb8016074, 0x0000f0f4);
  rtd_outl(0xb8016020, 0x00010000);
  rtd_outl(0xb8016024, 0x00000000);
  rtd_outl(0xb8016028, 0x00020000);
  rtd_outl(0xb801602c, 0x00000000);
  rtd_outl(0xb8016070, 0x00030000);
  rtd_outl(0xb8016074, 0x8000f3a0);
//  rtd_inl(0xb8016074, 0x0000f3a0);
  rtd_outl(0xb8016070, 0x00000000);
  rtd_outl(0xb8016074, 0x8000f3a4);
//  rtd_inl(0xb8016074, 0x0000f3a4);
  rtd_outl(0xb8016070, 0x00040000);
  rtd_outl(0xb8016074, 0x8000f3a8);
//  rtd_inl(0xb8016074, 0x0000f3a8);
  rtd_outl(0xb8016070, 0x00000000);
  rtd_outl(0xb8016074, 0x8000f3ac);
//  rtd_inl(0xb8016074, 0x0000f3ac);
  rtd_outl(0xb8016054, 0x0400f3ac);
  rtd_outl(0xb80160e4, 0x00080000);
  rtd_outl(0xb80160e8, 0x00000000);
  rtd_outl(0xb801603c, 0x0000ffff);
//  rtd_inl(0xb801603c, 0x0000ffff);
  rtd_outl(0xb80160e0, 0x00002c61);
  rtd_outl(0xb801603c, 0x00000000);
  rtd_outl(0xb80160d8, 0x30000000);
//  rtd_inl(0xb801606c, 0xf0b00084);
  rtd_outl(0xb80160ec, 0x0000000f);
  rtd_outl(0xb801603c, 0x00200000);
  rtd_outl(0xb8016070, 0x00080002);
//  rtd_inl(0xb801603c, 0x00000000);
  rtd_outl(0xb8016074, 0x8000f0c8);
  rtd_outl(0xb801603c, 0x0000ffff);
//  rtd_inl(0xb8016074, 0x0000f0c8);
  rtd_outl(0xb8016070, 0x02000038);
  rtd_outl(0xb8016074, 0x8000f0cc);
//  rtd_inl(0xb8016074, 0x0000f0cc);
  rtd_outl(0xb8016070, 0x00000048);
  rtd_outl(0xb8016074, 0x8000f0d0);
//  rtd_inl(0xb8016074, 0x0000f0d0);
  rtd_outl(0xb8016070, 0x00100006);
  rtd_outl(0xb8016074, 0x8000f0e8);
//  rtd_inl(0xb8016074, 0x0000f0e8);
  rtd_outl(0xb8016074, 0x0000f0dc);
//  rtd_inl(0xb8016074, 0x8000f0dc);
//  rtd_inl(0xb8016070, 0x001f0021);
  rtd_outl(0xb8016070, 0x001f000d);
  rtd_outl(0xb8016074, 0x8000f0dc);
//  rtd_inl(0xb8016074, 0x0000f0dc);
  rtd_outl(0xb8016040, 0x03000480);
//  rtd_inl(0xb80160d0, 0x32000021);
//  rtd_inl(0xb8016044, 0x0002480f);
  rtd_outl(0xb8016044, 0x0002080e);
//  rtd_inl(0xb8016034, 0x00000000);
  rtd_outl(0xb8016034, 0x8c02080e);
  rtd_outl(0xb8016034, 0x0c000000);
  rtd_outl(0xb8016044, 0x0002080e);

  // #######################
  // ##### from case.c #####
  // #######################
  // ### back-door register setting ###

//  rtd_inl(0xb8016c14, 0x00000000);
  rtd_outl(0xb8016c14, 0x00000004);
  // ### mbist setting ###
  rtd_outl(0xb8016000, 0x00006001);
  rtd_outl(0xb8016004, 0x0000ea06);
  rtd_outl(0xb8016000, 0x00036001);
  rtd_outl(0xb8016004, 0x0000ea20);
  rtd_outl(0xb8016000, 0x08006001);
  rtd_outl(0xb8016004, 0x0000ea04);
  rtd_outl(0xb8016000, 0x00086001);
  rtd_outl(0xb8016004, 0x0000ea02);
  rtd_outl(0xb8016000, 0x08006001);
  rtd_outl(0xb8016004, 0x0000ea00);
  rtd_outl(0xb8016000, 0x00086001);
  rtd_outl(0xb8016004, 0x0000ea04);
  rtd_outl(0xb8016000, 0x08086001);
  rtd_outl(0xb8016004, 0x0000ea02);
  rtd_outl(0xb8016000, 0x08046001);
  rtd_outl(0xb8016004, 0x0000ea00);
  rtd_outl(0xb8016000, 0x04086001);

  udelay(1000);
 
  // ### wait all bist done ###
  // add delay here
  // ### check register ###
  // sram
  rtd_outl(0xb8016004, 0x0000ea20);
  rtd_outl(0xb8016000, 0x00004001);
//  printk("%x\n",rtd_inl(0xb8016000));     // 0x09000005
  
  if (rtd_inl(0xb8016000) != 0x09000005)
	result = 1; 



  rtd_outl(0xb8016004, 0x0000ea04);
  rtd_outl(0xb8016000, 0x00004001);
//  printk("%x\n",rtd_inl(0xb8016000));     // 0x09090005
 
  if (rtd_inl(0xb8016000) != 0x09090005)
	result = 1;

 rtd_outl(0xb8016004, 0x0000ea02);
  rtd_outl(0xb8016000, 0x00004001);
//  printk("%x\n",rtd_inl(0xb8016000));     // 0x09050005
 
  if (rtd_inl(0xb8016000) != 0x09050005)
	result = 1;

 rtd_outl(0xb8016004, 0x0000ea00);
  rtd_outl(0xb8016000, 0x00004001);
//  printk("%x\n",rtd_inl(0xb8016000));     // 0x05090005


  if (rtd_inl(0xb8016000) != 0x05090005) 
	 result = 1;


  // ROM
  rtd_outl(0xb8016004, 0x0000ea14);
  rtd_outl(0xb8016000, 0x00004001);
//  printk("%x\n",rtd_inl(0xb8016000));    //0xd0510005
 
  if (rtd_inl(0xb8016000) != 0xd0510005)
	result = 1;

 rtd_outl(0xb8016004, 0x0000ea16);
  rtd_outl(0xb8016000, 0x00004001);
//  printk("%x\n",rtd_inl(0xb8016000));    //0xd61e0005


  if (rtd_inl(0xb8016000) != 0xd61e0005)
	result = 1;

  rtd_outl(0xb8016004, 0x0000ea10);
  rtd_outl(0xb8016000, 0x00004001);
//  printk("%x\n",rtd_inl(0xb8016000));    //8c4d0005

  if (rtd_inl(0xb8016000) != 0x8c4d0005)
	result = 1;



  rtd_outl(0xb8016004, 0x0000ea12);
  rtd_outl(0xb8016000, 0x00004001);
//  printk("%x\n",rtd_inl(0xb8016000));    //576a0005
  if (rtd_inl(0xb8016000) != 0x576a0005)
	result = 1;


  rtd_outl(0xb8016004, 0x0000ea0c);
  rtd_outl(0xb8016000, 0x00004001);
//  printk("%x\n",rtd_inl(0xb8016000));    //0x03820005

  if (rtd_inl(0xb8016000) != 0x03820005)
    result = 1;




  rtd_outl(0xb8016004, 0x0000ea0e);
  rtd_outl(0xb8016000, 0x00004001);
//  printk("%x\n",rtd_inl(0xb8016000));    //0x5cf70005

  if (rtd_inl(0xb8016000) != 0x5cf70005)
	result = 1;

#if (CONFIG_Merlin6_ETN)
	//Merlin6_Standby SPEC
	
	//rtd_maskl(0xb8060010,0xbfffffff,0x00000000);
	rtd_outl(0xb8060010,0x90000042);   // GETN_RSTN_MASK bit[30]=0,ETN_LDO_EN bit[6]=1	  
	//rtd_outl(0xb8060010,0x90000002);	  // GETN_RSTN_MASK bit[30]=0,ETN_LDO_EN bit[6]=0
	//ScalerTimer_DelayXms(5);
	rtd_outl(0xb8060040,0x00000901);	// CLKEN_ETN_RBUS bit[11]=1,CLKEN_GETN bit[8]=1
	//ScalerTimer_DelayXms(5);
	rtd_outl(0xb8060030,0x00000101);	// RSTN_ETNbit[8]=1
	//ScalerTimer_DelayXms(5);
	rtd_outl(0xb8016c04,0x00000006);	//CLKEN_ETN_MAC bit[2]=0,CLKEN_ETNSYS bit[1]=0
	//ScalerTimer_DelayXms(5);
	rtd_outl(0xb8060040,0x00000900);	// CLKEN_ETN_RBUS bit[11]=0,CLKEN_ETN bit[8]=0
	//ScalerTimer_DelayXms(5);
	rtd_outl(0xb8060030,0x00000100);	// RSTN_ETN bit[8] = 0
	//ScalerTimer_DelayXms(5);
	rtd_outl(0xb8060030,0x00000101);	// RSTN_ETN bit[8] = 1
	//ScalerTimer_DelayXms(5);
	rtd_outl(0xb8060040,0x00000901);	// CLKEN_ETNRBUS bit[11]=1,CLKEN_ETN bit[8]=1
	//ScalerTimer_DelayXms(5);
	rtd_outl(0xb8016c00,0x0000000f);	//RSTN_ETNPHY bit[3]=1,RSTN_ETNMAC bit[2]=1,RSTN_ETNSYS bit[1]=1
	//ScalerTimer_DelayXms(5);
	rtd_outl(0xb8016c04,0x00000007);	//CLKEN_ETNMAC bit[2]=1,CLKEN_ETNSYS bit[1]=1
	//ScalerTimer_DelayXms(5);
	//rtd_outl(0xb8016c14,0x00000000);	   //SEL_XTAL27M_CEN_L bit[5]=0, 24MHz 
	//rtd_outl(0xb8016c14,0x00000020);	   //SEL_XTAL27M_CEN_L bit[5]=1, 27MHz
	//ScalerTimer_DelayXms(5);



	//  AFE Power state
	//rtd_outl(0xb8016c3c, 0x00000000);   // Afe_pow_state bit[29]=0
	//ScalerTimer_DelayXms(10);
	rtd_outl(0xb8016c3c, 0x20000005);  // por_avth_cen bit[1:0]=1, por_dvth_cen bit[3:2]=1, afe_pow_state bit[29]=1
	mdelay(1);

	//rtd_outl(0xb8016c18,0x00c00000);  //plletn_pow bit[23]=1,plletn_powldo bit[22]=1
	//ScalerTimer_DelayXms(50);

	rtd_inl(0xb8016c1c);    // init_autoload_done ,check bit[0]=1
	mdelay(5);

#else
/*************revocer etn link ****************************/

	rtd_maskl(0xb8060010,0xbfffffff,0x00000000);    // GETN_RSTN_MASK bit[30]=0
	udelay(5);
//	rtd_outl(0xb8060010,0x90000002);
//	udelay(500);	
	rtd_outl(0xb8060040,0x00000901);    // CLKEN_ETN_RBUS bit[11]=1,CLKEN_GETN bit[8]=1
	udelay(5);
	rtd_outl(0xb8060030,0x00000101);    // RSTN_ETNbit[8]=1
	udelay(5);
	rtd_outl(0xb8016c04,0x00000006);    //CLKEN_ETN_MAC bit[2]=0,CLKEN_ETNSYS bit[1]=0
	udelay(5);
	rtd_outl(0xb8060040,0x00000900);    // CLKEN_ETN_RBUS bit[11]=0,CLKEN_ETN bit[8]=0
	udelay(5);
	rtd_outl(0xb8060030,0x00000100);    // RSTN_ETN bit[8] = 0
	udelay(5);
	rtd_outl(0xb8060030,0x00000101);    // RSTN_ETN bit[8] = 1
	udelay(5);
	rtd_outl(0xb8060040,0x00000901);    // CLKEN_ETNRBUS bit[11]=1,CLKEN_ETN bit[8]=1
	udelay(5);
	rtd_outl(0xb8016c00,0x0000000f);    //RSTN_ETNPHY bit[3]=1,RSTN_ETNMAC bit[2]=1,RSTN_ETNSYS bit[1]=1,
	udelay(5);
	rtd_outl(0xb8016c04,0x00000007);    //CLKEN_ETNMAC bit[2]=1,CLKEN_ETNSYS bit[1]=1
	udelay(5);


//  AFE Power state
	rtd_outl(0xb8016c3c, 0x00000000);   // Afe_pow_state bit[29]=0
	udelay(10);
	rtd_outl(0xb8016c3c, 0x20000000);   // afe_pow_state bit[29]=1
#endif

  if (result)
	return 1;
  else
    return 0;
}


static int dphy_ram_bist(struct rtl8168_private *tp)
{

	int i = 0;
	int value = 0;



		/////   Mac5p Memory_RAM BIST Test  //////////////////////
	mdio_write(tp,0x1F,0x0a00);    // change Page a00
	mdio_write(tp,0x17,0x0000);	   //reg23 ,rg_ram_bist_rstn = 0
	mdio_write(tp,0x17,0x0020);	   //reg23 ,rg_ram_bist_rstn = 1

	mdio_write(tp,0x17,0x00a0);	   //reg23 ,rg_ram_bist_mode = 1


	for (i = 0 ; i < 20 ; ++i)
	{
		value = mdio_read(tp,0x17);
		if (value & 0x1000) {   //polling ram bist done[12] = 1 ,check ram bist[10] fail=0
			if (value & 0x0400) {
				printk("ram bist fail\n");
				return 1;
			}
			else
				break; //pass
		}
		udelay(50);
	}

    if (i == 20) {
        printk("ram bist timout \n");
		return 1;
	}

	mdio_write(tp,0x17,0x0020);  //reg23 ,rg_ram_bist_mode = 0
	mdio_write(tp,0x17,0x0000);  //reg23 ,rg_ram_bist_rstn = 0
	mdio_write(tp,0x17,0x0020);  //reg23 ,rg_ram_bist_rstn = 1
	mdio_write(tp,0x17,0x0120);  //reg23 , rg_ram_drf_bist_mode = 1


	for (i = 0 ; i < 20 ; ++i)
	{
		value = mdio_read(tp,0x17);
		if (value & 0x2000) {  //polling drf_ram_start_pause[13]
			break;
		}
		udelay(50);
	}

	if (i == 20) {
		printk("drf_ram_start_pause fail\n");
		return 1;
	}

	mdio_write(tp,0x17,0x0160); //rg_ram_drf_bist_resume = 1

	for (i = 0 ; i < 20 ; ++i)
	{
		value = mdio_read(tp,0x17);  //polling drf_ram_start_pause
		if (value & 0x2000) { //reg23 bit[13]=0x1
			break;
		}
		udelay(50);
	}

	if (i == 20) {
		printk("drf_ram_start_pause fail\n");
		return 1;
	}

	mdio_write(tp,0x17,0x0160);  //rg_ram_drf_bist_resume = 1


	for (i = 0 ; i < 20 ; ++i)
	{
		value = mdio_read(tp,0x17);  //polling drf_ram_done[11] = 1 ,check drf_ram bist[9] fail=0
		if (value & 0x0800) { //reg23 bit[11]=1
			if (value & 0x0200) { //bit[9] = 1
				printk("drm_ram bist fail\n");
				return 1;
			}
			else
				break; //pass
		}
		udelay(50);
	}


	mdio_write(tp,0x17,0x0020); //reg23 ,rg_ram_bist_mode = 0
	mdio_write(tp,0x17,0x0000); //reg23 ,rg_ram_bist_rstn = 0

	return 0;
}

int dphy_rom_bist(struct rtl8168_private *tp)
{

	unsigned int value = 0;
	int i = 0, result = 0;

	mdio_write(tp,0x1f,0x0a46);	   //// change Page a46

	value = mdio_read(tp,0x15);
	value = value | 0x0002;    //  reg 21,  set lock_main,bit[1]=1 
	mdio_write(tp,0x15,value);


	mdio_write(tp,0x1f,0x0a60);    /// change Page a60

	

	for (i = 0 ; i < 20 ; ++i)
    {
        value = mdio_read(tp,0x10);  // polling reg 16 NCTL enters MAIN state
        if (!(value & 0xff)) {       //  NCTL enters MAIN state bit[7:0]=0x00
                break;
        }
        
        udelay(50);
    }

	if (i == 20) {
		printf("timeout to enter MAIN state");
		return 1;

	}

	mdio_write(tp,0x1f,0x0d01);  // change Page d01
	mdio_write(tp,0x10,0x3133);  //// reg 16 ,set rg_nctl_rst = 1, rg_nctl_hold = 1


	mdio_write(tp,0x1f,0x0a00);  // change Page a00

	mdio_write(tp,0x14,0x0000);  // reg 20 ,set rg_rom_bist_rstn = 0
	mdio_write(tp,0x14,0x0020);  // reg 20 ,set rg_rom_bist_rstn = 1
	mdio_write(tp,0x14,0x0060);  // reg 20 ,set rg_rom_bist_mode = 1

	for (i = 0 ; i < 20 ; ++i)
    {
        value = mdio_read(tp,0x14);  // reg20 ,polling rom_bist_done =1
        if (value & 0x80) {  //rom bist done
                break;
        }

        udelay(50);
    }

	if (i == 20) {
		printf("timeout to polling rom bist done\n");
		return 1;
	}

	value = mdio_read(tp,0x15);
//	printf("value=%x\n",value);

//	if (!(value == 0x16f6)) // check misr15-0
    if (!(value == 0xc12f)) // check misr15-0
		result = 1;

	value = mdio_read(tp,0x16);  // check misr31-16
//	printf("value=%x\n",value);

//	if (!(value == 0x3617))
	if (!(value == 0xc314))
		result = 1;

	mdio_write(tp,0x14,0x0020);  //reg 20 ,set rg_rom_bist_mode = 0
	mdio_write(tp,0x14,0x0000);  // reg 20 ,set rg_rom_bist_rstn = 0

	return result;

}

#endif

//1. X=HEX2DEC(cb0)
//2. y=2363*X^2+606100*X+2966200
//3. length=(130*100000000-573*y/5)/100000000
#if CONFIG_FT
static int cable_len(struct rtl8168_private *tp)
{

    int x = 0, y = 0, len = 0;

    mdio_write(tp, 0x1F, 0x0AA1);
    x = mdio_read(tp,0x10);

    x = (x & 0xff00) >> 8;

//  printk("x=%d\n",x);

    y = 2363 * (x * x) + 606100 * x + 2966200;

//  printk("y1=%d\n",y);

    y = y/50000;

//  printk("y2=%d\n",y);

    len = 130 - (573 * y / 10000);

    printk("evaluate cable len=%d\n",len);

    mdio_write(tp, 0x1F, 0x0A43);

    return len;

}
#endif

static int  snr_test(struct rtl8168_private *tp)
{
	int snr_check_count = 0;
	int value = 0;
	mdio_write(tp, 0x1F, 0x0A43);
        mdio_write(tp, 0x18, 0xA398);

	mdelay(50);

	value = mdio_read(tp,0x1a);

	printk("link status %x\n",value);
	if (value & 0x4)
	{
		for (snr_check_count = 0 ; snr_check_count < 3 ; ++snr_check_count)
		{
			mdio_write(tp, 0x1f, 0x0aa0);
			value = mdio_read(tp,0x10);
			printk("snr = %x\n",value);
			//23.5db (0x492) --> 22db (0x677) the same as cn
			if (value >  0x677)
				return 1;
	
		}			
	}
	else
	{
		printk("error snr test no link\n");
		return 1;
	}

	mdio_write(tp, 0x1F, 0x0A43);
        mdio_write(tp, 0x18, 0x2198);

	return 0;

}

//eeep and pll enable/disable

static int EEEP_10M_test(struct rtl8168_private *tp, u8 mode)
{
	int res = 0;
	mdio_write(tp, 0x1F, 0xA44);
	mdio_write(tp, 0x15, 0x0002); //EEEP mode

	if (!rtl8168_check_phy_status(tp, _10bps)) {
		rtl8168_halt(tp);
		res = 1;
	}

	mdio_write(tp, 0x1F, 0xA44);
	mdio_write(tp, 0x15, 0x0000);

	if (res)
		return 1;
	else
		return 0;

}

static void enable_pll_10M(struct rtl8168_private *tp,int enable)
{
	mdio_write(tp, 0x1F, 0x0A43);
	if (enable) {
		mdio_write(tp, 0x18, 0x2199); //enable 10M DCO
	}
	else {
		mdio_write(tp, 0x18, 0x2198);
	}
}

#define  STB_ST_CLKEN0_reg_addr                                                  "0xB8060040"
#define  STB_ST_SRST0_reg_addr                                                   "0xB8060030"

int qc_rtl8168_initialize(struct rtl8168_private *tp)
{
	int count = 0;
	//clear GETN RBUS & GETN clock
	rtd_outl(STB_ST_CLKEN0_reg, 0x00000900);
	mdelay(10);
	rtd_outl(STB_ST_SRST0_reg, 0x00000100);
	mdelay(10);

	//enable GETN rbus clock
	rtd_outl(STB_ST_SRST0_reg, 0x00000801);
	
	//GETN clock reset flow
	rtd_outl(STB_ST_SRST0_reg, 0x00000101);
	mdelay(10);
	rtd_outl(STB_ST_SRST0_reg, 0x00000100);
	mdelay(10);
	rtd_outl(STB_ST_SRST0_reg, 0x00000101);
	mdelay(10);
	rtd_outl(STB_ST_SRST0_reg, 0x00000101);
	mdelay(10);

    /* PLL ETN */
	rtd_outl(0xb8016c00, 0x0000000e); //clear GETN bit
	mdelay(1);

	rtd_outl(0xb8016c00, 0x0000000f);  //RSTN_GETN_PHY bit[3],RSTN_GETN_MAC bit[2],RSTN_GETN_SYS[1]
	rtd_outl(0xb8016c04, 0x00000007);  //CLKEN_GETN_MAC bit[2],CLKEN_GETN_SYS[1]
	
	rtd_outl(0xb8016c18, 0x00c00000);  //plletn_pow bit[23],plletn_powldo bit[22]
    
	while(!(rtd_inl(0xb8016c1c) & _BIT0) && count < 500) {    // wait plletn init done
		mdelay(1);
		count++;
	}	

	if(count == 500) {
		printf("[ETH] initdone failed!(%08x)\n", rtd_inl(0xb8016c1c));
	}	

	    /* fuse ready */
	mdio_write(tp, 0x1f, 0x0a46);  //Change page to 0xa46
	mdio_write(tp, 0x14, 0x0003);  //set bit[0] fuse_rdy=1, set bit[1] ext_ini_done=1 , to skip ext_ini
#ifdef DEBUG_RTL8168
	printf("fuse ready = %04x\n", mdio_read(tp, 0x14)); //read fuse_rdy
#endif
	mdio_write(tp, 0x1F, 0x0000);
	return 0;
}

#ifdef LINKP_TEST
int linkpartner_test(struct rtl8168_private *tp,void *packet)
{
        int i = 0;

   //     printf("100M test\n");

//	  rtl8168_set_speed_duplex(tp, BMCR_ANENABLE, BMCR_FULLDPLX);
        rtl8168_set_speed_duplex(tp, 0x0, BMCR_FULLDPLX);
        if (!rtl8168_check_phy_status(tp, _10bps)) {
                rtl8168_halt(tp);
                printf("QC_test_100M failed\n");
                return FAILED;
        }

        mdelay(5000);
        printf("10M transmit\n");

//	memset((void *)packet, 0x55 , 10);
//	rtl8168_send(tp, packet, 10);

//	memset((void *)packet, 0x77 , 10);
//	rtl8168_send(tp, packet, 10);

        for (i = 0 ; i < 128 ; ++i) {
                printf("send data i=%d\n",i);
                memset((void *)packet, i , 64);  //0x00 0x10 0x20 0x30 0x40
                rtl8168_send(tp, packet, 64);
                rtl8168_recv_packet(tp);
                mdelay(100);
        }


}
#endif


#ifdef SMARTBIT_TEST
int smartbit_test(struct rtl8168_private *tp,void *packet)
{
	int i = 0;

	printf("100M test\n");

	rtl8168_set_speed_duplex(tp, BMCR_SPEED100, BMCR_FULLDPLX);
	if (!rtl8168_check_phy_status(tp, _100bps)) {
		rtl8168_halt(tp);
		printf("QC_test_100M failed\n");
		return FAILED;
	}

	mdelay(50);
	printf("100M transmit\n");

	for (i = 0 ; i < 5 ; ++i) {
		printf("send data %x\n",i*16);
		memset((void *)packet, i*16 , LAN_QC_VERIFY_LEN);  //0x00 0x10 0x20 0x30 0x40
		rtl8168_send(tp, packet, LAN_QC_VERIFY_LEN);
		mdelay(8000);
	}	


}
#endif

static int rtl8168_verify_statics(struct rtl8168_private *tp)
{
    u32 ioaddr  = tp->mmio_addr;
    GDUMP_TALLY* pdump = NULL;
    u32 tally   = 0;
    u32 i = 0;
    u32 value = 0;

    tally = RTL_R32(CounterAddrLow) & ~ALIGN_64;
    pdump = tally;
    ETH_DBG("tally phy verify=0x%x pdump=0x%x\n",tally,pdump);
    RTL_W32(CounterAddrHigh,0);
    tally = tally | CounterDump;
    ETH_DBG("tally phy dump=0x%x\n",tally);
    RTL_W32(CounterAddrLow,cpu_to_le32(tally));

    for(i=0;i<200;i++)
    {
        value = RTL_R32(CounterAddrLow) & 0x8;
        mdelay(10);
        if(!value)
        {
            ETH_DBG("tally dump complete\n");
            break;
        }
    }
    if(i>=200)
    {
        ETH_DBG("tally dump failed\n");
    }
    invalidate_dcache_range(dummy,dummy+sizeof( GDUMP_TALLY ) + ALIGN_64); 
	dmb();
    ETH_DBG("TxOK=%llu TxERR=%llu Tx1Col=%u TxAbt=%u TxMCol=%u TxUndrn=%u\n"
            "RxOK=%llu RxERR=%u RxOKBrd=%llu RxOKMul=%u RxOKPhy=%llu\n"
            ,pdump->TxOK
            ,pdump->TxERR
            ,pdump->Tx1Col
            ,pdump->TxAbt
            ,pdump->TxMCol
            ,pdump->TxUndrn
            ,pdump->RxOK
            ,pdump->RxERR
            ,pdump->RxOKBrd
            ,pdump->RxOKMul
            ,pdump->RxOKPhy);
//    if(tp->tx_count != pdump->TxOK)
#if CONFIG_FT
	if(tp->tx_count != pdump->TxOK)
#else
	if(tp->tx_count+1 != pdump->TxOK)
#endif
    {
        printf("txok is wrong\n");
        return -1;
    }

//    if(tp->rx_count != pdump->RxOK)
#if CONFIG_FT
	if(tp->rx_count != pdump->RxOK)
#else
	if(tp->rx_count+1 != pdump->RxOK)
#endif
    {
        printf("rxok is wrong\n");
        return -1;
    }

    return 0;
}

static int QC_100M_packet_test_item(struct rtl8168_private *tp)
{
    u8 *packet  = NULL;
    int status  = 0;
    int i       = 0;
    int run     = 0;
    int max_run = 0;
    int count   = 0;
    int total_packet = 50;
    tp->tx_count = tp->rx_count = 0;

    packet = txbuffer;
    memset((void*)packet,   0xff, 6);
    memset((void*)packet+6, 0x68, LAN_QC_VERIFY_LEN);

    max_run = (total_packet/NUM_TX_DESC);

    printf("100M transmit, total_packet=%d\n",total_packet);
    for(run=0;run<max_run;run++)
    {
        for(i=0;i<NUM_TX_DESC;i++)
        {
            tp->tx_count++;
            ETH_DBG("tx count=%d\n",tp->tx_count);
            status = rtl8168_send(tp, packet, LAN_QC_VERIFY_LEN);
            if(status <= 0)
            {
                printf("QC_test_100M failed\n");
                return -1;
            }
        }
        printf("tx done count=%d\n",tp->tx_count);

        if ((status = rtl8168_recv_multi_packet(tp,NUM_TX_DESC)) < 0)
        {
            if (status < 0) {
                printf("QC_test_100M failed\n");
                return -2;
            }
        }
    }

    max_run = total_packet%NUM_TX_DESC;
    for(i=0;i<max_run;i++)
    {
        tp->tx_count++;
        ETH_DBG("tx count=%d\n",tp->tx_count);
        status = rtl8168_send(tp, packet, LAN_QC_VERIFY_LEN);
        if(status <= 0)
        {
            printf("QC_test_100M failed\n");
            return -1;
        }
    }
    printf("tx done count=%d\n",tp->tx_count);

    if ((status = rtl8168_recv_multi_packet(tp,max_run)) < 0)
    {
        if (status < 0) {
            printf("QC_test_100M failed\n");
            return -2;
        }
    }
    mdelay(100);
    if(rtl8168_verify_statics(tp) < 0)
    {
        return -3;
    }
    printf("QC_test_100M Success\n");
    return 1;
}

#if(CONFIG_Mac7p_ETN)
static void Parameter_modify(void)
{
	// Modify parameters
	
	// set R value
	rtd_outl(0xb8016060,0x801f0bc0);  // change page bc0
	mdelay(50);
	//rtd_outl(0xb8016060,0x8014ee08);	// set rg_rs_offset bit[15:12]=e, set rg_rsw_offset bit[11:8]=e, set R value bit[3:0]=8
	//ScalerTimer_DelayXms(50);
	rtd_outl(0xb8016060,0x80140018);  // set  bit[4] force R enable, set R value  bit[3:0]=8
	mdelay(50);
	rtd_outl(0xb8016060,0x00140000);  // read  R value
	mdelay(10);
	rtd_inl(0xb8016060); // R=0x8
	mdelay(10);  
	
	// 100M Swing		   
	rtd_outl(0xb8016060, 0x841f0bc0);	// change Page bc0
	mdelay(50);
	rtd_outl(0xb8016060, 0x84170055);	// bit[7:4]=MDIX, bit[3:0]=MDI
	mdelay(50);
	rtd_outl(0xb8016060,0x00170000);  // read  set value
	mdelay(50);
	rtd_inl(0xb8016060);  //   0x55
	mdelay(50);
	
	// 100M Tr/Tf
	rtd_outl(0xb8016060,0x801f0bd0);  // change page bd0
	mdelay(50);
	rtd_outl(0xb8016060, 0x841148cb);	//	Cf bit[13:11]=0x1
	mdelay(50);
	rtd_outl(0xb8016060,0x00110000);  // read  set value
	mdelay(50);
	rtd_part_inl(0xb8016060,13,11);    //	0x1
	mdelay(50);
	
	// 10M swing
	rtd_outl(0xb8016060,0x80100000);  // set  bit[9:8]=0
	mdelay(50);
	rtd_outl(0xb8016060,0x00100000);  // read  set value
	mdelay(50);
	rtd_part_inl(0xb8016060,9,8);	 //   0x0
	mdelay(50);
	
	// 10M Harmonic
	rtd_outl(0xb8016060, 0x841f0bc0);	// change Page bc0
	mdelay(50);
	rtd_outl(0xb8016060, 0x84160644);	//	ldvbias_10M bit[10:8]=0x644
	mdelay(50);
	rtd_outl(0xb8016060,0x00160000);  // read  set value
	mdelay(50);
	rtd_part_inl(0xb8016060,10,8);	//	 0x6
	mdelay(50);
}
#endif
int QC_Verifier_ETN(void)
{
	u32 ioaddr;
	u8 *packet = NULL;
	int status = 0;
	struct rtl8168_private *tp;

    printf("Mac Loopback\n");
	ioaddr = REG_BASE;
	
	tp = (struct rtl8168_private *)malloc(sizeof *tp);
	if (!tp) {
	    printf("Can not allocate memory of r8168\n");
		return FAILED;
	}
	memset(tp, 0, sizeof(*tp));
	tp->mmio_addr = ioaddr;
	tp->rx_buf_sz = RX_BUF_SIZE;
#if 0
	qc_rtl8168_initialize(tp);	
#endif
	rtl8168_init(tp);

#if (CONFIG_Mac7p_ETN)
	Parameter_modify;
#endif

	packet = txbuffer;

#ifdef LINKP_TEST
        linkpartner_test(tp,packet);
        return SUCCESS;
#endif

#ifdef SMARTBIT_TEST
	smartbit_test(tp,packet);
	return 0;
#endif



#ifdef QC_test_1000M	
	rtl8168_set_speed_duplex(tp, BMCR_SPEED1000, BMCR_FULLDPLX);
	rtl8168_giga_link_patch(tp);	
	if (!rtl8168_check_phy_status(tp, _1000bpsF)) {
		rtl8168_halt(tp);
		printf("QC_test_1000M failed\n");
		return FAILED;
	}
#ifdef CONFIG_DMA_TRANSFER
	memset((void *)packet, 0x5c, LAN_QC_VERIFY_LEN);
	rtl8168_send(tp, packet, LAN_QC_VERIFY_LEN);
	if (rtl8168_recv_packet(tp) < 0) {
		rtl8168_send(tp, packet, LAN_QC_VERIFY_LEN);
		if (rtl8168_recv_packet(tp) < 0) {
			rtl8168_halt(tp);
			printf("QC_test_1000M failed\n");
			return FAILED;
		}
	}
#endif
	printf("QC_test_1000M Success\n");
#endif

#ifdef QC_test_10M
	enable_pll_10M(tp,1);
	rtl8168_set_speed_duplex(tp, 0x0, BMCR_FULLDPLX);
	printf("10M test\n");

	if (!rtl8168_check_phy_status(tp, _10bps)) {
		rtl8168_halt(tp);
		printf("QC_test_10M failed\n");
		return FAILED;
	}

#ifdef CONFIG_DMA_TRANSFER
	memset((void *)packet, 0x97, LAN_QC_VERIFY_LEN);
	printf("10M transmit\n");
	rtl8168_send(tp, packet, LAN_QC_VERIFY_LEN);
	if ((status = rtl8168_recv_packet(tp)) < 0) {
	        if (status == -2) {
	            printf("QC_test_10M failed\n");
			return FAILED;
		}
	    printf("10M retransmit\n");
		rtl8168_send(tp, packet, LAN_QC_VERIFY_LEN);
		if (rtl8168_recv_packet(tp) < 0) {
			rtl8168_halt(tp);
			printf("QC_test_10M failed\n");
			return FAILED;
		}
	}
#endif
	printf("QC_test_10M Success\n");
	enable_pll_10M(tp,0);

	if (EEEP_10M_test(tp, _10bps)) {
	    printf("QC_test_EEEP failed\n");
		return FAILED;
	}
	printf("QC_test_10M EEEP Success\n");

#endif


#ifdef QC_test_100M

	printf("100M test\n");
	rtl8168_set_speed_duplex(tp, BMCR_SPEED100, BMCR_FULLDPLX);
	if (!rtl8168_check_phy_status(tp, _100bps)) {
		rtl8168_halt(tp);
		printf("QC_test_100M failed\n");
		return FAILED;
	}

	if(0 > QC_100M_packet_test_item(tp))
	{
	    return FAILED;
	}


	printf("100M SNR TEST\n");

	if (snr_test(tp))
	{
	    printf("SNR fail 1 time\n");
		if (snr_test(tp))
		{
		    printf("SNR fail 2 time\n");
			return FAILED;
		}

	}
	else
	    printf("SNR PASS\n");

#endif

	rtl8168_halt(tp);

#ifdef BIST_TEST

	if (!dphy_ram_bist(tp))
		printf("dphy ram bist test pass\n");
	else {
		printf("dphy ram bist test fail\n");
		return FAILED;
	}

	if (!dphy_rom_bist(tp))
		printf("dphy rom bist test pass\n");
	else {
		printf("dphy rom bist test fail\n");
		return FAILED;
	}	

	if (!mbist_test())
		printf("mbist test pass\n");
	else {
		printf("mbist test fail\n");
		return FAILED;
	}

#endif

	return SUCCESS;
}
