/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited.
*
*        (C) COPYRIGHT 2008-2009,2011,2013 ARM Limited.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited.
*
*     SVN Information
*
*     Checked In          : $Date$
*
*     Revision            : $Revision$
*
*     Release Information : r0p0-00rel0
*
*-----------------------------------------------------------------------------*/
#include "MaliFns.h"
#include "MaliDefines-t720.h"


int RunMaliTest_adv2_t720_part0 (void);
int RunMaliTest_adv2_t720_part1 (void);
void poll_reg (unsigned int my_reg, unsigned int my_bit_location, unsigned int my_val);

static int mali_step;

int RunMaliTest_adv2_t720 (int init) {
  if (init) { mali_step = 0; };
  switch (mali_step) {
    case 0  : return RunMaliTest_adv2_t720_part0();
    case 1  : return RunMaliTest_adv2_t720_part1();
    default : return 0;
  };
};

int RunMaliTest_adv2_t720_part0 (void) {
  int res = 0;
  int gpu_rawstat = Mali_RdReg(0x20,0,GPU_IRQ_RAWSTAT);
  int gpu_status = Mali_RdReg(0x20,0,GPU_IRQ_STATUS);
  int gpuid, num_cores, l2_size, axi_width, i, as_present, js_present, core_bitmap;

  printf("RUNNING TEST: adv2\n");
  printf("  Purpose: Tests correct operation of the GPU interrupt signal\n");
  printf("  A simple test is run which forces a GPU interrupt by sampling the performance counters.\n");

  // Get current configuration to allow testing all registers
  gpuid         = Mali_RdReg(0x20, 0, 0x0000);
  core_bitmap   = Mali_RdReg(0x20, 0, 0x0100);
  l2_size       = ((Mali_RdReg(0x20, 0, 0x0004) >> 16) & 0xFF);
  axi_width     = (1 <<((Mali_RdReg(0x20, 0, 0x0004) >> 24) & 0xFF));
  as_present    = Mali_RdReg(0x20, 0, 0x0018);
  js_present    = Mali_RdReg(0x20, 0, 0x001c);

  i = core_bitmap;
  num_cores = 0;
  while(i) {
    num_cores++;
    i >>= 1;
  }
  printf("Selected configuration:\n");
  printf("  Bus width: %d\n", axi_width);
  printf("  Number of shader cores: %d\n", num_cores);
  printf("  L2 cache size: %d kB\n", ((1<<l2_size)/1024));

  Mali_Reset();

  Mali_WrReg(0x20,0x0, JOB_IRQ_CLEAR       , 0xFFFFFFFF);
  Mali_WrReg(0x20,0x0, JOB_IRQ_MASK        , 0xFFFFFFFF);
  /* 
    ************************************************************
    cause a GPU interrupt by issuing a PRFCNT_SAMPLE GPU command 
    ************************************************************
  */
  /* ********************************** */
  /* ********************************** */
  printf ("Clear the GPU IRQ flags\n");
  Mali_WrReg(0x20,0x0, GPU_IRQ_CLEAR       , 0xFFFFFFFF);
  printf ("Set the GPU IRQ Mask to all 1s\n");
  Mali_WrReg(0x20,0x0, GPU_IRQ_MASK        , 0xFFFFFFFF);
  /* ********************************** */

  /* ********************************** */
  printf ("Power on the shader cores\n");
  Mali_WrReg(0x20,0x0, SHADER_PWRON_LO     , 0xFFFFFFFF);
  printf ("Power on the Tiler\n");
  Mali_WrReg(0x20,0x0, TILER_PWRON_LO      , 0xFFFFFFFF);

  printf ("Poll the power changed all flag until it is set\n");
  poll_reg (GPU_IRQ_RAWSTAT, 10, 1);
  gpu_rawstat = Mali_RdReg(0x20,0,GPU_IRQ_RAWSTAT);
  printf ("GPU IRQ RAWSTAT = %x\n", gpu_rawstat);
  /* ********************************** */

  /* ********************************** */
  printf ("Poll the performance counters until they are no longer active\n");
  poll_reg (GPU_STATUS, 2, 0);
  gpu_status = Mali_RdReg(0x20,0,GPU_STATUS);
  printf ("GPU_STATUS = %x\n", gpu_status);
  printf ("The performance counters are inactive\n");
  /* ********************************** */

  /* ********************************** */
  printf ("Poll the power active flag until it is no longer active\n");
  poll_reg (GPU_STATUS, 1, 0);
  gpu_status = Mali_RdReg(0x20,0,GPU_STATUS);
  printf ("GPU_STATUS = %x\n", gpu_status);
  printf ("The power controller is inactive\n");
  /* ********************************** */

  /* ********************************** */
  printf ("Clear the GPU IRQ flags again\n");
  Mali_WrReg(0x20,0x0, GPU_IRQ_CLEAR       , 0xFFFFFFFF);
  printf ("Set the GPU IRQ Mask to all 1s again\n");
  Mali_WrReg(0x20,0x0, GPU_IRQ_MASK        , 0xFFFFFFFF);
  
  /* print out the status and rawstat registers and they should be zero */
  gpu_rawstat = Mali_RdReg(0x20,0,GPU_IRQ_RAWSTAT);
  gpu_status = Mali_RdReg(0x20,0,GPU_STATUS);
  printf ("GPU IRQ RAWSTAT = %x\n", gpu_rawstat);
  printf ("GPU STATUS = %x\n", gpu_status);
  /* ********************************** */

  /* ********************************** */
  printf ("Enable all Performance counters\n");
  Mali_WrReg(0x20,0x0, PRFCNT_JM_EN        , 0xFFFFFFFF);
  Mali_WrReg(0x20,0x0, PRFCNT_SHADER_EN    , 0xFFFFFFFF);
  Mali_WrReg(0x20,0x0, PRFCNT_TILER_EN     , 0xFFFFFFFF);
  Mali_WrReg(0x20,0x0, PRFCNT_L3_CACHE_EN  , 0xFFFFFFFF);
  Mali_WrReg(0x20,0x0, PRFCNT_MMU_L2_EN    , 0xFFFFFFFF);

  printf ("Configure the performance counters\n");
  Mali_WrReg(0x20,0x0, PRFCNT_CONFIG       , 0x00000001); /* addr_space=0,mode=MANUAL */
  printf ("Poll the performance counters active flag until it is no longer active\n");
  poll_reg (GPU_STATUS, 2, 0);
  
  printf ("Clear the performance counters\n");
  Mali_WrReg(0x20,0x0, GPU_COMMAND, 0x04);
  printf ("Now sample the performance counters and wait for the interrupt\n");
  Mali_WrReg(0x20,0x0, GPU_COMMAND, 0x10);
  /* ********************************** */
  
  mali_step++;
  if (res == 0) {
   return 255;
  }
  return res;
}

int RunMaliTest_adv2_t720_part1 (void) {
  int res = 0;
  int gpu_irq_rawstat = Mali_RdReg(0x20,0,GPU_IRQ_RAWSTAT);
  int job_irq_rawstat = Mali_RdReg(0x20,0,JOB_IRQ_RAWSTAT);
  int mmu_irq_rawstat = Mali_RdReg(0x20,0,MMU_IRQ_RAWSTAT);
  int gpu_irq_status = Mali_RdReg(0x20,0,GPU_IRQ_STATUS);
  int job_irq_status = Mali_RdReg(0x20,0,JOB_IRQ_STATUS);
  int mmu_irq_status = Mali_RdReg(0x20,0,MMU_IRQ_STATUS);

  /* Check for PRFCNT_SAMPLE_COMPLETED and the GPU_FAULT interrupt flags */
  if (gpu_irq_rawstat == 0x10001)
  {
   printf ("GPU interrupt seen -> PRFCNT_SAMPLE_COMPLETED and GPU_FAULT flags were set correctly gpu_rawstat = %x\n", gpu_irq_rawstat);
  }
  else
  {
   printf ("GPU  interrupt -> PRFCNT_SAMPLE_COMPLETED/GPU_FAULT was NOT set correctly, expected 0x10001 got gpu_rawstat=%x\n", gpu_irq_rawstat);
   res = 1;
  }
    
   /* DEBUG printout */
   printf ("job_irq_rawstat = %x\n", job_irq_rawstat);
   printf ("mmu_irq_rawstat = %x\n", mmu_irq_rawstat);
   printf ("gpu_irq_status = %x\n", gpu_irq_status);
   printf ("job_irq_status = %x\n", job_irq_status);
   printf ("mmu_irq_status = %x\n", mmu_irq_status);

  Mali_MaskAllInterrupts();
  mali_step++;

  
  return res;
}

void poll_reg (unsigned int my_reg, unsigned int my_bit_location, unsigned int my_val) {
  int my_reg_value =  Mali_RdReg(0x20,0,my_reg);
 printf ("Polling (for %d) of register 0x%x bit %d started\n", my_val, my_reg, my_bit_location);
  
  if (my_val)
  {
    while ((my_reg_value & (1<<my_bit_location)) != (1<<my_bit_location))
    {
      my_reg_value = Mali_RdReg(0x20,0,my_reg);
    }  
  }
  else
  {
    while ((my_reg_value & (1<<my_bit_location)) == (1<<my_bit_location))
    {
      my_reg_value = Mali_RdReg(0x20,0,my_reg);
    }  
  }
  printf ("Polling (for %d) of register 0x%x bit %d ended\n", my_val, my_reg, my_bit_location);
}
