#include "config.h"
#include "../tb_util.h"

#if POWER_INDICATIVE_PEAK_POWER

static test_vector power_indicative_peak_power_vector = {
	.vector_name = "power_indicative_peak_power",
	.cmd_string = "\ 
timeout -1\n\
#buslog 1\n\
reset\n\
writereg 01000018 0000001c # enable_cache (L2 #0)\n\
writereg 0100001c 00000003 # enable_cache (L2 #0)\n\
writereg 00000028 00707bfe # Clear interrupt status for GP\n\
writereg 02000018 00000003 # Clear MMU interrupt status for GP\n\
writereg 80013000 00030003 # Enable all desired PPs and PP MMUs for broadcast writes\n\
writereg 80017024 00000fff # Clear interrupt status for all PPs\n\
writereg 80013004 00000003 # Setup interrupt mask filter so we only get one interrupt for the PPs combined\n\
writereg 80014018 00000003 # PP enable mask\n\
writereg 80015018 00000003 # Clear MMU interrupt status for all PPs\n\
writereg 0200001C 00000003 # Enable MMU page fault interrupts for GP\n\
writereg 1200001C 00000003 # Enable MMU page fault interrupts for PP0\n\
writereg 1201001C 00000003 # Enable MMU page fault interrupts for PP1\n\
\n\
# Dumping Start gp Job nr: 1, subjob 0 from Frame nr: 2\n\
# MMU page tables\n\
load_mem mali_dump_0002_gp_001_start_mmu.hex # Page Tables\n\
# MMU registers\n\
writereg 02000000 80000000\n\
writereg 02000008 00000004\n\
writereg 02000008 00000000\n\
# MMU end\n\
#\n\
load_mem mali_dump_0002_gp_001_start.hex\n\
writereg 00 10613000\n\
writereg 04 10613050\n\
writereg 08 10615000\n\
writereg 0c 10615088\n\
writereg 10 1060a400\n\
writereg 14 10612400\n\
writereg 2c 000003fe  # Set interrupt mask: wait for PLBU\n\
writereg 20 00000013  # Start GP Core\n\
#\n\
# Dumping Start pp Job nr: 1, subjob 0 from Frame nr: 2\n\
# MMU page tables\n\
load_mem mali_dump_0002_pp_001_start_mmu.hex # Page Tables\n\
# MMU registers\n\
writereg 80015000 9f000000 \n\
writereg 80015008 00000004 \n\
writereg 80015008 00000000 \n\
# MMU end\n\
#\n\
load_mem mali_dump_0002_pp_001_start.hex\n\
#\n\
# Register writes for the Mali450 Dynamic Load Balancing Unit\n\
writereg 80014000 9F869001 # Physical address of DLB unit\n\
writereg 80014004 fff00000 # Virtual  address of DLB unit\n\
writereg 80014008 10627000 # Tile list virtual base address\n\
writereg 8001400c 00030000 # Framebuffer dimensions\n\
writereg 80014010 20000000 # Tile list configuration\n\
writereg 80014014 03000000 # Start tile positions\n\
writereg 80014018 000000ff # DLBU PP enable mask\n\
#\n\
writereg 80016000 fff00000  # M200_FRAME_REG_REND_LIST_ADDR\n\
writereg 80016004 10602000  # M200_FRAME_REG_REND_RSW_BASE\n\
writereg 80016008 00000000  # M200_FRAME_REG_REND_VERTEX_BASE\n\
writereg 8001600c 00000002  # M200_FRAME_REG_FEATURE_ENABLE\n\
writereg 80016010 00ffffff  # M200_FRAME_REG_Z_CLEAR_VALUE\n\
writereg 80016014 000000f0  # M200_FRAME_REG_STENCIL_CLEAR_VALUE\n\
writereg 80016018 ff0000ff  # M200_FRAME_REG_ABGR_CLEAR_VALUE_0\n\
writereg 8001601c ff0000ff  # M200_FRAME_REG_ABGR_CLEAR_VALUE_1\n\
writereg 80016020 ff0000ff  # M200_FRAME_REG_ABGR_CLEAR_VALUE_2\n\
writereg 80016024 ff0000ff  # M200_FRAME_REG_ABGR_CLEAR_VALUE_3\n\
writereg 80016028 00000000  # M200_FRAME_REG_BOUNDING_BOX_LEFT_RIGHT\n\
writereg 8001602c 00000000  # M200_FRAME_REG_BOUNDING_BOX_BOTTOM\n\
writereg 80020030 00000000  # M200_FRAME_REG_FS_STACK_ADDR (PP0)\n\
writereg 80022030 00000000  # M200_FRAME_REG_FS_STACK_ADDR (PP1)\n\
writereg 80016034 00000000  # M200_FRAME_REG_FS_STACK_SIZE_AND_INIT_VAL\n\
writereg 80016038 00000000  # M200_FRAME_REG_REND_CLIP_VERTEX_BASE\n\
writereg 8001603c 00000000  # M200_FRAME_REG_REND_FIRST_CLIP_VERTEX\n\
writereg 80016040 00000001  # M200_FRAME_REG_ORIGIN_OFFSET_X\n\
writereg 80016044 0000007f  # M200_FRAME_REG_ORIGIN_OFFSET_Y\n\
writereg 80016048 00000077  # M200_FRAME_REG_SUBPIXEL_SPECIFIER\n\
writereg 8001604c 00000001  # M200_FRAME_REG_TIEBREAK_MODE\n\
writereg 80016050 00000000  # M400_FRAME_REG_PLIST_CONFIG\n\
writereg 80016054 00000e0c  # M400_FRAME_REG_SCALING_CONFIG\n\
writereg 80016058 00008888  # M400_FRAME_REG_TILEBUFFER_BITS\n\
writereg 80016100 00000002  # M200_WB0_REG_SOURCE_SELECT\n\
writereg 80016104 10000000  # M200_WB0_REG_TARGET_ADDR\n\
writereg 80016108 00000003  # M200_WB0_REG_TARGET_PIXEL_FORMAT\n\
writereg 8001610c 00000000  # M200_WB0_REG_TARGET_AA_FORMAT\n\
writereg 80016110 00000000  # M200_WB0_REG_TARGET_LAYOUT\n\
writereg 80016114 00000200  # M200_WB0_REG_TARGET_SCANLINE_LENGTH\n\
writereg 80016118 00000000  # M200_WB0_REG_TARGET_FLAGS\n\
writereg 8001611c 00000000  # M200_WB0_REG_MRT_ENABLE\n\
writereg 80016120 00000000  # M200_WB0_REG_MRT_OFFSET\n\
writereg 80016124 00000000  # MALI470_RESERVED\n\
writereg 80016128 00000000  # MALI470_RESERVED\n\
writereg 8001612c 00000000  # MALI470_RESERVED\n\
writereg 80016200 00000000  # M200_WB1_REG_SOURCE_SELECT\n\
writereg 80016204 00000000  # M200_WB1_REG_TARGET_ADDR\n\
writereg 80016208 00000000  # M200_WB1_REG_TARGET_PIXEL_FORMAT\n\
writereg 8001620c 00000000  # M200_WB1_REG_TARGET_AA_FORMAT\n\
writereg 80016210 00000000  # M200_WB1_REG_TARGET_LAYOUT\n\
writereg 80016214 00000000  # M200_WB1_REG_TARGET_SCANLINE_LENGTH\n\
writereg 80016218 00000000  # M200_WB1_REG_TARGET_FLAGS\n\
writereg 8001621c 00000000  # M200_WB1_REG_MRT_ENABLE\n\
writereg 80016220 00000000  # M200_WB1_REG_MRT_OFFSET\n\
writereg 80016224 00000000  # MALI470_RESERVED\n\
writereg 80016228 00000000  # MALI470_RESERVED\n\
writereg 8001622c 00000000  # MALI470_RESERVED\n\
writereg 80016300 00000000  # M200_WB2_REG_SOURCE_SELECT\n\
writereg 80016304 00000000  # M200_WB2_REG_TARGET_ADDR\n\
writereg 80016308 00000000  # M200_WB2_REG_TARGET_PIXEL_FORMAT\n\
writereg 8001630c 00000000  # M200_WB2_REG_TARGET_AA_FORMAT\n\
writereg 80016310 00000000  # M200_WB2_REG_TARGET_LAYOUT\n\
writereg 80016314 00000000  # M200_WB2_REG_TARGET_SCANLINE_LENGTH\n\
writereg 80016318 00000000  # M200_WB2_REG_TARGET_FLAGS\n\
writereg 8001631c 00000000  # M200_WB2_REG_MRT_ENABLE\n\
writereg 80016320 00000000  # M200_WB2_REG_MRT_OFFSET\n\
writereg 80016324 00000000  # MALI470_RESERVED\n\
writereg 80016328 00000000  # MALI470_RESERVED\n\
writereg 8001632c 00000000  # MALI470_RESERVED\n\
writereg 80017028 000001ff    # Set MGMT_IRQ_MASK_ALL\n\
writereg 8001700c 00000040    # Set MGMT_CMD_START_RENDER\n\
\n\
wait irq 7ff18000 00010000 # join\n\
dump_reg 10001000 10001010\n\
dump_reg 10001020 10001030\n\
dump_reg 10001050 10001054\n\
dump_reg 10001080 10001090\n\
dump_reg 100010a0 100010b0\n\
dump_reg 10001000 10001010 > PP0_status.hex\n\
dump_reg 10001020 10001030 >> PP0_status.hex\n\
dump_reg 10001050 10001054 >> PP0_status.hex\n\
dump_reg 10001080 10001090 >> PP0_status.hex\n\
dump_reg 100010a0 100010b0 >> PP0_status.hex\n\
dump_reg 10011000 10011010\n\
dump_reg 10011020 10011030\n\
dump_reg 10011050 10011054\n\
dump_reg 10011080 10011090\n\
dump_reg 100110a0 100110b0\n\
dump_reg 10011000 10011010 > PP1_status.hex\n\
dump_reg 10011020 10011030 >> PP1_status.hex\n\
dump_reg 10011050 10011054 >> PP1_status.hex\n\
dump_reg 10011080 10011090 >> PP1_status.hex\n\
dump_reg 100110a0 100110b0 >> PP1_status.hex\n\
dump_reg 00000024 00000034\n\
dump_reg 0000003c 0000005c\n\
dump_reg 00000068 00000070\n\
dump_reg 00000094 00000098\n\
dump_reg 00000024 00000034 > GP0_status.hex\n\
dump_reg 0000003c 0000005c >> GP0_status.hex\n\
dump_reg 00000068 00000070 >> GP0_status.hex\n\
dump_reg 00000094 00000098 >> GP0_status.hex\n\
dump_written_mem > GP_PP_written_0.hex # join\n\
\n",
};
mali_error_code power_indicative_peak_power(void){
	mali_error_code err = MALI_ERROR_NO_ERROR;

	IPRINTF("\n\n\n\n>>>>>enter function power_indicative_peak_power <<<<<\n");
	err = cmd_read_configtxt(&power_indicative_peak_power_vector);
	return err;

}

#endif
