/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited or its affiliates.
*
*        (C) COPYRIGHT 2016-2017 ARM Limited or its affiliates.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited or its affiliates.
*
* Release   : r0p0-00eac0
*-----------------------------------------------------------------------------*/

/* 

BRIEF DESCRIPTION, PURPOSE AND STRATEGY
=======================================

This test checks DBGEN and NIDEN connectivity.


DEBUG HINTS IN CASE OF FAILURE
===========================
a) Check the AXI bus connections as this is the bus that is used to communicate with the Job manager.
b) Check that the GPU's reset signal is not asserted.
c) Check that the GPU input clock is toggling as expected.
d) Check that the DFT signals are disabled.
e) Check that MBIST is disabled.
f) Check the GPU interrupt signals are connected correctly i.e. IRQGPU, IRQMMU, IRQJOB and IRQEVENT.

*/

#include "MaliFns.h"


int RunMaliTest_dbgen_niden_DVALIN (void);
void mali_delay (void);

int RunMaliTest_dbgen_niden_DVALIN (void) {
  int res = 0;
  int gpu_status = 0;
  int dbgen_niden = 0;

  int num_cores, i, core_bitmap;

  printf("RUNNING TEST: dbgen_niden\n");
  printf("Purpose: Check DBGEN and NIDEN connectivity\n");

  // Get current configuration to allow testing all registers
  core_bitmap   = Mali_RdReg(0x0100);

  i = core_bitmap;
  num_cores = 0;
  while(i) {
    num_cores++;
    i &= i-1;
  }

  gpu_status  = Mali_RdReg(0x0034);
  printf("GPU_STATUS is %08x\n", gpu_status);

  /* Inserting Delay */
  mali_delay ();

  gpu_status  = Mali_RdReg(0x0034);
  dbgen_niden = gpu_status >>8;
  printf("bit[8] of GPU_STATUS is %x\n", dbgen_niden);

  if (dbgen_niden == 1)
    printf("Either DBGEN is HIGH or NIDEN is HIGH or both are HIGH\n");
  else
    printf("Both DBGEN and NIDEN are LOW\n");

  return 0;
}

void mali_delay (void) {
  int n;
  int tmp = 0;

  printf("Start of Delay\n");

#ifdef MALI_MINIMUM_DELAYS
  for (n=0;n<(2);n++)
#else  
  for (n=0;n<(1024);n++)
#endif  
  {
    /* do something to take up time */
    tmp = tmp + 1;
  }
  printf("End of Delay\n");
}
