
/************************************************************************
 *
 *  otp.c
 *
 *  OTP routine for Magellan2
 *
 *
 * Copyright (c) 2013 Realtek Semiconductor Corp. All rights reserved.
 *
 ************************************************************************/

/************************************************************************
 *  Include files
 ************************************************************************/

#include <sysdefs.h>
#include "otp_reg.h"
#include "mis_reg.h"
#include "pin_share_reg.h"
#include "util.h"


/************************************************************************
 *  Definitions
 ************************************************************************/

/************************************************************************
 *  Public variables
 ************************************************************************/

/************************************************************************
 *  Static variables
 ************************************************************************/

/************************************************************************
 *  Public function prototypes
 ************************************************************************/

/************************************************************************
 *  Static function prototypes
 ************************************************************************/

/************************************************************************
 *  Function body
 ************************************************************************/
#if 0
static int decode_2state_otp(unsigned int value, int check_virgin_state)
{
	switch (value) {
		case 0x0:	// virgin state
			if (check_virgin_state) {
				if (OTP_VIRGIN_STATE_ILLEGAL)	// virgin state is most secure state
					return TWO_STATE_MOST_SECURE;
				else	// virgin state is least secure state
					return TWO_STATE_LEAST_SECURE;
			}
			else // virgin state is least secure state
				return TWO_STATE_LEAST_SECURE;
		case 0x1:
			return TWO_STATE_LEAST_SECURE;
		case 0x3:
		default:
			return TWO_STATE_MOST_SECURE;
	}
}


static int use_pin_select_boot_option(void)
{
	if (decode_2state_otp(OTP_BOOT_OPT_SEL_EN, 1) == TWO_STATE_LEAST_SECURE)
		return 1;
	else
		return 0;
/*
	switch (OTP_BOOT_OPT_SEL_EN) {
		case 0: // virgin state
			if (OTP_VIRGIN_STATE_ILLEGAL)
				return 0;	// virgin state is most secure state
			else
				return 1;	// virgin state is least secure state
		case 0x1:
			return 1;
		case 0x3:
			return 0;
		default:
			return 0;
	}
*/
}
#endif


int get_boot_SWPOL(void)
{
	/* get boot flash type from GPIO */
	unsigned int pin_mux1, pin_mux2;
	unsigned int gpio_dir;
	unsigned int reg_val, tmp;

	// from "Mag2_ROM Code_1027.pptx"
	// FLASH_TYPE[0]: PAD_GPIO_90
	// 0xB800_0858[15:12]=0xF
	// 0xB801_BD08[26]=0x0 (direction:input)
	// 0xB801_BD38[26] (Input data)
	//
	// FLASH_TYPE[1]: PAD_GPIO_92
	// 0xB800_085C[31:28]=0xF
	// 0xB801_BD08[28]=0x0 (direction:input)
	// 0xB801_BD38[28] (Input data)	
	pin_mux1 = REG32(0xb8000858);	// save pin mux1
	pin_mux2 = REG32(0xb800085c);	// save pin mux2
	gpio_dir = REG32(0xb801BD08);		// save GPIO direction
	REG32(0xb8000858) = pin_mux1 | (0x0f << 12);
	REG32(0xb800085c) = pin_mux2 | (0x0f << 28);
	REG32(0xb801BD08) &= ~((0x1 << 26) | (0x1 << 28));
	udelay(500);
	
	//PAD_GPIO_45 (Flash type) bit0
	//PAD_GPIO_39 (Flash type) bit1
	//(00):SPI (01):SPI-NAND (10):NAND (11):eMMC
	reg_val = REG32(0xb801BD38);
	tmp = ((reg_val >> (28 - 1)) & 0x2) | ((reg_val >> (26 - 0)) & 0x1);

	REG32(0xb801BD08) = gpio_dir;	// restore GPIO direction
	REG32(0xb800085c) = pin_mux2;	// restore pin mux
	REG32(0xb8000858) = pin_mux1;	// restore pin mux

	// Flash_Type[1:0]
	// 00: SPI
	// 01: SPI NAND	
	// 10: NAND
	// 11: eMMC
	switch (tmp) {
		case 0:
			return SPI_FLASH_BOOT;
			break;
		case 0x1:
			return SPI_NAND_FLASH_BOOT;
			break;
		case 0x2:
			return NAND_FLASH_BOOT;
			break;
		case 0x3:
			return EMMC_BOOT;
			break;
	}

	// Should impossible
	return SPI_FLASH_BOOT;
}

int get_boot_flash_type(void)
{
	if(OTP_BOOT_OPT_SEL_EN < 2) {			// use default, sw auto detect + HW POL
		return AUTO_SELECT_BOOT_MEDIA;				// SW Auto Detect
	}
	else if(OTP_BOOT_OPT_SEL_EN == 2) {	// HW POL
		return get_boot_SWPOL();
	}
	else {									// OTP
		switch (OTP_BOOT_TYPE_SEL) {
			case 0:
				return SPI_FLASH_BOOT;
				break;

			case 0x1:
				return SPI_NAND_FLASH_BOOT;
				break;

			case 0x2:
				return NAND_FLASH_BOOT;
				break;

			case 0x3:
				return EMMC_BOOT;
				break;
				
			case 0x4:
				return EMMC_BOOT;
				break;

			default:
				return EMMC_BOOT;
				break;
		}
	}
}

static const char* get_flash_type_name(unsigned int flash_type)
{
	switch(flash_type){
		case NAND_FLASH_BOOT:
			return "NAND";
		case SPI_FLASH_BOOT:
			return "SPI";
		case SPI_NAND_FLASH_BOOT:
			return "SPI_NAND";
		case EMMC_BOOT:
			return "EMMC";
		default:	// should be impossible
			return "NAND";
	}
}

unsigned char get_secure_boot_type(void)
{
	return NONE_SECURE_BOOT;
}

int get_burn_boot_flash_type(void)
{
	unsigned int boot_type;

	mdelay(500);
	boot_type = get_boot_flash_type();

	printf("auto flash type: %s\n", get_flash_type_name(boot_type));

	switch (boot_type) {
		case NAND_FLASH_BOOT:
			return FLASH_TYPE_NAND;
			break;
		case SPI_FLASH_BOOT:
			return FLASH_TYPE_SPI;
			break;
		case SPI_NAND_FLASH_BOOT:
			return FLASH_TYPE_SPI_NAND;
			break;
		case EMMC_BOOT:
			return FLASH_TYPE_EMMC;
			break;
		default:	// should be impossible
			return FLASH_TYPE_NAND;
			break;
	}

	return -1;
}
