#include <common.h>
#include <command.h>
#include <drivers/i2c/rtk_i2c_lib.h>
#include <asm/io.h>
DECLARE_GLOBAL_DATA_PTR;

#include <platform_lib/board/pcb_mgr.h>
#ifdef CONFIG_RUN_FLASH_WRITER
PCB_ENUM_T pcb_enum[0];
#endif
extern unsigned long strtoul(	const char *str, char **endptr, int base);

static int i2c_info_flag = 0;
extern rtk_i2c_device_addr i2c_dev_addr[];
extern unsigned char I2C_GetBusCount(void);
unsigned char rtk_i2c_num=0;

void rtk_i2c_port_read(int portId, int devaddr, int len)
{
    int ret = 0 ;
    int i;

    unsigned char data[64] = {0};
    unsigned char regAddr[2] = {0, 0};
    
    if(len > 64){
        printf("change len to 64bytes, not support read len > 64, len=%d\n", len);
    }

    if(I2C_SetSpeed(portId, i2c_dev_addr[portId].I2C_speed_KHz) == S_FALSE)
        return;


    ret=I2C_Read_EX(portId, devaddr>>1 , 2, regAddr, len, data);
    printf("[i2c_%d] = %s test~~I2C_Read_EX with speed:%dKhz, deviceAddress = 0x%X, data:",
            portId,( 0 == ret )?("pass"):("fail"),
            i2c_dev_addr[portId].I2C_speed_KHz,
            devaddr);
    for(i = 0; i < len; i++)
        printf("%02X ",data[i]);
    printf("\n");

}

void rtk_i2c_port_write(int portId, int devaddr, char data0, char data1)
{
    int ret = 0 ;

    unsigned char data[5];
    data[0] = data0;
    data[1] = data1;

    if(I2C_SetSpeed(portId, i2c_dev_addr[portId].I2C_speed_KHz) == S_FALSE)
        return;

    ret=I2C_Write_EX(portId, devaddr>>1 , 2, data);
    printf("[i2c_%d] = %s test~~I2C_Write_EX with speed:%dKhz , deviceAddress = 0x%X, %02X, %02X\n",
            portId,( 0 == ret )?("pass"):("fail"), i2c_dev_addr[portId].I2C_speed_KHz, devaddr,
            data[0],data[1]);
}

void rtk_i2c_port_set_speed(int portId, int KHz)
{
    i2c_dev_addr[portId].I2C_speed_KHz = KHz;
    if(I2C_SetSpeed(portId, KHz) == S_FALSE)
    {
        printf("set speed failed\n");
    }
}

/*------------------------------------------------------------------
 * Func : get_i2c_device_type
 * Desc : get i2c device type
 * Parm : p_id the i2c id
 * Retn : i2c device type
 *------------------------------------------------------------------*/
char *get_i2c_device_type(unsigned char id)
{
    int i = 0;

    if( get_i2c_port_phy_id(id) == PCB_I2C_PHY_UNUSED) {
        return "UnKnow";
    }

    if(NULL == pcb_enum){
        printf("ppcb_enum is NULL\n");
        return "Unknow";
    }
    do {
        if((pcb_enum[i].name == NULL) || (0 == pcb_enum[i].name[0])){
            break;
        }
        //printf("debug ==  %s\n", &pcb_enum[i].name[strlen(pcb_enum[i].name)-4]);
        if(0 == memcmp(&pcb_enum[i].name[strlen(pcb_enum[i].name)-4], "_I2C", 4)){
            if(id == (0x0f & pcb_enum[i].info ))
                return pcb_enum[i].name;
        }
        i++;
    } while(1);

    return "Unknow";


}

void i2c_trace_device(int speed)
{
    int portId = 0;

    int deviceAddr = 0;
    int addrLength = 0;
    int dataLength = 1;
    unsigned char data[5] = {0};
    unsigned char regAddr[2] = {0, 0};


    for(portId = 0; portId < rtk_i2c_num; portId++)
    {
        printf("\nTrace port %d type %s...\n", portId,get_i2c_device_type(portId));

        if(I2C_SetSpeed(portId, speed) == S_FALSE)
            continue;
        i2c_dev_addr[portId].I2C_speed_KHz = speed;
        i2c_dev_addr[portId].port_id = portId;
        i2c_dev_addr[portId].dev_Counter = 0;
        for(deviceAddr = 0x10; deviceAddr <= 0x7f; deviceAddr++)
        {
            if(I2C_Read_EX(portId, deviceAddr, addrLength, regAddr, dataLength, data) == S_OK)
            {
                printf("\nFind device , Addr=0x%02x(8bit addr)\n", (deviceAddr & 0x7F) << 1);

                if( i2c_dev_addr[portId].dev_Counter < MAX_TEST_I2C_DEV_ONE_PORT)
                {
                    i2c_dev_addr[portId].device_addr[i2c_dev_addr[portId].dev_Counter] = deviceAddr << 1;
                    i2c_dev_addr[portId].dev_Counter++;
                }
                else
                {
                    printf("\n por %d , too many device , no need to show \n",portId );
                }

            }

            printf(".");
            mdelay(3);
        }

    }

    printf("\nI2C trace finish.\n");

}

void i2c_get_device_within_current_bus(int portId)
{

    int deviceAddr = 0;
    int addrLength = 0;
    int dataLength = 1;

    unsigned char data[5] = {0};
    unsigned char regAddr[2] = {0, 0};

    if(I2C_SetSpeed(portId, 100) == S_FALSE)
        return;
    i2c_dev_addr[portId].dev_Counter = 0;
    i2c_dev_addr[portId].I2C_speed_KHz = 100;
    for(deviceAddr = 0x10; deviceAddr <= 0x7f; deviceAddr++)
    {
        if(I2C_Read_EX(portId, deviceAddr, addrLength, regAddr, dataLength, data) == S_OK)
        {
            if( i2c_dev_addr[portId].dev_Counter < MAX_TEST_I2C_DEV_ONE_PORT)
            {
                i2c_dev_addr[portId].device_addr[i2c_dev_addr[portId].dev_Counter] = deviceAddr << 1;
                i2c_dev_addr[portId].dev_Counter++;
            }
            else
            {
                printf("\n por %d , too many device , no need to show \n",portId );
            }
        }
        mdelay(1);
    }
}

int rtk_i2c_platform_info_get(void)
{
    int                i = 0, k = 0;
    unsigned long long ullPcbPinValue = 0;
    char               portname[24];
    PCB_I2C_TYPE_T     i2c_type=PCB_I2C_TYPE_UNDEF;
    unsigned char      i2c_phy_addr=0xFF;
    unsigned char      i2c_bus_src=0xFF;

    if(1 == i2c_info_flag)
        return 0;

#if 0 //[CID-313996]: Logically dead code
    if(NULL == pcb_enum){
        printf("ppcb_enum is NULL\n");
        return -1;
    }
#endif
    //printf("===========================I2C info ================================\n");
    do {
        if((pcb_enum[i].name == NULL) || (0 == pcb_enum[i].name[0])){
            break;
        }
        for(k = 0; k < rtk_i2c_num; k++) {
            //printf("pcb name:%s, lgid:%d, device addr:0x%x\n", pcb_enum[i].name, i2c_logicId, i2c_device_addr);
            snprintf(portname, 24, "I2C_PORT_%d", k);
            if(strcmp(pcb_enum[i].name, portname) == 0){
                //strcpy(g_i2c_pcb_info[i2c_logicId].port_pcb_name, pcb_enum[i].name);
                ullPcbPinValue = pcb_enum[i].info;
                i2c_type = (ullPcbPinValue >> 32) & 0x1;
                if(PCB_I2C_TYPE_HW_I2C == i2c_type){
                    i2c_phy_addr = (ullPcbPinValue >> 4) & 0x7;
                    i2c_bus_src  = ullPcbPinValue & 0xF;
                }
                else if (PCB_I2C_TYPE_GPIO_I2C == i2c_type){
                    i2c_phy_addr = (ullPcbPinValue >> 16) & 0xFFFF;
                    i2c_bus_src  = ullPcbPinValue & 0xFFFF;
                }
                //else{ //[CID-313995]: i2c_type is either 0 or 1.
                //    printf(" i2c type error !");
                //}
                i2c_get_device_within_current_bus(k);
                //printf("pcb name:%s, i2c_type:%d, i2c_phy_addr:%d, i2c_bus_src:%d\n", pcb_enum[i].name, i2c_type, i2c_phy_addr, i2c_bus_src);
                i2c_dev_addr[k].port_id            = k;
                i2c_dev_addr[k].i2c_type           = i2c_type;
                i2c_dev_addr[k].i2c_phy_addr       = i2c_phy_addr;
                i2c_dev_addr[k].i2c_bus_src        = i2c_bus_src;
                i2c_dev_addr[k].flag = 1;
            }
            //printf("================================================================\n");
        }
        i ++;
    } while(1);

    i2c_info_flag = 1;

    return 0;
}

char* rtk_i2c_device_name_get(void)
{
    int                i = 0;
#if 0 //[CID-313963]: Logically dead code
    if(NULL == pcb_enum){
        printf("ppcb_enum is NULL\n");
        return NULL;
    }
#endif
    do {
        if((pcb_enum[i].name == NULL) || (0 == pcb_enum[i].name[0])){
            break;
        }
        //printf("debug ==  %s\n", &pcb_enum[i].name[strlen(pcb_enum[i].name)-4]);
        if(0 == memcmp(&pcb_enum[i].name[strlen(pcb_enum[i].name)-4], "_I2C", 4)){
            printf("\t |----%s ==> port %lld\n", pcb_enum[i].name, pcb_enum[i].info);
        }
        i ++;
    } while(1);

    return NULL;
}

void rtk_i2c_platform_info_dump(void)
{
    int i;
    //int k;
    printf("|----i2c port list\n");
    for ( i = 0; i < rtk_i2c_num; i++){
        if(1 == i2c_dev_addr[i].flag){
            printf("\t |----portId:%d, type:%s, phy_id:%d, bus_src:%d \n",
                    i2c_dev_addr[i].port_id,
                    (i2c_dev_addr[i].i2c_type == PCB_I2C_TYPE_HW_I2C)? "HW_I2C":"GPIO_I2C",
                    i2c_dev_addr[i].i2c_phy_addr,
                    i2c_dev_addr[i].i2c_bus_src);
            //printf("\t\t |---- devices searched :");
            //for(k = 0; k < i2c_dev_addr[i].dev_Counter; k++){
                //printf("[8bit addr:0x%x] ", i2c_dev_addr[i].device_addr[k]);
            //}
            //if(0 == k) printf(" No device searched");
            //printf("\n");
        }
    }        
    printf("|----i2c device list\n");
    rtk_i2c_device_name_get();
}

int rtk_i2c_scan(void)
{
    i2c_trace_device(100);
    return 0;
}

int do_i2c_test (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int speed = 100;
	int logicId = 0;
    int devaddr = 0xFF;
    char data0, data1;
    I2C_Init();
    rtk_i2c_num = I2C_GetBusCount();
    rtk_i2c_platform_info_get();

	if(argc == 2)
	{
	    if(0 == strcmp(argv[1], "list")){
            rtk_i2c_platform_info_dump();
            return CMD_RET_SUCCESS;
	    }
        else if(0 == strcmp(argv[1], "scan")){
            rtk_i2c_scan();
            return CMD_RET_SUCCESS;
	    }            
        else{
            return CMD_RET_USAGE;
        }
	}
    if(argc >= 3){
        if(0 == strcmp(argv[1], "speed")){
            if(0 == strcmp(argv[2], "set")){
                logicId = simple_strtoul(argv[3],NULL,10);
		        speed=simple_strtoul(argv[4], NULL, 10);
                speed= (speed == 0)?100:speed;
		        printf("set port %d with speed %d \n", logicId, speed);            
                rtk_i2c_port_set_speed(logicId, speed);
                return CMD_RET_SUCCESS;
            }
            else if(0 == strcmp(argv[2], "get")){
                logicId = simple_strtoul(argv[3],NULL,10);
                printf("the port %d speed is %dKhz \n", logicId, i2c_dev_addr[logicId].I2C_speed_KHz);
                return CMD_RET_SUCCESS;
            }
            else
                return CMD_RET_USAGE;
        }
        else if(0 == strcmp(argv[1], "read")){
            logicId = simple_strtoul(argv[2],NULL,10);
            devaddr = simple_strtoul(argv[3],NULL,10);
            data0   = simple_strtoul(argv[4],NULL,10);
            rtk_i2c_port_read(logicId, devaddr, data0);
            return CMD_RET_SUCCESS;
        }
        else if(0 == strcmp(argv[1], "write")){
            logicId = simple_strtoul(argv[2],NULL,10);
            devaddr = simple_strtoul(argv[3],NULL,10);
            data0   = simple_strtoul(argv[4],NULL,10);
            data1   = simple_strtoul(argv[5],NULL,10);
            rtk_i2c_port_write(logicId, devaddr, data0, data1);
            return CMD_RET_SUCCESS;
        }
        else{
            return CMD_RET_USAGE;
        }
    }
	return CMD_RET_USAGE;
}

U_BOOT_CMD(
        i2c ,    6,    0,     do_i2c_test,
        "i2c test",
        "Usage:\n"
        "\t i2c scan: run i2c scan test with speed 100k\n"
        "\t i2c list: list i2c pcbenum info\n"
        "\t i2c speed set [port] [speed]: set port with speed Khz\n"
        "\t i2c speed get [port] : get port speed\n"
        "\t i2c read [port] [addr] [len]: like: i2c read 0 0x34 4 (Addr Note:8bit&Hex)\n"
        "\t i2c help: show help info\n"
);

