
/*
 *  mcp.c
 *
 *  Implementation of hardware CP module
 *
 **/
#include <common.h>
#include <asm/arch/system.h>
#include <asm/arch/error_type.h>
#include <drivers/mcp/rtk_mcp_core.h>
#include <drivers/mcp/rtk_mcp_scpu.h>
#include <drivers/cw/rtk_cw_util.h>
#include <drivers/mcp/rtk_rcp.h>
#include <malloc.h>
#include <error_type.h>

void * MCP_MALLOC(unsigned int len, MEM_ALLOC_TYPE cache_type)
{
    unsigned char * pdata = NULL;
    if(cache_type == MEM_ALLOC_UNCACHE) {
        pdata = (unsigned char *)malloc_noncache_align(len, 16);
    } else {
        pdata = (unsigned char *)malloc(len);
    }
    if(pdata == NULL) {
        printf("%s[%d] memory alloc failed..\n", __FUNCTION__, __LINE__);
    }

    return pdata;
}

void MCP_FREE(void *data, unsigned int size, MEM_ALLOC_TYPE cache_type)
{
    if(cache_type == MEM_ALLOC_UNCACHE)
        free_noncache(size);
    else
        free(data);
}


unsigned int mcp_dscpt_addr;

unsigned int alloc_mcp_descriptor_buffer(MEM_ALLOC_TYPE cache_type)
{
    unsigned int alloc_mem;
    alloc_mem = (unsigned int)MCP_MALLOC(CP_DSCPT_POOL_SIZE, cache_type);
    mcp_dscpt_addr = (alloc_mem / 16 + 1) * 16;
    return  alloc_mem;
}

void free_mcp_descriptor_buffer(unsigned int alloc_mem, MEM_ALLOC_TYPE cache_type)
{
    MCP_FREE((void *)alloc_mem, CP_DSCPT_POOL_SIZE, cache_type);
}


t_mcp_descriptor *alloc_mcp_descriptor(void)
{

    //static unsigned int mcp_dscpt_addr;
    t_mcp_descriptor *current;

#ifdef MCP_DEBUG
    printf("===> mcp_dscpt_addr: 0x%x\n", mcp_dscpt_addr);
#endif
#ifdef IS_FPGA
    // check if pool is full (descriptor address overflow)
    if ((mcp_dscpt_addr + sizeof(t_mcp_descriptor) > CP_DSCPT_POOL_MAX_ADDR)  || (mcp_dscpt_addr == 0)) {
        // wrap-around (allocate from base)
        mcp_dscpt_addr = CP_DSCPT_POOL_BASE_ADDR;
        // reset pool
        memset((char *)CP_DSCPT_POOL_BASE_ADDR, 0, CP_DSCPT_POOL_SIZE);
    }
#endif
    if ((mcp_dscpt_addr % CP_DESCRIPTOR_HW_ALIGN) != 0)
        mcp_dscpt_addr = mcp_dscpt_addr + CP_DESCRIPTOR_HW_ALIGN - (mcp_dscpt_addr % CP_DESCRIPTOR_HW_ALIGN);

    current = (t_mcp_descriptor *)mcp_dscpt_addr;
    memset(current, 0, sizeof(t_mcp_descriptor));

    mcp_dscpt_addr += CP_DESCRIPTOR_HW_ALIGN; // move forward

#ifdef MCP_DEBUG
    printf("%s\n\tmcp_dscpt_addr: 0x%x, return dscpt: 0x%x\n", __FUNCTION__, mcp_dscpt_addr, (unsigned int) current);
#endif

    return current;

}

#ifdef PATTERN_DECRYPT_TEST

int MCP_AES_Pattern_Decryption(
    unsigned int           BCM,
    unsigned int           Key[4],
    unsigned int           IV[4],
    unsigned long           DataIn,
    unsigned long           DataOut,
    unsigned long           Len,
    unsigned char           crypt_byte_block,
    unsigned char           skip_byte_block
)
{
    int ret = 0;
    t_mcp_descriptor *dscpt;
    unsigned int alloc_mem;

    alloc_mem = alloc_mcp_descriptor_buffer(DEFUALT_MEM_ALLOC_TYPE);
    dscpt = alloc_mcp_descriptor();
    if (dscpt == NULL) {
        free_mcp_descriptor_buffer(alloc_mem, DEFUALT_MEM_ALLOC_TYPE);
        return ERR_ALLOC_FAILED;
    }


    MCP_DESCRIPT_INIT(dscpt, (unsigned int)MCP_MODE_AES, BCM, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT,
                      Key, IV);
    //printf("dscpt mode 0x%x\n",dscpt->mode);

    if (crypt_byte_block && skip_byte_block) {
        //printf("c s\n");
        dscpt->mode = (dscpt->mode | MCP_CRYPT_SKIP);
        //printf("dscpt mode 0x%x\n",dscpt->mode);
    }
    else if (crypt_byte_block) {
        printf("c\n");
        dscpt->mode = (dscpt->mode | MCP_FULL_CRYPT);
        printf("dscpt mode 0x%x\n", dscpt->mode);
    }
    else {
        printf("s\n");
        dscpt->mode = (dscpt->mode | MCP_FULL_SKIP);
        printf("dscpt mode 0x%x\n", dscpt->mode);
    }
    dscpt->src_addr  = DataIn;
    dscpt->dst_addr = DataOut;
    dscpt->length   = MCP_BC(Len) | MCP_CRYPT_BLK_CNT(crypt_byte_block) | MCP_SKIP_BLK_CNT(skip_byte_block);

    ret = rtk_mcp_do_hw_crypt(dscpt);
    free_mcp_descriptor_buffer(alloc_mem, DEFUALT_MEM_ALLOC_TYPE);
    return ret;
}

#define GET_MCP_AES_INI_KEY0()  			rtd_inl(0x18015124)
#define GET_MCP_AES_INI_KEY1()  			rtd_inl(0x18015124 + 4)
#define GET_MCP_AES_INI_KEY2()  			rtd_inl(0x18015124 + 4 + 4)
#define GET_MCP_AES_INI_KEY3()  			rtd_inl(0x18015124 + 4 + 4 + 4)

int mcp_aes_peekIV(unsigned int IV[4])
{
    IV[0] = GET_MCP_AES_INI_KEY0();
    IV[1] = GET_MCP_AES_INI_KEY1();
    IV[2] = GET_MCP_AES_INI_KEY2();
    IV[3] = GET_MCP_AES_INI_KEY3();
    return 0;
}

int MCP_AES_Pattern_Decryption_DADR(
    unsigned int            BCM,
    unsigned int            Key[4],
    unsigned int            IV[4],
    unsigned long           DataIn,
    unsigned long           DataOut[2],
    unsigned long           Len[2],
    unsigned char           crypt_byte_block,
    unsigned char           skip_byte_block)
{
    int ret = 0;
    t_mcp_descriptor *dscpt;
    unsigned int alloc_mem;

    alloc_mem = alloc_mcp_descriptor_buffer(DEFUALT_MEM_ALLOC_TYPE);
    dscpt = alloc_mcp_descriptor();
    if (dscpt == NULL) {
        free_mcp_descriptor_buffer(alloc_mem, DEFUALT_MEM_ALLOC_TYPE);
        return ERR_ALLOC_FAILED;
    }


    MCP_DESCRIPT_INIT(dscpt, (unsigned int)MCP_MODE_AES, BCM, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT,
                      Key, IV);

    if (crypt_byte_block && skip_byte_block) {
        //printf("c s\n");
        dscpt->mode = (dscpt->mode | MCP_CRYPT_SKIP);
        //printf("dscpt mode 0x%x\n",dscpt->mode);
    }
    else if (crypt_byte_block) {
        printf("c\n");
        dscpt->mode = (dscpt->mode | MCP_FULL_CRYPT);
        printf("dscpt mode 0x%x\n", dscpt->mode);
    }
    else {
        printf("s\n");
        dscpt->mode = (dscpt->mode | MCP_FULL_SKIP);
        printf("dscpt mode 0x%x\n", dscpt->mode);
    }
    dscpt->mode = (dscpt->mode | MCP_TWO_DADR);
    //printf("dscpt mode 0x%x\n",dscpt->mode);

    dscpt->src_addr  = DataIn;
    dscpt->dst_addr = DataOut[0];
    dscpt->length   = MCP_BC(Len[0] + Len[1]) | MCP_CRYPT_BLK_CNT(crypt_byte_block) | MCP_SKIP_BLK_CNT(skip_byte_block);
    dscpt->dst_addr2 = DataOut[1];
    dscpt->dst_len = Len[0];
    //rtk_mcp_dump_descriptor(dscpt);

    ret = rtk_mcp_do_hw_crypt(dscpt);

    free_mcp_descriptor_buffer(alloc_mem, DEFUALT_MEM_ALLOC_TYPE);

    return ret;
}
#endif



int AES_CBC_decrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry, unsigned int ini_key[4])
{
    int ret = 0;
    unsigned int cur_pos = 0;
    unsigned int cur_iv_key[4];
    unsigned int last_iv_key[4];
    while(cur_pos < length) {
        if((length - cur_pos) > MAX_CP_EN_DE_SEGMENT_SIZE) {
            unsigned char *plast_raw_iv = src_addr + cur_pos + MAX_CP_EN_DE_SEGMENT_SIZE - 16;
            last_iv_key[0]  = MCP_B4TL(plast_raw_iv[0], plast_raw_iv[1], plast_raw_iv[2], plast_raw_iv[3]);
            last_iv_key[1]  = MCP_B4TL(plast_raw_iv[4], plast_raw_iv[5], plast_raw_iv[6], plast_raw_iv[7]);
            last_iv_key[2]  = MCP_B4TL(plast_raw_iv[8], plast_raw_iv[9], plast_raw_iv[10], plast_raw_iv[11]);
            last_iv_key[3]  = MCP_B4TL(plast_raw_iv[12], plast_raw_iv[13], plast_raw_iv[14], plast_raw_iv[15]);
            if(cur_pos == 0)
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT,
                                (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr + cur_pos, MAX_CP_EN_DE_SEGMENT_SIZE,
                                dst_addr + cur_pos, (unsigned int *)&key_entry, ini_key);
            else
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT,
                                (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr + cur_pos, MAX_CP_EN_DE_SEGMENT_SIZE,
                                dst_addr + cur_pos, (unsigned int *)&key_entry, cur_iv_key);
            if(ret != 0)
                break;
            cur_pos += MAX_CP_EN_DE_SEGMENT_SIZE;
            memcpy(cur_iv_key, last_iv_key, 16);
        } else {
            if(cur_pos == 0)
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT,
                                (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr + cur_pos, length - cur_pos,
                                dst_addr + cur_pos, (unsigned int *)&key_entry, ini_key);
            else
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT,
                                (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr + cur_pos, length - cur_pos,
                                dst_addr + cur_pos, (unsigned int *)&key_entry, cur_iv_key);
            cur_pos = length;/*no need to do this step*/
            break;
        }
    }
    return ret;

}
int AES_ECB_decrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry)
{
    return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, (unsigned int *)&key_entry, NULL);
}

int AES_ECB_encrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry)
{
    return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, (unsigned int *)&key_entry, NULL);
}


int DES_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int DES_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int DES_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int DES_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int *key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int TDES_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int *key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int TDES_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int *key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int TDES_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int *key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int TDES_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int *key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_3DES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

//=================================================================================================================================================
int AES_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_OTP, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, NULL);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int AES_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_OTP, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, NULL);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}


int AES_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_OTP, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_OTP, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_CTR_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_OTP, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_CTR_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_OTP, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}


int AES_ECB_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int AES_ECB_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}


int AES_CBC_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_CBC_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_CTR_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_CTR_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}


int AES_ECB_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int AES_ECB_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}


int AES_CBC_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_CBC_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_CTR_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_CTR_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int SHA1_hash(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int *iv)
{
    if(iv != NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA1, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, iv, iv + 4);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA1, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, NULL);
}

int SHA256_hash(unsigned char * src_addr, unsigned int length, unsigned char *dst_addr, unsigned int iv[8])
{
    int ret = 0;
    unsigned int cur_pos = 0;
    unsigned int cur_iv_key[8] = {0};
    if ((src_addr == NULL) || (dst_addr == NULL))
        return ERR_INVALID_PARAM;
    if(iv != NULL)
        memcpy(cur_iv_key, iv, 32);
    while(cur_pos < length) {
        if((length - cur_pos) > MAX_CP_SHA_SEGMENT_SIZE) {
            if(cur_pos != 0 || iv != NULL)
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr + cur_pos, MAX_CP_SHA_SEGMENT_SIZE, dst_addr, &cur_iv_key[0], &cur_iv_key[4]);
            else
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr + cur_pos, MAX_CP_SHA_SEGMENT_SIZE, dst_addr, NULL, NULL);

            if(ret != 0)
                break;
            cur_pos += MAX_CP_SHA_SEGMENT_SIZE;
#if 1
            memcpy(cur_iv_key, dst_addr, 32);
#else
            cur_iv_key[0]  = MCP_B4TL(dst_addr[0], dst_addr[1], dst_addr[2], dst_addr[3]);
            cur_iv_key[1]  = MCP_B4TL(dst_addr[4], dst_addr[5], dst_addr[6], dst_addr[7]);
            cur_iv_key[2]  = MCP_B4TL(dst_addr[8], dst_addr[9], dst_addr[10], dst_addr[11]);
            cur_iv_key[3]  = MCP_B4TL(dst_addr[12], dst_addr[13], dst_addr[14], dst_addr[15]);
            cur_iv_key[4]  = MCP_B4TL(dst_addr[16], dst_addr[17], dst_addr[18], dst_addr[19]);
            cur_iv_key[5]  = MCP_B4TL(dst_addr[20], dst_addr[21], dst_addr[22], dst_addr[23]);
            cur_iv_key[6]  = MCP_B4TL(dst_addr[24], dst_addr[25], dst_addr[26], dst_addr[27]);
            cur_iv_key[7]  = MCP_B4TL(dst_addr[28], dst_addr[29], dst_addr[30], dst_addr[31]);
#endif
        } else {
            if(cur_pos != 0 || iv != NULL)
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr + cur_pos, length - cur_pos, dst_addr, &cur_iv_key[0], &cur_iv_key[4]);
            else
                ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr + cur_pos, length - cur_pos, dst_addr, NULL, NULL);
            cur_pos = length;/*no need to do this step*/
            break;
        }
    }
    return ret;
}

////
int CAM_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_OTP, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, NULL);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int CAM_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_OTP, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, NULL);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}


int CAM_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_OTP, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_OTP, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CTR_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_OTP, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CTR_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_OTP, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, NULL, ini_key);
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}


int CAM_ECB_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int CAM_ECB_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}


int CAM_CBC_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CBC_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CTR_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CTR_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_192, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}


int CAM_ECB_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}

int CAM_ECB_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key)
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, NULL);
}


int CAM_CBC_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CBC_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CBC, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CTR_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_DECRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int CAM_CTR_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4])
{
    if(key == NULL)
        return -1;
    else if((unsigned long)key <= MCP_MAX_CW_ENTRY)
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
    else
        return GenerateMCPDescriptor((unsigned int)MCP_MODE_CAM_256, (unsigned int)MCP_BCM_CTR, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, key, ini_key);
}

int AES_hash(unsigned char * src_addr, int length, unsigned char * dst_addr, unsigned int block_size)
{
    int res;
    t_mcp_descriptor dscpt;
    unsigned int *init_key = NULL;
    if ((src_addr == NULL) || (dst_addr == NULL))
        return -1;
    if ((length <= 0) || (block_size <= 0))
        return -1;

    while(length > 0) {
        unsigned int cur_size = length < block_size ? length : block_size;
        length -= cur_size;
        res = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES_H, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DSCPT, MCP_IV_SEL_DSCPT, src_addr, cur_size, dst_addr, NULL, init_key);
        if(res)
            break;
        if (length > 0) {
            // update descriptor for next block
            update_iv(&dscpt, (unsigned char *)dst_addr, 1);
            init_key = dscpt.ini_key;
            src_addr += cur_size;
        } else {
            break;
        }
    }
    return res;
}


int SHA1_padding(char* p_buff, unsigned long buff_len, const unsigned int data_len)
{
    unsigned long long bit_len = ((unsigned long long) data_len) << 3;
    int pad_sz = 64 - (data_len & 0x3f);

    if (pad_sz <= 8)
        pad_sz += 64;

    if (buff_len < pad_sz) {
        printf("padding SHA1 data failed, no enough space left for data padding. pad_sz = %d, remain_sz = %ld\n", pad_sz, buff_len);
        return -1;
    }

    memset(p_buff, 0, pad_sz);

    *p_buff |= 0x80;

    // pad bit length in big endian format
    p_buff   += pad_sz - 8;
    *p_buff++ = (bit_len >> 56) & 0xFF;
    *p_buff++ = (bit_len >> 48) & 0xFF;
    *p_buff++ = (bit_len >> 40) & 0xFF;
    *p_buff++ = (bit_len >> 32) & 0xFF;
    *p_buff++ = (bit_len >> 24) & 0xFF;
    *p_buff++ = (bit_len >> 16) & 0xFF;
    *p_buff++ = (bit_len >>  8) & 0xFF;
    *p_buff++ = (bit_len)       & 0xFF;

    return pad_sz;
}

int SHA256_padding(char *p_buff, const unsigned int length)
{
    unsigned long long bit_len;
    int pad_len;

    if ((length == 0) || (p_buff == NULL)) {
        return -1;
    }

    // calculate padding length
    pad_len = 64 - (length & 0x3f);
    if (pad_len <= 8)
        pad_len += 64;

    bit_len = (unsigned long long)(length << 3);

    // fill padding pattern
    memset(p_buff, 0, pad_len);
    *p_buff = 0x80;

    // pad bit length in big endian format
    p_buff   += pad_len - 8;
    *p_buff++ = (bit_len >> 56) & 0xFF;
    *p_buff++ = (bit_len >> 48) & 0xFF;
    *p_buff++ = (bit_len >> 40) & 0xFF;
    *p_buff++ = (bit_len >> 32) & 0xFF;
    *p_buff++ = (bit_len >> 24) & 0xFF;
    *p_buff++ = (bit_len >> 16) & 0xFF;
    *p_buff++ = (bit_len >>  8) & 0xFF;
    *p_buff++ = (bit_len)       & 0xFF;

    return pad_len;
}

int SHA384_padding(char *p_buff, const unsigned int length)
{
    return SHA512_padding(p_buff, length);
}

int SHA512_padding(char *p_buff, const unsigned int length)
{
    unsigned long long bit_len;
    int pad_len;

    if ((length == 0) || (p_buff == NULL)) {
        return -1;
    }

    // calculate padding length
    pad_len = 128 - (length & 0x7f);
    if (pad_len <= 16)
        pad_len += 128;

    bit_len = (unsigned long long)(length << 3);
    *p_buff = 0x80;

    // pad bit length in big endian format
    p_buff   += pad_len - 8;
    /* for test, pad_ bit length will in 8 byte */
    *p_buff++ = (bit_len >> 56) & 0xFF;
    *p_buff++ = (bit_len >> 48) & 0xFF;
    *p_buff++ = (bit_len >> 40) & 0xFF;
    *p_buff++ = (bit_len >> 32) & 0xFF;
    *p_buff++ = (bit_len >> 24) & 0xFF;
    *p_buff++ = (bit_len >> 16) & 0xFF;
    *p_buff++ = (bit_len >>  8) & 0xFF;
    *p_buff++ = (bit_len)       & 0xFF;
    return pad_len;
}

void swap(unsigned char * in, unsigned int len)
{
	int i = 0;
	unsigned char tmp1;
	unsigned char tmp2;
	for (i = 0; i < len; i += 4) {
		tmp1 = in[i];
		tmp2 = in[i+1];
		in[i] = in[i+3];
		in[i+1] = in[i+2];
		in[i+2] = tmp2;
		in[i+3] = tmp1;
	}
}

int SHA512_hash(unsigned char * src_addr, unsigned int length, unsigned char *dst_addr, unsigned int iv[16])
{
    int ret = 0;
    unsigned int *cur_iv_key;
    int hash512iv[] = {
        0x6a09e667,0xf3bcc908,
        0xbb67ae85,0x84caa73b,
        0x3c6ef372,0xfe94f82b,
        0xa54ff53a,0x5f1d36f1,
        0x510e527f,0xade682d1,
        0x9b05688c,0x2b3e6c1f,
        0x1f83d9ab,0xfb41bd6b,
        0x5be0cd19,0x137e2179};
    if ((src_addr == NULL) || (dst_addr == NULL))
        return ERR_INVALID_PARAM;
    if(iv != NULL) {
        cur_iv_key = iv;
    }
    else {
        cur_iv_key = MCP_MALLOC(64, DEFUALT_MEM_ALLOC_TYPE);
        memcpy(cur_iv_key, hash512iv, 64);
    }
    swap((unsigned char *)cur_iv_key, 64);
    //printf("before sha512 iv:\n");
    //mcp_dump_mem((unsigned char *)cur_iv_key, 64);
    ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA512, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, (unsigned int)MCP_IV_SEL_DSCPT,  src_addr, length, dst_addr, &cur_iv_key[0], NULL);
    //printf("after sha512 iv:\n");
    //mcp_dump_mem((unsigned char *)cur_iv_key, 64);
    if (iv == NULL && cur_iv_key != NULL)
        MCP_FREE(cur_iv_key, 64, DEFUALT_MEM_ALLOC_TYPE);
    return ret;
}

int SHA384_hash(unsigned char * src_addr, unsigned int length, unsigned char *dst_addr, unsigned int iv[16])
{
    int ret = 0;
    unsigned int *cur_iv_key;
    int hash384iv[] = {
        0xcbbb9d5d,0xc1059ed8,
        0x629a292a,0x367cd507,
        0x9159015a,0x3070dd17,
        0x152fecd8,0xf70e5939,
        0x67332667,0xffc00b31,
        0x8eb44a87,0x68581511,
        0xdb0c2e0d,0x64f98fa7,
        0x47b5481d,0xbefa4fa4};
    if ((src_addr == NULL) || (dst_addr == NULL))
        return ERR_INVALID_PARAM;
    if(iv != NULL) {
        cur_iv_key = iv;
    }
    else {
        cur_iv_key = MCP_MALLOC(64, DEFUALT_MEM_ALLOC_TYPE);
        memcpy(cur_iv_key, hash384iv, 64);
    }
    swap((unsigned char *)cur_iv_key, 64);
    //printf("before sha512 iv:\n");
    //mcp_dump_mem((unsigned char *)cur_iv_key, 64);
    ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_SHA384, (unsigned int)MCP_BCM_ECB, (unsigned int)MCP_ENCRYPT, (unsigned int)MCP_KEY_FROM_DDR, (unsigned int)MCP_IV_SEL_DSCPT, src_addr, length, dst_addr, &cur_iv_key[0], NULL);
    if (iv == NULL && cur_iv_key != NULL)
        MCP_FREE(cur_iv_key, 64, DEFUALT_MEM_ALLOC_TYPE);
    return ret;
}

/*======================================================================
 * Func : MCP_HMAC-SHA256
 *
 * Desc : Generating message authentication code via HMAC-SHA256
 *
 * Parm : keyIn         : the key msg provider and you are owner.
 *        keyIn_len     : the length of key
 *        msgIn         : the msg need to verify whether correct or not
 *        msgIn_len     : the length of msg
 *        p_out         : the msg authertication code
 *
 * Retn : >=0 : success
 *        < 0 : failed,
 *======================================================================*/
/* +-----------------------+---------------------+                 +-----------------+
 * |inner_key(key^0x36)64B | Message(msg)        | Do SHA-256 Hash | First_Hash(32B) |
 * +-----------------------+---------------------+                 +-----------------+
 *                                __________________________________________|
 *                                |
 * +-----------------------+-----------------+                 +------------------------+
 * |outer_key(key^0x5c)64B | First_Hash(32B) | Do SHA-256 Hash | Second_Hash(p_out)32B  |
 * +-----------------------+-----------------+                 +------------------------+
 */
int MCP_HMAC_SHA256( unsigned char* keyIn,
                     unsigned int keyIn_len,
                     unsigned char* msg,
                     unsigned int msg_len,
                     unsigned char* p_out )
{
    char*  tmp = NULL;
    unsigned char   k_ipad[MCP_SHA256_BLOCK_SIZE] = {0}; /* inner padding - key XORd with ipad */
    unsigned char   k_opad[MCP_SHA256_BLOCK_SIZE] = {0}; /* outer padding - key XORd with opad */
    unsigned char   key[MCP_SHA256_BLOCK_SIZE] = {0};
    unsigned char   hash[MCP_HMAC_SHA256_SIZE] = {0};

    int i;
    unsigned int    error = 0;
    unsigned int    pad_sz = 0;

    tmp = malloc(MCP_SHA256_BLOCK_SIZE + msg_len + 72);

    /* 1.0 if key is longer than 64 bytes reset it to key=SAH256(key), new key is Ko*/
    if (keyIn_len > MCP_SHA256_BLOCK_SIZE) {
        pad_sz += SHA256_padding((char *)&keyIn[keyIn_len], keyIn_len);

//        DCACHE_FLUSH();

        error = SHA256_hash((unsigned char *)PHYS(keyIn), keyIn_len + pad_sz, (unsigned char *)PHYS(key), NULL);
        if( error ) {
            printf("[ERR] %s: caculate hash256 fail(%d)\n", __FUNCTION__, error );
            free(tmp);
            return 1;
        }

        keyIn_len = MCP_SHA256_BLOCK_SIZE;
    } else {
        memcpy( key, keyIn, keyIn_len );
        keyIn_len = MCP_SHA256_BLOCK_SIZE;
    }

    /* 2. XOR key with ipad and opad values */
    for ( i = 0; i < MCP_SHA256_BLOCK_SIZE; i++ ) {
        k_ipad[i] = 0x36 ^ key[i];
        k_opad[i] = 0x5c ^ key[i];
    }

    /* 3. First-Hash = H(Ko XOR Ipad(0x36) || (data to auth)) */
    memcpy(tmp, k_ipad, keyIn_len);
    memcpy(tmp + keyIn_len, msg, msg_len);
    pad_sz = SHA256_padding((char *)&tmp[keyIn_len + msg_len], keyIn_len + msg_len);

    DCACHE_FLUSH();

    error = SHA256_hash((unsigned char *)PHYS(tmp), keyIn_len + msg_len + pad_sz, (unsigned char *)PHYS(hash), NULL);
    if( error ) {
        printf("[ERR] %s: caculate hash1 fail(%d)\n", __FUNCTION__, error );
        free(tmp);
        return 1;
    }

    /* 4. Second-Hash = H(Ko XOR Opad(0x5c) || First-Hash) */
    memcpy(tmp, k_opad, keyIn_len);
    memcpy(tmp + keyIn_len, hash, 32);
    pad_sz = SHA256_padding((char *)&tmp[keyIn_len + 32], keyIn_len + 32);

    DCACHE_FLUSH();

    error = SHA256_hash((unsigned char *)PHYS(tmp), keyIn_len + 32 + pad_sz, (unsigned char *)PHYS(p_out), NULL);
    if( error ) {
        printf("[ERR] %s: caculate hash1 fail(%d)\n", __FUNCTION__, error );
        free(tmp);
        return 1;
    }
    free(tmp);
    return 0;
}

void reverse_signature( unsigned char * pSignature )
{
    unsigned char temp;
    unsigned int i;

    for( i = 0; i < 128; i ++ )  {
        temp = pSignature[i];
        pSignature[i] = pSignature[255 - i];
        pSignature[255 - i] = temp;
    }
}

static int get_sha256_value(unsigned char * IV,unsigned char *p_img, unsigned int img_size, unsigned int done_size, unsigned char *sha256_value)
{
    int res = ERR_UNKNOWN_TYPE;
    unsigned char *result = sha256_value;   // MCP SHA256 buffer
    unsigned char *tmp_pad = malloc(128);

    if (tmp_pad == NULL) {
        res = ERR_INVALID_PARAM;
        goto ERR;
    }
    int align_size = img_size & (~0x3f);
    int pad_sz = 0;
    unsigned char *tmp_result = IV; /* first time IV is set,
                                       then take result as IV */
    memcpy(tmp_pad, p_img + align_size, img_size - align_size);
    pad_sz = SHA256_padding((char *)tmp_pad + img_size - align_size, img_size + done_size);

    if (align_size) {
        int hw_done_size = 0;
        int hw_tmp_size = 0;
        while (hw_done_size < align_size) {
            hw_tmp_size = (hw_done_size + MAX_CP_SHA_SEGMENT_SIZE) > align_size ? align_size - hw_done_size : MAX_CP_SHA_SEGMENT_SIZE;
            res = SHA256_hash((unsigned char*)(p_img + hw_done_size), hw_tmp_size, result, (unsigned int *)tmp_result);
            if (res < 0)
                goto ERR;
            hw_done_size += hw_tmp_size;
            tmp_result = result;
            //hexdump("result1", result, 32);
        }
    }
    res = SHA256_hash(tmp_pad, img_size - align_size + pad_sz, result, (unsigned int *)result);
    //hexdump("result2", result, 32);
ERR:
    if(tmp_pad)
        free(tmp_pad);
    return res;
}

int CP_SHA256(unsigned char *p_img, unsigned int img_size,unsigned char *sha256_value)
{
    return get_sha256_value(NULL, p_img, img_size, 0, sha256_value);
}

int HMAC_SHA256(unsigned char *key, unsigned int key_len, unsigned char *buff, unsigned int len, unsigned char *output)
{
    unsigned int i;
    int ret = 0;
    unsigned char *hmac_padding;
    unsigned char *hmac_temp_out;
    hmac_padding = malloc(96);
    if (hmac_padding == NULL)
        return ERR_ALLOC_FAILED;
    memset(hmac_padding, 0, 64);
    hmac_temp_out = hmac_padding + 64;
    if(key_len > MCP_SHA256_BLOCK_SIZE)
        ret = get_sha256_value(NULL, key, key_len, 0, hmac_padding);
    else
        memcpy(hmac_padding, key, key_len);
    if (ret)
        goto ERR;
    for (i = 0; i < MCP_SHA256_BLOCK_SIZE; i++) {
        hmac_padding[i] ^= 0x36;
    }
    //mcp_hexdump("hmac_padding",hmac_padding,64);

    ret = SHA256_hash(hmac_padding, MCP_SHA256_BLOCK_SIZE, hmac_temp_out, NULL);
    if (ret)
        goto ERR;
    ret = get_sha256_value(hmac_temp_out, buff, len, MCP_SHA256_BLOCK_SIZE, hmac_temp_out);
    //mcp_hexdump("hmac_temp_out add buff",hmac_temp_out,32);
    
    if (ret)
        goto ERR;
    for (i = 0; i < 64; i++) {
        hmac_padding[i] ^= 0x36;
        hmac_padding[i] ^= 0x5c;
    }
    ret = SHA256_hash(hmac_padding, MCP_SHA256_BLOCK_SIZE, output, NULL);
    if (ret)
        goto ERR;
    ret = get_sha256_value(output, hmac_temp_out, 32, MCP_SHA256_BLOCK_SIZE, output);
    //mcp_hexdump("output final",output,32);
ERR:
    if(hmac_padding)
        free(hmac_padding);
    return ret;
}


int HKDF_EXTRACT(unsigned char * ikm, unsigned int ikm_len,
                     unsigned char * salt, unsigned int salt_len,
                     unsigned char * prk)
{
    if(salt == NULL || salt_len == 0) {
        salt_len = SHA256_SIZE;
        salt = prk;
        memset(salt,0,salt_len);
    }
    return HMAC_SHA256(salt, salt_len, ikm, ikm_len, prk);
}

int HKDF_EXPAND(unsigned char * prk,
        unsigned char * info, unsigned int info_len,
        unsigned char * okm, unsigned int okm_len)
{
    unsigned int i = 0;
    unsigned int N = 0;             //
    unsigned char * msg = NULL;
    unsigned int msg_len = 0;
    unsigned int msg_size = 0;
    unsigned int hash_len = SHA256_SIZE;
    int ret = 0;

    if(prk == NULL || okm == NULL || okm_len == 0)
        return ERR_INVALID_PARAM;

    if(info_len > 0x80)
        return ERR_INVALID_PARAM;
    
    N = (okm_len + SHA256_SIZE - 1)/SHA256_SIZE;
    msg_size = SHA256_SIZE+info_len+1;
    
    msg = malloc(msg_size);
    if (msg == NULL)
        ret = ERR_ALLOC_FAILED;
    msg_size = hash_len+info_len+1;
    memset(msg,0,msg_size);
    if(info && info_len)
        memcpy(msg+SHA256_SIZE,info,info_len);
    msg_len = info_len+1;
    //msg = T(0) | info | 0x01 ; tmp(i) | info | i + 1
    for(i = 0; i < N; i++) {
        *(msg+msg_size-1) = i + 1;
        //DHEXDUMP(msg,msg_size);
        ret = HMAC_SHA256(prk, SHA256_SIZE, msg+msg_size-msg_len,
                                                msg_len,msg);
        if(ret)
            goto ERR;
        msg_len = msg_size;
        memcpy(okm + i*SHA256_SIZE, msg, (i!=N-1)?SHA256_SIZE:okm_len-i*SHA256_SIZE);
        //T = T(0) | T(1) | ... | T(i) 
        //okm = T[:out_lenth]
        //DHEXDUMP(okm,okm_len);
    }
ERR:
    if(msg)
        free(msg);
    return ret;   
}

int HKDF_SHA256(
        unsigned char * ikm, unsigned int ikm_len,
        unsigned char * salt, unsigned int salt_len,
        unsigned char * info, unsigned int info_len,
        unsigned char * okm, unsigned int okm_len)
{
    unsigned char prk[32];
    int ret = 0;
    ret = HKDF_EXTRACT(ikm, ikm_len, salt, salt_len, prk);
    if (ret)
        goto ERR;
    ret = HKDF_EXPAND(prk, info, info_len, okm, okm_len);
ERR:
    return ret;
}

#define CP_CW_DEBUG
#ifdef CP_CW_DEBUG
#define CW_TP_REG_INFO      (0xb8014118)
#define CW_TP_REG_CTRL      (0xb8014120)

void RCP_WRITE_SRAM(unsigned int id, unsigned int data[2])
{
    WRITE_REG32(CW_TP_REG_INFO,data[0]);
    WRITE_REG32(CW_TP_REG_INFO + 4,data[1]);
    WRITE_REG32(CW_TP_REG_CTRL,(id | 0x80));  // write 8 bytes
    WRITE_REG32(CW_TP_REG_INFO, 0);
    WRITE_REG32(CW_TP_REG_INFO + 4, 0);
}
        
void RCP_READ_SRAM(unsigned int id, unsigned int data[2])
{
    WRITE_REG32(CW_TP_REG_INFO, 0);
    WRITE_REG32(CW_TP_REG_INFO + 4, 0);
    WRITE_REG32(CW_TP_REG_CTRL, (id));  // write 8 bytes
    data[0] =  READ_REG32(CW_TP_REG_INFO);
    data[1]  = READ_REG32(CW_TP_REG_INFO + 4);
}

        
int RCP_GET_CW(unsigned int id, unsigned int* pCW, unsigned int len)
{
    while (len) {
        if (len < 8) {
            RCP_READ_SRAM(id++, pCW);
            break;
        }

        RCP_READ_SRAM(id++, pCW);
        pCW += 2;
        len -= 8;
    }

    return 0;
}

int RCP_SET_CW(unsigned int id, unsigned int* pCW, unsigned int len)
{
    while (len) {
        if (len < 8) {
            RCP_WRITE_SRAM(id++, pCW);
            break;
        }

        RCP_WRITE_SRAM(id++, pCW);
        pCW += 2;
        len -= 8;
    }

    return 0;
}

int CP_SET_CW_KEY(unsigned int cw_key_id, unsigned char key[], unsigned int key_len)
{
    unsigned char key2[32];
    int i = 0;
    for(i = 0; i < key_len/4; i++) {
        *(key2 + i*4 + 3) = *(key + i*4 + 0);
        *(key2 + i*4 + 2) = *(key + i*4 + 1);
        *(key2 + i*4 + 1) = *(key + i*4 + 2);
        *(key2 + i*4 + 0) = *(key + i*4 + 3);
    }
    return RCP_SET_CW(cw_key_id, (unsigned int *)key2, key_len);
}


int CP_GET_CW_KEY(unsigned int cw_key_id,unsigned char key[], unsigned int key_len)
{
    int ret = 0, i = 0;
    unsigned char key2[32];
    if((ret = RCP_GET_CW(cw_key_id, (unsigned int *)key2, key_len)) < 0){
        return ret;
    }
    for(i = 0; i < key_len/4; i++) {
        *(key + i*4 + 3) = *(key2 + i*4 + 0);
        *(key + i*4 + 2) = *(key2 + i*4 + 1);
        *(key + i*4 + 1) = *(key2 + i*4 + 2);
        *(key + i*4 + 0) = *(key2 + i*4 + 3);
    }
    return ret;
}
#endif

int CP_AES_Engine_from_CW(unsigned char algo_id,
            unsigned int enc_dec,
            unsigned int key_entry, unsigned int iv_entry,
            void *src_addr, void *dst_addr,
            unsigned long length,unsigned int update_iv)
{
    unsigned int mode;
    int ret = 0;
    int hw_done_size = 0;
    int hw_tmp_size = 0;

#if defined(DEBUG)
    printf("%s(0x%x, 0x%x, 0x%x, 0x%x, key_entry=0x%x)\n", __FUNCTION__, algo_id, (unsigned int)src_addr, length, (unsigned int)dst_addr, key_entry);
#endif
    if ((src_addr == NULL) || (dst_addr == NULL))
        return ERR_INVALID_PARAM;

    if (algo_id == ALGO_ID_AES_ECB)
        mode = MCP_BCM_ECB;
    else if (algo_id == ALGO_ID_AES_CBC)
        mode = MCP_BCM_CBC;
    else
        return ERR_UNKNOWN_TYPE;

    if(!update_iv)
        RTK_RCP_CW_XOR(key_entry, iv_entry, CP_RESERVE_CW);
    while (hw_done_size < length) {
        hw_tmp_size = (hw_done_size + MAX_CP_EN_DE_SEGMENT_SIZE) > length ? length - hw_done_size : MAX_CP_EN_DE_SEGMENT_SIZE;
        ret = GenerateMCPDescriptor((unsigned int)MCP_MODE_AES,
                     (unsigned int)mode, (unsigned int)enc_dec,
                     (unsigned int)MCP_KEY_FROM_CW, MCP_IV_SEL_CW, src_addr + hw_done_size,
                     hw_tmp_size, dst_addr + hw_done_size, &key_entry, &iv_entry);
        if (ret != 0)
            break;
        hw_done_size += hw_tmp_size;
    }
    if(!update_iv)
        RTK_RCP_CW_XOR(key_entry, CP_RESERVE_CW, iv_entry);
    //hexdump("aes src",src_addr,48);
    //hexdump("aes dst",dst_addr,48);
    return ret;
}

int RT_AES_Decrypt_from_CW(unsigned char algo_id,
            unsigned int key_entry, unsigned int iv_entry,
            void *src_addr, void *dst_addr,
            unsigned long length,unsigned int update_iv)
{

    int ret = 0;
    unsigned int sblock_len = 0,loop = 0;
    unsigned char *sblock_src_addr = NULL,*sblock_dst_addr = NULL;
    unsigned char * src_tmp;
    unsigned char * dst_tmp;
    src_tmp = malloc(32);
    dst_tmp = src_tmp + 16;
    /*get short block size*/
    if (src_tmp == NULL) {
        ret = ERR_ALLOC_FAILED;
        goto ERR;
    }
    sblock_len = length & 0xf;
    if (sblock_len) {
        sblock_src_addr = (unsigned char *)src_addr + (length - sblock_len);
        sblock_dst_addr = (unsigned char *)dst_addr + (length - sblock_len);
        memcpy(src_tmp, sblock_src_addr - 0x10, 0x10);
    }
    if ((ret = CP_AES_Engine_from_CW(algo_id,MCP_DECRYPT,CW_KEY_ENTRY(key_entry),CW_KEY_ENTRY(iv_entry),src_addr,dst_addr,
        length - sblock_len,update_iv)) != 0) {
        printf("aes d fail\n");
        goto ERR;
    }

    if (sblock_len) {
        //printf("%s %d imglen %d sblock_len %d\n", __FUNCTION__, __LINE__,length,sblock_len);
        if ((ret = CP_AES_Engine_from_CW(ALGO_ID_AES_ECB,MCP_ENCRYPT,CW_KEY_ENTRY(key_entry),CW_KEY_ENTRY(iv_entry),
            (src_tmp),dst_tmp,0x10,update_iv))!= 0){
            printf("aes e fail\n");
            goto ERR;
        }
        // XOR with short block data to generate final result
        for (loop = 0; loop < sblock_len; loop++)
        {
            sblock_dst_addr[loop] = dst_tmp[loop] ^ sblock_src_addr[loop];
        }
    }
ERR:
    if(src_tmp)
        free(src_tmp);
    return ret;
}
