#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define GET_BIT(value, bit)    (((value) >> (bit)) & (0x01))
static void byte_crc8(unsigned char *crc,unsigned char data)
{
	unsigned char b00 = GET_BIT(data, 7) ^ GET_BIT(*crc, 7);
	unsigned char c00 = b00 ^ GET_BIT(*crc, 1);
	unsigned char a00 = b00 ^ GET_BIT(*crc, 6);

	unsigned char b01 = GET_BIT(data, 6) ^ a00;
	unsigned char c01 = GET_BIT(*crc, 0) ^ b01;
	unsigned char a01 = GET_BIT(*crc, 5) ^ b01;

	unsigned char b02 = GET_BIT(data, 5) ^ a01;
	unsigned char c02 = b00 ^ b02;
	unsigned char a02 = GET_BIT(*crc, 4) ^ b02;

	unsigned char b03 = GET_BIT(data, 4) ^ a02;
	unsigned char crc_o_6 = b03 ^ b01;
	unsigned char a03 = GET_BIT(*crc, 3) ^ b03;	

	unsigned char b04 = GET_BIT(data, 3) ^ a03;
	unsigned char crc_o_5 = b04 ^ b02;
	unsigned char a04 = GET_BIT(*crc, 2) ^ b04;

	unsigned char b05 = GET_BIT(data, 2) ^ a04;
	unsigned char crc_o_4 = b05 ^ b03;
	unsigned char a05 = c00 ^ b05;

	unsigned char crc_o_1 = GET_BIT(data, 1) ^ a05;
	unsigned char crc_o_3 = b04 ^ crc_o_1;
	unsigned char a06 = c01 ^ crc_o_1;	

	unsigned char crc_o_0 = GET_BIT(data, 0) ^ a06;
	unsigned char crc_o_2 = b05 ^ crc_o_0;
	unsigned char crc_o_7 = c02 ^ crc_o_0;

	*crc = (crc_o_7 << 7) | (crc_o_6 << 6) | (crc_o_5 << 5) 
		| (crc_o_4 << 4) | (crc_o_3 << 3) | (crc_o_2 << 2) 
		|(crc_o_1 << 1) |(crc_o_0 << 0);
	
}

static void byte_crc16(unsigned short *crc,unsigned short data)
{

	unsigned short b00 = GET_BIT(data, 15) ^ GET_BIT(*crc, 15);
	unsigned short c00 = GET_BIT(*crc, 1) ^ b00;
	unsigned short a00 = GET_BIT(*crc, 14) ^ b00;

	unsigned short b01 = GET_BIT(data, 14) ^ a00;
	unsigned short c01 = GET_BIT(*crc, 0) ^ b01;
	unsigned short a01 = GET_BIT(*crc, 13) ^ b01;

	unsigned short b02 = GET_BIT(data, 13) ^ a01;
	unsigned short c02 = b00 ^ b02;
	unsigned short a02 = GET_BIT(*crc, 12) ^ b02;

	unsigned short b03 = GET_BIT(data, 12) ^ a02;
	unsigned short crc_o_14 = b03 ^ b01;
	unsigned short a03 = GET_BIT(*crc, 11) ^ b03;	

	unsigned short b04 = GET_BIT(data, 11) ^ a03;
	unsigned short crc_o_13 = b04 ^ b02;
	unsigned short a04 = GET_BIT(*crc, 10) ^ b04;

	unsigned short b05 = GET_BIT(data, 10) ^ a04;
	unsigned short crc_o_12 = b05 ^ b03;
	unsigned short a05 = GET_BIT(*crc, 9) ^ b05;

	unsigned short b06 = GET_BIT(data, 9) ^ a05;
	unsigned short crc_o_11 = b06 ^ b04;
	unsigned short a06 = GET_BIT(*crc, 8) ^ b06;

	unsigned short b07 = GET_BIT(data, 8) ^ a06;
	unsigned short crc_o_10 = b07 ^ b05;
	unsigned short a07 = GET_BIT(*crc, 7) ^ b07;

	unsigned short b08 = GET_BIT(data, 7) ^ a07;
	unsigned short crc_o_9 = b08 ^ b06;
	unsigned short a08 = GET_BIT(*crc, 6) ^ b08;

	unsigned short b09 = GET_BIT(data, 6) ^ a08;
	unsigned short crc_o_8 = b09 ^ b07;
	unsigned short a09 = GET_BIT(*crc, 5) ^ b09;

	unsigned short b10 = GET_BIT(data, 5) ^ a09;
	unsigned short crc_o_7 = b10 ^ b08;
	unsigned short a10 = GET_BIT(*crc, 4) ^ b10;

	unsigned short b11 = GET_BIT(data, 4) ^ a10;
	unsigned short crc_o_6 = b11 ^ b09;
	unsigned short a11 = GET_BIT(*crc, 3) ^ b11;

	unsigned short b12 = GET_BIT(data, 3) ^ a11;
	unsigned short crc_o_5 = b12 ^ b10;
	unsigned short a12 = GET_BIT(*crc, 2) ^ b12;
	
	unsigned short b13 = GET_BIT(data, 2) ^ a12;
	unsigned short crc_o_4 = b13 ^ b11;
	unsigned short a13 = c00 ^ b13;

	unsigned short crc_o_1 = GET_BIT(data, 1) ^ a13;
	unsigned short crc_o_3 = b12 ^ crc_o_1;
	unsigned short a14 = c01 ^ crc_o_1;	

	unsigned short crc_o_0 = GET_BIT(data, 0) ^ a14;
	unsigned short crc_o_2 = b13 ^ crc_o_0;
	unsigned short crc_o_15 = c02 ^ crc_o_0;

	*crc = (crc_o_15 << 15) | (crc_o_14 << 14) | (crc_o_13 << 13) | (crc_o_12 << 12) 
		| (crc_o_11 << 11) | (crc_o_10 << 10) |(crc_o_9 << 9) |(crc_o_8 << 8) 
		| (crc_o_7 << 7) | (crc_o_6 << 6) | (crc_o_5 << 5) | (crc_o_4 << 4) 
		| (crc_o_3 << 3) | (crc_o_2 << 2) | (crc_o_1 << 1) |(crc_o_0 << 0);
	
}

static unsigned char _get_crc8(unsigned char *data, unsigned int len)
{
	int i = 0;
	unsigned char  crc = 0xFF;
	if(data == NULL || len <= 0)
		return 0;
	for(i = 0; i < len; i++)
		byte_crc8(&crc, data[i]);
	return crc;
}

static unsigned short _get_crc16(unsigned short *data, unsigned int len)
{
	int i = 0;
	unsigned short  crc = 0xFFFF;
	
	if(data == NULL || len <= 0)
		return 0;
	for(i = 0; i < len; i++)
		{
			byte_crc16(&crc, data[i]);
		}
	return crc;
}

int count_file_crc(unsigned char crc_type, char *file_name)
{
	unsigned char buf[2064] = {0};
	FILE *fp;
	int num;
	if(file_name == NULL) {
		printf("%s: file_name == NULL\n", __FUNCTION__);
		return -1;
	}
	memset(buf, 0, 2064);
	fp = fopen(file_name, "rb");
	if (fp == NULL) {
		printf("%s: open %s fail\n", __FUNCTION__, file_name);
		return -1;
	}
	num = fread(buf, 1, 2064, fp);
	fclose(fp);
	if(num <= 0) {
		printf("%s: Don't read efficient data\n", __FUNCTION__);
		return -1;
	}
	if(crc_type == 0) {//crc8
		return _get_crc8(buf, num);
	}
	else {
		if((crc_type == 1) && (num % 2 == 0))//crc16
			return _get_crc16((unsigned short *)buf, num/2);
		else
			return -1;
	}
}
int count_crc(unsigned char crc_type, unsigned char *buf, unsigned int len)
{
	if(buf == NULL || len == 0) {
		printf("%s: Params error\n", __FUNCTION__);
		return -1;
	}
	if(crc_type == 0) {
		return _get_crc8(buf, len);
	}
	else {
		if((crc_type == 1) && (len % 2 == 0))//crc16
			return _get_crc16((unsigned short *)buf, len/2);
		else
			return -1;
	}
}

