#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "otp_bitmap.h"
#include "otp_param.h"
#include "util.h"
#include "auxfun.h"



static char *get_config_value(char *line)
{
	char delim[] = " =\"\t";
	char *token;
	char *s = line;

	strtok(line, delim);	// CONFIG_*
	token = strtok(NULL, delim);	// string after "="
//	debug_print("%s:%d:%s, %p, %p [%s]\n", __FILE__, __LINE__, __FUNCTION__ , line, token, token);
	return s + (token - line) + 2;
}

inline unsigned int hexstr2int(char *s)
{
	return (unsigned int) strtoul (s, (char **) 0, 16);
}

int get_otp_id_by_name(const char* name)
{
}

static unsigned char BIT_MASK[] = { 0x1, 0x3, 0x7, 0xF, 0x1F, 0x3F, 0x7F, 0xFF };

#if 0

int write_otp_bitmap(unsigned int msb, unsigned int lsb, unsigned char* pVal, unsigned int len, otp_bitmap_t *map)
{
    unsigned char burn_val[1024];
    unsigned long write_bytes = (msb - (lsb & ~0x7) +7) >>3;  // compute how may bytes should be set
    int i;
    
    printf("write_otp_bitmap[%d:%d]=%x,%d - %x,%d (write_bytes=%d)\n", 
            msb, lsb, msb>>3, msb & 0x7, lsb>>3, lsb & 0x7, write_bytes);
    
    if (pVal==NULL || len==0 || msb < lsb)
        return -1;            
  
    if (write_bytes > 1 &&  (lsb & 0x7))      
    {
        printf("error, un-aligned write\n");
        return -1;
    }

    // load burn value
    memset(burn_val, 0, sizeof(burn_val));
    for (i=0; i<len; i++)
        burn_val[i] = pVal[len -i -1];    // inverse order
        
    if (lsb & 0x7)        
        array_bits_shift(1, lsb & 0x7, burn_val, write_bytes);    // shift bits to make the data is byte alignemnt
        
    //dump_mem_with_text("Burn Val", burn_val, write_bytes);        
        
    i=0;
    while(lsb < msb)
    {
        //printf("lsb=%x - %x\n", lsb, msb);
        // fill in the dat byte by byte...
        unsigned int msb_byte = msb >> 3;
        unsigned int lsb_byte = lsb >> 3;                        
        unsigned char msb_bit  = msb & 0x7;  
        unsigned char lsb_bit  = lsb & 0x7;                                                    
        unsigned char bit_mask;  
        unsigned char bits;        
            
        if (msb_byte > lsb_byte)
            bits = 8 - lsb_bit;            
        else
            bits = msb_bit - lsb_bit +1;

        bit_mask = BIT_MASK[bits-1];
           
        map->otp_bitmap[lsb_byte] &= ~(bit_mask<<lsb_bit);
        map->otp_bitmap[lsb_byte] |= (burn_val[i] & bit_mask) <<lsb_bit;

        //printf("otp_bitmap[%d]=%02x\n", lsb_byte, map->otp_bitmap[lsb_byte]);
        i++;                       
        lsb += bits;        
    }

    return 0;
}

#else

int write_otp_bitmap(unsigned int msb, unsigned int lsb, unsigned char* pVal, unsigned int len, otp_bitmap_t *map)
{
    unsigned char  burn_val[1024];
    unsigned int   write_bytes = (msb - (lsb & ~0x7) +8) >>3;  // compute how may bytes should be set    
    unsigned int   msb_shift  = msb & 0x7;
    unsigned int   lsb_byte   = lsb >> 3;
    unsigned int   lsb_shift  = lsb & 0x7;    
    unsigned char* p_otp  = &map->otp_bitmap[lsb_byte];   
    unsigned char* p_data = burn_val;
    unsigned char  bit_mask;   
    int i;
    
    printf("write_otp_bitmap[%5d:%5d]=%x,%d - %x,%d (write_bytes=%d)\n", 
            msb, lsb, msb>>3, msb & 0x7, lsb>>3, lsb & 0x7, write_bytes);
    
    if (pVal==NULL || len==0 || msb < lsb) {
        printf("write_otp_bitmap failed - invalid parameter\n");
        return -1;                  
    }
        
    if ((msb +7) >>3 > sizeof(map->otp_bitmap)) 
    {
        printf("write_otp_bitmap failed - msb out of range (%d > )\n", msb, sizeof(map->otp_bitmap) <<3 );
        return -1;
    }        

    /*------------------------------------------------
      convert the value from MSB first to LSB first
    -------------------------------------------------*/
    
    if (write_bytes > sizeof(burn_val)) 
    {
        printf("write_otp_bitmap failed - out of burn data buff (has %d, need %d)\n", sizeof(burn_val), write_bytes );
        return -1;
    }
    
    memset(burn_val, 0, sizeof(burn_val));
    
    for (i=0; i<len; i++)
        burn_val[i] = pVal[len -i -1];    // inverse data order
            
    /*------------------------------------------------
      shift bits to make the data is byte alignemnt
    -------------------------------------------------*/
    if (lsb_shift)        
        array_bits_shift(1, lsb_shift, burn_val, write_bytes);    
        
    /*------------------------------------------------
      start write data
    -------------------------------------------------*/
    if (write_bytes==1)
    {
        bit_mask = BIT_MASK[msb - lsb]<<lsb_shift;           
        *p_otp &= ~bit_mask;
        *p_otp |= (*p_data)  & bit_mask;
    }
    else
    {                               
        // process 1st non byte aligned bits                
        if (lsb_shift !=0)
        {            
            bit_mask = BIT_MASK[7 - lsb_shift];           
            *p_otp &= ~(bit_mask<<lsb_shift);
            *p_otp |= (*p_data & (bit_mask <<lsb_shift));           
            write_bytes--;
            p_otp++;
            p_data++;
        }
        
        // process last non byte aligned bits                
        if (msb_shift < 0x7)      // the msb is not byte aligned
        {
            bit_mask = BIT_MASK[msb_shift];
            p_otp[write_bytes-1] &= ~(bit_mask>>(7-msb_shift));
            p_otp[write_bytes-1] |= (p_data[write_bytes-1] & bit_mask);
            write_bytes--;            
        }    
        
        // write byte aligned otp bits
        memcpy(p_otp, p_data, write_bytes);        
    }
    
    return 0;
}

#endif

int write_otp_bitmap_by_name(char* otp_name, char* val_str,  otp_bitmap_t *map)
{
    unsigned char val[1024];
    int i=0; 
    int len;
    for (i=0; i<sizeof(gOTPDescList)/sizeof(OTP_PARAM_DESC); i++)
    {
        if (strcmp(otp_name, gOTPDescList[i].name)==0)
        {         
            /*                   
            printf("OTP Name = %s, bits=%d:%d, val=%s\n", 
                gOTPDescList[i].name, 
                OTP_MSB(gOTPDescList[i].id),
                OTP_LSB(gOTPDescList[i].id),
                val_str);
            */
            len = _str_to_bytes(val_str, val, sizeof(val));
            
            if (len <0)
            {
                printf("Invalud value = %s\n", val_str);
                return -1;
            }
            
            write_otp_bitmap(OTP_MSB(gOTPDescList[i].id), OTP_LSB(gOTPDescList[i].id), val, len, map);
            
            //dump_mem_with_text("val", val, len);
            
            return 0;
        }
    }
    
    printf("Unknow OTP name = %s\n", otp_name);
    
    return -1;
}



int parse_config(const char *f_name, otp_bitmap_t *map)
{
	FILE *fp;
	char *line;
	const char prefix[] = "#CONFIG_";
	char name_str[256];	// config name in each line
	char *value_str;	// config value in each line
	int res = -1;
	int i;

	debug_print("%s(%s)\n", __FUNCTION__, f_name);

	fp = fopen(f_name, "r");
	if (fp == NULL) {
		printf("open file %s failed\n", f_name);
		return -1;
	}

#define LINE_SIZE 4096
	line = (char *)malloc(LINE_SIZE);
	if (line == NULL) {
		printf("malloc(%d) failed\n", LINE_SIZE);
		fclose(fp);
		return -1;
	}

	// get each line
	while (fgets(line, LINE_SIZE, fp) != NULL) {
		// skip comment or other stuff
		if (strncmp(prefix, line, strlen(prefix)) != 0)
			continue;
#undef LINE_SIZE
		/* parse each CONFIG_* line... */		                    
		value_str = get_config_value(line);
        if (sscanf(line, "#CONFIG_%s", name_str)!=1)
            continue;
        for(i=0; i<strlen(name_str); i++)                    
        {        
            if (name_str[i]=='=')
                name_str[i]=0;            
        }		
		debug_print("%s:%d:%s, %s\n", __FILE__, __LINE__, __FUNCTION__, name_str);
		//debug_print("%s:%d:%s, %s\n", __FILE__, __LINE__, __FUNCTION__, line);
		debug_print("%s:%d:%s, %s\n", __FILE__, __LINE__, __FUNCTION__, value_str);


        write_otp_bitmap_by_name(name_str, value_str, map);                       
	}

	res = 0;
	
final:
	if (res != 0)
		printf("parse error: %s\n", line);

	free(line);
	fclose(fp);
	return res;
}

