#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "otp_bitmap.h"
#include "otp_info.h"
#include "util.h"
#include "auxfun.h"

static const char *msg_2state_enc_dis[2] = {
	"enable",
	"disable"
};

static const char *msg_3state_enc_tmp_dis[3] = {
	"enable",
	"disable, can re-enable temporarily with passwd",
	"disable"
};

static const char *msg_2state_dis_enc[2] = {
	"disable",
	"enable"
};

static const char *msg_2state_lock[2] = {
	"lock is disable",
	"lock is enable"
};

static const char *msg_boot_flash_type[4] = {
	"NOR SPI",
	"NAND",
	"eMMC",
	"SPI NAND"
};

static const char *msg_nand_ecc_size[2] = {
	"6 bit",
	"12 bit"
};

static const char *msg_secure_type[4] = {
	"Nagra secure boot",
	"xxx security boot enable",
	"Realtek secure boot",
	"DCAS secure boot"
};

static const char *msg_otp_clk[2] = {
	"27MHz crystal",
	"free running ring oscillator"
};

static const char *msg_rcic_length[2] = {
	"1MB",
	"512KB"
};

static const char *msg_cw_sram_cpu_access[4] = {
	"KCPU & SCPU R/W",
	"KCPU R/W",
	"KCPU & SCPU W/O",
	"KCPU W/O"
};

static const char *msg_cp_key_rd_en[2] = {
	"enable KCPU & SCPU RD",
	"disable KCPU & SCPU RD"
};

static const char *msg_rtk_deg_mode[4] = {
	"enable",
	"disable but can be re-enabled by password temporarily",
	"disable (backdoor is not allowed)",
	"disable (backdoor is not allowed)"
};

static const char *msg_arm_core_number_selection[4] = {
	"4 core",
	"3 core",
	"2 core",
	"1 core"
};

static const char *msg_tv_stb_mode_sel[2] = {
	"CA on TV chipset",
	"pure STB box chipset with TV function"
};

static const char *msg_nagra_master_root_key_select[2] = {
	"MasterRootKeyStorage1",
	"MasterRootKeyStorage2"
};

static const char *msg_cert_scan_disable[2] = {
	"CERT IP scan chain is enable",
	"CERT IP scan chain is disable"
};

static const char *msg_cert_disable[2] = {
	"CERT IP is enable",
	"CERT IP is disable"
};

static const char *msg_nagra_dsc_disable[2] = {
	"Nagra DSC IP is enable",
	"Nagra DSC IP is disable"
};

static const char *msg_nagra_dvbcsa3_disable[2] = {
	"Nagra DVB-CSA3 IP is enable",
	"Nagra DVB-CSA3 IP is disable"
};

static const char *msg_nagra_secret_debug_interface_deactivation[2] = {
	"secret debug interface is open",
	"secret debug interface is closed"
};

static const char *msg_nagra_descrambler_selection[2] = {
	"Realtek's CP engine",
	"Nagra Crypto Engine"
};

static const char *msg_nagra_dbgif_protection_level[3] = {
	"open",
	"pwassword protected",
	"completely close"
};



static const char * decode(int value, const char** msg, int num_of_state)
{
	if ((value < 0) || (msg == NULL) || (value >= num_of_state))
		return "illegal";
		
    //if (msg==msg_boot_flash_type)		
    //{
    //    printf("value=%d, msg[%d]=%s\n", value, value, msg[value]);
    //}        

	return msg[value];
}

static void print_keys(const char *name, unsigned int *ptr, unsigned int len)
{
	unsigned int i;
	unsigned char sb = len & 0x3;

	printf("%s:\n", name);
	for (i = 0; i < (len / 4); i++) {
		printf("\t0x%08X\n", ptr[i]);
	}
	
	if (sb)
    {
        unsigned int mask[4] = {
            0x00000000,
            0x000000FF,
            0x0000FFFF,
            0x00FFFFFF,
        };
        
        printf("\t0x%08X\n", ptr[i] & mask[sb]);
    }	    
}


static char *show_bits(char *buf, unsigned int value, unsigned int bit_width)
{
	char *idx = buf;
	unsigned int mask = 1 << (sizeof(unsigned int) * 8 - 1);	// match exactly one bit
	unsigned int printable = (1 << bit_width) - 1;	// filter out unwanted bits

	//debug_print("mask: 0x%x, value: 0x%x, printable: 0x%x\n", mask, value, printable);
	while (mask) {
		if (mask & printable) {
			*idx++ = (value & mask) ? '1' : '0';
		}

		mask >>= 1;
	}

	*idx = '\0';
	return buf;
}


static unsigned char BIT_MASK[] = { 0x1, 0x3, 0x7, 0xF, 0x1F, 0x3F, 0x7F, 0xFF };

int read_otp_bitmap(unsigned int msb, unsigned int lsb, unsigned char* pVal, unsigned int len, otp_bitmap_t *map)
{
    unsigned char  otp_val[1024];
    unsigned int   read_bytes = (msb - (lsb & ~0x7) +8) >>3;  // compute how may bytes should be set    
    unsigned int   msb_shift  = msb & 0x7;
    unsigned int   lsb_byte   = lsb >> 3;
    unsigned int   lsb_shift  = lsb & 0x7;    
    unsigned char* p_otp  = &map->otp_bitmap[lsb_byte];       
    unsigned char  bit_mask;   
    int i;
    
#if 0    
    printf("read_otp_bitmap[%5d:%5d]=%x,%d - %x,%d (read_bytes=%d)\n", 
            msb, lsb, msb>>3, msb & 0x7, lsb>>3, lsb & 0x7, read_bytes);
#endif
    
    if (pVal==NULL || len==0 || msb < lsb) {
        printf("read_otp_bitmap failed - invalid parameter\n");
        return -1;                  
    }
        
    if ((msb +7) >>3 > sizeof(map->otp_bitmap)) 
    {
        printf("read_otp_bitmap failed - msb out of range (%d > )\n", msb, sizeof(map->otp_bitmap) <<3 );
        return -1;
    }        
                
    memcpy(otp_val, p_otp, read_bytes);     // read data bytes...
       
    if (read_bytes > len) 
    {
        printf("read_otp_bitmap failed - out of burn data buff (has %d, need %d)\n", len, read_bytes );
        return -1;
    }    
    
    if (msb_shift < 0x7)        
        otp_val[read_bytes-1] &= BIT_MASK[msb_shift];  // mask un-wanted msb bits  
        
    if (lsb_shift)                                     // ignore un-wanged lsb bits
        array_bits_shift(0, lsb_shift, otp_val, read_bytes);
        
    read_bytes = (msb - lsb + 8) >> 3;                    
    
    /*------------------------------------------------
      convert the value from LSB first to MSB first
    -------------------------------------------------*/
    for (i=0; i<read_bytes; i++)
        pVal[i] = otp_val[read_bytes - i -1] ;    // inverse data order    
    
    return read_bytes;
}


int print_bitmap(otp_bitmap_t *map)
{
    int i;            

	char buf[sizeof(unsigned int) * 8 + 1];

	if (map == NULL)
		return -1;
		
    dump_mem_with_text("OTP value", map->otp_bitmap, sizeof(map->otp_bitmap));
  
    for (i=0; i<sizeof(gOTPDescList)/sizeof(OTP_PARAM_DESC); i++)
    {                
        unsigned char val[1024];
        int len;        
                                
        len = read_otp_bitmap(OTP_MSB(gOTPDescList[i].id), OTP_LSB(gOTPDescList[i].id), val, sizeof(val), map);
                
        if (len>=0)
        {
            int bit_sz = OTP_MSB(gOTPDescList[i].id)-OTP_LSB(gOTPDescList[i].id) +1;
            char* str;
            
#if 1            
            printf("\n#====================================\n");
            printf("# bit %u~%u (%d bits) \n", OTP_MSB(gOTPDescList[i].id), OTP_LSB(gOTPDescList[i].id), bit_sz);
            printf("#====================================\n");       
            str = gOTPDescList[i].description;
#if 0			
            while(*str !=0)
            {
                printf("# %s \n", str);                  
                str += strlen(str);
            }
#else
	    	if(str != NULL) {
			int i = 0;
			printf("#");
			for(i = 0; i < strlen(str); i++) {
				if(str[i] == 0x11)
					;
				else if(str[i] == 0x12)
					printf("\n#");
				else
					printf("%c", str[i]);
				
			}	
			printf("\n");
		}
		else {
			printf("#\n");
		}
#endif
            printf("#====================================\n");
#endif
            printf("OTP[%5d:%5d] %s[%d]= [ ", 
                OTP_MSB(gOTPDescList[i].id),
                OTP_LSB(gOTPDescList[i].id),                
                gOTPDescList[i].name, bit_sz);      
                
            if (bit_sz<=4)                       
                printf("%X", val[0]);
            else                
                dump_array(val, len);
            printf(" ]\n");            
        }            
    }  
                                                                                                                                         
	return 0;	
}

