#include <common.h>
#include <asm/arch/system.h>
#include <platform_lib/board/gpio.h>
#include <drivers/otp/rtk_otp_debug.h>
#include <drivers/otp/rtk_otp_util.h>
#include <drivers/otp/rtk_otp_sw_crc.h>
#include <platform_lib/board/pcb_mgr.h>

/*burn some regions with few bits*/
typedef struct _OTP_REGION_BURN_INFO {
    unsigned int ofst;
    unsigned int msb;
    unsigned int lsb;
    unsigned int val;
    unsigned int in_use;
    unsigned char name[32];
} OTP_REGION_BURN_INFO;
OTP_REGION_BURN_INFO otp_region_table[] = {
    {0, 0, 0, 0, 0, "none"},
};

typedef struct _OTP_PINMUX_INFO {
    unsigned int start_bit;
    unsigned int end_bit;
    unsigned int pinmux_reg;
    unsigned int pinmux_get_value;
    unsigned int pinmux_set_value;
    unsigned int gpio_pin_type;
    unsigned int gpio_pin_num;
    unsigned int gpio_set_value;
    unsigned int gpio_save_value;
    unsigned int is_init;
} OTP_PINMUX_INFO;

OTP_PINMUX_INFO otp_pinmux_info = {0};

extern unsigned char builtin_otp_map_array[];
extern unsigned char builtin_otp_map_array_end;

#ifdef CONFIG_SYS_SUPPORT_MULTITHREAD
u32 get_random(u32 limit)
{
    static u32 seed = 0;
    u32 next, res;

    if( seed == 0 )
        seed = (arch_counter_get_ticks() + 51284);
    next = seed;

    next *= 1103515245;
    next += 12345;
    res = (unsigned int) (next / 65536) % 2048;

    next *= 1103515245;
    next += 12345;
    res <<= 10;
    res ^= (unsigned int) (next / 65536) % 1024;

    next *= 1103515245;
    next += 12345;
    res <<= 10;
    res ^= (unsigned int) (next / 65536) % 1024;

    seed = next;

    return res % limit;
}
#endif
extern unsigned int rtd_part_inl(unsigned int addr, unsigned int B, unsigned int C);
extern void rtd_part_outl(unsigned int addr, unsigned int B, unsigned int C, unsigned int D);
/*Before burn otp, we may operate gpio to supply power*/
int otp_prepare_burn(void)
{
    int ret = 0;
    if(otp_pinmux_info.is_init == 0) {
        unsigned long long ullPcbMgrValue = 0;
        if (pcb_mgr_get_enum_info_byname("PIN_OTP_POWER_INFO", &ullPcbMgrValue) != 0) {
            OTP_WARNING("no PIN_OTP_POWER_INFO!\n");
            return -1;
        }
        if(ullPcbMgrValue == 0)
            return -1;
        otp_pinmux_info.start_bit = GET_PINMUX_START_BIT(ullPcbMgrValue);
        otp_pinmux_info.end_bit = GET_PINMUX_END_BIT(ullPcbMgrValue);
        otp_pinmux_info.pinmux_reg = GET_PINMUX_REG(ullPcbMgrValue);
        otp_pinmux_info.pinmux_set_value = GET_PINMUX_GPIO_VALUE(ullPcbMgrValue);
        otp_pinmux_info.gpio_pin_type = GET_PINMUX_GPIO_TYPE(ullPcbMgrValue);
        otp_pinmux_info.gpio_pin_num = GET_PINMUX_GPIO_INDEX(ullPcbMgrValue);
        otp_pinmux_info.gpio_set_value = GET_PINMUX_GPIO_INIT_VALUE(ullPcbMgrValue);
        otp_pinmux_info.is_init = 1;
    }
    otp_pinmux_info.pinmux_get_value = rtd_part_inl(otp_pinmux_info.pinmux_reg,
                                       otp_pinmux_info.end_bit, otp_pinmux_info.start_bit);
    rtd_part_outl(otp_pinmux_info.pinmux_reg, otp_pinmux_info.end_bit,
                  otp_pinmux_info.start_bit, otp_pinmux_info.pinmux_set_value);

    if (otp_pinmux_info.gpio_pin_type == PCB_PIN_TYPE_GPIO) {
        if((otp_pinmux_info.gpio_save_value = getGPIO(otp_pinmux_info.gpio_pin_num)) != 1) {
            setGPIO(otp_pinmux_info.gpio_pin_num, otp_pinmux_info.gpio_set_value);
            udelay(100000);
        }
    } else if(otp_pinmux_info.gpio_pin_type == PCB_PIN_TYPE_ISO_GPIO) {
        if((otp_pinmux_info.gpio_save_value = getISOGPIO(otp_pinmux_info.gpio_pin_num)) != 1) {
            setISOGPIO(otp_pinmux_info.gpio_pin_num, otp_pinmux_info.gpio_set_value);
            udelay(100000);
        }
    } else {
        ret = -1;
    }
//set_gpio_fail:
    return ret;

}

/*Before burn otp, we may operate gpio to power donw power*/
int otp_post_burn(void)
{
    if(otp_pinmux_info.is_init == 1) {
        if(otp_pinmux_info.gpio_pin_type == PCB_PIN_TYPE_GPIO) {
            setGPIO(otp_pinmux_info.gpio_pin_num, otp_pinmux_info.gpio_save_value);
            udelay(100000);
        } else if(otp_pinmux_info.gpio_pin_type == PCB_PIN_TYPE_ISO_GPIO) {
            setISOGPIO(otp_pinmux_info.gpio_pin_num, otp_pinmux_info.gpio_save_value);
            udelay(100000);
        }
        rtd_part_outl(otp_pinmux_info.pinmux_reg, otp_pinmux_info.end_bit,
                      otp_pinmux_info.start_bit, otp_pinmux_info.pinmux_get_value);
    }
    return 0;
}

int otp_check_and_program_special_regions(void)
{

    int table_len = 0;
    table_len = sizeof(otp_region_table) / sizeof(otp_region_table[0]);
    OTP_DBG("table_len == %d\n", table_len);
    if(table_len > 0) {
        int i = 0;
        otp_prepare_burn();
        for(i = 0; i < table_len; i++) {
            OTP_DBG("ofst:%lx, msb:%lu, lsb:%lu, val:%lu,in_use:%lu\n", otp_region_table[i].ofst,
                otp_region_table[i].msb, otp_region_table[i].lsb, otp_region_table[i].val, 
                otp_region_table[i].in_use);
            if(otp_region_table[i].in_use) {
                int ret = 0;
                unsigned int value = 0x0;
                ret = otp_read_bits(otp_region_table[i].ofst, otp_region_table[i].msb,
                                    otp_region_table[i].lsb, &value);
                if(ret != 0) {
                    OTP_WARNING("Read REGION[%s] failed\n", otp_region_table[i].name);
                    continue;
                }
                if(value == 0) {
                    ret = otp_write_bits(otp_region_table[i].ofst, otp_region_table[i].msb,
                                         otp_region_table[i].lsb, otp_region_table[i].val);
                    if(ret != 0)
                        OTP_WARNING("Program REGION[%s] failed\n", otp_region_table[i].name);
                }
                value = 0x0;
                ret = otp_read_bits(otp_region_table[i].ofst, otp_region_table[i].msb,
                                    otp_region_table[i].lsb, &value);
                if(ret == 0)
                    OTP_WARNING("REGION[%s] == 0x%08x\n",
                                otp_region_table[i].name, value);

            }
        }
        otp_post_burn();
    }
    return 0;
}




/*-----------------------------------------------------------------------------
 * Func : otp_is_ever_programed
 *
 * Desc : test otp is ever programed
 *
 * Parm :
 * Retn : OTP words
 *-----------------------------------------------------------------------------*/

int otp_is_ever_programed(void)
{
    unsigned int value = 0;
    otp_read(0x40, &value);
    if(value != 0)
        return 1;
    return 0;

}




/*-----------------------------------------------------------------------------
 * Func : otp_program_with_bitmap
 *
 * Desc : program with default bitmap
 *
 * Parm :
 * Retn : OTP words
 *-----------------------------------------------------------------------------*/
int otp_program_with_bitmap(void)
{
    int len = 0;
    int ret = 0;
    unsigned int *map_head = (unsigned int *)builtin_otp_map_array;
    len = (&builtin_otp_map_array_end - builtin_otp_map_array) / sizeof(unsigned int);
    OTP_DBG("len == %d\n", len);
    otp_prepare_burn();
    ret = otp_program_region(0, map_head, len);
    otp_post_burn();
    return ret;
}

/*Some regions may need be dealed with  specially*/
static void otp_pre_deal_with_specail_regions(void)
{
#ifdef CONFIG_SYS_SUPPORT_MULTITHREAD
    random(1000000);
#endif
}


/*-----------------------------------------------------------------------------
 * Func : otp_program_with_bitmap_with_verify
 *
 * Desc : program with default bitmap
 *
 * Parm :
 * Retn : OTP words
 *-----------------------------------------------------------------------------*/

int otp_program_with_bitmap_with_verify(void)
{
    int len = 0;
    int ret = 0;
    unsigned int *map_head = (unsigned int *)builtin_otp_map_array;

    len = (&builtin_otp_map_array_end - builtin_otp_map_array) / sizeof(unsigned int);

    otp_pre_deal_with_specail_regions();

    OTP_WARNING("Now Auto burn otp bitmap.....\n");

    otp_prepare_burn();
    ret = otp_program_region(0, map_head, len);
    otp_post_burn();

    if (ret == 0) {
        unsigned int hw_crc = 0x0;
        unsigned int sw_crc = 0x1;
        count_sw_crc(CRC_TYPE_CRC16, builtin_otp_map_array,
                     len * sizeof(unsigned int), &sw_crc);
        otp_crc_auto_region(&hw_crc);
        if (hw_crc == (unsigned long)sw_crc) {
            OTP_WARNING("otp auto burn sucess\n");
            ret = 0;
        } else {
            OTP_WARNING("otp auto burn done, but crc check failed, hw_crc = %08x, sw_crc= %08x\n", hw_crc, sw_crc);
            ret = -1;
        }
    } else {
        OTP_WARNING("otp auto burn failed\n");
    }
    return ret;
}
