/*
 * USB HOST EHCI Controller
 */
#include <common.h>
#include <malloc.h>
#include <usb.h>
#include <watchdog.h>
#include <asm-generic/errno.h>
#include <linux/compat.h>
#include <rbus/usb2_top_reg.h>
#include <rbus/sys_reg_reg.h>

#include "ehci.h"
#include "ehci_reg.h"

extern int rtk_usb2_hw_iniitalize(void);
extern int rtk_usb2_hw_disable(void);

unsigned long ehci_base_register[] = {
    HMAC_HOST_ehci_ctrl_start_reg,
};

int ehci_hcd_init(int index, struct ehci_hccr **hccr, struct ehci_hcor **hcor)
{
    if (index >= CONFIG_USB_MAX_CONTROLLER_COUNT) {
        printf( "index out of range. index=%d\n"
                " MAX_CONTROLLER_COUNT=%d\n"
                " MAX_EHCI_CONTROLLER=%d \n",
                index,
                CONFIG_USB_MAX_CONTROLLER_COUNT,
                CONFIG_USB_EHCI_CONTROLLER_COUNT);
        return -1;
    }

    printf("#@# %s(%d) ehci mac%d\n", __func__, __LINE__, index);
    rtk_usb2_hw_iniitalize();

    /* map ehci capacity registers and  operational registers */
    *hccr = (struct ehci_hccr *)ehci_base_register[index];
    *hcor = (struct ehci_hcor *)((uint32_t) *hccr
                + HC_LENGTH(ehci_readl((unsigned int *)&(*hccr)->cr_capbase)));

    debug("ehci: init hccr %x and hcor %x hc_length %d\n",
            (uint32_t)*hccr, (uint32_t)*hcor,
            (uint32_t)HC_LENGTH(ehci_readl((unsigned int *)&(*hccr)->cr_capbase)));

    return 0;
}

int ehci_hcd_stop(int index)
{
    rtk_usb2_hw_disable();
    return 0;
}
