#include "emmc_wrap_reg.h"

#ifndef _EMMC_WRAP_PARSER_H_
#define _EMMC_WRAP_PARSER_H_

/*
 * Rbus Register mapping (0x1801_0800~0x1801_0890)
 */

/* Non-security CPU registers: 0x1801_0800 ~ 0x1801_080C (all CPU can access) */
#define EM_SCPU_SEL                 EMMC_WRAP_emmc_scpu_sel               //0x18010800
#define EM_INT_STATUS               EMMC_WRAP_emmc_int_status             //0x18010804
#define EM_INT_MASK                 EMMC_WRAP_emmc_int_mask               //0x18010808
#define EM_INT_DUMMY0               EMMC_WRAP_emmc_int_dummy0             //0x1801080C

/* Security CPU (KCPU) registers: 0x1801_0810 ~ 0x1802_082C (only KCPU can access) */
#define EM_KCPU_SEL                 EMMC_WRAP_emmc_kcpu_sel               //0x18010810
#define EM_START_PROTECT_ADDR       EMMC_WRAP_emmc_start_protect_addr     //0x18010814
#define EM_END_PROTECT_ADDR         EMMC_WRAP_emmc_end_protect_addr       //0x18010818
#define EM_INT_STATUS_KCPU          EMMC_WRAP_emmc_int_status_kcpu        //0x1801081C
#define EM_INT_MASK_KCPU            EMMC_WRAP_emmc_int_mask_kcpu          //0x18010820

/*
 * Only KCPU can access when kcpu_status = 1 (1801_0830~1801_08FF)
 * All CPUs can access when kcpu_status = 0
*/
#define EM_WARP_CTRL                EMMC_WRAP_emmc_wrap_ctrl              //0x18010830
#define EM_WARP_STATUS0             EMMC_WRAP_emmc_wrap_status0           //0x18010834
#define EM_WARP_STATUS1             EMMC_WRAP_emmc_wrap_status1           //0x18010838

/* MBIST */
#define EM_IP_BIST                  EMMC_WRAP_emmc_ip_bist                //0x18010840
#define EM_WARP_BIST                EMMC_WRAP_emmc_wrap_bist              //0x18010844

/* DS tune */
#define EM_DS_TUNE_CTRL             EMMC_WRAP_ds_tune_ctrl                //0x18010850

/* Delay_cal */
#define EM_HALF_CYCLE_CAL_EN        EMMC_WRAP_half_cycle_cal_en           //0x1801088C
#define EM_HALF_CYCLE_CAL_RESULT    EMMC_WRAP_half_cycle_cal_result       //0x18010890


/* ====== Register Bit define ====== */
/* Wapper */
//EM_SCPU_SEL               //0x18010800
#define PS_SCPU_STOP                   EMMC_WRAP_emmc_scpu_sel_scpu_stop_mask                      //_BIT2  0x00000004
#define PS_SCPU_STATUS                 EMMC_WRAP_emmc_scpu_sel_scpu_status_mask                    //_BIT1  0x00000002
#define PS_SCPU_SEL                    EMMC_WRAP_emmc_scpu_sel_scpu_sel_mask                       //_BIT0  0x00000001

//EM_INT_STATUS             //0x18010804
#define PS_DMA_RD_DONE_S               EMMC_WRAP_emmc_int_status_dma_rd_done_scpu_mask             //_BIT2  0x00000004
#define PS_DMA_WR_DONE_S               EMMC_WRAP_emmc_int_status_dma_wr_done_scpu_mask             //_BIT1  0x00000002
#define PS_STOP_BY_KCPU_FORCE_S        EMMC_WRAP_emmc_int_status_stop_by_kcpu_force_scpu_mask      //_BIT0  0x00000001

//EM_INT_MASK               //0x18010808
#define PS_DMA_RD_DONE_MASK_S          EMMC_WRAP_emmc_int_mask_dma_rd_done_mask_scpu_mask          //_BIT2  0x00000004
#define PS_DMA_WD_DONE_MASK_S          EMMC_WRAP_emmc_int_mask_dma_wr_done_mask_scpu_mask          //_BIT1  0x00000002
#define PS_STOP_BY_KCPU_FORCE_MASK_S   EMMC_WRAP_emmc_int_mask_stop_by_kcpu_force_mask_scpu_mask   //_BIT0  0x00000001

//EM_KCPU_SEL               //0x18010810
#define PS_KCPU_FORCE_EN               EMMC_WRAP_emmc_kcpu_sel_kcpu_force_en_mask                  //_BIT3  0x00000008
#define PS_PROTECT_ADDR_EN             EMMC_WRAP_emmc_kcpu_sel_protect_addr_en_mask                //_BIT2  0x00000004
#define PS_KCPU_STATUS                 EMMC_WRAP_emmc_kcpu_sel_kcpu_status_mask                    //_BIT1  0x00000002
#define PS_KCPU_SEL                    EMMC_WRAP_emmc_kcpu_sel_kcpu_sel_mask                       //_BIT0  0x00000001

//EM_INT_STATUS_KCPU        //0x1801081C
#define PS_DMA_RD_DONE_K               EMMC_WRAP_emmc_int_status_kcpu_dma_rd_done_kcpu_mask        //_BIT2  0x00000004
#define PS_DMA_WR_DONE_K               EMMC_WRAP_emmc_int_status_kcpu_dma_wr_done_kcpu_mask        //_BIT1  0x00000002
#define PS_PROTECT_HIT_K               EMMC_WRAP_emmc_int_status_kcpu_protect_hit_kcpu_mask        //_BIT0  0x00000001

//EM_INT_MASK_KCPU          //0x18010820
#define PS_DMA_RD_DONE_MASK_K          EMMC_WRAP_emmc_int_mask_kcpu_dma_rd_done_mask_kcpu_mask     //_BIT2  0x00000004
#define PS_DMA_WR_DONE_MASK_K          EMMC_WRAP_emmc_int_mask_kcpu_dma_wr_done_mask_kcpu_mask     //_BIT1  0x00000002
#define PS_PROTECT_HIT_MASK_K          EMMC_WRAP_emmc_int_mask_kcpu_protect_hit_mask_kcpu_mask     //_BIT0  0x00000001

#endif // _EMMC_WRAP_PARSER_H_