#include <common.h>
#include <asm/arch/io.h>
#include <rbus/se_reg.h>
#include <sys_reg_reg.h>
#include <se_RegCmd.h>
#include <rtk_gdma_export.h>
#include <SeStretchUtils.h>
#include <panel_config_parameter.h>
#include <system.h>

//======================================
//for public test
#if !defined(SE_INSTALL_UI_SURFACE1_ADDR)
#define SE_INSTALL_UI_SURFACE1_ADDR         (0x02100000)   // size is 0x7e9000
#endif
#if !defined(SE_INSTALL_UI_SURFACE2_ADDR)
#define SE_INSTALL_UI_SURFACE2_ADDR         (0x02900000)   // size is 0x465000
#endif
#if !defined(SE_CMD_BUFFER_START_ADDR)
#define SE_CMD_BUFFER_START_ADDR            (0x03100000)   // size is 0x4000
#endif

#define SE_CMD_SHIFT            (0xC0000000)
#define SE_CMD_SIZE             (0x4000)
#define SE_CMD_DELAY_MS_ATOMIC  (1)
#define SE_SURFACE_PITCH        (1920*4)
#define SE_SURFACE_W            (1920)
#define SE_SURFACE_H            (1080)

#define SE_TEXT_SHIFT           (0)


#define FONT_COLOR              (0x0000FeFF)   //blue
#define FONT_COLOR_AA           (0x0000Fedd)   //blue

#define FONT_COLOR_AA2          (0x0000f1aa)   //blue

#define FONT_COLOR_BG           (0x99CCFFFF)

#define TEXT_SURFACE            SE_INSTALL_UI_SURFACE2_ADDR
#define WORD_HEIGHT             (80)

#define SE_EXT
#define SEINFO_OPCODE_NOP	(0xf)
#define SE_EXT_RSLT_ORDER	(0x1d)
//#define TEXTDEBUG
#undef TEXTDEBUG
#if defined(TEXTDEBUG)
#define print_textdebug(fmt,args...)	 	printf(fmt,## args)
#else
#define print_textdebug(fmt,args...)
#endif

#define SEBIT(x)                 (1 << (x))
static bool _gSEState = false;  // false:Not Ready,   true:Ready,
static uint32_t* _gCmdBuffer = (unsigned int*)SE_CMD_BUFFER_START_ADDR;
static uint32_t _gtextwidthshift = SE_TEXT_SHIFT;


void Draw_ClearMsg(void);

int DisplayArea(void);

bool se_state_Guard(void) {
    if(_gSEState == false) {
        printf("SE is not READY, skip...");
        return false;
    }
    return true;
}

#define SEBIT(x)                 (1 << (x))
void se_power(void) {
	printf("%s.....", __func__);

	rtd_outl(SYS_REG_SYS_SRST1_reg, SEBIT(SYS_REG_SYS_SRST1_rstn_se_shift) | SYS_REG_SYS_CLKEN1_write_data_mask);
	udelay(10);

	rtd_outl(SYS_REG_SYS_CLKEN1_reg, SEBIT(SYS_REG_SYS_CLKEN1_clken_se_shift) | SYS_REG_SYS_CLKEN1_write_data_mask);
	rtd_outl(SYS_REG_SYS_CLKEN1_reg, SEBIT(SYS_REG_SYS_CLKEN1_clken_se2_shift) | SYS_REG_SYS_CLKEN1_write_data_mask);
	udelay(10);

	rtd_outl(SYS_REG_SYS_CLKEN1_reg, SEBIT(SYS_REG_SYS_CLKEN1_clken_se_shift));
	rtd_outl(SYS_REG_SYS_CLKEN1_reg, SEBIT(SYS_REG_SYS_CLKEN1_clken_se2_shift));
	udelay(10);

	rtd_outl(SYS_REG_SYS_SRST1_reg, SEBIT(SYS_REG_SYS_SRST1_rstn_se_shift));
	udelay(10);

	rtd_outl(SYS_REG_SYS_SRST1_reg, SEBIT(SYS_REG_SYS_SRST1_rstn_se_shift) | SYS_REG_SYS_SRST1_write_data_mask);
	udelay(10);

	rtd_outl(SYS_REG_SYS_CLKEN1_reg, SEBIT(SYS_REG_SYS_CLKEN1_clken_se_shift) | SYS_REG_SYS_CLKEN1_write_data_mask);
	rtd_outl(SYS_REG_SYS_CLKEN1_reg, SEBIT(SYS_REG_SYS_CLKEN1_clken_se2_shift) | SYS_REG_SYS_CLKEN1_write_data_mask);

	printf("OK \n");

}

void se_hw_reg_init(void) {
    se_ctrl_RBUS se_ctrl;
    printf("%s.....", __func__);
    rtd_outl(SE_CMDBASE_0_reg, SE_CMD_BUFFER_START_ADDR);
    rtd_outl(SE_CMDLMT_0_reg, SE_CMD_BUFFER_START_ADDR+SE_CMD_SIZE);
    rtd_outl(SE_CMDRPTR_0_reg, SE_CMD_BUFFER_START_ADDR);
    rtd_outl(SE_CMDWPTR_0_reg, SE_CMD_BUFFER_START_ADDR);
    rtd_outl(SE_INSTCNT_L_0_reg, 0x0);
    rtd_outl(SE_INSTCNT_H_0_reg, 0x0);
    se_ctrl.write_data = 1;
    se_ctrl.endiswap = 1;
    rtd_outl(SE_CTRL_0_reg, se_ctrl.regValue);

    printf("OK \n");
}

void se_checkfinish(void){
	int delaycounter=0;
	bool ret=false;

	while(!ret) {
        if((rtd_inl(SE_IDLE_0_reg) & 0x1) &&
                !(rtd_inl(SE_CTRL_0_reg) & 0x2) &&
                (rtd_inl(SE_CMDRPTR_0_reg) == rtd_inl(SE_CMDWPTR_0_reg))){
            ret = true;
            break;
  		}
        mdelay(SE_CMD_DELAY_MS_ATOMIC);
        delaycounter++;
        if(delaycounter >=100) {
            printf("Error! %s timeout! call baker.. idle=%08x, ctrl=%08x, rptr=%08x, wptr=%08x.\n",
			__func__, rtd_inl(SE_IDLE_0_reg), rtd_inl(SE_CTRL_0_reg)
			,rtd_inl(SE_CMDRPTR_0_reg), rtd_inl(SE_CMDWPTR_0_reg));
            ret = true;
            break;
        }
	}
}

void se_writecmd(int32_t lCommandLength) {
    uint32_t dwDataCounter = 0;
    uint32_t wrptr = 0;
    uint32_t u32Alignment = 0;
    se_ctrl_RBUS se_ctrl;

    lCommandLength = lCommandLength*4;   //bytes base

    rtd_outl(SE_CMDRPTR_0_reg, SE_CMD_BUFFER_START_ADDR);
    rtd_outl(SE_CMDWPTR_0_reg, SE_CMD_BUFFER_START_ADDR);
    wrptr = rtd_inl(SE_CMDWPTR_0_reg) + lCommandLength;   //bytes base

    if(se_state_Guard() == false) {
        printf("%s! \n",__func__);
        return;
    }

    if(lCommandLength%16 != 0)
        u32Alignment = ((lCommandLength^0xf)&0xf) + 1;

    //printf("%s %d wrptr = %08x, lCommandLength =%d bytes, u32Alignment = %d bytes\n", __func__, __LINE__,wrptr, lCommandLength,u32Alignment);
    for(dwDataCounter = 0; dwDataCounter < (uint32_t)u32Alignment; dwDataCounter += sizeof(uint32_t)) {
        *(uint32_t*)wrptr = 0xf;
        wrptr += sizeof(uint32_t);
        if(wrptr >= (SE_CMD_BUFFER_START_ADDR + SE_CMD_SIZE))
			wrptr = SE_CMD_BUFFER_START_ADDR;
    }

    //flush_dcache_all();
    flush_dcache_range(rtd_inl(SE_CMDRPTR_0_reg), wrptr);
    //printf("%s -  flush from %08x to %08x \n", __func__,rtd_inl(SE_CMDRPTR_0_reg), wrptr);


	rtd_outl(SE_CMDWPTR_0_reg, wrptr);

    se_ctrl.write_data = 1;
    se_ctrl.go = 1;
    se_ctrl.endiswap = 1;
    rtd_outl(SE_CTRL_0_reg, se_ctrl.regValue);
}

int se_module_init(void) {
    uint32_t u32Ctrl=0, u32Idle=0;
    se_power();
    se_hw_reg_init();
    u32Ctrl = rtd_inl(SE_CTRL_0_reg);
    u32Idle = rtd_inl(SE_IDLE_0_reg);
    if(u32Ctrl == 0x4 && u32Idle == 0x1) {
        _gSEState = true;
        return 0;
    }
    printf("%s...Fail!", __func__);
    return -1;

};


//=============================================================================Draw API


static uint32_t fontarray[240][60] = {
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0xf00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0xf00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0xf00, 0x0, 0x3fc0000, 0x3f00000, 0x3, 0xfe000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f, 0xfc000000, 0x0, 0x3ffe000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3f, 0xfc000000},
    {0x1f, 0x80000001, 0xf80fc000, 0x7f8, 0xfe00000, 0xffe0, 0x0, 0x7ff0000, 0x7f00000, 0xf, 0xffc00000, 0x3f00, 0x3, 0xe000000f, 0x80000000, 0x3e000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7f00, 0xff, 0x80000000, 0xf8, 0x0, 0x1ff800, 0x0, 0x7fc0000, 0x0, 0x0, 0x0, 0x0, 0x7fc, 0x0, 0x0, 0x0, 0xff8000, 0x0, 0x7fc00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff, 0xff800000, 0x0, 0x7ffffe00, 0x0, 0x1ff0, 0x0, 0x7fffff, 0xc0000000, 0x1ff, 0xff800000},
    {0x1f, 0x80000001, 0xf80fc000, 0x7f0, 0xfe00000, 0x7fffc, 0x0, 0x1fff8000, 0x7f00000, 0x3f, 0xffe00000, 0x3f00, 0x3, 0xe000000f, 0x80000000, 0x3e000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7e00, 0x7ff, 0xf0000000, 0xf8, 0x0, 0xffff00, 0x0, 0x3fff8000, 0x0, 0x7e0000, 0x1fff, 0xfff00000, 0x3fff, 0x80000000, 0x3fffffff, 0xe0000000, 0x7fff000, 0x1, 0xfff80000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1ff, 0xffe00000, 0x1, 0xffffffc0, 0x0, 0x3ff8, 0x0, 0x7fffff, 0xfc000000, 0x7ff, 0xffe00000},
    {0x1f, 0x80000001, 0xf80fc000, 0x7f0, 0xfe00000, 0xfffff, 0x0, 0x1fffc000, 0xfe00000, 0x7f, 0xfff00000, 0x3f00, 0x7, 0xe000000f, 0xc0000000, 0x3e000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7e00, 0xfff, 0xfc000000, 0x1f8, 0x0, 0x3ffffc0, 0x0, 0xffffc000, 0x0, 0x7e0000, 0x1fff, 0xfff00000, 0xffff, 0xc0000000, 0x3fffffff, 0xe0000000, 0x1ffffc00, 0x7, 0xfffe0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7ff, 0xfff00000, 0x7, 0xfffffff0, 0x0, 0x3ff8, 0x0, 0x7fffff, 0xff000000, 0x1fff, 0xfff00000},
    {0x1f, 0x80000001, 0xf80fc000, 0x7f0, 0xfe00000, 0x3fffff, 0xc0000000, 0x3fffe000, 0xfe00000, 0xff, 0xfff80000, 0x3f00, 0xf, 0xc0000007, 0xe0000000, 0x3e000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7e00, 0x3fff, 0xfe000000, 0x3f8, 0x0, 0x7ffffe0, 0x1, 0xfffff000, 0x0, 0xfe0000, 0x1fff, 0xfff00000, 0x1ffff, 0xe0000000, 0x3fffffff, 0xe0000000, 0x3ffffe00, 0xf, 0xffff0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfff, 0xfff80000, 0x1f, 0xfffffffc, 0x0, 0x3ff8, 0x0, 0x7fffff, 0xff800000, 0x7fff, 0xfffc0000},
    {0x1f, 0x80000001, 0xf80fc000, 0x7f0, 0xfe00000, 0x7fffff, 0xe0000000, 0x7f8ff000, 0x1fc00000, 0xff, 0xfffc0000, 0x3f00, 0xf, 0xc0000007, 0xe0000000, 0x3e000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfe00, 0x7fff, 0xff000000, 0x3f8, 0x0, 0xffffff0, 0x3, 0xfffff800, 0x0, 0x1fe0000, 0x1fff, 0xfff00000, 0x3ffff, 0xf0000000, 0x3fffffff, 0xe0000000, 0x7fffff00, 0x1f, 0xffff8000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfff, 0xfffc0000, 0x3f, 0xfffffffe, 0x0, 0x7ffc, 0x0, 0x7fffff, 0xffc00000, 0xffff, 0xfffe0000},
    {0x1f, 0x80000001, 0xf80fc000, 0xff0, 0x1fe00000, 0xffffff, 0xe0000000, 0x7f07f000, 0x1fc00000, 0x1ff, 0xfffc0000, 0x3f00, 0x1f, 0x80000003, 0xf0000000, 0x3e000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfc00, 0x7fff, 0xff800000, 0x7f8, 0x0, 0x1ffffff8, 0x7, 0xfffffc00, 0x0, 0x3fe0000, 0x1fff, 0xfff00000, 0x7ffff, 0xf8000000, 0x3fffffff, 0xe0000000, 0xffffff80, 0x3f, 0xffffc000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fff, 0xfffe0000, 0xff, 0xffffffff, 0x0, 0x7ffc, 0x0, 0x7fffff, 0xffe00000, 0x1ffff, 0xffff0000},
    {0x1f, 0x80000001, 0xf80fc000, 0xfe0, 0x1fc00000, 0xffffff, 0xf0000000, 0x7e03f000, 0x3f800000, 0x3ff, 0xdffe0000, 0x3f00, 0x3f, 0x80000003, 0xf8000000, 0x383e0e0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfc00, 0xffff, 0xff800000, 0xff8, 0x0, 0x3ffffffc, 0xf, 0xfffffc00, 0x0, 0x3fe0000, 0x1fff, 0xfff00000, 0xfffff, 0xfc000000, 0x3fffffff, 0xe0000000, 0xffffff80, 0x7f, 0xffffe000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3fff, 0x3ffe0000, 0x1ff, 0xffe3ffff, 0xc0000000, 0x7ffc, 0x0, 0x7fffff, 0xffe00000, 0x3fffc, 0x7fff0000},
    {0x1f, 0x80000001, 0xf80fc000, 0xfe0, 0x1fc00000, 0x1ffffff, 0xf8000000, 0xfe03f800, 0x3f800000, 0x3fe, 0x3fe0000, 0x3f00, 0x3f, 0x1, 0xf8000000, 0x3f3e7e0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fc00, 0x1ffff, 0xffc00000, 0x1ff8, 0x0, 0x3ffffffe, 0xf, 0xfffffe00, 0x0, 0x7fe0000, 0x3fff, 0xfff00000, 0x1fffff, 0xfc000000, 0x3fffffff, 0xe0000001, 0xffffffc0, 0xff, 0xfffff000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3ff0, 0x7ff0000, 0x3ff, 0xf8000fff, 0xe0000000, 0xfffe, 0x0, 0x7f003f, 0xfff00000, 0x7ffc0, 0x7ff8000},
    {0x1f, 0x80000001, 0xf80fc000, 0xfe0, 0x1fc00000, 0x1ff8f1f, 0xf8000000, 0xfc01f800, 0x7f000000, 0x3fc, 0x1fe0000, 0x3f00, 0x7f, 0x1, 0xfc000000, 0x7fffff0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fc00, 0x1ff80, 0xffc00000, 0x3ff8, 0x0, 0x7ff00ffe, 0x1f, 0xf803fe00, 0x0, 0xffe0000, 0x3fff, 0xfff00000, 0x1ff807, 0xfe000000, 0x3fffffff, 0xe0000001, 0xff007fc0, 0xff, 0xe01ff000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7fe0, 0x1ff0000, 0x7ff, 0xc00001ff, 0xe0000000, 0xfffe, 0x0, 0x7f0000, 0x3ff00000, 0x7ff00, 0x1ffc000},
    {0x1f, 0x80000001, 0xf80fc000, 0xfe0, 0x1fc00000, 0x3fe0f0f, 0xfc000000, 0xfc01f800, 0x7f000000, 0x3f8, 0xfe0000, 0x3f00, 0x7e, 0x0, 0xfc000000, 0x7fffff0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f800, 0x3fe00, 0x3fe00000, 0xfff8, 0x0, 0x7fc003fe, 0x1f, 0xf001ff00, 0x0, 0xffe0000, 0x3f80, 0x0, 0x3fe003, 0xfe000000, 0x1f, 0xc0000003, 0xfe003fe0, 0x1ff, 0x8007f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7fc0, 0xff8000, 0xfff, 0x7f, 0xf0000000, 0xfefe, 0x0, 0x7f0000, 0x1ff80000, 0xffc00, 0xffc000},
    {0x1f, 0x80000001, 0xf80fc000, 0x1fe0, 0x3fc00000, 0x3fc0f03, 0xfc000000, 0xfc01f800, 0xfe000000, 0x3f8, 0xfe0000, 0x3f00, 0xfe, 0x0, 0xfe000000, 0x7fffff0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x1f800, 0x3fc00, 0x1fe00000, 0x1fff8, 0x0, 0x7f8001ff, 0x3f, 0xe000ff00, 0x0, 0x1ffe0000, 0x3f80, 0x0, 0x3fc000, 0xff000000, 0x3f, 0x80000003, 0xfc001fe0, 0x1ff, 0x3f800, 0x0, 0x0, 0x0, 0xc, 0x0, 0x0, 0x600, 0x0, 0x7f80, 0x7f8000, 0x1ffc, 0x3f, 0xf8000000, 0x1feff, 0x0, 0x7f0000, 0xff80000, 0x1ff800, 0x7fe000},
    {0x1f, 0x80000001, 0xf80fc000, 0x1fc0, 0x3f800000, 0x3fc0f03, 0xfc000000, 0xfc01f800, 0xfe000000, 0x3f8, 0xfe0000, 0x3f00, 0xfc, 0x0, 0x7e000000, 0x7fffff0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x3f800, 0x3f800, 0xfe00000, 0x7fff8, 0x0, 0xff0000ff, 0x3f, 0xc0007f00, 0x0, 0x3ffe0000, 0x3f80, 0x0, 0x7f8000, 0xff000000, 0x7f, 0x3, 0xf8000fe0, 0x3fe, 0x3fc00, 0x0, 0x0, 0x0, 0x3c, 0x0, 0x0, 0x780, 0x0, 0xff00, 0x7f8000, 0x1ff8, 0xf, 0xfc000000, 0x1feff, 0x0, 0x7f0000, 0x7f80000, 0x1ff000, 0x3fe000},
    {0x1f, 0x80000001, 0xf80fc000, 0x1fc0, 0x3f800000, 0x3f80f01, 0xfc000000, 0xfc01f801, 0xfc000000, 0x3f8, 0xfe0000, 0x3f00, 0x1fc, 0x0, 0x7f000000, 0x1ffffc0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x3f000, 0x7f800, 0xff00000, 0xffff8, 0x0, 0xff00007f, 0x3f, 0xc0007f00, 0x0, 0x7ffe0000, 0x3f80, 0x0, 0x7f8000, 0x7f000000, 0x7f, 0x3, 0xf8000fe0, 0x3fc, 0x1fc00, 0x0, 0x0, 0x0, 0xfc, 0x0, 0x0, 0x7e0, 0x0, 0xff00, 0x3f8000, 0x3fe0, 0x7, 0xfc000000, 0x3fc7f, 0x80000000, 0x7f0000, 0x7f80000, 0x3fe000, 0x3ff000},
    {0x1f, 0x80000001, 0xf80fc000, 0x1fc0, 0x3f800000, 0x3f80f01, 0xfc000000, 0xfc01f801, 0xfc000000, 0x3f8, 0xfe0000, 0x3f00, 0x1f8, 0x0, 0x3f000000, 0x1ffc00, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x3f000, 0x7f000, 0x7f00000, 0x3ffff8, 0x0, 0xfe00007f, 0x3f, 0x80007f00, 0x0, 0x7ffe0000, 0x7f80, 0x0, 0xff0000, 0x7f000000, 0xfe, 0x3, 0xf8000fe0, 0x3fc, 0xfc00, 0x0, 0x0, 0x0, 0x3fc, 0x0, 0x0, 0x7f8, 0x0, 0xfe00, 0x3f8000, 0x7fc0, 0x3, 0xfe000000, 0x3fc7f, 0x80000000, 0x7f0000, 0x7f80000, 0x3fe000, 0x1ff000},
    {0x1f, 0x80000001, 0xf80fc000, 0x1fc0, 0x7f800000, 0x3f80f01, 0xf0000000, 0xfc01f803, 0xf8000000, 0x3f8, 0xfe0000, 0x3f00, 0x1f8, 0x0, 0x3f000000, 0xff800, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x3f000, 0x7f000, 0x7f00000, 0x3ffdf8, 0x0, 0xfe00007f, 0x3f, 0x80007f00, 0x0, 0xfffe0000, 0x7f00, 0x0, 0xff0000, 0x7f000000, 0x1fc, 0x3, 0xf8000fe0, 0x3f8, 0xfc00, 0x0, 0x0, 0x0, 0x1ffc, 0x0, 0x0, 0x7ff, 0x0, 0xfe00, 0x3f8000, 0x7f80, 0x1f80001, 0xfe000000, 0x3fc7f, 0x80000000, 0x7f0000, 0x7f80000, 0x3fc000, 0x1ff000},
    {0x1f, 0x80000001, 0xf80fc000, 0xffffff, 0xfffc0000, 0x3f80f00, 0x0, 0xfc01f803, 0xf8000000, 0x3fc, 0x1fe0000, 0x3f00, 0x3f8, 0x0, 0x3f800000, 0x1ffc00, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x7f000, 0x7f000, 0x7f00000, 0x3ff1f8, 0x0, 0xfe00007f, 0xf, 0x80007f00, 0x1, 0xff7e0000, 0x7f00, 0x0, 0xfe0000, 0x0, 0x1fc, 0x3, 0xf8000fe0, 0x3f8, 0xfe00, 0x1f80, 0x7e, 0x0, 0x7ffc, 0x0, 0x0, 0x7ff, 0xc0000000, 0xfe00, 0x3f8000, 0xff00, 0xffe0fe0, 0xff000000, 0x7f83f, 0xc0000000, 0x7f0000, 0x7f80000, 0x7fc000, 0xff000},
    {0x1f, 0x80000001, 0xf80fc000, 0xffffff, 0xfffc0000, 0x3f80f00, 0x0, 0xfc01f807, 0xf0000000, 0x3fe, 0x3fc0000, 0x3f00, 0x3f0, 0x0, 0x1f800000, 0x3ffe00, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x7e000, 0xff000, 0x7f80000, 0x3fe1f8, 0x0, 0x7e00007f, 0x0, 0x7f00, 0x1, 0xfe7e0000, 0x7f00, 0x0, 0xfe0000, 0x0, 0x3f8, 0x3, 0xf8000fe0, 0x3f8, 0xfe00, 0x1f80, 0x7e, 0x0, 0x1fffc, 0x7f, 0xffffffc0, 0x7ff, 0xf0000000, 0x7e00, 0x3f8000, 0xfe00, 0x3fff8fe0, 0x7f000000, 0x7f83f, 0xc0000000, 0x7f0000, 0x7f80000, 0x7f8000, 0xfc000},
    {0x1f, 0x80000000, 0xf8078000, 0xffffff, 0xfffc0000, 0x3f80f00, 0x0, 0xfc01f807, 0xf0000000, 0x1fe, 0x7fc0000, 0x3e00, 0x7f0, 0x0, 0x1fc00000, 0x7f7f00, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x7e000, 0xfe000, 0x3f80000, 0x3f81f8, 0x0, 0x7f, 0x0, 0x7f00, 0x3, 0xfe7e0000, 0x7f00, 0x0, 0xfe0000, 0x0, 0x7f0, 0x3, 0xf8000fe0, 0x3f8, 0xfe00, 0x1f80, 0x7e, 0x0, 0x7fffc, 0x7f, 0xffffffc0, 0x7ff, 0xfc000000, 0x0, 0x3f8000, 0x1fe00, 0x7fffcfe0, 0x7f800000, 0x7f83f, 0xc0000000, 0x7f0000, 0x7f80000, 0x7f8000, 0xc0000},
    {0x1f, 0x80000000, 0xf0078000, 0xffffff, 0xfffc0000, 0x3f80f00, 0x0, 0xfc01f80f, 0xe0000000, 0x1ff, 0xff80000, 0x1e00, 0x7f0, 0x0, 0x1fc00000, 0xff7f80, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0xfe000, 0xfe000, 0x3f80000, 0x3f01f8, 0x0, 0x7f, 0x0, 0xff00, 0x7, 0xfc7e0000, 0xff00, 0x0, 0x1fc0000, 0x0, 0x7f0, 0x3, 0xfc001fe0, 0x3f8, 0xfe00, 0x1f80, 0x7e, 0x0, 0x1ffff0, 0x7f, 0xffffffc0, 0x1ff, 0xff000000, 0x0, 0x3f8000, 0x1fc00, 0xffffefe0, 0x3f800000, 0xff01f, 0xe0000000, 0x7f0000, 0x7f80000, 0xff8000, 0x0},
    {0x1f, 0x80000000, 0xf0078000, 0xffffff, 0xfffc0000, 0x3fc0f00, 0x0, 0xfc01f80f, 0xe0000000, 0xff, 0xbff00000, 0x1e00, 0x7f0, 0x0, 0x1fc00000, 0xfe3fc0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0xfe000, 0xfe000, 0x3f80000, 0x3c01f8, 0x0, 0x7f, 0x0, 0x1fe00, 0xf, 0xf87e0000, 0xff1f, 0xf8000000, 0x1fc0000, 0x0, 0xfe0, 0x3, 0xfc001fe0, 0x3f8, 0xfe00, 0x1f80, 0x7e, 0x0, 0x7fff80, 0x7f, 0xffffffc0, 0x3f, 0xffc00000, 0x0, 0x7f8000, 0x3f801, 0xffffefe0, 0x3f800000, 0xff01f, 0xe0000000, 0x7f0000, 0xff00000, 0xff0000, 0x0},
    {0x1f, 0x80000000, 0xf0078000, 0xffffff, 0xfffc0000, 0x3fe0f00, 0x0, 0xfc01f81f, 0xc0000000, 0x7f, 0xfff00000, 0x1e00, 0x7e0, 0x0, 0xfc00000, 0x1fc1fc0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0xfe000, 0xfe000, 0x3f80000, 0x3001f8, 0x0, 0xff, 0x0, 0x7fe00, 0xf, 0xf87e0000, 0xfe7f, 0xfe000000, 0x1fc0ffe, 0x0, 0xfe0, 0x1, 0xff007fc0, 0x3f8, 0xfe00, 0x1f80, 0x7e, 0x0, 0x3fffe00, 0x7f, 0xffffffc0, 0xf, 0xfff80000, 0x0, 0x7f8000, 0x3f803, 0xffffffe0, 0x1fc00000, 0xfe00f, 0xe0000000, 0x7f0000, 0xff00000, 0xff0000, 0x0},
    {0x1f, 0x80000000, 0x0, 0xffffff, 0xfffc0000, 0x1ff8f00, 0x0, 0x7e03f01f, 0xc0000000, 0x7f, 0xffe00000, 0x0, 0xfe0, 0x0, 0xfe00000, 0xfc1f80, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0xfc000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0xff, 0x0, 0x1ffc00, 0x1f, 0xf07e0000, 0xffff, 0xff800000, 0x1fc3fff, 0x80000000, 0x1fc0, 0x1, 0xffc1ffc0, 0x3f8, 0xfe00, 0x1f80, 0x7e, 0x0, 0xffff000, 0x7f, 0xffffffc0, 0x1, 0xfffe0000, 0x0, 0xff0000, 0x7f007, 0xffffffc0, 0x1fc00000, 0x1fe00f, 0xf0000000, 0x7f0000, 0x3fe00000, 0xff0000, 0x0},
    {0x1f, 0x80000000, 0x0, 0xffffff, 0xfffc0000, 0x1ffff00, 0x0, 0x7f07f03f, 0x80000000, 0x3f, 0xff800000, 0x0, 0xfe0, 0x0, 0xfe00000, 0x380e00, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x1fc000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x1fe, 0x0, 0x3fff800, 0x3f, 0xe07e0000, 0xffff, 0xffc00000, 0x1fc7fff, 0xe0000000, 0x1fc0, 0x0, 0xffffff80, 0x3f8, 0xfe00, 0x1f80, 0x7e, 0x0, 0x3fffc000, 0x7f, 0xffffffc0, 0x0, 0x7fff8000, 0x0, 0x1ff0000, 0x7f00f, 0xfe07ffc0, 0xfc00000, 0x1fe00f, 0xf0000000, 0x7f0000, 0xffe00000, 0xff0000, 0x0},
    {0x1f, 0x80000000, 0x0, 0x7f00, 0xfe000000, 0xffff00, 0x0, 0x7f8ff03f, 0x80000000, 0x1f, 0xff000000, 0x0, 0xfe0, 0x0, 0xfe00000, 0x180c00, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x1f8000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x3fe, 0x0, 0x3fff000, 0x3f, 0xe07e0000, 0xffff, 0xffe00000, 0x1fdffff, 0xf0000000, 0x3f80, 0x0, 0x7fffff00, 0x3f8, 0xfe00, 0x0, 0x0, 0x0, 0xffff0000, 0x7f, 0xffffffc0, 0x0, 0x1fffe000, 0x0, 0x3fe0000, 0x7e00f, 0xf803ffc0, 0xfc00000, 0x3fc007, 0xf8000000, 0x7fffff, 0xffc00000, 0xff0000, 0x0},
    {0x1f, 0x80000000, 0x0, 0x7f01, 0xfe000000, 0x7fffe0, 0x0, 0x3fffe07f, 0x0, 0x7f, 0xfc000000, 0x0, 0xfc0, 0x0, 0x7e00000, 0x0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x1f8000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x3fc, 0x0, 0x3ffe000, 0x7f, 0xc07e0000, 0x1ffff, 0xfff00000, 0x1ffffff, 0xf8000000, 0x3f80, 0x0, 0x3ffffe00, 0x3fc, 0x1fe00, 0x0, 0x0, 0x3, 0xfff80000, 0x0, 0x0, 0x0, 0x3fff800, 0x0, 0x7fc0000, 0xfe01f, 0xf001ffc0, 0xfc00000, 0x3fc007, 0xf8000000, 0x7fffff, 0xff000000, 0xff0000, 0x0},
    {0x1f, 0x80000000, 0x0, 0xff01, 0xfe000000, 0x3ffffe, 0x0, 0x3fffc07f, 0x0, 0xff, 0xfc000000, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x7ff, 0xfffffe00, 0x0, 0x0, 0x0, 0x0, 0x1f8000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x7fc, 0x0, 0x3fff800, 0xff, 0x807e0000, 0x1ffff, 0xfff80000, 0x1ffffff, 0xfc000000, 0x7f00, 0x0, 0x1ffffc00, 0x3fe, 0x3fe00, 0x0, 0x0, 0x7, 0xffe00000, 0x0, 0x0, 0x0, 0xfffc00, 0x0, 0xffc0000, 0xfc01f, 0xe000ffc0, 0xfc00000, 0x3fc007, 0xf8000000, 0x7fffff, 0xfe000000, 0xff0000, 0x0},
    {0x1f, 0x80000000, 0x0, 0xfe01, 0xfc000000, 0x1fffff, 0x80000000, 0x1fff80fe, 0xfc0000, 0x3ff, 0xfc000000, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x7ff, 0xfffffe00, 0x0, 0x0, 0x0, 0x0, 0x3f8000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0xff8, 0x0, 0x3fffc00, 0xff, 0x7e0000, 0x1ffe0, 0x7ff80000, 0x1ffffff, 0xfc000000, 0x7f00, 0x0, 0x7ffffe00, 0x1fe, 0x3fe00, 0x0, 0x0, 0x7, 0xff800000, 0x0, 0x0, 0x0, 0x3ffc00, 0x0, 0x1ff80000, 0xfc03f, 0xc0007f80, 0xfc00000, 0x7f8003, 0xfc000000, 0x7fffff, 0xff800000, 0xff0000, 0x0},
    {0x1f, 0x80000000, 0x0, 0xfe01, 0xfc000000, 0x7ffff, 0xc0000000, 0x7ff00fe, 0x3ff8000, 0x7ff, 0xfe000000, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x7ff, 0xfffffe00, 0x0, 0x0, 0x0, 0x0, 0x3f0000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x1ff0, 0x0, 0x3fffe00, 0x1ff, 0x7e0000, 0x1ff80, 0xffc0000, 0x1fffc1f, 0xfe000000, 0xfe00, 0x0, 0xffffff80, 0x1ff, 0x800ffe00, 0x0, 0x0, 0x7, 0xfc000000, 0x0, 0x0, 0x0, 0x7fc00, 0x0, 0x3ff00000, 0xfc03f, 0xc0007f80, 0xfc00000, 0x7f8003, 0xfc000000, 0x7fffff, 0xffe00000, 0xff0000, 0x0},
    {0xf, 0x0, 0x0, 0xfe01, 0xfc000000, 0x1ffff, 0xe0000000, 0x3fc01fc, 0x7ffc000, 0xfff, 0xff000000, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x7ff, 0xfffffe00, 0x0, 0x0, 0x0, 0x0, 0x3f0000, 0xfc000, 0x3f80000, 0x1f8, 0x0, 0x3ff0, 0x0, 0x3ffff00, 0x3fe, 0x7e0000, 0x1ff00, 0x7fc0000, 0x1ffe003, 0xff000000, 0xfe00, 0x1, 0xffffffc0, 0xff, 0xc01ffe00, 0x0, 0x0, 0x7, 0xf8000000, 0x0, 0x0, 0x0, 0x3fc00, 0x0, 0x7fe00000, 0x1fc07f, 0x80007f80, 0xfc00000, 0x7f8003, 0xfc000000, 0x7fffff, 0xfff00000, 0xff0000, 0x0},
    {0xf, 0x0, 0x0, 0xfe03, 0xfc000000, 0x7fff, 0xf0000000, 0x1fc, 0xfffe000, 0x1ffd, 0xff00c000, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x7ff, 0xfffffe00, 0x0, 0x0, 0x0, 0x0, 0x7f0000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x7fe0, 0x0, 0x1ff80, 0x7fc, 0x7e0000, 0x1fe00, 0x3fc0000, 0x1ff8001, 0xff000000, 0x1fe00, 0x3, 0xffc3ffe0, 0xff, 0xfffffe00, 0x0, 0x0, 0x7, 0xfe000000, 0x0, 0x0, 0x0, 0xffc00, 0x0, 0xffc00000, 0x1f807f, 0x80007f80, 0xfc00000, 0xff0001, 0xfe000000, 0x7fffff, 0xfff00000, 0xff0000, 0x0},
    {0xf, 0x0, 0x0, 0x1fe03, 0xf8000000, 0xfff, 0xf8000000, 0x3f8, 0x1ffff000, 0x3ff0, 0xff80fc00, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x7ff, 0xfffffe00, 0x0, 0x0, 0x0, 0x0, 0x7e0000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0xffc0, 0x0, 0x7fc0, 0x7fc, 0x7e0000, 0x1fc00, 0x1fe0000, 0x1ff0000, 0xff000000, 0x1fc00, 0x3, 0xfe007fe0, 0x7f, 0xfffffe00, 0x0, 0x0, 0x7, 0xffc00000, 0x0, 0x0, 0x0, 0x7ffc00, 0x1, 0xff800000, 0x1f807f, 0x7f80, 0xfc00000, 0xff0001, 0xfe000000, 0x7f0003, 0xfff80000, 0xff0000, 0x0},
    {0xf, 0x0, 0x0, 0x1fc03, 0xf8000000, 0xfff, 0xfc000000, 0x3f8, 0x1ffff000, 0x3fe0, 0x7fc0fe00, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x7ff, 0xfffffe00, 0x0, 0x0, 0x0, 0x0, 0x7e0000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x1ff80, 0x0, 0x3fc0, 0xff8, 0x7e0000, 0x0, 0x1fe0000, 0x1fe0000, 0x7f800000, 0x1fc00, 0x7, 0xfc001ff0, 0x7f, 0xfffffe00, 0x0, 0x0, 0x7, 0xfff00000, 0x0, 0x0, 0x0, 0x1fffc00, 0x1, 0xff000000, 0x1f80ff, 0x7f00, 0xfc00000, 0xffffff, 0xfe000000, 0x7f0000, 0x1ffc0000, 0xff0000, 0x0},
    {0xf, 0x0, 0x0, 0x1fc03, 0xf8000000, 0xf3f, 0xfc000000, 0x7f0, 0x3f83f800, 0x7fc0, 0x3fe1fe00, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x7ff, 0xfffffe00, 0x0, 0x7ff, 0xff000000, 0x0, 0x7e0000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x3ff00, 0x0, 0x3fe0, 0x1ff0, 0x7e0000, 0x0, 0xfe0000, 0x1fe0000, 0x7f800000, 0x3f800, 0x7, 0xf8000ff0, 0x3f, 0xfffefe00, 0x0, 0x0, 0x1, 0xfffe0000, 0x0, 0x0, 0x0, 0xffff000, 0x3, 0xfe000000, 0x1f80ff, 0x7f00, 0xfc00000, 0x1ffffff, 0xff000000, 0x7f0000, 0xffc0000, 0xff0000, 0x0},
    {0xf, 0x0, 0x0, 0x1fc03, 0xf8000000, 0xf0f, 0xfc000000, 0x7f0, 0x3f01f800, 0x7f80, 0x3fe1fe00, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x7ff, 0xff000000, 0x0, 0xfe0000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x7fe00, 0x0, 0x1fe0, 0x1fe0, 0x7e0000, 0x0, 0xfe0000, 0x1fc0000, 0x3f800000, 0x3f800, 0xf, 0xf00007f8, 0x1f, 0xfffcfe00, 0x0, 0x0, 0x0, 0x7fff8000, 0x7f, 0xffffffc0, 0x0, 0x3fffc000, 0x3, 0xfc000000, 0x1f80fe, 0x7f00, 0xfc00000, 0x1ffffff, 0xff000000, 0x7f0000, 0x7fe0000, 0xff0000, 0x0},
    {0xf, 0x0, 0x0, 0xffffff, 0xfffc0000, 0xf03, 0xfe000000, 0xfe0, 0x7f01fc00, 0xff00, 0x1ff3fe00, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x7ff, 0xff000000, 0x0, 0xfc0000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0xffc00, 0x0, 0xfe0, 0x3fe0, 0x7e0000, 0x0, 0xfe0000, 0x1fc0000, 0x3f800000, 0x7f800, 0xf, 0xf00007f8, 0x7, 0xfff8fe00, 0x0, 0x0, 0x0, 0x1fffe000, 0x7f, 0xffffffc0, 0x0, 0xffff0000, 0x7, 0xf8000000, 0x1f80fe, 0x7f00, 0xfc00000, 0x3ffffff, 0xff800000, 0x7f0000, 0x3fe0000, 0xff0000, 0x0},
    {0xf, 0x0, 0x0, 0xffffff, 0xfffc0000, 0xf01, 0xfe000000, 0xfe0, 0x7e00fc00, 0xff00, 0xffbfc00, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x7ff, 0xff000000, 0x0, 0xfc0000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x1ff800, 0x0, 0xfe0, 0x3fff, 0xfffff800, 0x0, 0xfe0000, 0x1fc0000, 0x3f800000, 0x7f000, 0xf, 0xe00003f8, 0x3, 0xffe0fe00, 0x0, 0x0, 0x0, 0x7fffc00, 0x7f, 0xffffffc0, 0x7, 0xfffc0000, 0x7, 0xf8000000, 0x1f80fe, 0x7f00, 0x1fc00000, 0x3ffffff, 0xff800000, 0x7f0000, 0x1fe0000, 0xff0000, 0x60000},
    {0xf, 0x0, 0x0, 0xffffff, 0xfffc0000, 0xf00, 0xfe000000, 0x1fc0, 0x7e00fc00, 0xfe00, 0x7fffc00, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x7ff, 0xff000000, 0x0, 0x1fc0000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x3ff000, 0x0, 0xfe0, 0x3fff, 0xfffff800, 0x0, 0xfe0000, 0x1fc0000, 0x3f800000, 0x7f000, 0xf, 0xe00003f8, 0x0, 0x7f80fe00, 0x0, 0x0, 0x0, 0x1ffff00, 0x7f, 0xffffffc0, 0x1f, 0xfff00000, 0x7, 0xf0000000, 0x1f80fe, 0x7e00, 0x1fc00000, 0x3ffffff, 0xff800000, 0x7f0000, 0x1fe0000, 0xff8000, 0x7e000},
    {0xf, 0x0, 0x0, 0xffffff, 0xfffc0000, 0xf00, 0xfe000000, 0x1fc0, 0x7e00fc00, 0xfe00, 0x7fffc00, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x7ff, 0xff000000, 0x0, 0x1fc0000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x7fe000, 0x0, 0xfe0, 0x3fff, 0xfffff800, 0x0, 0xfe0000, 0x1fc0000, 0x3f800000, 0x7f000, 0xf, 0xe00003f8, 0x0, 0xfe00, 0x0, 0x0, 0x0, 0x3fffc0, 0x7f, 0xffffffc0, 0x7f, 0xff800000, 0x7, 0xf0000000, 0x1f80fe, 0xfe00, 0x3f800000, 0x7ffffff, 0xffc00000, 0x7f0000, 0x1fe0000, 0x7f8000, 0x7f800},
    {0xf, 0x0, 0x0, 0xffffff, 0xfffc0000, 0x700f00, 0xfe000000, 0x3f80, 0x7e00fc00, 0xfe00, 0x3fff800, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x7ff, 0xff000000, 0x0, 0x1f80000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x1ffc000, 0x0, 0xfe0, 0x3fff, 0xfffff800, 0x0, 0xfe0000, 0x1fc0000, 0x3f800000, 0xfe000, 0xf, 0xe00003f8, 0x0, 0xfe00, 0x0, 0x0, 0x0, 0xffff8, 0x7f, 0xffffffc0, 0x3ff, 0xfe000000, 0x7, 0xf0000000, 0x1f80fe, 0xfe00, 0x3f800000, 0x7f80000, 0x3fc00000, 0x7f0000, 0x1fe0000, 0x7f8000, 0x7f800},
    {0xf, 0x0, 0x0, 0xffffff, 0xfffc0000, 0x7f00f00, 0xfe000000, 0x3f80, 0x7e00fc00, 0xfe00, 0x1fff800, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x1f80000, 0xfe000, 0x3f80000, 0x1f8, 0x0, 0x3ff8000, 0x3, 0x80000fe0, 0x3fff, 0xfffff800, 0x0, 0xfe0000, 0x1fc0000, 0x3f800000, 0xfe000, 0xf, 0xe00003f8, 0x0, 0x1fc00, 0x0, 0x0, 0x0, 0x3fffc, 0x7f, 0xffffffc0, 0x7ff, 0xf8000000, 0x7, 0xf0000000, 0x1f80fe, 0xfe00, 0x7f800000, 0x7f80000, 0x3fc00000, 0x7f0000, 0x1fe0000, 0x7fc000, 0xff800},
    {0xf, 0x0, 0x0, 0xffffff, 0xfffc0000, 0x7f00f00, 0xfe000000, 0x7f00, 0x7e00fc00, 0xfe00, 0xfff000, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x3f80000, 0xff000, 0x7f80000, 0x1f8, 0x0, 0x7ff0000, 0x3f, 0x80000fe0, 0x3fff, 0xfffff800, 0x3f800, 0xfe0000, 0x1fc0000, 0x3f800000, 0xfe000, 0xf, 0xe00003f8, 0x0, 0x1fc00, 0x0, 0x0, 0x0, 0xfffc, 0x0, 0x0, 0x7ff, 0xe0000000, 0xf, 0xf0000000, 0x1f80fe, 0x1fe00, 0x7f000000, 0xff80000, 0x3fe00000, 0x7f0000, 0x1fe0000, 0x3fc000, 0xff800},
    {0xf, 0x0, 0x0, 0x7f00f, 0xe0000000, 0x7f00f00, 0xfe000000, 0x7f00, 0x7e00fc00, 0xfe00, 0xffe000, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x3f00000, 0x7f000, 0x7f00000, 0x1f8, 0x0, 0x7fe0000, 0x3f, 0x80000fe0, 0x3fff, 0xfffff800, 0x3f800, 0xfe0000, 0xfc0000, 0x3f800000, 0x1fe000, 0xf, 0xe00003f8, 0x1f8, 0x1fc00, 0x0, 0x0, 0x0, 0x3ffc, 0x0, 0x0, 0x7ff, 0x80000000, 0xf, 0xf0000000, 0x1f80fe, 0x3fc00, 0xff000000, 0xff00000, 0x1fe00000, 0x7f0000, 0x1fe0000, 0x3fe000, 0x1ff000},
    {0xf, 0x0, 0x0, 0x7f00f, 0xe0000000, 0x7f00f00, 0xfe000000, 0x7e00, 0x7e00fc00, 0xfe00, 0x7fe000, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x3f00000, 0x7f000, 0x7f00000, 0x1f8, 0x0, 0xffc0000, 0x3f, 0x80000fe0, 0x0, 0x7e0000, 0x3f800, 0xfe0000, 0xfe0000, 0x3f800000, 0x1fc000, 0xf, 0xe00003f8, 0x1fc, 0x3fc00, 0x0, 0x0, 0x0, 0xffc, 0x0, 0x0, 0x7fe, 0x0, 0xf, 0xf0000000, 0x1f80fe, 0x3fc01, 0xfe000000, 0xff00000, 0x1fe00000, 0x7f0000, 0x1fe0000, 0x3fe000, 0x1ff000},
    {0x0, 0x0, 0x0, 0x7f00f, 0xe0000000, 0x7f80f00, 0xfe000000, 0xfe00, 0x7e00fc00, 0xff00, 0x7ff000, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x3f00000, 0x7f800, 0xff00000, 0x1f8, 0x0, 0x1ff80000, 0x3f, 0xc0001fe0, 0x0, 0x7e0000, 0x3f800, 0x1fe0000, 0xfe0000, 0x7f800000, 0x1fc000, 0xf, 0xe00003f8, 0x1fc, 0x3f800, 0x0, 0x0, 0x0, 0x1fc, 0x0, 0x0, 0x7f0, 0x0, 0x0, 0x0, 0x1f80ff, 0x7fc03, 0xfe000000, 0x1ff00000, 0x1ff00000, 0x7f0000, 0x3fe0000, 0x1ff000, 0x3ff000},
    {0x0, 0x0, 0x0, 0x7f01f, 0xe0000000, 0x7f80f01, 0xfe000000, 0xfc00, 0x7e00fc00, 0xff00, 0x7ff800, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x7f00000, 0x7f800, 0xff00000, 0x1f8, 0x0, 0x3ff00000, 0x3f, 0xc0001fe0, 0x0, 0x7e0000, 0x3fc00, 0x1fc0000, 0xff0000, 0x7f800000, 0x1fc000, 0xf, 0xf00007f8, 0x1fc, 0x7f800, 0x0, 0x0, 0x0, 0x7c, 0x0, 0x0, 0x7c0, 0x0, 0x0, 0x0, 0x1f80ff, 0xffc07, 0xfc000000, 0x1fe00000, 0xff00000, 0x7f0000, 0x3fe0000, 0x1ff800, 0x7fe000},
    {0x0, 0x0, 0x0, 0xff01f, 0xc0000000, 0x7fc0f03, 0xfe000000, 0x1fc00, 0x7e00fc00, 0xff80, 0xfff800, 0x0, 0x1fc0, 0x0, 0x7f00000, 0x0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x7e00000, 0x3fc00, 0x1fe00000, 0x1f8, 0x0, 0x7fe00000, 0x3f, 0xe0003fc0, 0x0, 0x7e0000, 0x3fc00, 0x3fc0000, 0x7f8000, 0xff000000, 0x1fc000, 0xf, 0xf8000ff8, 0x1fe, 0x7f800, 0x0, 0x0, 0x0, 0x1c, 0x0, 0x0, 0x700, 0x0, 0x0, 0x0, 0x1fc07f, 0x801ffc0f, 0xf8000000, 0x3fe00000, 0xff80000, 0x7f0000, 0x7fc0000, 0xffc00, 0xffe000},
    {0x0, 0x0, 0x0, 0xfe01f, 0xc0000000, 0x3fe0f07, 0xfc000000, 0x1f800, 0x7e00fc00, 0x7fc0, 0x3fffc00, 0x0, 0xfe0, 0x0, 0xfe00000, 0x0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0x7e00000, 0x3fe00, 0x3fe00000, 0x1f8, 0x0, 0x7fc00000, 0x1f, 0xf0007fc0, 0x0, 0x7e0000, 0x1fe00, 0x7f80000, 0x7fc001, 0xff000000, 0x3fc000, 0x7, 0xf8001ff0, 0x1ff, 0xff000, 0x0, 0x0, 0x0, 0x4, 0x0, 0x0, 0x400, 0x0, 0x0, 0x0, 0x1fc07f, 0xc03ffc1f, 0xf8000000, 0x3fe00000, 0xff80000, 0x7f0000, 0xffc0000, 0xffe00, 0x1ffc000},
    {0x0, 0x0, 0x0, 0xfe01f, 0xc0000000, 0x3ff0f0f, 0xfc000000, 0x3f800, 0x7e00fc00, 0x7fe0, 0x7fffe00, 0x0, 0xfe0, 0x0, 0xfe00000, 0x0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x0, 0xfe00000, 0x3ff00, 0x7fe00000, 0x1f8, 0x0, 0xff800000, 0x1f, 0xf800ff80, 0x0, 0x7e0000, 0x1ff00, 0xff80000, 0x3fe003, 0xfe000000, 0x3f8000, 0x7, 0xfe003ff0, 0xff, 0x803ff000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfc07f, 0xf0fffe7f, 0xf0000000, 0x3fc00000, 0x7f80000, 0x7f0000, 0x7ffc0000, 0x7ff80, 0x7ffc000},
    {0x1f, 0x80000000, 0x0, 0xfe01f, 0xc0000000, 0x1ffcf3f, 0xf8000000, 0x3f000, 0x7f01fc00, 0x3ff8, 0x3fffff00, 0x0, 0xfe0, 0x0, 0xfe00000, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x1f800000, 0xfe00000, 0x1ffc1, 0xffc00000, 0x1f8, 0x0, 0xffffffff, 0xf, 0xfe03ff80, 0x0, 0x7e0000, 0xffe0, 0x7ff00000, 0x3ff80f, 0xfe000000, 0x3f8000, 0x3, 0xff80ffe0, 0xff, 0xc0ffe000, 0x1f80, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0xfe03f, 0xffffffff, 0xe0000000, 0x7fc00000, 0x7fc0000, 0x7fffff, 0xfff80000, 0x3fff0, 0x1fff8000},
    {0x1f, 0x80000000, 0x0, 0xfe03f, 0xc0000000, 0x1ffffff, 0xf8000000, 0x7f000, 0x3f01f800, 0x3fff, 0xfffdff80, 0x0, 0xfe0, 0x0, 0xfe00000, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x1f800000, 0xfc00000, 0x1ffff, 0xffc00000, 0x1f8, 0x1, 0xffffffff, 0xf, 0xffffff00, 0x0, 0x7e0000, 0xffff, 0xfff00000, 0x1fffff, 0xfc000000, 0x3f8000, 0x3, 0xffffffe0, 0x7f, 0xffffe000, 0x1f80, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0xfe03f, 0xffffffff, 0xc0000000, 0x7fc00000, 0x7fc0000, 0x7fffff, 0xfff00000, 0x3ffff, 0xffff0000},
    {0x1f, 0x80000000, 0x0, 0x1fe03f, 0x80000000, 0xffffff, 0xf0000000, 0x7e000, 0x3fc7f800, 0x1fff, 0xfff8ffc0, 0x0, 0x7e0, 0x0, 0x1fc00000, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x1f800000, 0xfc00000, 0xffff, 0xff800000, 0x1f8, 0x1, 0xffffffff, 0x7, 0xfffffe00, 0x0, 0x7e0000, 0x7fff, 0xffe00000, 0xfffff, 0xfc000000, 0x3f8000, 0x1, 0xffffffc0, 0x7f, 0xffffc000, 0x1f80, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0xfe01f, 0xfffdffff, 0x80000000, 0x7f800000, 0x3fc0000, 0x7fffff, 0xfff00000, 0x1ffff, 0xfffe0000},
    {0x1f, 0x80000000, 0x0, 0x1fc03f, 0x80000000, 0x7fffff, 0xe0000000, 0xfe000, 0x1ffff000, 0xfff, 0xfff07fc0, 0x0, 0x7f0, 0x0, 0x1fc00000, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x1f800000, 0x1fc00000, 0x7fff, 0xff000000, 0x1f8, 0x1, 0xffffffff, 0x3, 0xfffffe00, 0x0, 0x7e0000, 0x3fff, 0xffc00000, 0x7ffff, 0xf8000000, 0x3f8000, 0x0, 0xffffff80, 0x3f, 0xffff8000, 0x1f80, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x7f00f, 0xfff9ffff, 0x0, 0xff800000, 0x3fe0000, 0x7fffff, 0xffe00000, 0xffff, 0xfffc0000},
    {0x1f, 0x80000000, 0x0, 0x1fc03f, 0x80000000, 0x3fffff, 0xc0000000, 0xfc000, 0x1ffff000, 0x7ff, 0xffe07f80, 0x0, 0x7f0, 0x0, 0x1fc00000, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x1f800000, 0x1f800000, 0x7fff, 0xfe000000, 0x1f8, 0x1, 0xffffffff, 0x1, 0xfffffc00, 0x0, 0x7e0000, 0x1fff, 0xff800000, 0x3ffff, 0xf0000000, 0x3f8000, 0x0, 0x7fffff00, 0x1f, 0xffff0000, 0x1f80, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x7f807, 0xfff1fffe, 0x0, 0xff000000, 0x1fe0000, 0x7fffff, 0xff800000, 0x3fff, 0xfff80000},
    {0x1f, 0x80000000, 0x0, 0x1fc03f, 0x80000000, 0x1fffff, 0x80000000, 0x1fc000, 0xfffe000, 0x3ff, 0xffc03f80, 0x0, 0x3f0, 0x0, 0x1f800000, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x1f800000, 0x1f800000, 0x1fff, 0xfc000000, 0x1f8, 0x1, 0xffffffff, 0x0, 0xfffff000, 0x0, 0x7e0000, 0xfff, 0xff000000, 0x1ffff, 0xe0000000, 0x3f8000, 0x0, 0x3ffffe00, 0xf, 0xfffe0000, 0x1f80, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x7f803, 0xffe0fff8, 0x3f00000, 0xff000000, 0x1fe0000, 0x7fffff, 0xff000000, 0x1fff, 0xfff00000},
    {0x1f, 0x80000000, 0x0, 0x1fc07f, 0x80000000, 0xfffff, 0x0, 0x1fc000, 0x7ffc000, 0x1ff, 0xff801f00, 0x0, 0x3f8, 0x0, 0x3f800000, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x1f800000, 0x1f800000, 0xfff, 0xf8000000, 0x1f8, 0x1, 0xffffffff, 0x0, 0x7fffe000, 0x0, 0x7e0000, 0x7ff, 0xfe000000, 0xffff, 0x80000000, 0x3f8000, 0x0, 0xffff800, 0x7, 0xfff80000, 0x1f80, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x3fc00, 0xff807fe0, 0x7f00001, 0xff000000, 0x1ff0000, 0x7fffff, 0xf8000000, 0x7ff, 0xffc00000},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x1fffc, 0x0, 0x3f8000, 0x1ff0000, 0x3f, 0xfc000e00, 0x0, 0x3f8, 0x0, 0x3f800000, 0x0, 0x0, 0x0, 0x780, 0x0, 0x0, 0x0, 0x0, 0x3ff, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0xfff8000, 0x0, 0x0, 0x1ff, 0xf0000000, 0x3ffe, 0x0, 0x0, 0x0, 0x3ffe000, 0x1, 0xffe00000, 0x0, 0x1e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3fe00, 0x0, 0xff00000, 0x0, 0x0, 0x0, 0x0, 0xff, 0xfe000000},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x3f8000, 0x300000, 0x3, 0xc0000400, 0x0, 0x1f8, 0x0, 0x3f000000, 0x0, 0x0, 0x0, 0x780, 0x0, 0x0, 0x0, 0x0, 0x3c, 0x0, 0x0, 0x0, 0x0, 0x0, 0xf00000, 0x0, 0x0, 0xf, 0x0, 0x1e0, 0x0, 0x0, 0x0, 0x3e0000, 0x0, 0xe000000, 0x0, 0x1e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1ff00, 0x0, 0x1fe00000, 0x0, 0x0, 0x0, 0x0, 0x3, 0xc0000000},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0xf00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fc, 0x0, 0x7f000000, 0x0, 0x0, 0x0, 0x780, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff80, 0x0, 0x3fe00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0xf00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfc, 0x0, 0x7e000000, 0x0, 0x0, 0x0, 0x780, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xffc0, 0x0, 0x7fc00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0xf00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfc, 0x0, 0x7e000000, 0x0, 0x0, 0x0, 0x780, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7fe0, 0x1, 0xff800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0xf00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7e, 0x0, 0xfc000000, 0x0, 0x0, 0x0, 0x780, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3ff8, 0x3, 0xff000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0xf00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7f, 0x1, 0xf8000000, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfff, 0xc000007f, 0xfe000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3f, 0x1, 0xf8000000, 0x0, 0x0, 0x0, 0x1f00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7ff, 0xfc0003ff, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f, 0x80000003, 0xf0000000, 0x0, 0x0, 0x0, 0x1f00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3ff, 0xffffffff, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f, 0x80000003, 0xf0000000, 0x0, 0x0, 0x0, 0x1e00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x78, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1ff, 0xffffffff, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xf, 0xc0000007, 0xe0000000, 0x0, 0x0, 0x0, 0xc00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x30, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff, 0xffffffff, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xf, 0xc0000007, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3f, 0xffffffff, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe000000f, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xf, 0xfffffffc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xe000000f, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0xfffffff0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xe000000f, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3fffff80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfff000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xf, 0xff800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfff, 0x0, 0x0, 0x0, 0x1, 0xffc00000, 0x0, 0x0, 0x0, 0x3ffc00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfe, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0x0, 0xfffff, 0xffff0000, 0x3ffff, 0xffff0000, 0xff, 0xfff00000, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x7fe, 0x7f, 0x0, 0x7ff8, 0x3ff, 0xc0000001, 0xfe000001, 0xfc000000, 0x7fff, 0xe0000000, 0x3ffff, 0xff800000, 0xf, 0xfff80000, 0x7f, 0xfffffc00, 0x0, 0x1ffff80, 0xf, 0xffffffff, 0xf8000001, 0xfc000001, 0xfc000003, 0xfe000000, 0x3fe0000, 0x7f800003, 0xff000003, 0xfc00001f, 0xf000001f, 0xf000001f, 0xf0000001, 0xff000000, 0xffffffff, 0xf0000003, 0xffe00000, 0xfe000000, 0xfff800, 0xfe, 0x0, 0x0, 0x1f, 0xf0000000, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xe0000000, 0xfffff, 0xffff0000, 0x3ffff, 0xffff0000, 0x3ff, 0xfffc0000, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0xffe, 0x7f, 0x0, 0x7ff8, 0x3ff, 0xc0000001, 0xff000001, 0xfc000000, 0x1ffff, 0xf8000000, 0x3ffff, 0xfff00000, 0x3f, 0xfffe0000, 0x7f, 0xffffff80, 0x0, 0x7ffffe0, 0xf, 0xffffffff, 0xf8000001, 0xfc000001, 0xfc000003, 0xfe000000, 0x3fe0000, 0x7f800003, 0xff800003, 0xfc00001f, 0xf800003f, 0xf000001f, 0xf8000003, 0xff000000, 0xffffffff, 0xf0000003, 0xffe00000, 0x7e000000, 0xfff800, 0xfe, 0x0, 0x0, 0x1f, 0xf0000000, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xf8000000, 0xfffff, 0xffff0000, 0x3ffff, 0xffff0000, 0xfff, 0xffff0000, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x1ffc, 0x7f, 0x0, 0x7ff8, 0x3ff, 0xc0000001, 0xff000001, 0xfc000000, 0x7ffff, 0xfe000000, 0x3ffff, 0xfffc0000, 0xff, 0xffff8000, 0x7f, 0xffffffe0, 0x0, 0x1ffffff8, 0xf, 0xffffffff, 0xf8000001, 0xfc000001, 0xfc000001, 0xfe000000, 0x3fc0000, 0x7f800003, 0xff800003, 0xfc00000f, 0xf800007f, 0xe000000f, 0xf8000003, 0xfe000000, 0xffffffff, 0xf0000003, 0xffe00000, 0x7e000000, 0xfff800, 0x1ff, 0x0, 0x0, 0xf, 0xf8000000, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xfe000000, 0xfffff, 0xffff0000, 0x3ffff, 0xffff0000, 0x1fff, 0xffff8000, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x3ff8, 0x7f, 0x0, 0x7ffc, 0x7ff, 0xc0000001, 0xff800001, 0xfc000000, 0xfffff, 0xff000000, 0x3ffff, 0xfffe0000, 0x3ff, 0xffffe000, 0x7f, 0xfffffff0, 0x0, 0x3ffffffc, 0xf, 0xffffffff, 0xf8000001, 0xfc000001, 0xfc000001, 0xff000000, 0x7fc0000, 0x7f800003, 0xff800003, 0xfc00000f, 0xfc00007f, 0xe000000f, 0xfc000007, 0xfe000000, 0xffffffff, 0xf0000003, 0xffe00000, 0x7e000000, 0xfff800, 0x1ff, 0x0, 0x0, 0xf, 0xf8000000, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xff000000, 0xfffff, 0xffff0000, 0x3ffff, 0xffff0000, 0x3fff, 0xffffc000, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x7ff0, 0x7f, 0x0, 0x7ffc, 0x7ff, 0xc0000001, 0xffc00001, 0xfc000000, 0x3fffff, 0xffc00000, 0x3ffff, 0xffff0000, 0x7ff, 0xfffff000, 0x7f, 0xfffffff8, 0x0, 0x7ffffffe, 0xf, 0xffffffff, 0xf8000001, 0xfc000001, 0xfc000001, 0xff000000, 0x7fc0000, 0x7fc00007, 0xffc00007, 0xfc000007, 0xfe0000ff, 0xc0000007, 0xfc000007, 0xfc000000, 0xffffffff, 0xf0000003, 0xffe00000, 0x7f000000, 0xfff800, 0x3ff, 0x80000000, 0x0, 0x7, 0xfc000000, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xff800000, 0xfffff, 0xffff0000, 0x3ffff, 0xffff0000, 0xffff, 0xffffe000, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0xffe0, 0x7f, 0x0, 0x7ffc, 0x7ff, 0xc0000001, 0xffc00001, 0xfc000000, 0x7fffff, 0xffe00000, 0x3ffff, 0xffff8000, 0xfff, 0xfffff800, 0x7f, 0xfffffffc, 0x0, 0xffffffff, 0xf, 0xffffffff, 0xf8000001, 0xfc000001, 0xfc000000, 0xff000000, 0x7f80000, 0x3fc00007, 0xffc00007, 0xf8000007, 0xfe0001ff, 0xc0000007, 0xfe00000f, 0xfc000000, 0xffffffff, 0xf0000003, 0xffe00000, 0x3f000000, 0xfff800, 0x3ff, 0x80000000, 0x0, 0x3, 0xfc000000, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xffc00000, 0xfffff, 0xffff0000, 0x3ffff, 0xffff0000, 0x1ffff, 0xffff000, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x1ffc0, 0x7f, 0x0, 0x7ffe, 0xfff, 0xc0000001, 0xffe00001, 0xfc000000, 0xffff9f, 0xfff00000, 0x3ffff, 0xffffc000, 0x1fff, 0xe3fffc00, 0x7f, 0xfffffffc, 0x0, 0xfffe3fff, 0x8000000f, 0xffffffff, 0xf8000001, 0xfc000001, 0xfc000000, 0xff800000, 0xff80000, 0x3fc00007, 0xffc00007, 0xf8000003, 0xff0001ff, 0x80000003, 0xff00001f, 0xf8000000, 0xffffffff, 0xf0000003, 0xffe00000, 0x3f000000, 0xfff800, 0x3ff, 0x80000000, 0x0, 0x1, 0xfc000000, 0x0, 0x0, 0x0, 0x0},
    {0x3f800ff, 0xffe00000, 0xfe000, 0x0, 0x3f800, 0x0, 0x1fff0, 0xfff000, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x3ff80, 0x7f, 0x0, 0x7ffe, 0xfff, 0xc0000001, 0xffe00001, 0xfc000000, 0xfff001, 0xfff00000, 0x3f800, 0x3fffc000, 0x3ffe, 0x3ffe00, 0x7f, 0x1fffe, 0x1, 0xffc003ff, 0x80000000, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x7f800000, 0xff00000, 0x3fc00007, 0xffc00007, 0xf8000001, 0xff0003ff, 0x1, 0xff00001f, 0xf0000000, 0x3f, 0xf0000003, 0xf0000000, 0x3f800000, 0x1f800, 0x7ff, 0xc0000000, 0x0, 0x1, 0xfe000000, 0x0, 0x0, 0x0, 0x0},
    {0x3f80001, 0xfff00000, 0xfe000, 0x0, 0x3f800, 0x0, 0x3ffc0, 0x3ff800, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x7ff00, 0x7f, 0x0, 0x7ffe, 0xfff, 0xc0000001, 0xfff00001, 0xfc000000, 0x1ffc000, 0x7ff80000, 0x3f800, 0xffe000, 0x3ff8, 0xffe00, 0x7f, 0xffe, 0x1, 0xff8000ff, 0xc0000000, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x7f800000, 0xff00000, 0x3fe0000f, 0xffe0000f, 0xf8000001, 0xff8003ff, 0x1, 0xff80003f, 0xf0000000, 0x7f, 0xf0000003, 0xf0000000, 0x3f800000, 0x1f800, 0x7ef, 0xc0000000, 0x0, 0x0, 0xfe000000, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x7ff00000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7ff00, 0x1ffc00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0xfff00, 0x7f, 0x0, 0x7ffe, 0xfff, 0xc0000001, 0xfff80001, 0xfc000000, 0x3ff8000, 0x1ffc0000, 0x3f800, 0x3fe000, 0x7ff0, 0x7ff00, 0x7f, 0x3ff, 0x3, 0xfe00007f, 0xc0000000, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x7fc00000, 0x1ff00000, 0x1fe0000f, 0xefe0000f, 0xf0000000, 0xffc007fe, 0x0, 0xff80003f, 0xe0000000, 0x7f, 0xe0000003, 0xf0000000, 0x1f800000, 0x1f800, 0x7ef, 0xc0000000, 0x0, 0x0, 0x7f000000, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x1ff80000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7fe00, 0x7fc00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x1ffe00, 0x7f, 0x0, 0x7fff, 0x1fff, 0xc0000001, 0xfff80001, 0xfc000000, 0x3ff0000, 0xffc0000, 0x3f800, 0x3ff000, 0x7fc0, 0x3ff80, 0x7f, 0x1ff, 0x3, 0xfe00003f, 0xe0000000, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x3fc00000, 0x1fe00000, 0x1fe0000f, 0xefe0000f, 0xf0000000, 0x7fc00ffc, 0x0, 0xffc0007f, 0xe0000000, 0xff, 0xc0000003, 0xf0000000, 0x1f800000, 0x1f800, 0xfef, 0xe0000000, 0x0, 0x0, 0x3f000000, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff80000, 0xfe000, 0x0, 0x3f800, 0x0, 0xffc00, 0x7fc00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x1ffc00, 0x7f, 0x0, 0x7f7f, 0x1fdf, 0xc0000001, 0xfffc0001, 0xfc000000, 0x7fe0000, 0x7fe0000, 0x3f800, 0x1ff000, 0xff80, 0x1ff80, 0x7f, 0x1ff, 0x3, 0xfc00003f, 0xe0000000, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x3fc00000, 0x1fe00000, 0x1fe0000f, 0xefe0000f, 0xf0000000, 0x7fe00ffc, 0x0, 0x7fe000ff, 0xc0000000, 0x1ff, 0x80000003, 0xf0000000, 0x1fc00000, 0x1f800, 0xfc7, 0xe0000000, 0x0, 0x0, 0x3f000000, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x7fc0000, 0xfe000, 0x0, 0x3f800, 0x0, 0xff800, 0x3fe00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x3ff800, 0x7f, 0x0, 0x7f7f, 0x1fdf, 0xc0000001, 0xfffc0001, 0xfc000000, 0x7fc0000, 0x3fe0000, 0x3f800, 0xff000, 0xff80, 0xffc0, 0x7f, 0xff, 0x3, 0xfc00001f, 0xe0000000, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x3fe00000, 0x3fe00000, 0x1fe0001f, 0xeff0000f, 0xf0000000, 0x3ff01ff8, 0x0, 0x3fe000ff, 0x80000000, 0x3ff, 0x80000003, 0xf0000000, 0xfc00000, 0x1f800, 0x1fc7, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x7fc0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x1ff000, 0x1fe00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x7ff000, 0x7f, 0x0, 0x7f7f, 0x80003fdf, 0xc0000001, 0xfffe0001, 0xfc000000, 0xff80000, 0x1ff0000, 0x3f800, 0xff000, 0x1ff00, 0x7fc0, 0x7f, 0xff, 0x3, 0xfc00001f, 0xe0000000, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x1fe00000, 0x3fc00000, 0xff0001f, 0xc7f0001f, 0xe0000000, 0x1ff03ff8, 0x0, 0x3ff001ff, 0x80000000, 0x3ff, 0x3, 0xf0000000, 0xfc00000, 0x1f800, 0x1fc7, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x3fe0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x1ff000, 0x1fe00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0xffe000, 0x7f, 0x0, 0x7f3f, 0x80003f9f, 0xc0000001, 0xffff0001, 0xfc000000, 0xff00000, 0x1ff0000, 0x3f800, 0xff000, 0x1fe00, 0x7fc0, 0x7f, 0xff, 0x3, 0xfc00001f, 0xe0000000, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x1fe00000, 0x3fc00000, 0xff0001f, 0xc7f0001f, 0xe0000000, 0x1ff83ff0, 0x0, 0x1ff001ff, 0x0, 0x7fe, 0x3, 0xf0000000, 0xfc00000, 0x1f800, 0x1f83, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x3fe0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x1fe000, 0x1ff00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x1ffc000, 0x7f, 0x0, 0x7f3f, 0x80003f9f, 0xc0000001, 0xffff0001, 0xfc000000, 0x1ff00000, 0xff8000, 0x3f800, 0xff000, 0x3fe00, 0x3fe0, 0x7f, 0xff, 0x3, 0xfc00001f, 0xe0000000, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x1ff00000, 0x7fc00000, 0xff0003f, 0xc7f8001f, 0xe0000000, 0xffc7fe0, 0x0, 0x1ff803ff, 0x0, 0xffe, 0x3, 0xf0000000, 0xfe00000, 0x1f800, 0x3f83, 0xf8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x1fe0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x3fe000, 0xff00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x3ff8000, 0x7f, 0x0, 0x7f3f, 0x80003f9f, 0xc0000001, 0xfdff8001, 0xfc000000, 0x1fe00000, 0xff8000, 0x3f800, 0xff000, 0x3fc00, 0x3fe0, 0x7f, 0xff, 0x3, 0xfc000000, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0xff00000, 0x7f800000, 0xff0003f, 0xc7f8001f, 0xe0000000, 0xffc7fe0, 0x0, 0xffc07fe, 0x0, 0x1ffc, 0x3, 0xf0000000, 0x7e00000, 0x1f800, 0x3f83, 0xf8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x1fe0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x3fc000, 0xf000, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x7ff0000, 0x7f, 0x0, 0x7f3f, 0xc0007f9f, 0xc0000001, 0xfdff8001, 0xfc000000, 0x1fe00000, 0x7f8000, 0x3f800, 0xff000, 0x3fc00, 0x1fe0, 0x7f, 0xff, 0x3, 0xfe000000, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0xff80000, 0xff800000, 0xff8003f, 0x83f8003f, 0xe0000000, 0x7feffc0, 0x0, 0x7fc07fc, 0x0, 0x1ff8, 0x3, 0xf0000000, 0x7e00000, 0x1f800, 0x3f01, 0xf8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x1ff0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x3fc000, 0x0, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0xffe0000, 0x7f, 0x0, 0x7f1f, 0xc0007f1f, 0xc0000001, 0xfcffc001, 0xfc000000, 0x3fe00000, 0x7f8000, 0x3f800, 0xff000, 0x3fc00, 0x1ff0, 0x7f, 0xff, 0x3, 0xfe000000, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0xff80000, 0xff000000, 0x7f8003f, 0x83f8003f, 0xc0000000, 0x3ffff80, 0x0, 0x7fe0ffc, 0x0, 0x3ff0, 0x3, 0xf0000000, 0x7f00000, 0x1f800, 0x7f01, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7fc000, 0x0, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x1ffc0000, 0x7f, 0x0, 0x7f1f, 0xc0007f1f, 0xc0000001, 0xfc7fe001, 0xfc000000, 0x3fc00000, 0x7fc000, 0x3f800, 0xff000, 0x7f800, 0x1ff0, 0x7f, 0x1ff, 0x3, 0xff800000, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x7f80000, 0xff000000, 0x7f8007f, 0x83fc003f, 0xc0000000, 0x3ffff80, 0x0, 0x3fe0ff8, 0x0, 0x7ff0, 0x3, 0xf0000000, 0x7f00000, 0x1f800, 0x7f01, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7f8000, 0x0, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x3ffc0000, 0x7f, 0x0, 0x7f1f, 0xe000ff1f, 0xc0000001, 0xfc7fe001, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3f800, 0xff000, 0x7f800, 0xff0, 0x7f, 0x1ff, 0x1, 0xffe00000, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x7fc0001, 0xff000000, 0x7f8007f, 0x1fc003f, 0xc0000000, 0x1ffff00, 0x0, 0x3ff1ff8, 0x0, 0x7fe0, 0x3, 0xf0000000, 0x3f00000, 0x1f800, 0xfe00, 0xfe000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7f8000, 0x0, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x7ff80000, 0x7f, 0x0, 0x7f0f, 0xe000fe1f, 0xc0000001, 0xfc3ff001, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3f800, 0x1ff000, 0x7f800, 0xff0, 0x7f, 0x3fe, 0x1, 0xfffc0000, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x3fc0001, 0xfe000000, 0x7f8007f, 0x1fc003f, 0xc0000000, 0xfffe00, 0x0, 0x1ffbff0, 0x0, 0xffc0, 0x3, 0xf0000000, 0x3f00000, 0x1f800, 0xfe00, 0xfe000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7f8000, 0x0, 0x7ff, 0xfffffff0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0xfff00000, 0x7f, 0x0, 0x7f0f, 0xe000fe1f, 0xc0000001, 0xfc3ff001, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3f800, 0x1ff000, 0x7f800, 0xff0, 0x7f, 0x7fe, 0x0, 0xffff8000, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x3fc0001, 0xfe000000, 0x3fc007f, 0x1fc007f, 0x80000000, 0xfffe00, 0x0, 0xffbfe0, 0x0, 0x1ff80, 0x3, 0xf0000000, 0x3f80000, 0x1f800, 0xfe00, 0xfe000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfffff, 0xfffc0000, 0x3ffff, 0xfff00000, 0x7f8000, 0x0, 0x7ff, 0xfffffff0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0xffe00000, 0x7f, 0x0, 0x7f0f, 0xe000fe1f, 0xc0000001, 0xfc1ff801, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3f800, 0x3fe000, 0x7f800, 0xff0, 0x7f, 0x3ffc, 0x0, 0x7ffff800, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x3fe0003, 0xfe000000, 0x3fc00ff, 0x1fe007f, 0x80000000, 0x7ffc00, 0x0, 0xffffe0, 0x0, 0x3ff80, 0x3, 0xf0000000, 0x1f80000, 0x1f800, 0x1fc00, 0x7f000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfffff, 0xfffc0000, 0x3ffff, 0xfff00000, 0x7f8000, 0x0, 0x7ff, 0xfffffff0, 0x1f, 0xc0000000, 0x7f00, 0x1fd, 0xfff00000, 0x7f, 0x0, 0x7f0f, 0xf001fe1f, 0xc0000001, 0xfc0ff801, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3f800, 0x7fe000, 0x7f800, 0xff0, 0x7f, 0xfffffffc, 0x0, 0x3fffff00, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x1fe0003, 0xfc000000, 0x3fc00fe, 0xfe007f, 0x80000000, 0x3ffc00, 0x0, 0x7fffc0, 0x0, 0x3ff00, 0x3, 0xf0000000, 0x1f80000, 0x1f800, 0x1fc00, 0x7f000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfffff, 0xfffc0000, 0x3ffff, 0xfff00000, 0x7f8000, 0x0, 0x7ff, 0xfffffff0, 0x1f, 0xc0000000, 0x7f00, 0x1ff, 0xfff00000, 0x7f, 0x0, 0x7f07, 0xf001fc1f, 0xc0000001, 0xfc0ffc01, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3f800, 0x3ffe000, 0x7f800, 0xff0, 0x7f, 0xfffffff8, 0x0, 0x1fffffe0, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x1fe0003, 0xfc000000, 0x3fc00fe, 0xfe007f, 0x80000000, 0x3ff800, 0x0, 0x7fffc0, 0x0, 0x7fe00, 0x3, 0xf0000000, 0x1f80000, 0x1f800, 0x1fc00, 0x7f000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfffff, 0xfffc0000, 0x3ffff, 0xfff00000, 0x7f8000, 0x0, 0x7ff, 0xfffffff0, 0x1f, 0xc0000000, 0x7f00, 0x1ff, 0xfff80000, 0x7f, 0x0, 0x7f07, 0xf001fc1f, 0xc0000001, 0xfc07fe01, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3ffff, 0xffffc000, 0x7f800, 0xff0, 0x7f, 0xfffffff0, 0x0, 0xffffff8, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x1ff0007, 0xfc000000, 0x1fc00fe, 0xfe00ff, 0x0, 0x3ff800, 0x0, 0x3fff80, 0x0, 0xffc00, 0x3, 0xf0000000, 0x1fc0000, 0x1f800, 0x3f800, 0x3f800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfffff, 0xfffc0000, 0x3ffff, 0xfff00000, 0x7f8000, 0x1fffff00, 0x7ff, 0xfffffff0, 0x1f, 0xc0000000, 0x7f00, 0x1ff, 0xfffc0000, 0x7f, 0x0, 0x7f07, 0xf803fc1f, 0xc0000001, 0xfc07fe01, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3ffff, 0xffff8000, 0x7f800, 0xff0, 0x7f, 0xffffffe0, 0x0, 0x3fffffe, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0xff0007, 0xf8000000, 0x1fe01fe, 0xff00ff, 0x0, 0x3ff800, 0x0, 0x1fff00, 0x0, 0x1ffc00, 0x3, 0xf0000000, 0xfc0000, 0x1f800, 0x3f800, 0x3f800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfffff, 0xfffc0000, 0x3ffff, 0xfff00000, 0x7f8000, 0x1fffff00, 0x7ff, 0xfffffff0, 0x1f, 0xc0000000, 0x7f00, 0x1ff, 0xfffe0000, 0x7f, 0x0, 0x7f03, 0xf803f81f, 0xc0000001, 0xfc03ff01, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3ffff, 0xffff8000, 0x7f800, 0xff0, 0x7f, 0xffffff80, 0x0, 0x7fffff, 0x0, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0xff0007, 0xf8000000, 0x1fe01fc, 0x7f00ff, 0x0, 0x7ffc00, 0x0, 0x1fff00, 0x0, 0x1ff800, 0x3, 0xf0000000, 0xfc0000, 0x1f800, 0x3f800, 0x3f800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfffff, 0xfffc0000, 0x3ffff, 0xfff00000, 0x7f8000, 0x1fffff00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1ff, 0xfffe0000, 0x7f, 0x0, 0x7f03, 0xf803f81f, 0xc0000001, 0xfc01ff01, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3ffff, 0xffff0000, 0x7f800, 0xff0, 0x7f, 0xfffffe00, 0x0, 0xfffff, 0x80000000, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0xff800f, 0xf8000000, 0x1fe01fc, 0x7f00ff, 0x0, 0xfffc00, 0x0, 0xffe00, 0x0, 0x3ff000, 0x3, 0xf0000000, 0xfe0000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7f8000, 0x1fffff00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1ff, 0xf3ff0000, 0x7f, 0x0, 0x7f03, 0xf803f81f, 0xc0000001, 0xfc01ff81, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3ffff, 0xfffe0000, 0x7f800, 0xff0, 0x7f, 0xfffff800, 0x0, 0x1ffff, 0xc0000000, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x7f800f, 0xf0000000, 0xfe03fc, 0x7f80fe, 0x0, 0xfffe00, 0x0, 0xffe00, 0x0, 0x7ff000, 0x3, 0xf0000000, 0xfe0000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7f8000, 0x1fffff00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1ff, 0xe1ff8000, 0x7f, 0x0, 0x7f03, 0xfc07f81f, 0xc0000001, 0xfc00ffc1, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3ffff, 0xfff80000, 0x7f800, 0xff0, 0x7f, 0xfffc00, 0x0, 0x1fff, 0xc0000000, 0x7f000, 0x1, 0xfc000001, 0xfc000000, 0x7f800f, 0xf0000000, 0xff03fc, 0x7f81fe, 0x0, 0x1ffff00, 0x0, 0x7fc00, 0x0, 0xffe000, 0x3, 0xf0000000, 0x7e0000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7f8000, 0x1fffff00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1ff, 0xc1ff8000, 0x7f, 0x0, 0x7f01, 0xfc07f01f, 0xc0000001, 0xfc00ffc1, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3ffff, 0xffe00000, 0x7f800, 0xff0, 0x7f, 0x7fe00, 0x0, 0x3ff, 0xe0000000, 0x7f000, 0x1, 0xfe000003, 0xfc000000, 0x3fc01f, 0xe0000000, 0xff03f8, 0x3f81fe, 0x0, 0x3ffff00, 0x0, 0x3f800, 0x0, 0xffc000, 0x3, 0xf0000000, 0x7e0000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7f8000, 0x1fffff00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1ff, 0xc0ffc000, 0x7f, 0x0, 0x7f01, 0xfc07f01f, 0xc0000001, 0xfc007fe1, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3ffff, 0xf8000000, 0x7f800, 0xff0, 0x7f, 0x3ff00, 0x0, 0xff, 0xe0000000, 0x7f000, 0x1, 0xfe000003, 0xfc000000, 0x3fc01f, 0xe0000000, 0xff03f8, 0x3f81fe, 0x0, 0x3ffff80, 0x0, 0x3f800, 0x0, 0x1ff8000, 0x3, 0xf0000000, 0x7f0000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7f8000, 0x1fffff00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1ff, 0x807fe000, 0x7f, 0x0, 0x7f01, 0xfe0ff01f, 0xc0000001, 0xfc003fe1, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3f800, 0x0, 0x7f800, 0xff0, 0x7f, 0x1ff80, 0x0, 0x3f, 0xf0000000, 0x7f000, 0x1, 0xfe000003, 0xfc000000, 0x3fc01f, 0xe0000000, 0x7f07f8, 0x3fc1fc, 0x0, 0x7feffc0, 0x0, 0x3f800, 0x0, 0x3ff8000, 0x3, 0xf0000000, 0x3f0000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7f8000, 0x7f00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1ff, 0x7fe000, 0x7f, 0x0, 0x7f00, 0xfe0fe01f, 0xc0000001, 0xfc003ff1, 0xfc000000, 0x3fc00000, 0x3fc000, 0x3f800, 0x0, 0x7f800, 0xff0, 0x7f, 0xffc0, 0x0, 0x1f, 0xf0000000, 0x7f000, 0x1, 0xfe000003, 0xfc000000, 0x1fe03f, 0xc0000000, 0x7f07f0, 0x1fc1fc, 0x0, 0x7fe7fc0, 0x0, 0x3f800, 0x0, 0x3ff0000, 0x3, 0xf0000000, 0x3f0000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x1ff0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7fc000, 0x7f00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fe, 0x3ff000, 0x7f, 0x0, 0x7f00, 0xfe0fe01f, 0xc0000001, 0xfc001ff9, 0xfc000000, 0x3fe00000, 0x7fc000, 0x3f800, 0x0, 0x7fc00, 0x1ff0, 0x7f, 0x7fc0, 0xf, 0xf000000f, 0xf0000000, 0x7f000, 0x1, 0xfe000003, 0xfc000000, 0x1fe03f, 0xc0000000, 0x7f87f0, 0x1fc3fc, 0x0, 0xffc7fe0, 0x0, 0x3f800, 0x0, 0x7fe0000, 0x3, 0xf0000000, 0x3f8000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x1fe0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x3fc000, 0x7f00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7f00, 0x1fc, 0x1ff800, 0x7f, 0x0, 0x7f00, 0xfe0fe01f, 0xc0000001, 0xfc001ff9, 0xfc000000, 0x1fe00000, 0x7f8000, 0x3f800, 0x0, 0x3fc00, 0x1fe0, 0x7f, 0x7fe0, 0xf, 0xf000000f, 0xf0000000, 0x7f000, 0x1, 0xfe000003, 0xfc000000, 0x1fe03f, 0xc0000000, 0x7f87f0, 0x1fc3fc, 0x0, 0x1ff83ff0, 0x0, 0x3f800, 0x0, 0xffc0000, 0x3, 0xf0000000, 0x3f8000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x1fe0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x3fc000, 0x7f00, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7800ff00, 0x1fc, 0x1ff800, 0x7f, 0x0, 0x7f00, 0xff1fe01f, 0xc0000001, 0xfc000ffd, 0xfc000000, 0x1fe00000, 0x7f8000, 0x3f800, 0x0, 0x3fc00, 0x1fe0, 0x7f, 0x3fe0, 0xf, 0xf000000f, 0xf0000000, 0x7f000, 0x1, 0xfe000003, 0xfc000000, 0xff07f, 0x80000000, 0x7f8ff0, 0x1fe3fc, 0x0, 0x1ff81ff0, 0x0, 0x3f800, 0x0, 0x1ffc0000, 0x3, 0xf0000000, 0x1f8000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x3fe0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x3fe000, 0x7f00, 0x7f0, 0x7f0, 0x1f, 0xc0000007, 0xf800ff00, 0x1fc, 0xffc00, 0x7f, 0x0, 0x7f00, 0x7f1fc01f, 0xc0000001, 0xfc0007fd, 0xfc000000, 0x1ff00000, 0xff8000, 0x3f800, 0x0, 0x3fe00, 0x2003fe0, 0x7f, 0x1ff0, 0xf, 0xf000000f, 0xf0000000, 0x7f000, 0x1, 0xfe000003, 0xfc000000, 0xff07f, 0x80000000, 0x3f8fe0, 0xfe3f8, 0x0, 0x3ff01ff8, 0x0, 0x3f800, 0x0, 0x1ff80000, 0x3, 0xf0000000, 0x1f8000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x3fe0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x1fe000, 0x7f00, 0x7f0, 0x7f0, 0x1f, 0xc0000007, 0xf800ff00, 0x1fc, 0x7fe00, 0x7f, 0x0, 0x7f00, 0x7f1fc01f, 0xc0000001, 0xfc0007ff, 0xfc000000, 0xff00000, 0xff0000, 0x3f800, 0x0, 0x1fe00, 0x7c03fc0, 0x7f, 0x1ff8, 0xf, 0xf000000f, 0xf0000000, 0x7f000, 0x1, 0xfe000003, 0xfc000000, 0xff0ff, 0x80000000, 0x3fcfe0, 0xfe7f8, 0x0, 0x7fe00ff8, 0x0, 0x3f800, 0x0, 0x3ff00000, 0x3, 0xf0000000, 0x1fc000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x7fc0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x1ff000, 0x7f00, 0x7f0, 0x7f0, 0x1f, 0xc0000007, 0xf800ff00, 0x1fc, 0x7fe00, 0x7f, 0x0, 0x7f00, 0x7fbfc01f, 0xc0000001, 0xfc0003ff, 0xfc000000, 0xff80000, 0x1ff0000, 0x3f800, 0x0, 0x1ff00, 0x7f07fc0, 0x7f, 0xff8, 0xf, 0xf800000f, 0xf0000000, 0x7f000, 0x1, 0xfe000007, 0xfc000000, 0x7f8ff, 0x0, 0x3fdfe0, 0xff7f8, 0x0, 0x7fe00ffc, 0x0, 0x3f800, 0x0, 0x7fe00000, 0x3, 0xf0000000, 0xfc000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x7fc0000, 0xfe000, 0x0, 0x3f800, 0x0, 0x1ff800, 0x7f00, 0x7f0, 0x7f0, 0x1f, 0xc0000007, 0xf800ff00, 0x1fc, 0x3ff00, 0x7f, 0x0, 0x7f00, 0x3fbf801f, 0xc0000001, 0xfc0003ff, 0xfc000000, 0xffc0000, 0x3ff0000, 0x3f800, 0x0, 0x1ff00, 0x7f87fc0, 0x7f, 0xffc, 0xf, 0xf800000f, 0xf0000000, 0x7f000, 0x1, 0xff000007, 0xfc000000, 0x7f8ff, 0x0, 0x3fdfe0, 0xff7f8, 0x0, 0xffc007fe, 0x0, 0x3f800, 0x0, 0xffe00000, 0x3, 0xf0000000, 0xfc000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xff80000, 0xfe000, 0x0, 0x3f800, 0x0, 0xff800, 0x7f00, 0x7f0, 0x7f0, 0x1f, 0xc0000007, 0xf800ff00, 0x1fc, 0x1ff80, 0x7f, 0x0, 0x7f00, 0x3fbf801f, 0xc0000001, 0xfc0001ff, 0xfc000000, 0x7fc0000, 0x3fe0000, 0x3f800, 0x0, 0xff80, 0x7feff80, 0x7f, 0x7fc, 0x7, 0xfc00000f, 0xf0000000, 0x7f000, 0x0, 0xff000007, 0xf8000000, 0x7f8ff, 0x0, 0x1fdfc0, 0x7f7f0, 0x0, 0xffc003fe, 0x0, 0x3f800, 0x0, 0xffc00000, 0x3, 0xf0000000, 0xfc000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x1ff80000, 0xfe000, 0x0, 0x3f800, 0x0, 0xffc00, 0x1ff00, 0x7f0, 0x7f0, 0x1f, 0xc0000007, 0xf800ff00, 0x1fc, 0xffc0, 0x7f, 0x0, 0x7f00, 0x3fbf801f, 0xc0000001, 0xfc0000ff, 0xfc000000, 0x7fe0000, 0x7fe0000, 0x3f800, 0x0, 0xffc0, 0xfffff00, 0x7f, 0x7fe, 0x7, 0xfc00001f, 0xf0000000, 0x7f000, 0x0, 0xff80000f, 0xf8000000, 0x3fdfe, 0x0, 0x1fdfc0, 0x7f7f0, 0x1, 0xff8003ff, 0x0, 0x3f800, 0x1, 0xff800000, 0x3, 0xf0000000, 0xfe000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0x3ff00000, 0xfe000, 0x0, 0x3f800, 0x0, 0x7ff00, 0x3ff00, 0x7f0, 0x7f0, 0x1f, 0xc0000007, 0xfc01ff00, 0x1fc, 0xffc0, 0x7f, 0x0, 0x7f00, 0x3fff801f, 0xc0000001, 0xfc0000ff, 0xfc000000, 0x3ff0000, 0xffc0000, 0x3f800, 0x0, 0x7fe0, 0xfffff00, 0x7f, 0x3fe, 0x7, 0xfe00003f, 0xe0000000, 0x7f000, 0x0, 0xffc0000f, 0xf8000000, 0x3fdfe, 0x0, 0x1fffc0, 0x7fff0, 0x3, 0xff0001ff, 0x80000000, 0x3f800, 0x3, 0xff800000, 0x3, 0xf0000000, 0x7e000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80000, 0xfff00000, 0xfe000, 0x0, 0x3f800, 0x0, 0x3ff80, 0xfff00, 0x7f0, 0x7f0, 0x1f, 0xc0000007, 0xfc01ff00, 0x1fc, 0x7fe0, 0x7f, 0x0, 0x7f00, 0x1fff001f, 0xc0000001, 0xfc00007f, 0xfc000000, 0x1ffc000, 0x3ffc0000, 0x3f800, 0x0, 0x7ff0, 0x1fffe00, 0x7f, 0x1ff, 0x3, 0xff80007f, 0xe0000000, 0x7f000, 0x0, 0x7fe0003f, 0xf0000000, 0x1fdfc, 0x0, 0x1fff80, 0x3fff0, 0x3, 0xff0000ff, 0x80000000, 0x3f800, 0x7, 0xff000000, 0x3, 0xf0000000, 0x7e000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3f80007, 0xffe00000, 0xfe000, 0x0, 0x3f800, 0x0, 0x3ffe0, 0x3fff00, 0x7f0, 0x7f0, 0x1f, 0xc0000007, 0xfe03fe00, 0x1fc, 0x3ff0, 0x7f, 0x0, 0x7f00, 0x1fff001f, 0xc0000001, 0xfc00007f, 0xfc000000, 0x1ffe000, 0x7ff80000, 0x3f800, 0x0, 0x3ffc, 0x7ffc00, 0x7f, 0x1ff, 0x3, 0xffc000ff, 0xe0000000, 0x7f000, 0x0, 0x7ff0007f, 0xf0000000, 0x1fffc, 0x0, 0xfff80, 0x3ffe0, 0x7, 0xfe0000ff, 0xc0000000, 0x3f800, 0x7, 0xfe000000, 0x3, 0xf0000000, 0x7f000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xffc00000, 0xfffff, 0xffff8000, 0x3f800, 0x0, 0x1fffc, 0x1ffff00, 0x7f0, 0x7f0, 0x1f, 0xc0000003, 0xff07fe00, 0x1fc, 0x3ff0, 0x7f, 0xffffff00, 0x7f00, 0x1fff001f, 0xc0000001, 0xfc00003f, 0xfc000000, 0xfffc03, 0xfff00000, 0x3f800, 0x0, 0x1fff, 0x80fffc00, 0x7f, 0xff, 0x80000001, 0xfff807ff, 0xc0000000, 0x7f000, 0x0, 0x7ffc03ff, 0xe0000000, 0x1fffc, 0x0, 0xfff80, 0x3ffe0, 0xf, 0xfc00007f, 0xe0000000, 0x3f800, 0x7, 0xffffffff, 0xf8000003, 0xf0000000, 0x7f000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xffc00000, 0xfffff, 0xffff8000, 0x3f800, 0x0, 0xffff, 0xfffffe00, 0x7f0, 0x7f0, 0x1f, 0xc0000003, 0xfffffe00, 0x1fc, 0x1ff8, 0x7f, 0xffffff00, 0x7f00, 0xffe001f, 0xc0000001, 0xfc00001f, 0xfc000000, 0x7fffff, 0xffe00000, 0x3f800, 0x0, 0xfff, 0xfffffc00, 0x7f, 0xff, 0xc0000000, 0xffffffff, 0x80000000, 0x7f000, 0x0, 0x3fffffff, 0xe0000000, 0xfff8, 0x0, 0xfff80, 0x3ffe0, 0xf, 0xfc00003f, 0xe0000000, 0x3f800, 0x7, 0xffffffff, 0xf8000003, 0xf0000000, 0x3f000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xff800000, 0xfffff, 0xffff8000, 0x3f800, 0x0, 0x7fff, 0xfffff800, 0x7f0, 0x7f0, 0x1f, 0xc0000001, 0xfffffc00, 0x1fc, 0xffc, 0x7f, 0xffffff00, 0x7f00, 0xffe001f, 0xc0000001, 0xfc00001f, 0xfc000000, 0x3fffff, 0xffc00000, 0x3f800, 0x0, 0x7ff, 0xfffffe00, 0x7f, 0x7f, 0xc0000000, 0x7fffffff, 0x80000000, 0x7f000, 0x0, 0x1fffffff, 0xc0000000, 0xfff8, 0x0, 0xfff00, 0x1ffe0, 0x1f, 0xf800003f, 0xf0000000, 0x3f800, 0x7, 0xffffffff, 0xf8000003, 0xf0000000, 0x3f000, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xfe000000, 0xfffff, 0xffff8000, 0x3f800, 0x0, 0x3fff, 0xfffff000, 0x7f0, 0x7f0, 0x1f, 0xc0000001, 0xfffff800, 0x1fc, 0xffc, 0x7f, 0xffffff00, 0x7f00, 0xffe001f, 0xc0000001, 0xfc00000f, 0xfc000000, 0x1fffff, 0xff800000, 0x3f800, 0x0, 0x3ff, 0xffffff80, 0x7f, 0x7f, 0xe0000000, 0x3fffffff, 0x0, 0x7f000, 0x0, 0xfffffff, 0x80000000, 0xfff8, 0x0, 0x7ff00, 0x1ffc0, 0x3f, 0xf800001f, 0xf8000000, 0x3f800, 0x7, 0xffffffff, 0xf8000003, 0xf0000000, 0x3f800, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xfc000000, 0xfffff, 0xffff8000, 0x3f800, 0x0, 0xfff, 0xffffc000, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0xfffff800, 0x1fc, 0x7fe, 0x7f, 0xffffff00, 0x7f00, 0xffe001f, 0xc0000001, 0xfc00000f, 0xfc000000, 0xfffff, 0xff000000, 0x3f800, 0x0, 0x1ff, 0xffffffe0, 0x7f, 0x3f, 0xe0000000, 0x1ffffffc, 0x0, 0x7f000, 0x0, 0x7ffffff, 0x0, 0x7ff0, 0x0, 0x7ff00, 0x1ffc0, 0x3f, 0xf000001f, 0xf8000000, 0x3f800, 0x7, 0xffffffff, 0xf8000003, 0xf0000000, 0x1f800, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xf0000000, 0xfffff, 0xffff8000, 0x3f800, 0x0, 0x3ff, 0xffff0000, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x7ffff000, 0x1fc, 0x3ff, 0x7f, 0xffffff00, 0x7f00, 0x7fc001f, 0xc0000001, 0xfc000007, 0xfc000000, 0x3ffff, 0xfc000000, 0x3f800, 0x0, 0x7f, 0xfffffff8, 0x7f, 0x3f, 0xf0000000, 0xffffff8, 0x0, 0x7f000, 0x0, 0x3fffffe, 0x0, 0x7ff0, 0x0, 0x7ff00, 0x1ffc0, 0x7f, 0xe000000f, 0xfc000000, 0x3f800, 0x7, 0xffffffff, 0xf8000003, 0xf0000000, 0x1f800, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0x80000000, 0xfffff, 0xffff8000, 0x3f800, 0x0, 0xff, 0xfffc0000, 0x7f0, 0x7f0, 0x1f, 0xc0000000, 0x3fffc000, 0x1fc, 0x3ff, 0x7f, 0xffffff00, 0x7f00, 0x7fc001f, 0xc0000001, 0xfc000003, 0xfc000000, 0xffff, 0xf0000000, 0x3f800, 0x0, 0x3f, 0xfffe7ff8, 0x7f, 0x1f, 0xf0000000, 0x3ffffe0, 0x0, 0x7f000, 0x0, 0xfffff8, 0x0, 0x7ff0, 0x0, 0x7fe00, 0xffc0, 0x7f, 0xe0000007, 0xfc000000, 0x3f800, 0x7, 0xffffffff, 0xf8000003, 0xf0000000, 0x1f800, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f, 0xffe00000, 0x0, 0x0, 0x0, 0x0, 0xfff0000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fff, 0x80000000, 0x0, 0x0, 0x7, 0xfff01ff8, 0x0, 0x0, 0x0, 0x7fff80, 0x0, 0x0, 0x0, 0x1fffc0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xf0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x78000000, 0x0, 0x0, 0x0, 0x0, 0xf00000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xf0, 0x0, 0x0, 0x0, 0x0, 0x1c000ff8, 0x0, 0x0, 0x0, 0x1e000, 0x0, 0x0, 0x0, 0x7000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xf0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3f0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xf0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xf0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x30, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xf0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xf0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xf0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xf0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xffe00000, 0x0, 0xfff800, 0x0, 0x7, 0xffffffff, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xffe00000, 0x0, 0xfff800, 0x0, 0x7, 0xffffffff, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xffe00000, 0x0, 0xfff800, 0x0, 0x7, 0xffffffff, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xffe00000, 0x0, 0xfff800, 0x0, 0x7, 0xffffffff, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xffe00000, 0x0, 0xfff800, 0x0, 0x7, 0xffffffff, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xffe00000, 0x0, 0xfff800, 0x0, 0x7, 0xffffffff, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xffe00000, 0x0, 0xfff800, 0x0, 0x7, 0xffffffff, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3, 0xffe00000, 0x0, 0xfff800, 0x0, 0x7, 0xffffffff, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7ff000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7f0, 0x0, 0x7f0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0x1fff000, 0x0, 0x0, 0xfc00000, 0x3, 0xf0000000, 0xfc000000, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1ff0, 0x3e0, 0x7fc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0x3fff000, 0x0, 0x0, 0xfc00000, 0x3, 0xf0000000, 0xfc000000, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7ff0, 0x3e0, 0x7fe, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0x7fff000, 0x0, 0x0, 0xfc00000, 0x3, 0xf0000000, 0xfc000000, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x180, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7ff0, 0x3e0, 0x7ff, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0x7fff000, 0x0, 0x0, 0xfc00000, 0x3, 0xf0000000, 0xfc000000, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x780, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfff0, 0x3e0, 0x7ff, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0xffff000, 0x0, 0x0, 0xfc00000, 0x3, 0xf0000000, 0xfc000000, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xf80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfff0, 0x3e0, 0x7ff, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0xfffe000, 0x0, 0x0, 0xfc00000, 0x3, 0xf0000000, 0xfc000000, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fff0, 0x3e0, 0x7ff, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0xffc6000, 0x0, 0x0, 0xfc00000, 0x3, 0xf0000000, 0xfc000000, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1ffc0, 0x3e0, 0xff, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0xff00000, 0x0, 0x0, 0xfc00000, 0x0, 0x0, 0x0, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fe00, 0x3e0, 0x3f, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0xfe00000, 0x0, 0x0, 0xfc00000, 0x0, 0x0, 0x0, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fc00, 0x3e0, 0x1f, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0xfe00000, 0x0, 0x0, 0xfc00000, 0x0, 0x0, 0x0, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fc00, 0x3e0, 0x1f, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0xfe00000, 0x0, 0x0, 0xfc00000, 0x0, 0x0, 0x0, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fc00, 0x3e0, 0x1f, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0xfe00000, 0x0, 0x0, 0xfc00000, 0x0, 0x0, 0x0, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fc00, 0x3e0, 0xf, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0xfc00000, 0x0, 0x0, 0xfc00000, 0x0, 0x0, 0x0, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fc00, 0x3e0, 0xf, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x3, 0xf0000000, 0x0, 0x0, 0x0, 0x3f00000, 0x0, 0x0, 0xfc00000, 0x0, 0x0, 0xfc00000, 0x0, 0x0, 0x0, 0x3f000000, 0x7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f800, 0x3e0, 0xf, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1ff80, 0x3, 0xf01fc000, 0x0, 0xfe00000, 0xfe, 0x3f00000, 0x1fe0, 0x0, 0xfc00000, 0x1fc0, 0x0, 0xfc07f00, 0x0, 0x0, 0x0, 0x3f000000, 0x7e, 0x0, 0x7e0003, 0xf8000000, 0xfe, 0x0, 0x1fc00, 0x0, 0x3f8000, 0x0, 0x7e000000, 0xf, 0x80000000, 0x1fe00000, 0x1f80, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1f800, 0x3e0, 0xf, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0xffff0, 0x3, 0xf07ff800, 0x0, 0x7ffe0000, 0x7ff, 0x83f00000, 0xfffc, 0x1, 0xffff8000, 0xfff8, 0x7e000000, 0xfc1ffe0, 0x3, 0xf0000000, 0xfc000000, 0x3f0003fe, 0x7e, 0x1f, 0x83ffc00f, 0xff000000, 0x3f07ff, 0x80000000, 0xfffc0, 0x7, 0xe0fff000, 0x3, 0xffc1f800, 0x3e3f, 0xf0000000, 0xfffe0000, 0x3fffe, 0xfc, 0x1f800, 0x3f800, 0xfe0000, 0x3f8000fe, 0x3f800, 0xff0000, 0x7fc00007, 0xf00007f8, 0x1fff, 0xffff0000, 0x1f800, 0x3e0, 0xf, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffc, 0x3, 0xf1fffe00, 0x1, 0xffff0000, 0x1fff, 0xe3f00000, 0x3ffff, 0x1, 0xffff8000, 0x3fffc, 0x7e000000, 0xfc7fff8, 0x3, 0xf0000000, 0xfc000000, 0x3f0007fe, 0x7e, 0x1f, 0x87ffe03f, 0xff800000, 0x3f1fff, 0xe0000000, 0x3fffe0, 0x7, 0xe3fffc00, 0xf, 0xffe1f800, 0x3e7f, 0xf8000003, 0xffff8000, 0x3fffe, 0xfc, 0x1f800, 0x3f800, 0xfe0000, 0x3fc001ff, 0x7f800, 0xff8000, 0x7f800007, 0xf00007f8, 0x1fff, 0xffff0000, 0x1f800, 0x3e0, 0xf, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0xfffffe, 0x3, 0xf3ffff00, 0x7, 0xffffc000, 0x3fff, 0xf3f00000, 0x7ffff, 0x80000001, 0xffff8000, 0x7fffe, 0x7e000000, 0xfcffffc, 0x3, 0xf0000000, 0xfc000000, 0x3f000ffc, 0x7e, 0x1f, 0x9ffff07f, 0xffe00000, 0x3f3fff, 0xf0000000, 0xfffff8, 0x7, 0xe7fffe00, 0x1f, 0xfff9f800, 0x3eff, 0xf8000007, 0xffffc000, 0x3fffe, 0xfc, 0x1f800, 0x3fc00, 0x1fe0000, 0x3fc001ff, 0x7f800, 0xff8000, 0xff800007, 0xf80007f0, 0x1fff, 0xffff0000, 0x1f800, 0x3e0, 0xf, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1ffffff, 0x3, 0xf7ffff80, 0xf, 0xffffe000, 0x7fff, 0xfbf00000, 0x1fffff, 0xe0000001, 0xffff8000, 0xfffff, 0xfe000000, 0xfdffffe, 0x3, 0xf0000000, 0xfc000000, 0x3f001ff8, 0x7e, 0x1f, 0xbffff8ff, 0xffe00000, 0x3f7fff, 0xf8000000, 0x1fffffc, 0x7, 0xefffff00, 0x3f, 0xfff9f800, 0x3eff, 0xf800000f, 0xffffe000, 0x3fffe, 0xfc, 0x1f800, 0x3fc00, 0x1fe0000, 0x1fc001ff, 0x7f000, 0x7fc001, 0xff000007, 0xf80007f0, 0x1fff, 0xffff0000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0x80000003, 0xffffffc0, 0x1f, 0xfffff000, 0xffff, 0xfff00000, 0x1fffff, 0xf0000001, 0xffff8000, 0x1fffff, 0xfe000000, 0xfffffff, 0x3, 0xf0000000, 0xfc000000, 0x3f003ff0, 0x7e, 0x1f, 0xbffffdff, 0xfff00000, 0x3fffff, 0xfc000000, 0x3fffffe, 0x7, 0xefffff80, 0x7f, 0xfffdf800, 0x3fff, 0xf800001f, 0xfffff000, 0x3fffe, 0xfc, 0x1f800, 0x1fc00, 0x1fc0000, 0x1fc001ff, 0x7f000, 0x3fc001, 0xfe000003, 0xf8000ff0, 0x1fff, 0xffff0000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0x80000003, 0xffffffe0, 0x3f, 0xfffff000, 0x1ffff, 0xfff00000, 0x3fffff, 0xf0000001, 0xffff8000, 0x3fffff, 0xfe000000, 0xfffffff, 0x3, 0xf0000000, 0xfc000000, 0x3f007fe0, 0x7e, 0x1f, 0xffffffff, 0xfff00000, 0x3fffff, 0xfc000000, 0x7ffffff, 0x7, 0xffffffc0, 0xff, 0xfffff800, 0x3fff, 0xf000001f, 0xfffff000, 0x3fffe, 0xfc, 0x1f800, 0x1fe00, 0x3fc0000, 0x1fe001ff, 0x800ff000, 0x3fe003, 0xfe000003, 0xfc000fe0, 0x1fff, 0xffff0000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x7ffffff, 0xc0000003, 0xffffffe0, 0x3f, 0xfffff800, 0x1ffff, 0xfff00000, 0x7fffff, 0xf8000001, 0xffff8000, 0x3fffff, 0xfe000000, 0xfffffff, 0x3, 0xf0000000, 0xfc000000, 0x3f007fc0, 0x7e, 0x1f, 0xffffffff, 0xfff80000, 0x3fffff, 0xfe000000, 0x7ffffff, 0x7, 0xffffffc0, 0xff, 0xfffff800, 0x3fff, 0xf000003f, 0xfffff800, 0x3fffe, 0xfc, 0x1f800, 0x1fe00, 0x3fc0000, 0xfe003ff, 0x800fe000, 0x1ff007, 0xfc000003, 0xfc000fe0, 0x1fff, 0xffff0000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x7fe00ff, 0xc0000003, 0xffe07ff0, 0x7f, 0xf01ff800, 0x3ff01, 0xfff00000, 0xffe01f, 0xfc000001, 0xffff8000, 0x7ff03f, 0xfe000000, 0xfff01ff, 0x80000003, 0xf0000000, 0xfc000000, 0x3f00ffc0, 0x7e, 0x1f, 0xfe07fff8, 0x1ff80000, 0x3ffc07, 0xfe000000, 0xffe03ff, 0x80000007, 0xffc0ffe0, 0x1ff, 0x80fff800, 0x3ff0, 0x7000003f, 0xe01ff800, 0x3fffe, 0xfc, 0x1f800, 0xfe00, 0x3f80000, 0xfe003ff, 0x800fe000, 0xff007, 0xf8000001, 0xfc001fe0, 0x1fff, 0xffff0000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x1fe0000, 0x10000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0xff8003f, 0xc0000003, 0xff801ff0, 0x7f, 0xc007fc00, 0x3fe00, 0x7ff00000, 0xff8007, 0xfc000000, 0xfc00000, 0x7fc00f, 0xfe000000, 0xffc00ff, 0x80000003, 0xf0000000, 0xfc000000, 0x3f01ff80, 0x7e, 0x1f, 0xf801ffe0, 0x7f80000, 0x3ff003, 0xfe000000, 0xff800ff, 0x80000007, 0xff003fe0, 0x1ff, 0x3ff800, 0x3fe0, 0x2000003f, 0xc007f800, 0x1f80, 0xfc, 0x1f800, 0xff00, 0x7f80000, 0xff003ff, 0x801fe000, 0xff80f, 0xf8000001, 0xfe001fc0, 0x0, 0x7fe0000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x7ff8000, 0x30000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0xff0001f, 0xc0000003, 0xff000ff8, 0xff, 0x8003fc00, 0x7fc00, 0x3ff00000, 0x1ff0003, 0xfe000000, 0xfc00000, 0xff8007, 0xfe000000, 0xff8007f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f03ff00, 0x7e, 0x1f, 0xf001ffc0, 0x7f80000, 0x3fe001, 0xfe000000, 0x1ff0007f, 0xc0000007, 0xfe001ff0, 0x3fe, 0x1ff800, 0x3fc0, 0x3f, 0x8003f800, 0x1f80, 0xfc, 0x1f800, 0xff00, 0x7f80000, 0x7f003ff, 0x801fc000, 0x7fc1f, 0xf0000001, 0xfe001fc0, 0x0, 0x7fc0000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x1fffe000, 0x70000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0xfe0001f, 0xc0000003, 0xfe0007f8, 0xff, 0x3fc00, 0x7f800, 0x1ff00000, 0x1fe0001, 0xfe000000, 0xfc00000, 0xff0003, 0xfe000000, 0xff0003f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f07fe00, 0x7e, 0x1f, 0xe000ff80, 0x3f80000, 0x3fc000, 0xfe000000, 0x1fe0003f, 0xc0000007, 0xfc000ff0, 0x3fc, 0xff800, 0x3fc0, 0x3f, 0x8003f800, 0x1f80, 0xfc, 0x1f800, 0x7f00, 0x7f00000, 0x7f007ff, 0xc01fc000, 0x3fc1f, 0xe0000000, 0xfe003fc0, 0x0, 0xff80000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x3ffff800, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0xfe0001f, 0xc0000003, 0xfe0003f8, 0xfe, 0x1fc00, 0x7f000, 0x1ff00000, 0x1fc0000, 0xfe000000, 0xfc00000, 0xfe0001, 0xfe000000, 0xff0003f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f0ffc00, 0x7e, 0x1f, 0xe000ff80, 0x3f80000, 0x3fc000, 0xfe000000, 0x1fe0003f, 0xc0000007, 0xfc0007f0, 0x3f8, 0xff800, 0x3f80, 0x3f, 0x8003e000, 0x1f80, 0xfc, 0x1f800, 0x7f80, 0x7f00000, 0x7f007ff, 0xc03fc000, 0x3fe3f, 0xe0000000, 0xff003fc0, 0x0, 0x1ff00000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x3ffffc01, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3e0001f, 0xc0000003, 0xfc0003fc, 0x1fe, 0x1f000, 0xff000, 0xff00000, 0x3f80000, 0x7f000000, 0xfc00000, 0x1fe0001, 0xfe000000, 0xfe0003f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f1ff800, 0x7e, 0x1f, 0xc000ff00, 0x3f80000, 0x3f8000, 0xfe000000, 0x3fc0001f, 0xe0000007, 0xf80007f8, 0x7f8, 0x7f800, 0x3f80, 0x3f, 0xc0000000, 0x1f80, 0xfc, 0x1f800, 0x7f80, 0xff00000, 0x7f807ff, 0xc03fc000, 0x1ff7f, 0xc0000000, 0xff003f80, 0x0, 0x3ff00000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x7fffff03, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1f, 0xc0000003, 0xfc0003fc, 0x1fe, 0x0, 0xff000, 0xff00000, 0x3f80000, 0x7f000000, 0xfc00000, 0x1fc0001, 0xfe000000, 0xfe0003f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f3ff000, 0x7e, 0x1f, 0xc000ff00, 0x3f80000, 0x3f8000, 0xfe000000, 0x3fc0001f, 0xe0000007, 0xf80007f8, 0x7f8, 0x7f800, 0x3f80, 0x3f, 0xe0000000, 0x1f80, 0xfc, 0x1f800, 0x3f80, 0xfe00000, 0x3f807ef, 0xc03f8000, 0xffff, 0x80000000, 0x7f007f80, 0x0, 0x7fe00000, 0x7f800, 0x3e0, 0xf, 0xf0000000, 0x7fffffff, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1f, 0xc0000003, 0xf80001fc, 0x1fc, 0x0, 0xfe000, 0x7f00000, 0x3f80000, 0x7f000000, 0xfc00000, 0x1fc0000, 0xfe000000, 0xfe0003f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f7fe000, 0x7e, 0x1f, 0xc000ff00, 0x3f80000, 0x3f8000, 0xfe000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f80, 0x3f, 0xfc000000, 0x1f80, 0xfc, 0x1f800, 0x3fc0, 0xfe00000, 0x3f807ef, 0xe03f8000, 0xffff, 0x80000000, 0x7f807f80, 0x0, 0xffc00000, 0xff000, 0x3e0, 0x7, 0xf8000000, 0x7fffffff, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0xff, 0xc0000003, 0xf80001fc, 0x1fc, 0x0, 0xfe000, 0x7f00000, 0x3f80000, 0x7f000000, 0xfc00000, 0x1fc0000, 0xfe000000, 0xfe0003f, 0x80000003, 0xf0000000, 0xfc000000, 0x3fffc000, 0x7e, 0x1f, 0xc0007f00, 0x3f80000, 0x3f8000, 0xfe000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f80, 0x3f, 0xff800000, 0x1f80, 0xfc, 0x1f800, 0x3fc0, 0x1fe00000, 0x3fc0fef, 0xe07f8000, 0x7fff, 0x0, 0x7f807f00, 0x0, 0xff800000, 0x1ff000, 0x3e0, 0x7, 0xfc000000, 0x7fbfffff, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3fff, 0xc0000003, 0xf80001fc, 0x1fc, 0x0, 0xfe000, 0x7f00000, 0x3ffffff, 0xff000000, 0xfc00000, 0x1fc0000, 0xfe000000, 0xfe0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3fffe000, 0x7e, 0x1f, 0xc0007f00, 0x1f80000, 0x3f8000, 0xfe000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f80, 0x1f, 0xfff00000, 0x1f80, 0xfc, 0x1f800, 0x1fc0, 0x1fc00000, 0x1fc0fef, 0xe07f0000, 0x3ffe, 0x0, 0x3f807f00, 0x1, 0xff000000, 0x7fe000, 0x3e0, 0x3, 0xff000000, 0x7e03ffff, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0xfffff, 0xc0000003, 0xf80001fc, 0x1fc, 0x0, 0xfe000, 0x7f00000, 0x3ffffff, 0xff000000, 0xfc00000, 0x1fc0000, 0xfe000000, 0xfe0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3fffe000, 0x7e, 0x1f, 0xc0007f00, 0x1f80000, 0x3f8000, 0x7e000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f80, 0x1f, 0xfffe0000, 0x1f80, 0xfc, 0x1f800, 0x1fe0, 0x1fc00000, 0x1fc0fcf, 0xe07f0000, 0x3ffc, 0x0, 0x3fc0ff00, 0x3, 0xfe000000, 0xffe000, 0x3e0, 0x1, 0xff800000, 0x7801ffff, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x7fffff, 0xc0000003, 0xf80001fc, 0x1fc, 0x0, 0xfe000, 0x7f00000, 0x3ffffff, 0xff000000, 0xfc00000, 0x1fc0000, 0xfe000000, 0xfe0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3ffff000, 0x7e, 0x1f, 0xc0007e00, 0x1f80000, 0x3f8000, 0x7e000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f80, 0xf, 0xffffc000, 0x1f80, 0xfc, 0x1f800, 0x1fe0, 0x3fc00000, 0x1fe0fc7, 0xe0ff0000, 0x1ffc, 0x0, 0x3fc0fe00, 0x7, 0xfe000000, 0xffc000, 0x3e0, 0x1, 0xff800000, 0x70007fff, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0xffffff, 0xc0000003, 0xf80001fc, 0x1fc, 0x0, 0xfe000, 0x7f00000, 0x3ffffff, 0xff000000, 0xfc00000, 0x1fc0000, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3ffff800, 0x7e, 0x1f, 0xc0007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f00, 0x7, 0xffffe000, 0x1f80, 0xfc, 0x1f800, 0xfe0, 0x3f800000, 0xfe1fc7, 0xf0fe0000, 0xff8, 0x0, 0x1fc0fe00, 0xf, 0xfc000000, 0xff0000, 0x3e0, 0x0, 0x7f800000, 0x60001fff, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffffff, 0xc0000003, 0xf80001fc, 0x1fc, 0x0, 0xfe000, 0x7f00000, 0x3ffffff, 0xff000000, 0xfc00000, 0x1fc0000, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3ffff800, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f00, 0x3, 0xfffff800, 0x1f80, 0xfc, 0x1f800, 0xff0, 0x3f800000, 0xfe1fc7, 0xf0fe0000, 0xff8, 0x0, 0x1fe1fe00, 0x1f, 0xf8000000, 0xfe0000, 0x3e0, 0x0, 0x3f800000, 0x600007ff, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x7ffffff, 0xc0000003, 0xf80001fc, 0x1fc, 0x0, 0xfe000, 0x7f00000, 0x3ffffff, 0xff000000, 0xfc00000, 0x1fc0000, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3ff7fc00, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f00, 0x0, 0xfffff800, 0x1f80, 0xfc, 0x3f800, 0xff0, 0x7f800000, 0xfe1fc7, 0xf0fe0000, 0x1ffc, 0x0, 0x1fe1fe00, 0x1f, 0xf0000000, 0xff8000, 0x3e0, 0x0, 0xff800000, 0x400001fc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x7ffffff, 0xc0000003, 0xf80001fc, 0x1fc, 0x0, 0xfe000, 0x7f00000, 0x3ffffff, 0xff000000, 0xfc00000, 0x1fc0000, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3ff3fc00, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f00, 0x0, 0x3ffffc00, 0x1f80, 0xfc, 0x3f800, 0x7f0, 0x7f000000, 0x7f1f83, 0xf1fc0000, 0x3ffc, 0x0, 0xfe1fc00, 0x3f, 0xe0000000, 0xffc000, 0x3e0, 0x1, 0xff800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0xffffc1f, 0xc0000003, 0xf80001fc, 0x1fc, 0x0, 0xfe000, 0x7f00000, 0x3f80000, 0x0, 0xfc00000, 0x1fc0000, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3fe1fe00, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f00, 0x0, 0x7fffe00, 0x1f80, 0xfe, 0x3f800, 0x7f8, 0x7f000000, 0x7f3f83, 0xf9fc0000, 0x3ffe, 0x0, 0xff3fc00, 0x7f, 0xc0000000, 0xffe000, 0x3e0, 0x3, 0xff800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1fff001f, 0xc0000003, 0xf80001fc, 0x1fc, 0x0, 0xfe000, 0x7f00000, 0x3f80000, 0x0, 0xfc00000, 0x1fc0000, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3fc1ff00, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f00, 0x0, 0xfffe00, 0x1f80, 0xfe, 0x3f800, 0x7f8, 0xff000000, 0x7f3f83, 0xf9fc0000, 0x7fff, 0x0, 0xff3fc00, 0xff, 0xc0000000, 0x3fe000, 0x3e0, 0x7, 0xfe000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1ff0001f, 0xe0000003, 0xf80001fc, 0x1fc, 0xc000, 0xfe000, 0x7f00000, 0x3f80000, 0x0, 0xfc00000, 0x1fc0000, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f80ff00, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f00, 0x0, 0xffe00, 0x1f80, 0xfe, 0x3f800, 0x3f8, 0xfe000000, 0x7f3f83, 0xfbfc0000, 0xffff, 0x0, 0x7f3f800, 0x1ff, 0x80000000, 0xff000, 0x3e0, 0x7, 0xf8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1fe0001f, 0xe0000003, 0xf80001fc, 0x1fc, 0xfe00, 0xfe000, 0x7f00000, 0x3f80000, 0x0, 0xfc00000, 0x1fe0001, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f00ff80, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3f80000f, 0xe0000007, 0xf00003f8, 0x7f0, 0x3f800, 0x3f00, 0x2, 0x3fe00, 0x1f80, 0xfe, 0x3f800, 0x3f8, 0xfe000000, 0x3fbf03, 0xfbf80000, 0xffff, 0x80000000, 0x7fbf800, 0x3ff, 0x0, 0x7f000, 0x3e0, 0xf, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1fc0001f, 0xe0000003, 0xf80003fc, 0x1fe, 0xfe00, 0xff000, 0xff00000, 0x3fc0000, 0x0, 0xfc00000, 0x1fe0001, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f007f80, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3fc0001f, 0xe0000007, 0xf80007f8, 0x7f8, 0x7f800, 0x3f00, 0x7f, 0xfe00, 0x1f80, 0xfe, 0x3f800, 0x3fd, 0xfe000000, 0x3fbf01, 0xfbf80000, 0x1ff7f, 0xc0000000, 0x7fff800, 0x7fe, 0x0, 0x7f800, 0x3e0, 0xf, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1fc0003f, 0xe0000003, 0xfc0003f8, 0x1fe, 0xfe00, 0xff000, 0xff00000, 0x3fc0000, 0x7f000000, 0xfc00000, 0xff0003, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f003fc0, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3fc0001f, 0xe0000007, 0xf80007f0, 0x3f8, 0x7f800, 0x3f00, 0x7f, 0xfe00, 0x1f80, 0xfe, 0x3f800, 0x1fd, 0xfc000000, 0x3fff01, 0xfff80000, 0x3fe7f, 0xc0000000, 0x7fff000, 0x7fc, 0x0, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1fc0003f, 0xe0000003, 0xfc0007f8, 0xff, 0x1fe00, 0x7f800, 0x1ff00000, 0x1fe0000, 0xff000000, 0xfc00000, 0xff0003, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f003fe0, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x1fe0003f, 0xc0000007, 0xfc000ff0, 0x3fc, 0xff800, 0x3f00, 0x7f, 0xfe00, 0x1f80, 0xfe, 0x7f800, 0x1fd, 0xfc000000, 0x1fff01, 0xfff00000, 0x7fc3f, 0xe0000000, 0x3fff000, 0xff8, 0x0, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1fc0007f, 0xe0000003, 0xfe0007f8, 0xff, 0x1fe00, 0x7f800, 0x1ff00000, 0x1ff0000, 0xff000000, 0xfc00000, 0xff8007, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f001fe0, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x1fe0003f, 0xc0000007, 0xfc000ff0, 0x3fc, 0xff800, 0x3f00, 0x7f, 0x8000fe00, 0x1fc0, 0xff, 0xff800, 0x1ff, 0xfc000000, 0x1fff01, 0xfff00000, 0x7fc1f, 0xf0000000, 0x3fff000, 0x1ff8, 0x0, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1fc000ff, 0xe0000003, 0xff000ff0, 0xff, 0x8003fc00, 0x3fc00, 0x3ff00000, 0x1ff8001, 0xff000000, 0xfc00000, 0x7fc00f, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f001ff0, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x1ff0007f, 0xc0000007, 0xfe001fe0, 0x1fe, 0x1ff800, 0x3f00, 0x7f, 0xc001fe00, 0x1fc0, 0xff, 0xff800, 0xff, 0xf8000000, 0x1ffe01, 0xfff00000, 0xff81f, 0xf0000000, 0x3ffe000, 0x3ff0, 0x0, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1fe003ff, 0xe0000003, 0xff803ff0, 0x7f, 0xc007fc00, 0x3ff00, 0x7ff00000, 0xffc007, 0xfe000000, 0xfc00000, 0x7ff87f, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f000ff0, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0xffc01ff, 0x80000007, 0xff007fe0, 0x1ff, 0x803ff800, 0x3f00, 0x3f, 0xe003fe00, 0x1fc0, 0xff, 0x803ff800, 0xff, 0xf8000000, 0xffe00, 0xffe00000, 0x1ff00f, 0xf8000000, 0x1ffe000, 0x7fe0, 0x0, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1ff80fff, 0xe0000003, 0xffe0ffe0, 0x7f, 0xf81ffc00, 0x3ffc1, 0xfff00000, 0xfff01f, 0xfe000000, 0xfc00000, 0x3fffff, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f0007f8, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0xfff07ff, 0x80000007, 0xffc1ffc0, 0xff, 0xe0fff800, 0x3f00, 0x3f, 0xf80ffe00, 0x1fe0, 0xff, 0xe0fff800, 0xff, 0xf8000000, 0xffe00, 0xffe00000, 0x1fe007, 0xf8000000, 0x1ffe000, 0x7fff, 0xffff8000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1fffffff, 0xe0000003, 0xffffffe0, 0x3f, 0xfffff800, 0x1ffff, 0xfff00000, 0x7fffff, 0xfc000000, 0xfc00000, 0x1fffff, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f0007f8, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x7ffffff, 0x7, 0xffffffc0, 0xff, 0xfffff800, 0x3f00, 0x3f, 0xfffffc00, 0x1fff, 0x7f, 0xfffff800, 0x7f, 0xf0000000, 0xffe00, 0xffe00000, 0x3fe007, 0xfc000000, 0x1ffe000, 0x7fff, 0xffff8000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0xfffffff, 0xe0000003, 0xffffffc0, 0x1f, 0xfffff000, 0xffff, 0xfff00000, 0x3fffff, 0xf8000000, 0xfc00000, 0x1fffff, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f0003fc, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3fffffe, 0x7, 0xffffff80, 0x7f, 0xfffff800, 0x3f00, 0x1f, 0xfffff800, 0x1fff, 0x7f, 0xfffff800, 0x7f, 0xf0000000, 0xffe00, 0xffc00000, 0x7fc003, 0xfe000000, 0xffc000, 0x7fff, 0xffff8000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0xfffffef, 0xe0000003, 0xf7ffff80, 0x1f, 0xffffe000, 0x7fff, 0xfbf00000, 0x1fffff, 0xf8000000, 0xfc00000, 0xfffff, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f0003fe, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3fffffe, 0x7, 0xffffff00, 0x3f, 0xfffff800, 0x3f00, 0xf, 0xfffff800, 0x1fff, 0x3f, 0xfffdf800, 0x7f, 0xf0000000, 0x7fc00, 0x7fc00000, 0x7f8001, 0xfe000000, 0xffc000, 0x7fff, 0xffff8000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x7ffffcf, 0xf0000003, 0xf7ffff00, 0xf, 0xffffe000, 0x3fff, 0xfbf00000, 0xfffff, 0xf0000000, 0xfc00000, 0x7fffe, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f0001fe, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x1fffffc, 0x7, 0xeffffe00, 0x1f, 0xfffdf800, 0x3f00, 0x7, 0xfffff000, 0x1fff, 0x3f, 0xfff9f800, 0x3f, 0xe0000000, 0x7fc00, 0x7fc00000, 0xff8001, 0xff000000, 0xffc000, 0x7fff, 0xffff8000, 0x3f800, 0x3e0, 0xf, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x3ffff87, 0xf0000003, 0xf3fffe00, 0x3, 0xffff8000, 0x1fff, 0xf3f00000, 0x7ffff, 0xc0000000, 0xfc00000, 0x1fffc, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f0001ff, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x7ffff0, 0x7, 0xe7fffc00, 0xf, 0xfff9f800, 0x3f00, 0x3, 0xffffe000, 0xfff, 0x1f, 0xfff1f800, 0x3f, 0xe0000000, 0x7fc00, 0x7fc00000, 0x1ff0000, 0xff800000, 0x7f8000, 0x7fff, 0xffff8000, 0x1f800, 0x3e0, 0xf, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x1fffe07, 0xf0000003, 0xf0fffc00, 0x1, 0xffff0000, 0xfff, 0xe3f00000, 0x3ffff, 0x80000000, 0xfc00000, 0x7ff0, 0xfe000000, 0xfc0001f, 0x80000003, 0xf0000000, 0xfc000000, 0x3f0000ff, 0x7e, 0x1f, 0x80007e00, 0x1f80000, 0x3f0000, 0x7e000000, 0x3fffe0, 0x7, 0xe3fff800, 0x7, 0xfff1f800, 0x3f00, 0x1, 0xffff8000, 0x7ff, 0xf, 0xffe1f800, 0x3f, 0xe0000000, 0x3fc00, 0x7f800000, 0x1fe0000, 0x7f800000, 0x7f8000, 0x7fff, 0xffff8000, 0x1f800, 0x3e0, 0xf, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x7ff800, 0x0, 0x7ff000, 0x0, 0x7ffc0000, 0x3ff, 0x80000000, 0x7ffe, 0x0, 0x0, 0x0, 0xfe000000, 0x0, 0x0, 0x0, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfff80, 0x7, 0xe0ffc000, 0x1, 0xffc1f800, 0x0, 0x0, 0x7ffe0000, 0x3ff, 0x3, 0xff800000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7f8000, 0x0, 0x0, 0x1f800, 0x3e0, 0xf, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x38000, 0x0, 0x70000, 0x0, 0x3800000, 0x38, 0x0, 0x780, 0x0, 0x0, 0x0, 0xfe000000, 0x0, 0x0, 0x0, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xf000, 0x7, 0xe01c0000, 0x0, 0x1e01f800, 0x0, 0x0, 0x1c00000, 0x78, 0x0, 0x38000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7f0000, 0x0, 0x0, 0x1f800, 0x3e0, 0xf, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfe000000, 0x0, 0x0, 0x1, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7f0000, 0x0, 0x0, 0x1f800, 0x3e0, 0x1f, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xf00000, 0xfe000000, 0x0, 0x0, 0x1, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff0000, 0x0, 0x0, 0x1f800, 0x3e0, 0x1f, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfe0001, 0xfe000000, 0x0, 0x0, 0x1, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfe0000, 0x0, 0x0, 0x1f800, 0x3e0, 0x1f, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfe0001, 0xfe000000, 0x0, 0x0, 0x1, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfe0000, 0x0, 0x0, 0x1fc00, 0x3e0, 0x1f, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfe0003, 0xfc000000, 0x0, 0x0, 0x1, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fe0000, 0x0, 0x0, 0x1fc00, 0x3e0, 0x1f, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff0007, 0xfc000000, 0x0, 0x0, 0x1, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fc0000, 0x0, 0x0, 0x1fc00, 0x3e0, 0x1f, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xffc01f, 0xfc000000, 0x0, 0x0, 0x3, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3fc0000, 0x0, 0x0, 0x1fe00, 0x3e0, 0x3f, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xffffff, 0xf8000000, 0x0, 0x0, 0x7f, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfffc0000, 0x0, 0x0, 0x1fff0, 0x3e0, 0x7ff, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7fffff, 0xf8000000, 0x0, 0x0, 0x7f, 0xfc000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfff80000, 0x0, 0x0, 0x1fff0, 0x3e0, 0x7ff, 0xc0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3fffff, 0xf0000000, 0x0, 0x0, 0x7f, 0xf8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfff80000, 0x0, 0x0, 0xfff0, 0x3e0, 0x7ff, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x3fffff, 0xe0000000, 0x0, 0x0, 0x7f, 0xf8000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xfff00000, 0x0, 0x0, 0xfff0, 0x3e0, 0x7ff, 0x80000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fffff, 0xc0000000, 0x0, 0x0, 0x7f, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xffe00000, 0x0, 0x0, 0x7ff0, 0x3e0, 0x7ff, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7ffff, 0x0, 0x0, 0x0, 0xff, 0xf0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xffe00000, 0x0, 0x0, 0x3ff0, 0x3e0, 0x7fe, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
    {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1fffc, 0x0, 0x0, 0x0, 0xff, 0xe0000000, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x7, 0xe0000000, 0x0, 0x1f800, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff800000, 0x0, 0x0, 0x1ff0, 0x3e0, 0x7fc, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},

};

static int _grtkasciioffset[3][40] = {
    {0, 27, 63, 104, 165, 224, 304, 370, 403, 444, 485, 533, 595, 629, 675, 707, 748, 810, 863, 922, 978, 1038, 1095, 1154, 1212, 1270, 1331, 1369, 1405, 1465, 1525, 1584, 1643, 1727, 1801, 1864},
    {6, 76, 142, 201, 277, 347, 381, 439, 505, 561, 639, 706, 782, 845, 921, 988, 1052, 1119, 1182, 1249, 1337, 1403, 1469, 1534, 1568, 1608, 1646, 1693, 1755},
    {3, 62, 119, 172, 230, 287, 327, 388, 446, 472, 514, 569, 603, 682, 738, 797, 853, 914, 953, 1006, 1048, 1102, 1154, 1223, 1277, 1329, 1384, 1430, 1461, 1505},
};

static int _grtkasciiwidth[3][40] = {
    {10, 6, 19, 38, 34, 54, 42, 6, 16, 16, 23, 34, 6, 19, 6, 21, 33, 19, 33, 33, 35, 33, 34, 33, 33, 33, 6, 6, 33, 33, 33, 33, 65, 49, 38, 45},
    {42, 37, 34, 47, 39, 7, 27, 41, 31, 49, 39, 48, 38, 48, 43, 40, 41, 39, 49, 69, 45, 45, 40, 13, 21, 13, 27, 41, 13},
    {33, 32, 32, 32, 34, 21, 32, 29, 6, 14, 30, 6, 50, 29, 33, 32, 32, 19, 30, 18, 29, 33, 51, 35, 32, 32, 20, 5, 20, 35},

};
void Draw_Array(int arraynumber, int ordinaloffset, int* stroffset) {
#if defined(TEXTDEBUG)
    char rtkascii1[] = " !\"#$%&'()*+,-./0123456789:;<=>?@ABC";
#endif
    int rtkasciioffset[40];
    int rtkasciiwidth[40];
    int cntw=0, cnth=0, dw=0,strnum=0, endnum=0, pixelloc=0, reverseoffset=0, fontoffset=0;
    uint32_t fontaddr = TEXT_SURFACE + *stroffset*4;
    int f1 = 0;

    memcpy(rtkasciioffset, _grtkasciioffset[arraynumber], sizeof(rtkasciioffset));
    memcpy(rtkasciiwidth, _grtkasciiwidth[arraynumber], sizeof(rtkasciiwidth));
    fontoffset = arraynumber*WORD_HEIGHT;

    strnum = rtkasciioffset[ordinaloffset]/32;
    endnum = (rtkasciioffset[ordinaloffset] + rtkasciiwidth[ordinaloffset])/32;

    //printf("surface offset - %08x , array offset is %d , string offset is %d, char offset is %d, width = %d\n",fontaddr,  ordinaloffset, *stroffset, rtkasciioffset[ordinaloffset], rtkasciiwidth[ordinaloffset]);
    //printf("strnum = %d ,  endnum = %d\n", strnum, endnum);
    for(cnth = 0;cnth<WORD_HEIGHT;cnth++) {
        dw = rtkasciiwidth[ordinaloffset]; //reset
        pixelloc = rtkasciioffset[ordinaloffset];
        fontaddr = TEXT_SURFACE +1920*cnth*4 + *stroffset*4;
        f1 =0;
        for(cntw = strnum;cntw <=endnum;cntw++) {
            //printf("%s - %d cnth = %d, cntw = %d dw = %d, pixelloc = %d, ", __func__,__LINE__, cnth, cntw, dw, pixelloc);
            while(dw >=0) {
                reverseoffset = 32-((pixelloc+1)%32);
                if(reverseoffset == 32)
                    reverseoffset = 0;
                //printf(" value = %d, offset = %d \n", fontarray[cnth][cntw], reverseoffset );

                if((fontarray[cnth + fontoffset][cntw] >>reverseoffset)&0x1) {
                    if(f1 ==0){
                        *(uint32_t*)fontaddr = FONT_COLOR_AA2;
                    }else if(f1 ==1){
                        *(uint32_t*)fontaddr = FONT_COLOR_AA;

                    }else {
                        *(uint32_t*)(fontaddr) = FONT_COLOR;
                    }
                    f1++;

                }else {
                    if(f1 != 0) {
                        *(uint32_t*)(fontaddr-4) = FONT_COLOR_AA2;
                        *(uint32_t*)(fontaddr-8) = FONT_COLOR_AA;
                        //*(uint32_t*)(fontaddr-4) = 0x00ff00ff;
                    }
                    if((cnth+fontoffset) >=3 && (cnth+fontoffset) <=237){
                        if((fontarray[cnth + fontoffset-1][cntw] >>reverseoffset)&0x1 &&
                                (fontarray[cnth + fontoffset+1][cntw] >>reverseoffset)&0x1 )
                            *(uint32_t*)(fontaddr) = *(uint32_t*)(fontaddr - 1920*4);
                            //*(uint32_t*)(fontaddr) = FONT_COLOR_AA2;
                        else if((fontarray[cnth + fontoffset-1][cntw] >>reverseoffset)&0x1 ||
                            (fontarray[cnth + fontoffset+1][cntw] >>reverseoffset)&0x1)
                            *(uint32_t*)(fontaddr) = FONT_COLOR_AA2;

                    }

                    f1=0;

                }


                dw--;
                pixelloc++;;
                fontaddr+=4;
                if((pixelloc)%32 == 0)
                    break;
            }
        }
    }


    *stroffset += rtkasciiwidth[ordinaloffset]  + 3;
#if defined(TEXTDEBUG)
    printf("Get rtkascii ordinal offset %d is '%c', pixel offset is %d\n\n", ordinaloffset, rtkascii1[ordinaloffset], rtkasciioffset[ordinaloffset]);
#endif



};

#if defined(SE_EXT)
void DRAW_STRETCH_COPY(int ix, int iy, int iw, int ih, int ox, int oy, int ow, int oh){
	uint32_t cmdLength = 0;
	int instCount = 0;
	uint32_t vScalingRatio = 0;
	uint32_t hScalingRatio = 0;
	SECMD_EXT_STCH_BLIT	se_ext_stch;
	SECMD_WRITE_REGISTER SeCmdWriteRegister;
	SECMD_STRETCH SeCmdStretch;

	memset((void *)&SeCmdWriteRegister, 0, sizeof(SECMD_WRITE_REGISTER));
	memset((void *)&SeCmdStretch, 0, sizeof(SECMD_STRETCH));

//    printf("%s : From x=%d, y=%d,(%d,%d) To x=%d, y=%d, (%d,%d)\n", ix,iy,iw,ih,ox,oy,ow,oh);
	vScalingRatio = (ih << 14)/oh;
	hScalingRatio = (iw << 14)/ow;

	rtd_outl(SE_CMDRPTR_0_reg, SE_CMD_BUFFER_START_ADDR);
	rtd_outl(SE_CMDWPTR_0_reg, SE_CMD_BUFFER_START_ADDR);

	SeCmdWriteRegister.Word1.Reserved = 0x0;
	SeCmdWriteRegister.Word1.OpCode = 0x01;
	SeCmdWriteRegister.Word1.Clut = 0x0;

	/* --- SEINFO_REG_STRETCH*/
	SeCmdWriteRegister.Word1.RegisterAddress = 0x320;
	SeCmdWriteRegister.Word2.RegisterValue =((uint32_t)2 << 3) | ((uint32_t)1 << 2);

	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	/* --- SEINFO_REG_VDHS*/
	SeCmdWriteRegister.Word1.RegisterAddress = 0x340;
	SeCmdWriteRegister.Word2.RegisterValue = vScalingRatio & 0x0003FFFF;

	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	/* --- SEINFO_REG_HDHS*/
	SeCmdWriteRegister.Word1.RegisterAddress = 0x330;
	SeCmdWriteRegister.Word2.RegisterValue = hScalingRatio & 0x0003FFFF;

	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;


	memset(&se_ext_stch, 0x00, sizeof(SECMD_EXT_STCH_BLIT));

	se_ext_stch.Word1.opcode = 0xb;
	se_ext_stch.Word1.rslt_fmt = 0xc;
	se_ext_stch.Word2.rslt_out_x = ox;
	se_ext_stch.Word2.rslt_out_y = oy;
	se_ext_stch.Word3.rslt_addr_index = 0;

	se_ext_stch.Word4.src1_inpt_w = iw;
	se_ext_stch.Word4.src1_inpt_h = ih;
	se_ext_stch.Word4.src1_fmt = 0xc;

	se_ext_stch.Word5.src1_inpt_x = ix;
	se_ext_stch.Word5.src1_inpt_y = iy;
	se_ext_stch.Word5.src1_fcv_stch = 1;


	se_ext_stch.Word6.src1_addr_index = 1;

	se_ext_stch.Word9.lay_num_ext = 2;
	se_ext_stch.Word10.blend_sel_1 = 0x0;
	se_ext_stch.Word10.rop_code_1 = 0x3;
	se_ext_stch.Word12.stch1_out_w = ow;
	se_ext_stch.Word12.stch1_out_h = oh;
	se_ext_stch.Word12.stch1_src = 0x1;
	se_ext_stch.Word12.stch1_algo = 0x1;//0:FIR, 1:repeat
	se_ext_stch.Word12.stch1_h_en = 1;
	se_ext_stch.Word12.stch1_v_en = 1;
	se_ext_stch.Word13.src2_inpt_w = SEINFO_OPCODE_NOP;
	se_ext_stch.Word14.src2_out_x = SEINFO_OPCODE_NOP;
	se_ext_stch.Word15.stch2_out_w = SEINFO_OPCODE_NOP;
	se_ext_stch.Word16.Reserved = SEINFO_OPCODE_NOP;

	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&se_ext_stch, sizeof(SECMD_EXT_STCH_BLIT));
	cmdLength += sizeof(SECMD_EXT_STCH_BLIT)/4;
	instCount += 3;

	se_writecmd(cmdLength);
	se_checkfinish();

};

#else
//input x,y,w,h output x,y,w,h
void DRAW_STRETCH_COPY(int ix, int iy, int iw, int ih, int ox, int oy, int ow, int oh){
	uint32_t cmdLength = 0;
	uint32_t dwDataCounter = 0;
	int instCount = 0;
	uint32_t vScalingRatio = 0;
	uint32_t hScalingRatio = 0;
	unsigned short tmp_coef_4t16p[32];
	unsigned short tmp_coef_8t16p[64];
	SECMD_WRITE_REGISTER SeCmdWriteRegister;
	SECMD_STRETCH SeCmdStretch;

	memset((void *)&SeCmdWriteRegister, 0, sizeof(SECMD_WRITE_REGISTER));
	memset((void *)&SeCmdStretch, 0, sizeof(SECMD_STRETCH));

//    printf("%s : From x=%d, y=%d,(%d,%d) To x=%d, y=%d, (%d,%d)\n", ix,iy,iw,ih,ox,oy,ow,oh);
	vScalingRatio = (ih << 14)/oh;
	hScalingRatio = (iw << 14)/ow;

	rtd_outl(SE_CMDRPTR_0_reg, SE_CMD_BUFFER_START_ADDR);
	rtd_outl(SE_CMDWPTR_0_reg, SE_CMD_BUFFER_START_ADDR);

	SeCmdWriteRegister.Word1.Reserved = 0x0;
	SeCmdWriteRegister.Word1.OpCode = 0x01;
	SeCmdWriteRegister.Word1.Clut = 0x0;


	/* --- SEINFO_REG_STRETCH*/
	SeCmdWriteRegister.Word1.RegisterAddress = 0x2C0;
	SeCmdWriteRegister.Word2.RegisterValue =((uint32_t)2 << 3) | ((uint32_t)1 << 2);

	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	/* --- SEINFO_REG_VDHS*/
	SeCmdWriteRegister.Word1.RegisterAddress = 0x2E0;
	SeCmdWriteRegister.Word2.RegisterValue = vScalingRatio & 0x0003FFFF;

	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	/* --- SEINFO_REG_HDHS*/
	SeCmdWriteRegister.Word1.RegisterAddress = 0x2D0;
	SeCmdWriteRegister.Word2.RegisterValue = hScalingRatio & 0x0003FFFF;

	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	/* --- FIR Algo Vertical Scaling Coeffs*/
	SetVideoScalingCoeffs((unsigned short*)tmp_coef_4t16p,
												vScalingRatio,
												0,
												1 << 2,//4tap
												0);

	for(dwDataCounter=0; dwDataCounter < 16; dwDataCounter++){
		/* --- SEINFO_REG_VCOEF1*/
		SeCmdWriteRegister.Word1.RegisterAddress = 0x400 + (dwDataCounter << 2);
		SeCmdWriteRegister.Word2.RegisterValue = tmp_coef_4t16p[dwDataCounter];

	    memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	    cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	    instCount += 1;

		/* --- SEINFO_REG_VCOEF1_16*/
		SeCmdWriteRegister.Word1.RegisterAddress = 0x440 + (dwDataCounter << 2);
		SeCmdWriteRegister.Word2.RegisterValue = tmp_coef_4t16p[dwDataCounter+16];

	    memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	    cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	    instCount += 1;

	}

	/* --- FIR Algo Horizontal Scaling Coeffs*/
	SetVideoScalingCoeffs((unsigned short*)tmp_coef_8t16p,
												hScalingRatio,
												0,
												1 << 3,//8tap
												0);

	for(dwDataCounter=0; dwDataCounter < 32; dwDataCounter++){

		/* --- SEINFO_REG_HCOEF1*/
		SeCmdWriteRegister.Word1.RegisterAddress = 0x300 + (dwDataCounter << 2);
		SeCmdWriteRegister.Word2.RegisterValue = tmp_coef_8t16p[dwDataCounter];

	    memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	    cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	    instCount += 1;

		/* --- SEINFO_REG_HCOEF1_32*/
		SeCmdWriteRegister.Word1.RegisterAddress = 0x380 + (dwDataCounter << 2);
		SeCmdWriteRegister.Word2.RegisterValue = tmp_coef_8t16p[dwDataCounter+32];

	    memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	    cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	    instCount += 1;
	}

	/*SE_Stretch*/
	SeCmdStretch.Word1.SrcInterleaving = 0x0;
	SeCmdStretch.Word1.VScalingEnable = 0x01;
	SeCmdStretch.Word1.HScalingEnable = 0x01;
	SeCmdStretch.Word1.ColorKey = 0x0;
	SeCmdStretch.Word1.OutputY = oy;
	SeCmdStretch.Word1.OutputX = ox;
	SeCmdStretch.Word1.OpCode = 0x06;

	SeCmdStretch.Word2.ScalingAlgorithm = 0x01;
	SeCmdStretch.Word2.Reserved = 0x0;
	SeCmdStretch.Word2.StretchBitblit = 0x0;
	SeCmdStretch.Word2.directFB = 0x0;

	SeCmdStretch.Word2.OutputHeight = oh;
	SeCmdStretch.Word2.OutputWidth = ow;
    //printk("out x = %d, y = %d, w = %d, h = %d\n",SeCmdStretch.Word1.OutputX, SeCmdStretch.Word1.OutputY, ow, oh);

	SeCmdStretch.Word3.SrcAddressIndex = 0x1;
	SeCmdStretch.Word3.DestInterleaving = 0x0;
	SeCmdStretch.Word3.InputY = iy;
	SeCmdStretch.Word3.InputX = ix;

	SeCmdStretch.Word4.DestAddressIndex = 0x0;
	SeCmdStretch.Word4.InputHeight = ih;
	SeCmdStretch.Word4.InputWidth = iw;

    //printk("in x = %d, y = %d, w = %d, h = %d\n",SeCmdStretch.Word3.InputX, SeCmdStretch.Word3.InputY, iw, ih);
	SeCmdStretch.Word5.fb_sel_destclr_blend = 0x0f;

	SeCmdStretch.Word6.Reserved = 0x0f;
#if 0
	SeCmdStretch.Word7.Reserved = 0x0f;
	SeCmdStretch.Word8.Reserved = 0x0f;
#endif

	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdStretch, sizeof(SECMD_STRETCH));
	cmdLength += sizeof(SECMD_STRETCH)/4;
	instCount += 3;

	se_writecmd(cmdLength);
	se_checkfinish();

};
#endif



void Draw_message(char* msg) {
    int i=0, strsize=0, strshift = 0;
    if(msg == NULL) {
        printf("[%s] Get Null Pointer.\n",__func__);
        return;
    }

    _gtextwidthshift = SE_TEXT_SHIFT;  //reset text offset
    Draw_ClearMsg();
    strsize = strlen(msg);
    if(strsize == 0){
        return;
    }
    print_textdebug("Get String size = %d\n", strsize);
    for(i=0;i<strsize;i++){
        if(msg[i] >= 0x20 && msg[i] <= 0x43) {
            //ascii1
            print_textdebug("ascii0 %c, %02x\n", msg[i], msg[i]);
            Draw_Array(0, (int)msg[i] - 0x20, &strshift);
        }else if(msg[i] >= 0x44 && msg[i] <= 0x60) {
            //ascii2
            print_textdebug("ascii1 %c, %02x\n", msg[i], msg[i]);
            Draw_Array(1, (int)msg[i] - 0x44, &strshift);
        }else if(msg[i] >= 0x61 && msg[i] <= 0x7E) {
            //ascii3
            print_textdebug("ascii2 %c, %02x\n", msg[i], msg[i]);
            Draw_Array(2, (int)msg[i] - 0x61, &strshift);
        }else {
            //others, space
            print_textdebug("space , %c, %02x\n", msg[i], msg[i]);
            Draw_Array(0, 0, &strshift);
        }


    }
    flush_dcache_range(TEXT_SURFACE, TEXT_SURFACE + 1920*WORD_HEIGHT*4);
    //input x,y,w,h output x,y,w,h
    DRAW_STRETCH_COPY(0,0,strshift,WORD_HEIGHT, 150,180, strshift*2,WORD_HEIGHT*2);

    DisplayArea();

}




int DisplayArea(void) {

#ifdef CONFIG_SYS_GDMA_SUPPORT
#if defined(CONFIG_SYS_PANEL_PARAMETER)
	extern struct _PANEL_CONFIG_PARAMETER  default_panel_parameter;
#endif

	GDMA_PICTURE_OBJECT *picObj;
	PICTURE_LAYERS_OBJECT pic_layer;

	memset(&pic_layer, 0, sizeof(PICTURE_LAYERS_OBJECT));
	picObj = &pic_layer.layer[0].normal[0];

	pic_layer.layer_num = 1;
	pic_layer.layer[0].normal_num = 1;
	pic_layer.layer[0].onlineOrder = C0;

	picObj->src_type = SRC_NORMAL;
	picObj->format = VO_OSD_COLOR_FORMAT_RGBA8888_LITTLE;
	picObj->context = 0;
	picObj->key_en = 0;
	picObj->colorkey = -1;
	picObj->alpha = 0;
	picObj->x = 0;
	picObj->y = 0;
	picObj->width = SE_SURFACE_W;
	picObj->height = SE_SURFACE_H;
	picObj->address = SE_INSTALL_UI_SURFACE1_ADDR;
	picObj->pitch = picObj->width * 4;
	picObj->plane = VO_GRAPHIC_OSD1;
	picObj->dst_x = 0;
	picObj->dst_y = 0;

#if defined(CONFIG_SYS_PANEL_PARAMETER)
	picObj->dst_width = default_panel_parameter.iCONFIG_DISP_ACT_END_HPOS;
	picObj->dst_height = default_panel_parameter.iCONFIG_DISP_ACT_END_VPOS;
#else
	picObj->dst_width = 1920;
	picObj->dst_height = 1080;

#endif
	picObj->layer_used = 1;
	picObj->plane_ar.plane_alpha_a = 0xff;
	picObj->plane_ar.plane_alpha_r = 0xff;
	picObj->plane_gb.plane_alpha_g = 0xff;
	picObj->plane_gb.plane_alpha_b = 0xff;
	picObj->clear_x.value = 0;
	picObj->clear_y.value = 0;
	picObj->show = 1;
	picObj->rgb_order = VO_OSD_COLOR_RGB;
	picObj->syncInfo.syncstamp = 0;
	picObj->scale_factor = 1;
	GDMA_ReceivePicture(&pic_layer);
#endif
    return 0;
}
#if defined(SE_EXT)
void Draw_ClearMsg(void) {
	uint32_t cmdLength = 0;
	int instCount = 0;
	SECMD_WRITE_REGISTER SeCmdWriteRegister;
	SECMD_EXT_FMT_CONV se_ext_conv;

	rtd_outl(SE_CMDRPTR_0_reg, SE_CMD_BUFFER_START_ADDR);
	rtd_outl(SE_CMDWPTR_0_reg, SE_CMD_BUFFER_START_ADDR);


    //==================================================================================main message
	SeCmdWriteRegister.Word1.Reserved = 0x0;
	SeCmdWriteRegister.Word1.OpCode = 0x01;
	SeCmdWriteRegister.Word1.Clut = 0x0;
	SeCmdWriteRegister.Word1.RegisterAddress = 0xd0;
	SeCmdWriteRegister.Word2.RegisterValue = 0xFFFFCC99;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));

	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;
	se_ext_conv.Word1.rslt_argb_order = SE_EXT_RSLT_ORDER;
	se_ext_conv.Word2.rslt_out_x = 100;
	se_ext_conv.Word2.rslt_out_y = 140;
	se_ext_conv.Word3.rslt_addr_index = 0;

	se_ext_conv.Word4.src1_inpt_w = SE_SURFACE_W-200;
	se_ext_conv.Word4.src1_inpt_h = 300;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = 0;
	se_ext_conv.Word5.src1_inpt_y = 0;

	se_ext_conv.Word5.src1_argb_order = 0;
	se_ext_conv.Word5.src1_argb_order = 0x1d;
	se_ext_conv.Word1.src2_argb_order = 0x1d;

	se_ext_conv.Word6.src1_addr_index = 0;
	se_ext_conv.Word6.src1_inpt_sel = 1;
	se_ext_conv.Word7.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.Reserved = SEINFO_OPCODE_NOP;

	memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&se_ext_conv, sizeof(SECMD_EXT_FMT_CONV));
	cmdLength += sizeof(SECMD_EXT_FMT_CONV)/4;
	instCount += 2;


	//==================================================================================TEXT message
	SeCmdWriteRegister.Word1.Reserved = 0x0;
	SeCmdWriteRegister.Word1.OpCode = 0x01;
	SeCmdWriteRegister.Word1.Clut = 0x0;
	SeCmdWriteRegister.Word1.RegisterAddress = 0xd0;
	SeCmdWriteRegister.Word2.RegisterValue = 0xFFFFCC99;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;


	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));

	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;
	se_ext_conv.Word1.rslt_argb_order = SE_EXT_RSLT_ORDER;
	se_ext_conv.Word2.rslt_out_x = 0;
	se_ext_conv.Word2.rslt_out_y = 0;
	se_ext_conv.Word3.rslt_addr_index = 1;

	se_ext_conv.Word4.src1_inpt_w = SE_SURFACE_W-200;
	se_ext_conv.Word4.src1_inpt_h = 300;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = 0;
	se_ext_conv.Word5.src1_inpt_y = 0;

	se_ext_conv.Word5.src1_argb_order = 0;
	se_ext_conv.Word5.src1_argb_order = 0x1d;
	se_ext_conv.Word1.src2_argb_order = 0x1d;

	se_ext_conv.Word6.src1_addr_index = 0;
	se_ext_conv.Word6.src1_inpt_sel = 1;
	se_ext_conv.Word7.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.Reserved = SEINFO_OPCODE_NOP;

	memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&se_ext_conv, sizeof(SECMD_EXT_FMT_CONV));
	cmdLength += sizeof(SECMD_EXT_FMT_CONV)/4;
	instCount += 2;

	se_writecmd(cmdLength);
	se_checkfinish();


}
#else

void Draw_ClearMsg(void) {
	uint32_t cmdLength = 0;
	int instCount = 0;
	SECMD_WRITE_REGISTER SeCmdWriteRegister;
    SECMD_BITBLT SeCmdBlt;

    rtd_outl(SE_CMDRPTR_0_reg, SE_CMD_BUFFER_START_ADDR);
    rtd_outl(SE_CMDWPTR_0_reg, SE_CMD_BUFFER_START_ADDR);


    //==================================================================================main message
	SeCmdWriteRegister.Word1.Reserved = 0x0;
	SeCmdWriteRegister.Word1.OpCode = 0x01;
	SeCmdWriteRegister.Word1.Clut = 0x0;
	SeCmdWriteRegister.Word1.RegisterAddress = 0x90;
	SeCmdWriteRegister.Word2.RegisterValue = 0xFFFFCC99;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;


    //draw surface 1
    memset((void *)&SeCmdBlt , 0, sizeof(SECMD_BITBLT));
    SeCmdBlt.Word1.OpCode = 0x5;
    SeCmdBlt.Word1.Width = SE_SURFACE_W - 200;
    SeCmdBlt.Word1.Height = 300;
    SeCmdBlt.Word2.OutputX = 100;
    SeCmdBlt.Word2.OutputY = 140;
    SeCmdBlt.Word2.RopCode = 0x3;
    SeCmdBlt.Word3.RopMode = 0x1;
    SeCmdBlt.Word4.Reserved=0xf;

    memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdBlt, sizeof(SECMD_BITBLT));
    cmdLength += sizeof(SECMD_BITBLT)/4;
    instCount += 2;

    //==================================================================================TEXT message
	SeCmdWriteRegister.Word1.Reserved = 0x0;
	SeCmdWriteRegister.Word1.OpCode = 0x01;
	SeCmdWriteRegister.Word1.Clut = 0x0;
	SeCmdWriteRegister.Word1.RegisterAddress = 0x90;
	SeCmdWriteRegister.Word2.RegisterValue = 0xFFFFCC99;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;


    //draw surface 1
    memset((void *)&SeCmdBlt , 0, sizeof(SECMD_BITBLT));
    SeCmdBlt.Word1.OpCode = 0x5;
    SeCmdBlt.Word1.Width = SE_SURFACE_W - 200;
    SeCmdBlt.Word1.Height = 300;
    SeCmdBlt.Word2.OutputX = 0;
    SeCmdBlt.Word2.OutputY = 0;
    SeCmdBlt.Word2.RopCode = 0x3;
	SeCmdBlt.Word2.DestAddressIndex = 0x1;
    SeCmdBlt.Word3.RopMode = 0x1;
    SeCmdBlt.Word4.Reserved=0xf;

    memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdBlt, sizeof(SECMD_BITBLT));
    cmdLength += sizeof(SECMD_BITBLT)/4;
    instCount += 2;

	se_writecmd(cmdLength);
	se_checkfinish();
}
#endif
#if defined(SE_EXT)
void Draw_bg(void) {
	uint32_t cmdLength = 0;
	int instCount = 0;


	SECMD_WRITE_REGISTER SeCmdWriteRegister;
	SEREG_PITCH SeRegPitch;
	SEREG_CLR_FMT SeRegColorFormat;
	SECMD_EXT_FMT_CONV se_ext_conv;

	/*SE_SetColorFormat*/
	SeRegColorFormat.Value=0x0;
	SeRegColorFormat.Fields.Format=0x7;
	SeRegColorFormat.Fields.write_enable1 = 0x0;
	SeRegColorFormat.Fields.alpha_loc = 0x0; /*RIGHTMOST*/
	SeRegColorFormat.Fields.write_enable2 = 0x00;
	SeRegColorFormat.Fields.alpha_loc2 = 0x0; /*RIGHTMOST*/
	SeRegColorFormat.Fields.write_enable5 = 0x00;
	SeRegColorFormat.Fields.big_endian_o = 0x0;
	SeRegColorFormat.Fields.write_enable8 = 0x0;

	SeCmdWriteRegister.Word1.Reserved = 0x0;
	SeCmdWriteRegister.Word1.OpCode = 0x01;
	SeCmdWriteRegister.Word1.Clut = 0x0;
	SeCmdWriteRegister.Word1.RegisterAddress = 0x60;
	SeCmdWriteRegister.Word2.RegisterValue = SeRegColorFormat.Value;


	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	/*SE_SetDmaInfo - DMA0*/
	SeCmdWriteRegister.Word1.RegisterAddress = 0xB80;
	SeCmdWriteRegister.Word2.RegisterValue = SE_INSTALL_UI_SURFACE1_ADDR;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	SeRegPitch.Value = 0x0;
	SeRegPitch.Fields.pitch = SE_SURFACE_PITCH;
	SeRegPitch.Fields.interleave = 0x0;
	SeCmdWriteRegister.Word1.RegisterAddress = 0xC80;
	SeCmdWriteRegister.Word2.RegisterValue = SeRegPitch.Value;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	/*SE_SetDmaInfo - DMA1*/
	SeCmdWriteRegister.Word1.RegisterAddress = 0xB90;
	SeCmdWriteRegister.Word2.RegisterValue = SE_INSTALL_UI_SURFACE2_ADDR;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	SeCmdWriteRegister.Word1.RegisterAddress = 0xC90;

	SeCmdWriteRegister.Word2.RegisterValue = SeRegPitch.Value;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;


    //========================================================================================================clear surface 1,2
	SeCmdWriteRegister.Word1.RegisterAddress = 0xd0;
	SeCmdWriteRegister.Word2.RegisterValue = 0xfff0f0f0;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	//draw surface 1
	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));

	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;
	se_ext_conv.Word1.rslt_argb_order = SE_EXT_RSLT_ORDER;
	se_ext_conv.Word2.rslt_out_x = 0;
	se_ext_conv.Word2.rslt_out_y = 0;
	se_ext_conv.Word3.rslt_addr_index = 0;

	se_ext_conv.Word4.src1_inpt_w = SE_SURFACE_W;
	se_ext_conv.Word4.src1_inpt_h = SE_SURFACE_H;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = 0;
	se_ext_conv.Word5.src1_inpt_y = 0;

	se_ext_conv.Word5.src1_argb_order = 0;
	se_ext_conv.Word5.src1_argb_order = 0x1d;
	se_ext_conv.Word1.src2_argb_order = 0x1d;

	se_ext_conv.Word6.src1_addr_index = 0;
	se_ext_conv.Word6.src1_inpt_sel = 1;
	se_ext_conv.Word7.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.Reserved = SEINFO_OPCODE_NOP;

	memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&se_ext_conv, sizeof(SECMD_EXT_FMT_CONV));
	cmdLength += sizeof(SECMD_EXT_FMT_CONV)/4;
	instCount += 2;

	//========================================================================================================draw bg surface 1,2
	SeCmdWriteRegister.Word1.RegisterAddress = 0xd0;
	SeCmdWriteRegister.Word2.RegisterValue = 0xffff6600;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;


	//draw surface 1
	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));
	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;
	se_ext_conv.Word1.rslt_argb_order = SE_EXT_RSLT_ORDER;
	se_ext_conv.Word2.rslt_out_x = 0;
	se_ext_conv.Word2.rslt_out_y = 80;
	se_ext_conv.Word3.rslt_addr_index = 0;

	se_ext_conv.Word4.src1_inpt_w = SE_SURFACE_W;
	se_ext_conv.Word4.src1_inpt_h = SE_SURFACE_H-160;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = 0;
	se_ext_conv.Word5.src1_inpt_y = 0;

	se_ext_conv.Word5.src1_argb_order = 0;
	se_ext_conv.Word5.src1_argb_order = 0x1d;
	se_ext_conv.Word1.src2_argb_order = 0x1d;

	se_ext_conv.Word6.src1_addr_index = 0;
	se_ext_conv.Word6.src1_inpt_sel = 1;
	se_ext_conv.Word7.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.Reserved = SEINFO_OPCODE_NOP;

	memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&se_ext_conv, sizeof(SECMD_EXT_FMT_CONV));
	cmdLength += sizeof(SECMD_EXT_FMT_CONV)/4;
	instCount += 2;


    //========================================================================================================draw progress surface 1,2
	SeCmdWriteRegister.Word1.RegisterAddress = 0xd0;
	SeCmdWriteRegister.Word2.RegisterValue = 0xFFD0B3B3;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));
	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;
	se_ext_conv.Word1.rslt_argb_order = SE_EXT_RSLT_ORDER;
	se_ext_conv.Word2.rslt_out_x = 120;
	se_ext_conv.Word2.rslt_out_y = 700;
	se_ext_conv.Word3.rslt_addr_index = 0;

	se_ext_conv.Word4.src1_inpt_w = SE_SURFACE_W - 240;
	se_ext_conv.Word4.src1_inpt_h = 220;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = 0;
	se_ext_conv.Word5.src1_inpt_y = 0;

	se_ext_conv.Word5.src1_argb_order = 0;
	se_ext_conv.Word5.src1_argb_order = 0x1d;
	se_ext_conv.Word1.src2_argb_order = 0x1d;

	se_ext_conv.Word6.src1_addr_index = 0;
	se_ext_conv.Word6.src1_inpt_sel = 1;
	se_ext_conv.Word7.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.Reserved = SEINFO_OPCODE_NOP;

	memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&se_ext_conv, sizeof(SECMD_EXT_FMT_CONV));
	cmdLength += sizeof(SECMD_EXT_FMT_CONV)/4;
	instCount += 2;

	SeCmdWriteRegister.Word1.RegisterAddress = 0xd0;
	SeCmdWriteRegister.Word2.RegisterValue = 0xFFf0f0f0;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));
	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;
	se_ext_conv.Word1.rslt_argb_order = SE_EXT_RSLT_ORDER;
	se_ext_conv.Word2.rslt_out_x = 120+20;
	se_ext_conv.Word2.rslt_out_y = 700+20;
	se_ext_conv.Word3.rslt_addr_index = 0;

	se_ext_conv.Word4.src1_inpt_w = SE_SURFACE_W - 240 - 40;
	se_ext_conv.Word4.src1_inpt_h = 220-40;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = 0;
	se_ext_conv.Word5.src1_inpt_y = 0;

	se_ext_conv.Word5.src1_argb_order = 0;
	se_ext_conv.Word5.src1_argb_order = 0x1d;
	se_ext_conv.Word1.src2_argb_order = 0x1d;

	se_ext_conv.Word6.src1_addr_index = 0;
	se_ext_conv.Word6.src1_inpt_sel = 1;
	se_ext_conv.Word7.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.Reserved = SEINFO_OPCODE_NOP;

	memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&se_ext_conv, sizeof(SECMD_EXT_FMT_CONV));
	cmdLength += sizeof(SECMD_EXT_FMT_CONV)/4;
	instCount += 2;


	//printf("Ready to write %d Words, %d inst\n", cmdLength, instCount);
	se_writecmd(cmdLength);
	se_checkfinish();

}
#else
void Draw_bg(void) {
	uint32_t cmdLength = 0;
	int instCount = 0;

	SECMD_WRITE_REGISTER SeCmdWriteRegister;
	SEREG_PITCH SeRegPitch;
	SEREG_CLR_FMT SeRegColorFormat;
    SECMD_BITBLT SeCmdBlt;

	/*SE_SetColorFormat*/
	SeRegColorFormat.Value=0x0;
	SeRegColorFormat.Fields.Format=0x7;
    SeRegColorFormat.Fields.write_enable1 = 0x1;
	SeRegColorFormat.Fields.alpha_loc = 0x0; /*RIGHTMOST*/
	SeRegColorFormat.Fields.write_enable2 = 0x01;
	SeRegColorFormat.Fields.alpha_loc2 = 0x0; /*RIGHTMOST*/
	SeRegColorFormat.Fields.write_enable5 = 0x01;
    SeRegColorFormat.Fields.big_endian_o = 0x1;
    SeRegColorFormat.Fields.write_enable8 = 0x1;

	SeCmdWriteRegister.Word1.Reserved = 0x0;
	SeCmdWriteRegister.Word1.OpCode = 0x01;
	SeCmdWriteRegister.Word1.Clut = 0x0;
	SeCmdWriteRegister.Word1.RegisterAddress = 0x60;
	SeCmdWriteRegister.Word2.RegisterValue = SeRegColorFormat.Value;


	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	/*SE_SetDmaInfo - DMA0*/
	SeCmdWriteRegister.Word1.RegisterAddress = 0xD0;
	SeCmdWriteRegister.Word2.RegisterValue = SE_INSTALL_UI_SURFACE1_ADDR;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	SeRegPitch.Value = 0x0;
	SeRegPitch.Fields.pitch = SE_SURFACE_PITCH;
	SeRegPitch.Fields.interleave = 0x0;
	SeCmdWriteRegister.Word1.RegisterAddress = 0xE0;
	SeCmdWriteRegister.Word2.RegisterValue = SeRegPitch.Value;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	/*SE_SetDmaInfo - DMA1*/
	SeCmdWriteRegister.Word1.RegisterAddress = 0xD0 + (1 << 2);
	SeCmdWriteRegister.Word2.RegisterValue = SE_INSTALL_UI_SURFACE2_ADDR;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	SeCmdWriteRegister.Word1.RegisterAddress = 0xE0 + (1 << 2);

	SeCmdWriteRegister.Word2.RegisterValue = SeRegPitch.Value;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
    //printf("write reg word 1  %08x \n", SeCmdWriteRegister.Word1);
    //printf("write reg word 2  %08x \n", SeCmdWriteRegister.Word2);

    //printf("write reg word 1 shift = %08x  value = %08x \n", (uint32_t*)(_gCmdBuffer+cmdLength), *(uint32_t*)(_gCmdBuffer+cmdLength));
    //printf("write reg word 1 shift = %08x  value = %08x \n", (uint32_t*)(_gCmdBuffer+cmdLength+1), *(uint32_t*)(_gCmdBuffer+cmdLength+1));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;


    //========================================================================================================clear surface 1,2
	SeCmdWriteRegister.Word1.RegisterAddress = 0xd0;
	SeCmdWriteRegister.Word2.RegisterValue = 0xfff0f0f0;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;


    //draw surface 1
    memset((void *)&SeCmdBlt , 0, sizeof(SECMD_BITBLT));
    SeCmdBlt.Word1.OpCode = 0x5;
    SeCmdBlt.Word1.Width = SE_SURFACE_W;
    SeCmdBlt.Word1.Height = SE_SURFACE_H;
    SeCmdBlt.Word2.RopCode = 0x3;
    SeCmdBlt.Word3.RopMode = 0x1;
    SeCmdBlt.Word4.Reserved=0xf;

    memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdBlt, sizeof(SECMD_BITBLT));
    cmdLength += sizeof(SECMD_BITBLT)/4;
    instCount += 2;

    //========================================================================================================draw bg surface 1,2
	SeCmdWriteRegister.Word1.RegisterAddress = 0xd0;
	SeCmdWriteRegister.Word2.RegisterValue = 0xffff6600;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;


    //draw surface 1
    memset((void *)&SeCmdBlt , 0, sizeof(SECMD_BITBLT));
    SeCmdBlt.Word1.OpCode = 0x5;
    SeCmdBlt.Word1.Width = SE_SURFACE_W;
    SeCmdBlt.Word1.Height = SE_SURFACE_H-160;
    SeCmdBlt.Word2.OutputY = 80;
    SeCmdBlt.Word2.RopCode = 0x3;
    SeCmdBlt.Word3.RopMode = 0x1;
    SeCmdBlt.Word4.Reserved=0xf;

    memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdBlt, sizeof(SECMD_BITBLT));
    cmdLength += sizeof(SECMD_BITBLT)/4;
    instCount += 2;


    //========================================================================================================draw progress surface 1,2
	SeCmdWriteRegister.Word1.RegisterAddress = 0xd0;
	SeCmdWriteRegister.Word2.RegisterValue = 0xFFD0B3B3;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;


    //draw surface 1
    memset((void *)&SeCmdBlt , 0, sizeof(SECMD_BITBLT));
    SeCmdBlt.Word1.OpCode = 0x5;
    SeCmdBlt.Word1.Width = SE_SURFACE_W - 240;
    SeCmdBlt.Word1.Height = 220;
    SeCmdBlt.Word2.OutputX = 120;
    SeCmdBlt.Word2.OutputY = 700;
    SeCmdBlt.Word2.RopCode = 0x3;
    SeCmdBlt.Word3.RopMode = 0x1;
    SeCmdBlt.Word4.Reserved=0xf;

    memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdBlt, sizeof(SECMD_BITBLT));
    cmdLength += sizeof(SECMD_BITBLT)/4;
    instCount += 2;

	SeCmdWriteRegister.Word1.RegisterAddress = 0xd0;
	SeCmdWriteRegister.Word2.RegisterValue = 0xFFf0f0f0;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;


    //draw surface 1
    memset((void *)&SeCmdBlt , 0, sizeof(SECMD_BITBLT));
    SeCmdBlt.Word1.OpCode = 0x5;
    SeCmdBlt.Word1.Width = SE_SURFACE_W - 240 - 40;
    SeCmdBlt.Word1.Height = 220-40;
    SeCmdBlt.Word2.OutputX = 120+20;
    SeCmdBlt.Word2.OutputY = 700+20;
    SeCmdBlt.Word2.RopCode = 0x3;
    SeCmdBlt.Word3.RopMode = 0x1;
    SeCmdBlt.Word4.Reserved=0xf;

    memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdBlt, sizeof(SECMD_BITBLT));
    cmdLength += sizeof(SECMD_BITBLT)/4;
    instCount += 2;

    //printf("Ready to write %d Words, %d inst\n", cmdLength, instCount);
	se_writecmd(cmdLength);
	se_checkfinish();

}
#endif
#if defined(SE_EXT)

void Draw_progress(int percent) {
	uint32_t cmdLength = 0;
	int instCount = 0;
	int drawwidth = 0;
	SECMD_WRITE_REGISTER SeCmdWriteRegister;
	SECMD_EXT_FMT_CONV se_ext_conv;

	if(percent >=100){
		percent = 100;
		drawwidth = SE_SURFACE_W - 280;
	}else if(percent <= 0) {
		percent  = 0;
		return;
	}else{
		drawwidth = (SE_SURFACE_W - 280)/100*percent;
	}

	rtd_outl(SE_CMDRPTR_0_reg, SE_CMD_BUFFER_START_ADDR);
	rtd_outl(SE_CMDWPTR_0_reg, SE_CMD_BUFFER_START_ADDR);

	SeCmdWriteRegister.Word1.Reserved = 0x0;
	SeCmdWriteRegister.Word1.OpCode = 0x01;
	SeCmdWriteRegister.Word1.Clut = 0x0;
	SeCmdWriteRegister.Word1.RegisterAddress = 0xd0;
	SeCmdWriteRegister.Word2.RegisterValue = 0xFFFFFF00;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;

	memset((void *)&se_ext_conv, 0, sizeof(SECMD_EXT_FMT_CONV));

	se_ext_conv.Word1.opcode = 0xb;
	se_ext_conv.Word1.rslt_fmt = 0xc;
	se_ext_conv.Word1.rslt_argb_order = SE_EXT_RSLT_ORDER;
	se_ext_conv.Word2.rslt_out_x = 120+20;
	se_ext_conv.Word2.rslt_out_y = 700+20;
	se_ext_conv.Word3.rslt_addr_index = 0;

	se_ext_conv.Word4.src1_inpt_w = drawwidth;
	se_ext_conv.Word4.src1_inpt_h = 220-40;
	se_ext_conv.Word4.src1_fmt = 0xc;

	se_ext_conv.Word5.src1_inpt_x = 0;
	se_ext_conv.Word5.src1_inpt_y = 0;

	se_ext_conv.Word5.src1_argb_order = 0;
	se_ext_conv.Word5.src1_argb_order = 0x1d;
	se_ext_conv.Word1.src2_argb_order = 0x1d;

	se_ext_conv.Word6.src1_addr_index = 0;
	se_ext_conv.Word6.src1_inpt_sel = 1;
	se_ext_conv.Word7.Reserved = SEINFO_OPCODE_NOP;
	se_ext_conv.Word8.Reserved = SEINFO_OPCODE_NOP;

	memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&se_ext_conv, sizeof(SECMD_EXT_FMT_CONV));
	cmdLength += sizeof(SECMD_EXT_FMT_CONV)/4;
	instCount += 2;


	printf("Ready to write %d Words, %d inst\n", cmdLength, instCount);
	se_writecmd(cmdLength);
	se_checkfinish();
	DisplayArea();

}
#else
void Draw_progress(int percent) {
	uint32_t cmdLength = 0;
	int instCount = 0;
    int drawwidth = 0;
	SECMD_WRITE_REGISTER SeCmdWriteRegister;
    SECMD_BITBLT SeCmdBlt;
    if(percent >=100){
        percent = 100;
        drawwidth = SE_SURFACE_W - 280;
    }else if(percent <= 0) {
        percent  = 0;
        return;
    }else{
        drawwidth = (SE_SURFACE_W - 280)/100*percent;
    }

	rtd_outl(SE_CMDRPTR_0_reg, SE_CMD_BUFFER_START_ADDR);
	rtd_outl(SE_CMDWPTR_0_reg, SE_CMD_BUFFER_START_ADDR);

	SeCmdWriteRegister.Word1.Reserved = 0x0;
	SeCmdWriteRegister.Word1.OpCode = 0x01;
	SeCmdWriteRegister.Word1.Clut = 0x0;
	SeCmdWriteRegister.Word1.RegisterAddress = 0xd0;
	SeCmdWriteRegister.Word2.RegisterValue = 0xFFFFFF00;
	memcpy((uint32_t*)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdWriteRegister, sizeof(SECMD_WRITE_REGISTER));
	cmdLength += sizeof(SECMD_WRITE_REGISTER)/4;
	instCount += 1;


    //draw surface 1
    memset((void *)&SeCmdBlt , 0, sizeof(SECMD_BITBLT));
    SeCmdBlt.Word1.OpCode = 0x5;
    SeCmdBlt.Word1.Width = drawwidth;
    SeCmdBlt.Word1.Height = 220-40;
    SeCmdBlt.Word2.OutputX = 120+20;
    SeCmdBlt.Word2.OutputY = 700+20;
    SeCmdBlt.Word2.RopCode = 0x3;
    SeCmdBlt.Word3.RopMode = 0x1;
    SeCmdBlt.Word4.Reserved=0xf;

    memcpy((uint32_t *)_gCmdBuffer+cmdLength,(uint32_t*)&SeCmdBlt, sizeof(SECMD_BITBLT));
    cmdLength += sizeof(SECMD_BITBLT)/4;
    instCount += 2;

    printf("Ready to write %d Words, %d inst\n", cmdLength, instCount);
	se_writecmd(cmdLength);
	se_checkfinish();
    DisplayArea();

}
#endif

int se_module_test(void) {
    if (se_module_init() == -1) {
        printf("%s...Fail!", __func__);
        return -1;
    }
    Draw_bg();
    Draw_ClearMsg();

    Draw_message(" !\"#$%&'()*+,-./0123456789:;<=>?@ABC");
    mdelay(2000);
    Draw_message("DEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`");
    mdelay(2000);
    Draw_message("abcdefghijklmnopqrstuvwxyz{|}~");
    mdelay(2000);
    DisplayArea();

    Draw_message("Test$%& 10.. ");
    Draw_progress(10);
    mdelay(500);
    Draw_message("Test()- 30.. ");
    Draw_progress(30);
    mdelay(500);
    Draw_message("Test~`1! 50.. ");
    Draw_progress(50);
    mdelay(500);
    Draw_message("Test~*[} 70.. ");
    Draw_progress(70);
    mdelay(500);
    Draw_message("Test~*}\\'\" 90.. ");
    Draw_progress(90);
    mdelay(500);
    Draw_message("Test+=_- 100.. ");
    Draw_progress(100);


    return 0;
};

int do_seui(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = 0;
	se_module_test();
	return ret;
}

U_BOOT_CMD(
	callseui,	4,	1,	do_seui,
	"se ui",
	"se ui"
);
